@@ -53,7 +53,7 @@ impl OperationTemplateLanguageExtension for HexCounter {
         table.operation_methods.insert(
             "num_digits_in_id",
             |_language, _build_context, property, call| {
-                template_parser::expect_no_arguments(call)?;
+                call.expect_no_arguments()?;
                 Ok(L::wrap_integer(
                     property.map(|operation| num_digits_in_id(operation.id())),
                 ))
@@ -62,7 +62,7 @@ impl OperationTemplateLanguageExtension for HexCounter {
         table.operation_methods.insert(
             "num_char_in_id",
             |_language, _build_context, property, call| {
-                let [string_arg] = template_parser::expect_exact_arguments(call)?;
+                let [string_arg] = call.expect_exact_arguments()?;
                 let char_arg =
                     template_parser::expect_string_literal_with(string_arg, |string, span| {
                         let chars: Vec<_> = string.chars().collect();
