@@ -26,7 +26,7 @@ use jj_lib::op_heads_store::OpHeadResolutionError;
 use jj_lib::op_store::OpStoreError;
 use jj_lib::op_walk::OpsetEvaluationError;
 use jj_lib::repo::{CheckOutCommitError, EditCommitError, RepoLoaderError, RewriteRootCommit};
-use jj_lib::repo_path::{FsPathParseError, RepoPathBuf};
+use jj_lib::repo_path::{RepoPathBuf, UiPathParseError};
 use jj_lib::revset::{
     RevsetEvaluationError, RevsetParseError, RevsetParseErrorKind, RevsetResolutionError,
 };
@@ -36,10 +36,9 @@ use jj_lib::working_copy::{ResetError, SnapshotError, WorkingCopyStateError};
 use jj_lib::workspace::WorkspaceInitError;
 use thiserror::Error;
 
+use crate::diff_util::DiffRenderError;
 use crate::formatter::{FormatRecorder, Formatter};
-use crate::merge_tools::{
-    ConflictResolveError, DiffEditError, DiffGenerateError, MergeToolConfigError,
-};
+use crate::merge_tools::{ConflictResolveError, DiffEditError, MergeToolConfigError};
 use crate::revset_util::UserRevsetEvaluationError;
 use crate::template_parser::{TemplateParseError, TemplateParseErrorKind};
 use crate::ui::Ui;
@@ -361,9 +360,14 @@ impl From<DiffEditError> for CommandError {
     }
 }
 
-impl From<DiffGenerateError> for CommandError {
-    fn from(err: DiffGenerateError) -> Self {
-        user_error_with_message("Failed to generate diff", err)
+impl From<DiffRenderError> for CommandError {
+    fn from(err: DiffRenderError) -> Self {
+        match err {
+            DiffRenderError::DiffGenerate(_) => user_error(err),
+            DiffRenderError::Backend(err) => err.into(),
+            DiffRenderError::AccessDenied { .. } => user_error(err),
+            DiffRenderError::Io(err) => err.into(),
+        }
     }
 }
 
@@ -375,7 +379,20 @@ impl From<ConflictResolveError> for CommandError {
 
 impl From<MergeToolConfigError> for CommandError {
     fn from(err: MergeToolConfigError) -> Self {
-        user_error_with_message("Failed to load tool configuration", err)
+        match &err {
+            MergeToolConfigError::MergeArgsNotConfigured { tool_name } => {
+                let tool_name = tool_name.clone();
+                user_error_with_hint(
+                    err,
+                    format!(
+                        "To use `{tool_name}` as a merge tool, the config \
+                         `merge-tools.{tool_name}.merge-args` must be defined (see docs for \
+                         details)"
+                    ),
+                )
+            }
+            _ => user_error_with_message("Failed to load tool configuration", err),
+        }
     }
 }
 
@@ -433,15 +450,7 @@ impl From<RevsetEvaluationError> for CommandError {
 
 impl From<FilesetParseError> for CommandError {
     fn from(err: FilesetParseError) -> Self {
-        let hint = match err.kind() {
-            FilesetParseErrorKind::NoSuchFunction {
-                name: _,
-                candidates,
-            } => format_similarity_hint(candidates),
-            FilesetParseErrorKind::InvalidArguments { .. }
-            | FilesetParseErrorKind::Expression(_) => find_source_parse_error_hint(&err),
-            _ => None,
-        };
+        let hint = fileset_parse_error_hint(&err);
         let mut cmd_err =
             user_error_with_message(format!("Failed to parse fileset: {}", err.kind()), err);
         cmd_err.extend_hints(hint);
@@ -451,33 +460,7 @@ impl From<FilesetParseError> for CommandError {
 
 impl From<RevsetParseError> for CommandError {
     fn from(err: RevsetParseError) -> Self {
-        // Only for the bottom error, which is usually the root cause
-        let bottom_err = iter::successors(Some(&err), |e| e.origin()).last().unwrap();
-        let hint = match bottom_err.kind() {
-            RevsetParseErrorKind::NotPrefixOperator {
-                op: _,
-                similar_op,
-                description,
-            }
-            | RevsetParseErrorKind::NotPostfixOperator {
-                op: _,
-                similar_op,
-                description,
-            }
-            | RevsetParseErrorKind::NotInfixOperator {
-                op: _,
-                similar_op,
-                description,
-            } => Some(format!("Did you mean '{similar_op}' for {description}?")),
-            RevsetParseErrorKind::NoSuchFunction {
-                name: _,
-                candidates,
-            } => format_similarity_hint(candidates),
-            RevsetParseErrorKind::InvalidFunctionArguments { .. } => {
-                find_source_parse_error_hint(bottom_err)
-            }
-            _ => None,
-        };
+        let hint = revset_parse_error_hint(&err);
         let mut cmd_err =
             user_error_with_message(format!("Failed to parse revset: {}", err.kind()), err);
         cmd_err.extend_hints(hint);
@@ -487,18 +470,7 @@ impl From<RevsetParseError> for CommandError {
 
 impl From<RevsetResolutionError> for CommandError {
     fn from(err: RevsetResolutionError) -> Self {
-        let hint = match &err {
-            RevsetResolutionError::NoSuchRevision {
-                name: _,
-                candidates,
-            } => format_similarity_hint(candidates),
-            RevsetResolutionError::EmptyString
-            | RevsetResolutionError::WorkspaceMissingWorkingCopy { .. }
-            | RevsetResolutionError::AmbiguousCommitIdPrefix(_)
-            | RevsetResolutionError::AmbiguousChangeIdPrefix(_)
-            | RevsetResolutionError::StoreError(_)
-            | RevsetResolutionError::Other(_) => None,
-        };
+        let hint = revset_resolution_error_hint(&err);
         let mut cmd_err = user_error(err);
         cmd_err.extend_hints(hint);
         cmd_err
@@ -516,18 +488,7 @@ impl From<UserRevsetEvaluationError> for CommandError {
 
 impl From<TemplateParseError> for CommandError {
     fn from(err: TemplateParseError) -> Self {
-        // Only for the bottom error, which is usually the root cause
-        let bottom_err = iter::successors(Some(&err), |e| e.origin()).last().unwrap();
-        let hint = match bottom_err.kind() {
-            TemplateParseErrorKind::NoSuchKeyword { candidates, .. }
-            | TemplateParseErrorKind::NoSuchFunction { candidates, .. }
-            | TemplateParseErrorKind::NoSuchMethod { candidates, .. } => {
-                format_similarity_hint(candidates)
-            }
-            TemplateParseErrorKind::InvalidArguments { .. }
-            | TemplateParseErrorKind::Expression(_) => find_source_parse_error_hint(bottom_err),
-            _ => None,
-        };
+        let hint = template_parse_error_hint(&err);
         let mut cmd_err =
             user_error_with_message(format!("Failed to parse template: {}", err.kind()), err);
         cmd_err.extend_hints(hint);
@@ -535,8 +496,8 @@ impl From<TemplateParseError> for CommandError {
     }
 }
 
-impl From<FsPathParseError> for CommandError {
-    fn from(err: FsPathParseError) -> Self {
+impl From<UiPathParseError> for CommandError {
+    fn from(err: UiPathParseError) -> Self {
         user_error(err)
     }
 }
@@ -572,8 +533,18 @@ fn find_source_parse_error_hint(err: &dyn error::Error) -> Option<String> {
     let source = err.source()?;
     if let Some(source) = source.downcast_ref() {
         file_pattern_parse_error_hint(source)
+    } else if let Some(source) = source.downcast_ref() {
+        fileset_parse_error_hint(source)
+    } else if let Some(source) = source.downcast_ref() {
+        revset_parse_error_hint(source)
+    } else if let Some(source) = source.downcast_ref() {
+        revset_resolution_error_hint(source)
+    } else if let Some(UserRevsetEvaluationError::Resolution(source)) = source.downcast_ref() {
+        revset_resolution_error_hint(source)
     } else if let Some(source) = source.downcast_ref() {
         string_pattern_parse_error_hint(source)
+    } else if let Some(source) = source.downcast_ref() {
+        template_parse_error_hint(source)
     } else {
         None
     }
@@ -583,14 +554,73 @@ fn file_pattern_parse_error_hint(err: &FilePatternParseError) -> Option<String>
     match err {
         FilePatternParseError::InvalidKind(_) => None,
         // Suggest root:"<path>" if input can be parsed as repo-relative path
-        FilePatternParseError::FsPath(e) => RepoPathBuf::from_relative_path(&e.input)
-            .ok()
-            .map(|path| format!(r#"Consider using root:{path:?} to specify repo-relative path"#)),
+        FilePatternParseError::UiPath(UiPathParseError::Fs(e)) => {
+            RepoPathBuf::from_relative_path(&e.input).ok().map(|path| {
+                format!(r#"Consider using root:{path:?} to specify repo-relative path"#)
+            })
+        }
         FilePatternParseError::RelativePath(_) => None,
         FilePatternParseError::GlobPattern(_) => None,
     }
 }
 
+fn fileset_parse_error_hint(err: &FilesetParseError) -> Option<String> {
+    match err.kind() {
+        FilesetParseErrorKind::NoSuchFunction {
+            name: _,
+            candidates,
+        } => format_similarity_hint(candidates),
+        FilesetParseErrorKind::InvalidArguments { .. } | FilesetParseErrorKind::Expression(_) => {
+            find_source_parse_error_hint(&err)
+        }
+        _ => None,
+    }
+}
+
+fn revset_parse_error_hint(err: &RevsetParseError) -> Option<String> {
+    // Only for the bottom error, which is usually the root cause
+    let bottom_err = iter::successors(Some(err), |e| e.origin()).last().unwrap();
+    match bottom_err.kind() {
+        RevsetParseErrorKind::NotPrefixOperator {
+            op: _,
+            similar_op,
+            description,
+        }
+        | RevsetParseErrorKind::NotPostfixOperator {
+            op: _,
+            similar_op,
+            description,
+        }
+        | RevsetParseErrorKind::NotInfixOperator {
+            op: _,
+            similar_op,
+            description,
+        } => Some(format!("Did you mean '{similar_op}' for {description}?")),
+        RevsetParseErrorKind::NoSuchFunction {
+            name: _,
+            candidates,
+        } => format_similarity_hint(candidates),
+        RevsetParseErrorKind::InvalidFunctionArguments { .. }
+        | RevsetParseErrorKind::Expression(_) => find_source_parse_error_hint(bottom_err),
+        _ => None,
+    }
+}
+
+fn revset_resolution_error_hint(err: &RevsetResolutionError) -> Option<String> {
+    match err {
+        RevsetResolutionError::NoSuchRevision {
+            name: _,
+            candidates,
+        } => format_similarity_hint(candidates),
+        RevsetResolutionError::EmptyString
+        | RevsetResolutionError::WorkspaceMissingWorkingCopy { .. }
+        | RevsetResolutionError::AmbiguousCommitIdPrefix(_)
+        | RevsetResolutionError::AmbiguousChangeIdPrefix(_)
+        | RevsetResolutionError::StoreError(_)
+        | RevsetResolutionError::Other(_) => None,
+    }
+}
+
 fn string_pattern_parse_error_hint(err: &StringPatternParseError) -> Option<String> {
     match err {
         StringPatternParseError::InvalidKind(_) => {
@@ -600,6 +630,22 @@ fn string_pattern_parse_error_hint(err: &StringPatternParseError) -> Option<Stri
     }
 }
 
+fn template_parse_error_hint(err: &TemplateParseError) -> Option<String> {
+    // Only for the bottom error, which is usually the root cause
+    let bottom_err = iter::successors(Some(err), |e| e.origin()).last().unwrap();
+    match bottom_err.kind() {
+        TemplateParseErrorKind::NoSuchKeyword { candidates, .. }
+        | TemplateParseErrorKind::NoSuchFunction { candidates, .. }
+        | TemplateParseErrorKind::NoSuchMethod { candidates, .. } => {
+            format_similarity_hint(candidates)
+        }
+        TemplateParseErrorKind::InvalidArguments { .. } | TemplateParseErrorKind::Expression(_) => {
+            find_source_parse_error_hint(bottom_err)
+        }
+        _ => None,
+    }
+}
+
 const BROKEN_PIPE_EXIT_CODE: u8 = 3;
 
 pub(crate) fn handle_command_result(ui: &mut Ui, result: Result<(), CommandError>) -> ExitCode {
