@@ -12,26 +12,23 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::collections::HashSet;
+use std::collections::{HashMap, HashSet};
 use std::fmt;
 use std::io::Write as _;
 
 use clap::builder::NonEmptyStringValueParser;
 use itertools::Itertools;
-use jj_lib::backend::CommitId;
 use jj_lib::git;
 use jj_lib::object_id::ObjectId;
 use jj_lib::op_store::{RefTarget, RemoteRef};
 use jj_lib::repo::Repo;
-use jj_lib::revset::{self, RevsetExpression};
+use jj_lib::revset::RevsetExpression;
 use jj_lib::str_util::StringPattern;
 use jj_lib::view::View;
 
-use crate::cli_util::{
-    parse_string_pattern, user_error, user_error_with_hint, CommandError, CommandHelper,
-    RemoteBranchName, RemoteBranchNamePattern, RevisionArg,
-};
-use crate::formatter::Formatter;
+use crate::cli_util::{CommandHelper, RemoteBranchName, RemoteBranchNamePattern, RevisionArg};
+use crate::command_error::{user_error, user_error_with_hint, CommandError};
+use crate::commit_templater::{CommitTemplateLanguage, RefName};
 use crate::ui::Ui;
 
 /// Manage branches.
@@ -78,7 +75,7 @@ pub struct BranchDeleteArgs {
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// select branches by wildcard pattern. For details, see
     /// https://github.com/martinvonz/jj/blob/main/docs/revsets.md#string-patterns.
-    #[arg(required_unless_present_any(&["glob"]), value_parser = parse_string_pattern)]
+    #[arg(required_unless_present_any(&["glob"]), value_parser = StringPattern::parse)]
     pub names: Vec<StringPattern>,
 
     /// Deprecated. Please prefix the pattern with `glob:` instead.
@@ -100,15 +97,24 @@ pub struct BranchDeleteArgs {
 pub struct BranchListArgs {
     /// Show all tracking and non-tracking remote branches including the ones
     /// whose targets are synchronized with the local branches.
-    #[arg(long, short, conflicts_with_all = ["names", "revisions"])]
-    all: bool,
+    #[arg(long, short, alias = "all")]
+    all_remotes: bool,
+
+    /// Show remote tracked branches only. Omits local Git-tracking branches by
+    /// default.
+    #[arg(long, short, conflicts_with_all = ["all_remotes"])]
+    tracked: bool,
+
+    /// Show conflicted branches only.
+    #[arg(long, short, conflicts_with_all = ["all_remotes"])]
+    conflicted: bool,
 
     /// Show branches whose local name matches
     ///
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// select branches by wildcard pattern. For details, see
     /// https://github.com/martinvonz/jj/blob/main/docs/revsets.md#string-patterns.
-    #[arg(value_parser = parse_string_pattern)]
+    #[arg(value_parser = StringPattern::parse)]
     pub names: Vec<StringPattern>,
 
     /// Show branches whose local targets are in the given revisions.
@@ -117,6 +123,14 @@ pub struct BranchListArgs {
     /// wouldn't have a local target.
     #[arg(long, short)]
     revisions: Vec<RevisionArg>,
+
+    /// Render each branch using the given template
+    ///
+    /// All 0-argument methods of the `RefName` type are available as keywords.
+    ///
+    /// For the syntax, see https://github.com/martinvonz/jj/blob/main/docs/templates.md
+    #[arg(long, short = 'T')]
+    template: Option<String>,
 }
 
 /// Forget everything about a branch, including its local and remote
@@ -131,7 +145,7 @@ pub struct BranchForgetArgs {
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// select branches by wildcard pattern. For details, see
     /// https://github.com/martinvonz/jj/blob/main/docs/revsets.md#string-patterns.
-    #[arg(required_unless_present_any(&["glob"]), value_parser = parse_string_pattern)]
+    #[arg(required_unless_present_any(&["glob"]), value_parser = StringPattern::parse)]
     pub names: Vec<StringPattern>,
 
     /// Deprecated. Please prefix the pattern with `glob:` instead.
@@ -234,7 +248,7 @@ fn cmd_branch_create(
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
     let target_commit =
-        workspace_command.resolve_single_rev(args.revision.as_deref().unwrap_or("@"), ui)?;
+        workspace_command.resolve_single_rev(args.revision.as_ref().unwrap_or(&RevisionArg::AT))?;
     let view = workspace_command.repo().view();
     let branch_names = &args.names;
     if let Some(branch_name) = branch_names
@@ -249,8 +263,8 @@ fn cmd_branch_create(
 
     if branch_names.len() > 1 {
         writeln!(
-            ui.warning(),
-            "warning: Creating multiple branches: {}",
+            ui.warning_default(),
+            "Creating multiple branches: {}",
             branch_names.join(", "),
         )?;
     }
@@ -312,15 +326,17 @@ fn cmd_branch_rename(
         .any(|(_, remote_ref)| remote_ref.is_tracking())
     {
         writeln!(
-            ui.warning(),
-            "Warning: Branch {old_branch} has tracking remote branches which were not renamed."
-        )?;
-        writeln!(
-            ui.hint(),
-            "Hint: to rename the branch on the remote, you can `jj git push --branch \
-             {old_branch}` first (to delete it on the remote), and then `jj git push --branch \
-             {new_branch}`. `jj git push --all` would also be sufficient."
+            ui.warning_default(),
+            "Branch {old_branch} has tracking remote branches which were not renamed."
         )?;
+        if let Some(mut writer) = ui.hint_default() {
+            writeln!(
+                writer,
+                "to rename the branch on the remote, you can `jj git push --branch {old_branch}` \
+                 first (to delete it on the remote), and then `jj git push --branch \
+                 {new_branch}`. `jj git push --all` would also be sufficient."
+            )?;
+        }
     }
 
     Ok(())
@@ -333,7 +349,7 @@ fn cmd_branch_set(
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
     let target_commit =
-        workspace_command.resolve_single_rev(args.revision.as_deref().unwrap_or("@"), ui)?;
+        workspace_command.resolve_single_rev(args.revision.as_ref().unwrap_or(&RevisionArg::AT))?;
     let repo = workspace_command.repo().as_ref();
     let is_fast_forward = |old_target: &RefTarget| {
         // Strictly speaking, "all" old targets should be ancestors, but we allow
@@ -361,8 +377,8 @@ fn cmd_branch_set(
 
     if branch_names.len() > 1 {
         writeln!(
-            ui.warning(),
-            "warning: Updating multiple branches: {}",
+            ui.warning_default(),
+            "Updating multiple branches: {}",
             branch_names.join(", "),
         )?;
     }
@@ -478,7 +494,7 @@ fn cmd_branch_delete(
     let view = workspace_command.repo().view();
     if !args.glob.is_empty() {
         writeln!(
-            ui.warning(),
+            ui.warning_default(),
             "--glob has been deprecated. Please prefix the pattern with `glob:` instead."
         )?;
     }
@@ -491,7 +507,7 @@ fn cmd_branch_delete(
     }
     tx.finish(ui, format!("delete {}", make_branch_term(&names)))?;
     if names.len() > 1 {
-        writeln!(ui.stderr(), "Deleted {} branches.", names.len())?;
+        writeln!(ui.status(), "Deleted {} branches.", names.len())?;
     }
     Ok(())
 }
@@ -505,7 +521,7 @@ fn cmd_branch_forget(
     let view = workspace_command.repo().view();
     if !args.glob.is_empty() {
         writeln!(
-            ui.warning(),
+            ui.warning_default(),
             "--glob has been deprecated. Please prefix the pattern with `glob:` instead."
         )?;
     }
@@ -517,7 +533,7 @@ fn cmd_branch_forget(
     }
     tx.finish(ui, format!("forget {}", make_branch_term(&names)))?;
     if names.len() > 1 {
-        writeln!(ui.stderr(), "Forgot {} branches.", names.len())?;
+        writeln!(ui.status(), "Forgot {} branches.", names.len())?;
     }
     Ok(())
 }
@@ -532,7 +548,10 @@ fn cmd_branch_track(
     let mut names = Vec::new();
     for (name, remote_ref) in find_remote_branches(view, &args.names)? {
         if remote_ref.is_tracking() {
-            writeln!(ui.warning(), "Remote branch already tracked: {name}")?;
+            writeln!(
+                ui.warning_default(),
+                "Remote branch already tracked: {name}"
+            )?;
         } else {
             names.push(name);
         }
@@ -545,11 +564,60 @@ fn cmd_branch_track(
     tx.finish(ui, format!("track remote {}", make_branch_term(&names)))?;
     if names.len() > 1 {
         writeln!(
-            ui.stderr(),
+            ui.status(),
             "Started tracking {} remote branches.",
             names.len()
         )?;
     }
+
+    //show conflicted branches if there are some
+
+    if let Some(mut formatter) = ui.status_formatter() {
+        let template = {
+            let language = workspace_command.commit_template_language()?;
+            let text = command
+                .settings()
+                .config()
+                .get::<String>("templates.branch_list")?;
+            workspace_command
+                .parse_template(&language, &text, CommitTemplateLanguage::wrap_ref_name)?
+                .labeled("branch_list")
+        };
+
+        let mut remote_per_branch: HashMap<&str, Vec<&str>> = HashMap::new();
+        for n in names.iter() {
+            remote_per_branch
+                .entry(&n.branch)
+                .or_default()
+                .push(&n.remote);
+        }
+        let branches_to_list =
+            workspace_command
+                .repo()
+                .view()
+                .branches()
+                .filter(|(name, target)| {
+                    remote_per_branch.contains_key(name) && target.local_target.has_conflict()
+                });
+
+        for (name, branch_target) in branches_to_list {
+            let local_target = branch_target.local_target;
+            let ref_name = RefName::local(
+                name,
+                local_target.clone(),
+                branch_target.remote_refs.iter().map(|x| x.1),
+            );
+            template.format(&ref_name, formatter.as_mut())?;
+
+            for (remote_name, remote_ref) in branch_target.remote_refs {
+                if remote_per_branch[name].contains(&remote_name) {
+                    let ref_name =
+                        RefName::remote(name, remote_name, remote_ref.clone(), local_target);
+                    template.format(&ref_name, formatter.as_mut())?;
+                }
+            }
+        }
+    }
     Ok(())
 }
 
@@ -565,11 +633,14 @@ fn cmd_branch_untrack(
         if name.remote == git::REMOTE_NAME_FOR_LOCAL_GIT_REPO {
             // This restriction can be lifted if we want to support untracked @git branches.
             writeln!(
-                ui.warning(),
+                ui.warning_default(),
                 "Git-tracking branch cannot be untracked: {name}"
             )?;
         } else if !remote_ref.is_tracking() {
-            writeln!(ui.warning(), "Remote branch not tracked yet: {name}")?;
+            writeln!(
+                ui.warning_default(),
+                "Remote branch not tracked yet: {name}"
+            )?;
         } else {
             names.push(name);
         }
@@ -582,7 +653,7 @@ fn cmd_branch_untrack(
     tx.finish(ui, format!("untrack remote {}", make_branch_term(&names)))?;
     if names.len() > 1 {
         writeln!(
-            ui.stderr(),
+            ui.status(),
             "Stopped tracking {} remote branches.",
             names.len()
         )?;
@@ -611,18 +682,10 @@ fn cmd_branch_list(
         }
         if !args.revisions.is_empty() {
             // Match against local targets only, which is consistent with "jj git push".
-            let filter_expressions: Vec<_> = args
-                .revisions
-                .iter()
-                .map(|revision_str| workspace_command.parse_revset(revision_str, Some(ui)))
-                .try_collect()?;
-            let filter_expression = RevsetExpression::union_all(&filter_expressions);
+            let mut expression = workspace_command.parse_union_revsets(&args.revisions)?;
             // Intersects with the set of local branch targets to minimize the lookup space.
-            let revset_expression = RevsetExpression::branches(StringPattern::everything())
-                .intersection(&filter_expression);
-            let revset_expression = revset::optimize(revset_expression);
-            let revset = workspace_command.evaluate_revset(revset_expression)?;
-            let filtered_targets: HashSet<CommitId> = revset.iter().collect();
+            expression.intersect_with(&RevsetExpression::branches(StringPattern::everything()));
+            let filtered_targets: HashSet<_> = expression.evaluate_to_commit_ids()?.collect();
             branch_names.extend(
                 view.local_branches()
                     .filter(|(_, target)| {
@@ -636,134 +699,93 @@ fn cmd_branch_list(
         None
     };
 
-    let no_branches_template = workspace_command.parse_commit_template(
-        &command
-            .settings()
-            .config()
-            .get_string("templates.commit_summary_no_branches")?,
-    )?;
-    let print_branch_target =
-        |formatter: &mut dyn Formatter, target: &RefTarget| -> Result<(), CommandError> {
-            if let Some(id) = target.as_normal() {
-                write!(formatter, ": ")?;
-                let commit = repo.store().get_commit(id)?;
-                no_branches_template.format(&commit, formatter)?;
-                writeln!(formatter)?;
-            } else {
-                write!(formatter, " ")?;
-                write!(formatter.labeled("conflict"), "(conflicted)")?;
-                writeln!(formatter, ":")?;
-                for id in target.removed_ids() {
-                    let commit = repo.store().get_commit(id)?;
-                    write!(formatter, "  - ")?;
-                    no_branches_template.format(&commit, formatter)?;
-                    writeln!(formatter)?;
-                }
-                for id in target.added_ids() {
-                    let commit = repo.store().get_commit(id)?;
-                    write!(formatter, "  + ")?;
-                    no_branches_template.format(&commit, formatter)?;
-                    writeln!(formatter)?;
-                }
-            }
-            Ok(())
+    let template = {
+        let language = workspace_command.commit_template_language()?;
+        let text = match &args.template {
+            Some(value) => value.to_owned(),
+            None => command.settings().config().get("templates.branch_list")?,
         };
+        workspace_command
+            .parse_template(&language, &text, CommitTemplateLanguage::wrap_ref_name)?
+            .labeled("branch_list")
+    };
 
     ui.request_pager();
     let mut formatter = ui.stdout_formatter();
-    let formatter = formatter.as_mut();
 
-    let branches_to_list = view.branches().filter(|&(name, _)| {
+    let mut found_deleted_local_branch = false;
+    let mut found_deleted_tracking_local_branch = false;
+    let branches_to_list = view.branches().filter(|(name, target)| {
         branch_names_to_list
             .as_ref()
             .map_or(true, |branch_names| branch_names.contains(name))
+            && (!args.conflicted || target.local_target.has_conflict())
     });
     for (name, branch_target) in branches_to_list {
-        let (tracking_remote_refs, untracked_remote_refs) =
-            branch_target
-                .remote_refs
-                .into_iter()
-                .partition::<Vec<_>, _>(|&(_, remote_ref)| remote_ref.is_tracking());
+        let local_target = branch_target.local_target;
+        let remote_refs = branch_target.remote_refs;
+        let (mut tracking_remote_refs, untracked_remote_refs) = remote_refs
+            .iter()
+            .copied()
+            .partition::<Vec<_>, _>(|&(_, remote_ref)| remote_ref.is_tracking());
 
-        if branch_target.local_target.is_present() || !tracking_remote_refs.is_empty() {
-            write!(formatter.labeled("branch"), "{name}")?;
-            if branch_target.local_target.is_present() {
-                print_branch_target(formatter, branch_target.local_target)?;
-            } else {
-                writeln!(formatter, " (deleted)")?;
-            }
+        if args.tracked {
+            tracking_remote_refs
+                .retain(|&(remote, _)| remote != git::REMOTE_NAME_FOR_LOCAL_GIT_REPO);
+        } else if !args.all_remotes {
+            tracking_remote_refs.retain(|&(_, remote_ref)| remote_ref.target != *local_target);
+        }
+
+        if !args.tracked && local_target.is_present() || !tracking_remote_refs.is_empty() {
+            let ref_name = RefName::local(
+                name,
+                local_target.clone(),
+                remote_refs.iter().map(|&(_, remote_ref)| remote_ref),
+            );
+            template.format(&ref_name, formatter.as_mut())?;
         }
 
         for &(remote, remote_ref) in &tracking_remote_refs {
-            let synced = remote_ref.target == *branch_target.local_target;
-            if !args.all && synced {
-                continue;
-            }
-            write!(formatter, "  ")?;
-            write!(formatter.labeled("branch"), "@{remote}")?;
-            let local_target = branch_target.local_target;
-            if local_target.is_present() && !synced {
-                let remote_added_ids = remote_ref.target.added_ids().cloned().collect_vec();
-                let local_added_ids = local_target.added_ids().cloned().collect_vec();
-                let (remote_ahead_lower, remote_ahead_upper) =
-                    revset::walk_revs(repo.as_ref(), &remote_added_ids, &local_added_ids)?
-                        .count_estimate();
-                let (local_ahead_lower, local_ahead_upper) =
-                    revset::walk_revs(repo.as_ref(), &local_added_ids, &remote_added_ids)?
-                        .count_estimate();
-                let remote_ahead_message = match remote_ahead_upper {
-                    Some(0) => None,
-                    Some(upper) if upper == remote_ahead_lower => {
-                        Some(format!("ahead by {remote_ahead_lower} commits"))
-                    }
-                    _ => Some(format!("ahead by at least {remote_ahead_lower} commits")),
-                };
-                let local_ahead_message = match local_ahead_upper {
-                    Some(0) => None,
-                    Some(upper) if upper == local_ahead_lower => {
-                        Some(format!("behind by {local_ahead_lower} commits"))
-                    }
-                    _ => Some(format!("behind by at least {local_ahead_lower} commits")),
-                };
-                match (remote_ahead_message, local_ahead_message) {
-                    (Some(rm), Some(lm)) => {
-                        write!(formatter, " ({rm}, {lm})")?;
-                    }
-                    (Some(m), None) | (None, Some(m)) => {
-                        write!(formatter, " ({m})")?;
-                    }
-                    (None, None) => { /* do nothing */ }
-                }
-            }
-            print_branch_target(formatter, &remote_ref.target)?;
+            let ref_name = RefName::remote(name, remote, remote_ref.clone(), local_target);
+            template.format(&ref_name, formatter.as_mut())?;
         }
 
-        if branch_target.local_target.is_absent() && !tracking_remote_refs.is_empty() {
-            let found_non_git_remote = tracking_remote_refs
+        if local_target.is_absent() && !tracking_remote_refs.is_empty() {
+            found_deleted_local_branch = true;
+            found_deleted_tracking_local_branch |= tracking_remote_refs
                 .iter()
                 .any(|&(remote, _)| remote != git::REMOTE_NAME_FOR_LOCAL_GIT_REPO);
-            if found_non_git_remote {
-                writeln!(
-                    formatter.labeled("hint"),
-                    "  (this branch will be *deleted permanently* on the remote on the next `jj \
-                     git push`. Use `jj branch forget` to prevent this)"
-                )?;
-            } else {
-                writeln!(
-                    formatter.labeled("hint"),
-                    "  (this branch will be deleted from the underlying Git repo on the next `jj \
-                     git export`)"
-                )?;
-            }
         }
 
-        if args.all {
+        if args.all_remotes {
             for &(remote, remote_ref) in &untracked_remote_refs {
-                write!(formatter.labeled("branch"), "{name}@{remote}")?;
-                print_branch_target(formatter, &remote_ref.target)?;
+                let ref_name = RefName::remote_only(name, remote, remote_ref.target.clone());
+                template.format(&ref_name, formatter.as_mut())?;
             }
         }
     }
 
+    drop(formatter);
+
+    // Print only one of these hints. It's not important to mention unexported
+    // branches, but user might wonder why deleted branches are still listed.
+    if found_deleted_tracking_local_branch {
+        if let Some(mut writer) = ui.hint_default() {
+            writeln!(
+                writer,
+                "Branches marked as deleted will be *deleted permanently* on the remote on the \
+                 next `jj git push`. Use `jj branch forget` to prevent this."
+            )?;
+        }
+    } else if found_deleted_local_branch {
+        if let Some(mut writer) = ui.hint_default() {
+            writeln!(
+                writer,
+                "Branches marked as deleted will be deleted from the underlying Git repo on the \
+                 next `jj git export`."
+            )?;
+        }
+    }
+
     Ok(())
 }
