@@ -58,7 +58,7 @@ pub(crate) fn cmd_cat(
 
     // Try fast path for single file entry
     if let Some(path) = get_single_path(&fileset_expression) {
-        let value = tree.path_value(path);
+        let value = tree.path_value(path)?;
         if value.is_absent() {
             let ui_path = workspace_command.format_file_path(path);
             return Err(user_error(format!("No such path: {ui_path}")));
@@ -104,6 +104,13 @@ fn write_tree_entries<P: AsRef<RepoPath>>(
         let materialized = materialize_tree_value(repo.store(), path.as_ref(), value).block_on()?;
         match materialized {
             MaterializedTreeValue::Absent => panic!("absent values should be excluded"),
+            MaterializedTreeValue::AccessDenied(err) => {
+                let ui_path = workspace_command.format_file_path(path.as_ref());
+                writeln!(
+                    ui.warning_default(),
+                    "Path '{ui_path}' exists but access is denied: {err}"
+                )?;
+            }
             MaterializedTreeValue::File { mut reader, .. } => {
                 io::copy(&mut reader, &mut ui.stdout_formatter().as_mut())?;
             }
@@ -114,7 +121,7 @@ fn write_tree_entries<P: AsRef<RepoPath>>(
                 let ui_path = workspace_command.format_file_path(path.as_ref());
                 writeln!(
                     ui.warning_default(),
-                    "Path exists but is not a file: {ui_path}"
+                    "Path '{ui_path}' exists but is not a file"
                 )?;
             }
             MaterializedTreeValue::Tree(_) => panic!("entries should not contain trees"),
