@@ -12,12 +12,11 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use jj_lib::rewrite::merge_commit_trees;
 use tracing::instrument;
 
 use crate::cli_util::{print_unmatched_explicit_paths, CommandHelper, RevisionArg};
 use crate::command_error::CommandError;
-use crate::diff_util::{diff_formats_for, show_diff, DiffFormatArgs};
+use crate::diff_util::DiffFormatArgs;
 use crate::ui::Ui;
 
 /// Compare file contents between two revisions
@@ -72,22 +71,19 @@ pub(crate) fn cmd_diff(
     } else {
         let commit = workspace_command
             .resolve_single_rev(args.revision.as_ref().unwrap_or(&RevisionArg::AT))?;
-        let parents = commit.parents();
-        from_tree = merge_commit_trees(workspace_command.repo().as_ref(), &parents)?;
+        from_tree = commit.parent_tree(workspace_command.repo().as_ref())?;
         to_tree = commit.tree()?
     }
     let fileset_expression = workspace_command.parse_file_patterns(&args.paths)?;
     let matcher = fileset_expression.to_matcher();
-    let diff_formats = diff_formats_for(command.settings(), &args.format)?;
+    let diff_renderer = workspace_command.diff_renderer_for(&args.format)?;
     ui.request_pager();
-    show_diff(
+    diff_renderer.show_diff(
         ui,
         ui.stdout_formatter().as_mut(),
-        &workspace_command,
         &from_tree,
         &to_tree,
         matcher.as_ref(),
-        &diff_formats,
     )?;
     print_unmatched_explicit_paths(
         ui,
