@@ -14,21 +14,26 @@
 
 use std::io::Write;
 
+use itertools::Itertools;
 use jj_lib::matchers::EverythingMatcher;
 use jj_lib::object_id::ObjectId;
 use jj_lib::rewrite::merge_commit_trees;
 use tracing::instrument;
 
-use crate::cli_util::{CommandError, CommandHelper, RevisionArg};
+use crate::cli_util::{CommandHelper, RevisionArg};
+use crate::command_error::CommandError;
 use crate::ui::Ui;
 
 /// Touch up the content changes in a revision with a diff editor
 ///
-/// With the `-r` option, which is the default, starts a diff editor (`meld` by
-/// default) on the changes in the revision.
+/// With the `-r` option, which is the default, starts a [diff editor] on the
+/// changes in the revision.
 ///
-/// With the `--from` and/or `--to` options, starts a diff editor comparing the
-/// "from" revision to the "to" revision.
+/// With the `--from` and/or `--to` options, starts a [diff editor] comparing
+/// the "from" revision to the "to" revision.
+///
+/// [diff editor]:
+///     https://martinvonz.github.io/jj/latest/config/#editing-diffs
 ///
 /// Edit the right side of the diff until it looks the way you want. Once you
 /// close the editor, the revision specified with `-r` or `--to` will be
@@ -50,6 +55,9 @@ pub(crate) struct DiffeditArgs {
     /// Edit changes in this revision. Defaults to @ if --from is specified.
     #[arg(long, conflicts_with = "revision")]
     to: Option<RevisionArg>,
+    /// Specify diff editor to be used
+    #[arg(long, value_name = "NAME")]
+    tool: Option<String>,
 }
 
 #[instrument(skip_all)]
@@ -63,21 +71,23 @@ pub(crate) fn cmd_diffedit(
     let (target_commit, base_commits, diff_description);
     if args.from.is_some() || args.to.is_some() {
         target_commit =
-            workspace_command.resolve_single_rev(args.to.as_deref().unwrap_or("@"), ui)?;
+            workspace_command.resolve_single_rev(args.to.as_ref().unwrap_or(&RevisionArg::AT))?;
         base_commits =
-            vec![workspace_command.resolve_single_rev(args.from.as_deref().unwrap_or("@"), ui)?];
+            vec![workspace_command
+                .resolve_single_rev(args.from.as_ref().unwrap_or(&RevisionArg::AT))?];
         diff_description = format!(
             "The diff initially shows the commit's changes relative to:\n{}",
             workspace_command.format_commit_summary(&base_commits[0])
         );
     } else {
-        target_commit =
-            workspace_command.resolve_single_rev(args.revision.as_deref().unwrap_or("@"), ui)?;
-        base_commits = target_commit.parents();
+        target_commit = workspace_command
+            .resolve_single_rev(args.revision.as_ref().unwrap_or(&RevisionArg::AT))?;
+        base_commits = target_commit.parents().try_collect()?;
         diff_description = "The diff initially shows the commit's changes.".to_string();
     };
-    workspace_command.check_rewritable([&target_commit])?;
+    workspace_command.check_rewritable([target_commit.id()])?;
 
+    let diff_editor = workspace_command.diff_editor(ui, args.tool.as_deref())?;
     let mut tx = workspace_command.start_transaction();
     let instructions = format!(
         "\
@@ -91,9 +101,9 @@ don't make any changes, then the operation will be aborted.",
     );
     let base_tree = merge_commit_trees(tx.repo(), base_commits.as_slice())?;
     let tree = target_commit.tree()?;
-    let tree_id = tx.edit_diff(ui, &base_tree, &tree, &EverythingMatcher, &instructions)?;
+    let tree_id = diff_editor.edit(&base_tree, &tree, &EverythingMatcher, Some(&instructions))?;
     if tree_id == *target_commit.tree_id() {
-        writeln!(ui.stderr(), "Nothing changed.")?;
+        writeln!(ui.status(), "Nothing changed.")?;
     } else {
         let mut_repo = tx.mut_repo();
         let new_commit = mut_repo
@@ -103,11 +113,13 @@ don't make any changes, then the operation will be aborted.",
         // rebase_descendants early; otherwise `new_commit` would always have
         // a conflicted change id at this point.
         let num_rebased = tx.mut_repo().rebase_descendants(command.settings())?;
-        write!(ui.stderr(), "Created ")?;
-        tx.write_commit_summary(ui.stderr_formatter().as_mut(), &new_commit)?;
-        writeln!(ui.stderr())?;
-        if num_rebased > 0 {
-            writeln!(ui.stderr(), "Rebased {num_rebased} descendant commits")?;
+        if let Some(mut formatter) = ui.status_formatter() {
+            write!(formatter, "Created ")?;
+            tx.write_commit_summary(formatter.as_mut(), &new_commit)?;
+            writeln!(formatter)?;
+            if num_rebased > 0 {
+                writeln!(formatter, "Rebased {num_rebased} descendant commits")?;
+            }
         }
         tx.finish(ui, format!("edit commit {}", target_commit.id().hex()))?;
     }
