@@ -12,9 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::collections::HashSet;
+use std::collections::{HashMap, HashSet};
 use std::io::Write;
-use std::ops::Deref;
 use std::path::{Path, PathBuf};
 use std::sync::Arc;
 use std::{fmt, fs, io};
@@ -29,7 +28,7 @@ use jj_lib::git::{
 use jj_lib::object_id::ObjectId;
 use jj_lib::op_store::RefTarget;
 use jj_lib::refs::{
-    classify_branch_push_action, BranchPushAction, BranchPushUpdate, TrackingRefPair,
+    classify_branch_push_action, BranchPushAction, BranchPushUpdate, LocalAndRemoteRef,
 };
 use jj_lib::repo::{ReadonlyRepo, Repo};
 use jj_lib::repo_path::RepoPath;
@@ -38,21 +37,21 @@ use jj_lib::settings::{ConfigResultExt as _, UserSettings};
 use jj_lib::str_util::StringPattern;
 use jj_lib::view::View;
 use jj_lib::workspace::Workspace;
-use maplit::hashset;
 
 use crate::cli_util::{
-    parse_string_pattern, print_trackable_remote_branches, resolve_multiple_nonempty_revsets,
-    short_change_hash, short_commit_hash, start_repo_transaction, user_error, user_error_with_hint,
-    user_error_with_hint_opt, user_error_with_message, CommandError, CommandHelper, RevisionArg,
-    WorkspaceCommandHelper,
+    print_trackable_remote_branches, short_change_hash, short_commit_hash, start_repo_transaction,
+    CommandHelper, RevisionArg, WorkspaceCommandHelper, WorkspaceCommandTransaction,
+};
+use crate::command_error::{
+    user_error, user_error_with_hint, user_error_with_message, CommandError,
 };
 use crate::git_util::{
     get_git_repo, is_colocated_git_workspace, print_failed_git_export, print_git_import_stats,
-    with_remote_git_callbacks,
+    with_remote_git_callbacks, GitSidebandProgressMessageWriter,
 };
 use crate::ui::Ui;
 
-/// Commands for working with the underlying Git repo
+/// Commands for working with Git remotes and the underlying Git repo
 ///
 /// For a comparison with Git, including a table of commands, see
 /// https://github.com/martinvonz/jj/blob/main/docs/git-comparison.md.
@@ -115,7 +114,7 @@ pub struct GitRemoteListArgs {}
 pub struct GitInitArgs {
     /// The destination directory where the `jj` repo will be created.
     /// If the directory does not exist, it will be created.
-    /// If no directory is diven, the current directory is used.
+    /// If no directory is given, the current directory is used.
     ///
     /// By default the `git` repo is under `$destination/.jj`
     #[arg(default_value = ".", value_hint = clap::ValueHint::DirPath)]
@@ -125,12 +124,9 @@ pub struct GitInitArgs {
     /// `git` repo, allowing the use of both `jj` and `git` commands
     /// in the same directory.
     ///
-    /// This is done by placing the backing git repo into a `.git` directory
-    /// in the root of the `jj` repo along with the `.jj` directory.
-    ///
-    /// This option is only valid when creating new repos. To
-    /// reuse an existing `.git` directory in an existing git
-    /// repo, see the `--git-repo` param below.
+    /// This is done by placing the backing git repo into a `.git` directory in
+    /// the root of the `jj` repo along with the `.jj` directory. If the `.git`
+    /// directory already exists, all the existing commits will be imported.
     ///
     /// This option is mutually exclusive with `--git-repo`.
     #[arg(long, conflicts_with = "git_repo")]
@@ -150,13 +146,16 @@ pub struct GitInitArgs {
 }
 
 /// Fetch from a Git remote
+///
+/// If a working-copy commit gets abandoned, it will be given a new, empty
+/// commit. This is true in general; it is not specific to this command.
 #[derive(clap::Args, Clone, Debug)]
 pub struct GitFetchArgs {
     /// Fetch only some of the branches
     ///
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// expand `*` as a glob. The other wildcard characters aren't supported.
-    #[arg(long, default_value = "glob:*", value_parser = parse_string_pattern)]
+    #[arg(long, short, default_value = "glob:*", value_parser = StringPattern::parse)]
     branch: Vec<StringPattern>,
     /// The remote to fetch from (only named remotes are supported, can be
     /// repeated)
@@ -189,6 +188,18 @@ pub struct GitCloneArgs {
 /// `remote_branches(remote=<remote>)..@`. Use `--branch` to push specific
 /// branches. Use `--all` to push all branches. Use `--change` to generate
 /// branch names based on the change IDs of specific commits.
+///
+/// Before the command actually moves, creates, or deletes a remote branch, it
+/// makes several [safety checks]. If there is a problem, you may need to run
+/// `jj git fetch --remote <remote name>` and/or resolve some [branch
+/// conflicts].
+///
+/// [safety checks]:
+///     https://martinvonz.github.io/jj/latest/branches/#pushing-branches-safety-checks
+///
+/// [branch conflicts]:
+///     https://martinvonz.github.io/jj/latest/branches/#conflicts
+
 #[derive(clap::Args, Clone, Debug)]
 #[command(group(ArgGroup::new("specific").args(&["branch", "change", "revisions"]).multiple(true)))]
 #[command(group(ArgGroup::new("what").args(&["all", "deleted", "tracked"]).conflicts_with("specific")))]
@@ -201,7 +212,7 @@ pub struct GitPushArgs {
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// select branches by wildcard pattern. For details, see
     /// https://martinvonz.github.io/jj/latest/revsets#string-patterns.
-    #[arg(long, short, value_parser = parse_string_pattern)]
+    #[arg(long, short, value_parser = StringPattern::parse)]
     branch: Vec<StringPattern>,
     /// Push all branches (including deleted branches)
     #[arg(long)]
@@ -220,6 +231,9 @@ pub struct GitPushArgs {
     /// correspond to missing local branches.
     #[arg(long)]
     deleted: bool,
+    /// Allow pushing commits with empty descriptions
+    #[arg(long)]
+    allow_empty_description: bool,
     /// Push branches pointing to these commits (can be repeated)
     #[arg(long, short)]
     revisions: Vec<RevisionArg>,
@@ -233,6 +247,9 @@ pub struct GitPushArgs {
 }
 
 /// Update repo with changes made in the underlying Git repo
+///
+/// If a working-copy commit gets abandoned, it will be given a new, empty
+/// commit. This is true in general; it is not specific to this command.
 #[derive(clap::Args, Clone, Debug)]
 pub struct GitImportArgs {}
 
@@ -373,53 +390,75 @@ pub fn git_init(
     colocate: bool,
     git_repo: Option<&str>,
 ) -> Result<(), CommandError> {
-    let cwd = command.cwd().canonicalize().unwrap();
-    let relative_wc_path = file_util::relative_path(&cwd, workspace_root);
-
-    if colocate {
-        let (workspace, repo) = Workspace::init_colocated_git(command.settings(), workspace_root)?;
-        let workspace_command = command.for_loaded_repo(ui, workspace, repo)?;
-        maybe_add_gitignore(&workspace_command)?;
-        return Ok(());
+    #[derive(Clone, Debug)]
+    enum GitInitMode {
+        Colocate,
+        External(PathBuf),
+        Internal,
     }
 
-    if let Some(git_store_str) = git_repo {
-        let git_store_path = cwd.join(git_store_str);
-        let (workspace, repo) =
-            Workspace::init_external_git(command.settings(), workspace_root, &git_store_path)?;
-        // Import refs first so all the reachable commits are indexed in
-        // chronological order.
-        let colocated = is_colocated_git_workspace(&workspace, &repo);
-        let repo = init_git_refs(ui, command, repo, colocated)?;
-        let mut workspace_command = command.for_loaded_repo(ui, workspace, repo)?;
-        maybe_add_gitignore(&workspace_command)?;
-        workspace_command.maybe_snapshot(ui)?;
-        if !workspace_command.working_copy_shared_with_git() {
-            let mut tx = workspace_command.start_transaction();
-            jj_lib::git::import_head(tx.mut_repo())?;
-            if let Some(git_head_id) = tx.mut_repo().view().git_head().as_normal().cloned() {
-                let git_head_commit = tx.mut_repo().store().get_commit(&git_head_id)?;
-                tx.check_out(&git_head_commit)?;
-            }
-            if tx.mut_repo().has_changes() {
-                tx.finish(ui, "import git head")?;
+    let colocated_git_repo_path = workspace_root.join(".git");
+    let init_mode = if colocate {
+        if colocated_git_repo_path.exists() {
+            GitInitMode::External(colocated_git_repo_path)
+        } else {
+            GitInitMode::Colocate
+        }
+    } else if let Some(path_str) = git_repo {
+        let mut git_repo_path = command.cwd().join(path_str);
+        if !git_repo_path.ends_with(".git") {
+            git_repo_path.push(".git");
+            // Undo if .git doesn't exist - likely a bare repo.
+            if !git_repo_path.exists() {
+                git_repo_path.pop();
             }
         }
-        print_trackable_remote_branches(ui, workspace_command.repo().view())?;
+        GitInitMode::External(git_repo_path)
     } else {
-        if workspace_root.join(".git").exists() {
+        if colocated_git_repo_path.exists() {
             return Err(user_error_with_hint(
                 "Did not create a jj repo because there is an existing Git repo in this directory.",
-                format!(
-                    r#"To create a repo backed by the existing Git repo, run `jj git init --git-repo={}` instead."#,
-                    relative_wc_path.display()
-                ),
+                "To create a repo backed by the existing Git repo, run `jj git init --colocate` \
+                 instead.",
             ));
         }
+        GitInitMode::Internal
+    };
 
-        Workspace::init_internal_git(command.settings(), workspace_root)?;
+    match &init_mode {
+        GitInitMode::Colocate => {
+            let (workspace, repo) =
+                Workspace::init_colocated_git(command.settings(), workspace_root)?;
+            let workspace_command = command.for_loaded_repo(ui, workspace, repo)?;
+            maybe_add_gitignore(&workspace_command)?;
+        }
+        GitInitMode::External(git_repo_path) => {
+            let (workspace, repo) =
+                Workspace::init_external_git(command.settings(), workspace_root, git_repo_path)?;
+            // Import refs first so all the reachable commits are indexed in
+            // chronological order.
+            let colocated = is_colocated_git_workspace(&workspace, &repo);
+            let repo = init_git_refs(ui, command, repo, colocated)?;
+            let mut workspace_command = command.for_loaded_repo(ui, workspace, repo)?;
+            maybe_add_gitignore(&workspace_command)?;
+            workspace_command.maybe_snapshot(ui)?;
+            if !workspace_command.working_copy_shared_with_git() {
+                let mut tx = workspace_command.start_transaction();
+                jj_lib::git::import_head(tx.mut_repo())?;
+                if let Some(git_head_id) = tx.mut_repo().view().git_head().as_normal().cloned() {
+                    let git_head_commit = tx.mut_repo().store().get_commit(&git_head_id)?;
+                    tx.check_out(&git_head_commit)?;
+                }
+                if tx.mut_repo().has_changes() {
+                    tx.finish(ui, "import git head")?;
+                }
+            }
+            print_trackable_remote_branches(ui, workspace_command.repo().view())?;
+        }
+        GitInitMode::Internal => {
+            Workspace::init_internal_git(command.settings(), workspace_root)?;
+        }
     }
-
     Ok(())
 }
 
@@ -447,7 +486,7 @@ fn init_git_refs(
     if !tx.mut_repo().has_changes() {
         return Ok(repo);
     }
-    print_git_import_stats(ui, &stats)?;
+    print_git_import_stats(ui, tx.repo(), &stats, false)?;
     if colocated {
         // If git.auto-local-branch = true, local branches could be created for
         // the imported remote branches.
@@ -456,7 +495,7 @@ fn init_git_refs(
     }
     let repo = tx.commit("import git refs");
     writeln!(
-        ui.stderr(),
+        ui.status(),
         "Done importing changes from the underlying Git repo."
     )?;
     Ok(repo)
@@ -467,7 +506,7 @@ fn cmd_git_init(
     command: &CommandHelper,
     args: &GitInitArgs,
 ) -> Result<(), CommandError> {
-    let cwd = command.cwd().canonicalize().unwrap();
+    let cwd = command.cwd();
     let wc_path = cwd.join(&args.destination);
     let wc_path = file_util::create_or_reuse_dir(&wc_path)
         .and_then(|_| wc_path.canonicalize())
@@ -481,9 +520,9 @@ fn cmd_git_init(
         args.git_repo.as_deref(),
     )?;
 
-    let relative_wc_path = file_util::relative_path(&cwd, &wc_path);
+    let relative_wc_path = file_util::relative_path(cwd, &wc_path);
     writeln!(
-        ui.stderr(),
+        ui.status(),
         r#"Initialized repo in "{}""#,
         relative_wc_path.display()
     )?;
@@ -508,7 +547,7 @@ fn cmd_git_fetch(
     };
     let mut tx = workspace_command.start_transaction();
     for remote in &remotes {
-        let stats = with_remote_git_callbacks(ui, |cb| {
+        let stats = with_remote_git_callbacks(ui, None, |cb| {
             git::fetch(
                 tx.mut_repo(),
                 &git_repo,
@@ -537,7 +576,7 @@ fn cmd_git_fetch(
             GitFetchError::InternalGitError(err) => map_git_error(err),
             _ => user_error(err),
         })?;
-        print_git_import_stats(ui, &stats.import_stats)?;
+        print_git_import_stats(ui, tx.repo(), &stats.import_stats, true)?;
     }
     tx.finish(
         ui,
@@ -569,11 +608,9 @@ fn get_default_fetch_remotes(
     } else if let Some(remote) = get_single_remote(git_repo)? {
         // if nothing was explicitly configured, try to guess
         if remote != DEFAULT_REMOTE {
-            writeln!(
-                ui.hint(),
-                "Fetching from the only existing remote: {}",
-                remote
-            )?;
+            if let Some(mut writer) = ui.hint_default() {
+                writeln!(writer, "Fetching from the only existing remote: {remote}")?;
+            }
         }
         Ok(vec![remote])
     } else {
@@ -677,7 +714,7 @@ fn cmd_git_clone(
         };
         if let Err(err) = clean_up_dirs() {
             writeln!(
-                ui.warning(),
+                ui.warning_default(),
                 "Failed to clean up {}: {}",
                 canonical_wc_path.display(),
                 err
@@ -722,7 +759,7 @@ fn do_git_clone(
     };
     let git_repo = get_git_repo(repo.store())?;
     writeln!(
-        ui.stderr(),
+        ui.status(),
         r#"Fetching into new repo in "{}""#,
         wc_path.display()
     )?;
@@ -731,7 +768,7 @@ fn do_git_clone(
     git_repo.remote(remote_name, source).unwrap();
     let mut fetch_tx = workspace_command.start_transaction();
 
-    let stats = with_remote_git_callbacks(ui, |cb| {
+    let stats = with_remote_git_callbacks(ui, None, |cb| {
         git::fetch(
             fetch_tx.mut_repo(),
             &git_repo,
@@ -751,11 +788,18 @@ fn do_git_clone(
             unreachable!("we didn't provide any globs")
         }
     })?;
-    print_git_import_stats(ui, &stats.import_stats)?;
+    print_git_import_stats(ui, fetch_tx.repo(), &stats.import_stats, true)?;
     fetch_tx.finish(ui, "fetch from git remote into empty repo")?;
     Ok((workspace_command, stats))
 }
 
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
+enum BranchMoveDirection {
+    Forward,
+    Backward,
+    Sideways,
+}
+
 fn cmd_git_push(
     ui: &mut Ui,
     command: &CommandHelper,
@@ -771,13 +815,6 @@ fn cmd_git_push(
     };
 
     let repo = workspace_command.repo().clone();
-    let wc_commit_id = workspace_command.get_wc_commit_id().cloned();
-    let change_commits: Vec<_> = args
-        .change
-        .iter()
-        .map(|change_str| workspace_command.resolve_single_rev(change_str, ui))
-        .try_collect()?;
-
     let mut tx = workspace_command.start_transaction();
     let tx_description;
     let mut branch_updates = vec![];
@@ -815,65 +852,31 @@ fn cmd_git_push(
         }
         tx_description = format!("push all deleted branches to git remote {remote}");
     } else {
-        let mut seen_branches = hashset! {};
-        let branches_by_name =
-            find_branches_to_push(repo.view(), &args.branch, &remote, &mut seen_branches)?;
-        for (branch_name, targets) in branches_by_name {
+        let mut seen_branches: HashSet<&str> = HashSet::new();
+
+        // Process --change branches first because matching branches can be moved.
+        let change_branch_names = update_change_branches(
+            ui,
+            &mut tx,
+            &args.change,
+            &command.settings().push_branch_prefix(),
+        )?;
+        let change_branches = change_branch_names.iter().map(|branch_name| {
+            let targets = LocalAndRemoteRef {
+                local_target: tx.repo().view().get_local_branch(branch_name),
+                remote_ref: tx.repo().view().get_remote_branch(branch_name, &remote),
+            };
+            (branch_name.as_ref(), targets)
+        });
+        let branches_by_name = find_branches_to_push(repo.view(), &args.branch, &remote)?;
+        for (branch_name, targets) in change_branches.chain(branches_by_name.iter().copied()) {
+            if !seen_branches.insert(branch_name) {
+                continue;
+            }
             match classify_branch_update(branch_name, &remote, targets) {
                 Ok(Some(update)) => branch_updates.push((branch_name.to_owned(), update)),
                 Ok(None) => writeln!(
-                    ui.stderr(),
-                    "Branch {branch_name}@{remote} already matches {branch_name}",
-                )?,
-                Err(reason) => return Err(reason.into()),
-            }
-        }
-
-        for (change_str, commit) in std::iter::zip(args.change.iter(), change_commits) {
-            let mut branch_name = format!(
-                "{}{}",
-                command.settings().push_branch_prefix(),
-                commit.change_id().hex()
-            );
-            if !seen_branches.insert(branch_name.clone()) {
-                continue;
-            }
-            let view = tx.base_repo().view();
-            if view.get_local_branch(&branch_name).is_absent() {
-                // A local branch with the full change ID doesn't exist already, so use the
-                // short ID if it's not ambiguous (which it shouldn't be most of the time).
-                let short_change_id = short_change_hash(commit.change_id());
-                if tx
-                    .base_workspace_helper()
-                    .resolve_single_rev(&short_change_id, ui)
-                    .is_ok()
-                {
-                    // Short change ID is not ambiguous, so update the branch name to use it.
-                    branch_name = format!(
-                        "{}{}",
-                        command.settings().push_branch_prefix(),
-                        short_change_id
-                    );
-                };
-            }
-            if view.get_local_branch(&branch_name).is_absent() {
-                writeln!(
-                    ui.stderr(),
-                    "Creating branch {} for revision {}",
-                    branch_name,
-                    change_str.deref()
-                )?;
-            }
-            tx.mut_repo()
-                .set_local_branch_target(&branch_name, RefTarget::normal(commit.id().clone()));
-            let targets = TrackingRefPair {
-                local_target: tx.repo().view().get_local_branch(&branch_name),
-                remote_ref: tx.repo().view().get_remote_branch(&branch_name, &remote),
-            };
-            match classify_branch_update(&branch_name, &remote, targets) {
-                Ok(Some(update)) => branch_updates.push((branch_name.clone(), update)),
-                Ok(None) => writeln!(
-                    ui.stderr(),
+                    ui.status(),
                     "Branch {branch_name}@{remote} already matches {branch_name}",
                 )?,
                 Err(reason) => return Err(reason.into()),
@@ -882,38 +885,15 @@ fn cmd_git_push(
 
         let use_default_revset =
             args.branch.is_empty() && args.change.is_empty() && args.revisions.is_empty();
-        let revision_commit_ids: HashSet<_> = if use_default_revset {
-            let Some(wc_commit_id) = wc_commit_id else {
-                return Err(user_error("Nothing checked out in this workspace"));
-            };
-            let current_branches_expression = RevsetExpression::remote_branches(
-                StringPattern::everything(),
-                StringPattern::Exact(remote.to_owned()),
-            )
-            .range(&RevsetExpression::commit(wc_commit_id))
-            .intersection(&RevsetExpression::branches(StringPattern::everything()));
-            let current_branches_revset = tx
-                .base_workspace_helper()
-                .evaluate_revset(current_branches_expression)?;
-            current_branches_revset.iter().collect()
-        } else {
-            // TODO: Narrow search space to local target commits.
-            // TODO: Remove redundant CommitId -> Commit -> CommitId round trip.
-            resolve_multiple_nonempty_revsets(&args.revisions, tx.base_workspace_helper(), ui)?
-                .iter()
-                .map(|commit| commit.id().clone())
-                .collect()
-        };
-        let branches_targeted = repo
-            .view()
-            .local_remote_branches(&remote)
-            .filter(|(_, targets)| {
-                let mut local_ids = targets.local_target.added_ids();
-                local_ids.any(|id| revision_commit_ids.contains(id))
-            })
-            .collect_vec();
+        let branches_targeted = find_branches_targeted_by_revisions(
+            ui,
+            tx.base_workspace_helper(),
+            &remote,
+            &args.revisions,
+            use_default_revset,
+        )?;
         for &(branch_name, targets) in &branches_targeted {
-            if !seen_branches.insert(branch_name.to_owned()) {
+            if !seen_branches.insert(branch_name) {
                 continue;
             }
             match classify_branch_update(branch_name, &remote, targets) {
@@ -922,21 +902,6 @@ fn cmd_git_push(
                 Err(reason) => reason.print(ui)?,
             }
         }
-        if branches_targeted.is_empty() {
-            if use_default_revset {
-                writeln!(
-                    ui.warning(),
-                    "No branches found in the default push revset, \
-                     `remote_branches(remote={remote})..@`."
-                )?;
-            } else if !args.revisions.is_empty() {
-                writeln!(
-                    ui.warning(),
-                    "No branches point to the specified revisions."
-                )?;
-            } else { /* A plain "Nothing changed" message will suffice */
-            }
-        }
 
         tx_description = format!(
             "push {} to git remote {}",
@@ -950,27 +915,38 @@ fn cmd_git_push(
         );
     }
     if branch_updates.is_empty() {
-        writeln!(ui.stderr(), "Nothing changed.")?;
+        writeln!(ui.status(), "Nothing changed.")?;
         return Ok(());
     }
 
-    let mut new_heads = vec![];
-    let mut force_pushed_branches = hashset! {};
+    let mut branch_push_direction = HashMap::new();
     for (branch_name, update) in &branch_updates {
-        if let Some(new_target) = &update.new_target {
-            new_heads.push(new_target.clone());
-            let force = match &update.old_target {
-                None => false,
-                Some(old_target) => !repo.index().is_ancestor(old_target, new_target),
-            };
-            if force {
-                force_pushed_branches.insert(branch_name.to_string());
-            }
-        }
+        let BranchPushUpdate {
+            old_target: Some(old_target),
+            new_target: Some(new_target),
+        } = update
+        else {
+            continue;
+        };
+        assert_ne!(old_target, new_target);
+        branch_push_direction.insert(
+            branch_name.to_string(),
+            if repo.index().is_ancestor(old_target, new_target) {
+                BranchMoveDirection::Forward
+            } else if repo.index().is_ancestor(new_target, old_target) {
+                BranchMoveDirection::Backward
+            } else {
+                BranchMoveDirection::Sideways
+            },
+        );
     }
 
     // Check if there are conflicts in any commits we're about to push that haven't
     // already been pushed.
+    let new_heads = branch_updates
+        .iter()
+        .filter_map(|(_, update)| update.new_target.clone())
+        .collect_vec();
     let mut old_heads = repo
         .view()
         .remote_branches(&remote)
@@ -986,7 +962,7 @@ fn cmd_git_push(
     {
         let commit = commit?;
         let mut reasons = vec![];
-        if commit.description().is_empty() {
+        if commit.description().is_empty() && !args.allow_empty_description {
             reasons.push("it has no description");
         }
         if commit.author().name.is_empty()
@@ -1012,36 +988,40 @@ fn cmd_git_push(
         }
     }
 
-    writeln!(ui.stderr(), "Branch changes to push to {}:", &remote)?;
+    writeln!(ui.status(), "Branch changes to push to {}:", &remote)?;
     for (branch_name, update) in &branch_updates {
         match (&update.old_target, &update.new_target) {
             (Some(old_target), Some(new_target)) => {
-                if force_pushed_branches.contains(branch_name) {
-                    writeln!(
-                        ui.stderr(),
-                        "  Force branch {branch_name} from {} to {}",
-                        short_commit_hash(old_target),
-                        short_commit_hash(new_target)
-                    )?;
-                } else {
-                    writeln!(
-                        ui.stderr(),
-                        "  Move branch {branch_name} from {} to {}",
-                        short_commit_hash(old_target),
-                        short_commit_hash(new_target)
-                    )?;
-                }
+                let old = short_commit_hash(old_target);
+                let new = short_commit_hash(new_target);
+                // TODO(ilyagr): Add color. Once there is color, "Move branch ... sideways" may
+                // read more naturally than "Move sideways branch ...". Without color, it's hard
+                // to see at a glance if one branch among many was moved sideways (say).
+                // TODO: People on Discord suggest "Move branch ... forward by n commits",
+                // possibly "Move branch ... sideways (X forward, Y back)".
+                let msg = match branch_push_direction.get(branch_name).unwrap() {
+                    BranchMoveDirection::Forward => {
+                        format!("Move forward branch {branch_name} from {old} to {new}")
+                    }
+                    BranchMoveDirection::Backward => {
+                        format!("Move backward branch {branch_name} from {old} to {new}")
+                    }
+                    BranchMoveDirection::Sideways => {
+                        format!("Move sideways branch {branch_name} from {old} to {new}")
+                    }
+                };
+                writeln!(ui.status(), "  {msg}")?;
             }
             (Some(old_target), None) => {
                 writeln!(
-                    ui.stderr(),
+                    ui.status(),
                     "  Delete branch {branch_name} from {}",
                     short_commit_hash(old_target)
                 )?;
             }
             (None, Some(new_target)) => {
                 writeln!(
-                    ui.stderr(),
+                    ui.status(),
                     "  Add branch {branch_name} to {}",
                     short_commit_hash(new_target)
                 )?;
@@ -1053,26 +1033,32 @@ fn cmd_git_push(
     }
 
     if args.dry_run {
-        writeln!(ui.stderr(), "Dry-run requested, not pushing.")?;
+        writeln!(ui.status(), "Dry-run requested, not pushing.")?;
         return Ok(());
     }
 
-    let targets = GitBranchPushTargets {
-        branch_updates,
-        force_pushed_branches,
+    let targets = GitBranchPushTargets { branch_updates };
+    let mut writer = GitSidebandProgressMessageWriter::new(ui);
+    let mut sideband_progress_callback = |progress_message: &[u8]| {
+        _ = writer.write(ui, progress_message);
     };
-    with_remote_git_callbacks(ui, |cb| {
+    with_remote_git_callbacks(ui, Some(&mut sideband_progress_callback), |cb| {
         git::push_branches(tx.mut_repo(), &git_repo, &remote, &targets, cb)
     })
     .map_err(|err| match err {
         GitPushError::InternalGitError(err) => map_git_error(err),
-        GitPushError::NotFastForward => user_error_with_hint(
-            "The push conflicts with changes made on the remote (it is not fast-forwardable).",
+        GitPushError::RefInUnexpectedLocation(refs) => user_error_with_hint(
+            format!(
+                "Refusing to push a branch that unexpectedly moved on the remote. Affected refs: \
+                 {}",
+                refs.join(", ")
+            ),
             "Try fetching from the remote, then make the branch point to where you want it to be, \
              and push again.",
         ),
         _ => user_error(err),
     })?;
+    writer.flush(ui)?;
     tx.finish(ui, tx_description)?;
     Ok(())
 }
@@ -1087,7 +1073,9 @@ fn get_default_push_remote(
     } else if let Some(remote) = get_single_remote(git_repo)? {
         // similar to get_default_fetch_remotes
         if remote != DEFAULT_REMOTE {
-            writeln!(ui.hint(), "Pushing to the only existing remote: {}", remote)?;
+            if let Some(mut writer) = ui.hint_default() {
+                writeln!(writer, "Pushing to the only existing remote: {remote}")?;
+            }
         }
         Ok(remote)
     } else {
@@ -1103,9 +1091,11 @@ struct RejectedBranchUpdateReason {
 
 impl RejectedBranchUpdateReason {
     fn print(&self, ui: &Ui) -> io::Result<()> {
-        writeln!(ui.warning(), "{}", self.message)?;
+        writeln!(ui.warning_default(), "{}", self.message)?;
         if let Some(hint) = &self.hint {
-            writeln!(ui.hint(), "Hint: {hint}")?;
+            if let Some(mut writer) = ui.hint_default() {
+                writeln!(writer, "{hint}")?;
+            }
         }
         Ok(())
     }
@@ -1114,14 +1104,16 @@ impl RejectedBranchUpdateReason {
 impl From<RejectedBranchUpdateReason> for CommandError {
     fn from(reason: RejectedBranchUpdateReason) -> Self {
         let RejectedBranchUpdateReason { message, hint } = reason;
-        user_error_with_hint_opt(message, hint)
+        let mut cmd_err = user_error(message);
+        cmd_err.extend_hints(hint);
+        cmd_err
     }
 }
 
 fn classify_branch_update(
     branch_name: &str,
     remote_name: &str,
-    targets: TrackingRefPair,
+    targets: LocalAndRemoteRef,
 ) -> Result<Option<BranchPushUpdate>, RejectedBranchUpdateReason> {
     let push_action = classify_branch_push_action(targets);
     match push_action {
@@ -1146,12 +1138,49 @@ fn classify_branch_update(
     }
 }
 
+/// Creates or moves branches based on the change IDs.
+fn update_change_branches(
+    ui: &Ui,
+    tx: &mut WorkspaceCommandTransaction,
+    changes: &[RevisionArg],
+    branch_prefix: &str,
+) -> Result<Vec<String>, CommandError> {
+    let mut branch_names = Vec::new();
+    for change_arg in changes {
+        let workspace_command = tx.base_workspace_helper();
+        let commit = workspace_command.resolve_single_rev(change_arg)?;
+        let mut branch_name = format!("{branch_prefix}{}", commit.change_id().hex());
+        let view = tx.base_repo().view();
+        if view.get_local_branch(&branch_name).is_absent() {
+            // A local branch with the full change ID doesn't exist already, so use the
+            // short ID if it's not ambiguous (which it shouldn't be most of the time).
+            let short_change_id = short_change_hash(commit.change_id());
+            if workspace_command
+                .resolve_single_rev(&RevisionArg::from(short_change_id.clone()))
+                .is_ok()
+            {
+                // Short change ID is not ambiguous, so update the branch name to use it.
+                branch_name = format!("{branch_prefix}{short_change_id}");
+            };
+        }
+        if view.get_local_branch(&branch_name).is_absent() {
+            writeln!(
+                ui.status(),
+                "Creating branch {branch_name} for revision {change_arg}",
+            )?;
+        }
+        tx.mut_repo()
+            .set_local_branch_target(&branch_name, RefTarget::normal(commit.id().clone()));
+        branch_names.push(branch_name);
+    }
+    Ok(branch_names)
+}
+
 fn find_branches_to_push<'a>(
     view: &'a View,
     branch_patterns: &[StringPattern],
     remote_name: &str,
-    seen_branches: &mut HashSet<String>,
-) -> Result<Vec<(&'a str, TrackingRefPair<'a>)>, CommandError> {
+) -> Result<Vec<(&'a str, LocalAndRemoteRef<'a>)>, CommandError> {
     let mut matching_branches = vec![];
     let mut unmatched_patterns = vec![];
     for pattern in branch_patterns {
@@ -1166,8 +1195,7 @@ fn find_branches_to_push<'a>(
         if matches.peek().is_none() {
             unmatched_patterns.push(pattern);
         }
-        matching_branches
-            .extend(matches.filter(|&(name, _)| seen_branches.insert(name.to_owned())));
+        matching_branches.extend(matches);
     }
     match &unmatched_patterns[..] {
         [] => Ok(matching_branches),
@@ -1179,6 +1207,59 @@ fn find_branches_to_push<'a>(
     }
 }
 
+fn find_branches_targeted_by_revisions<'a>(
+    ui: &Ui,
+    workspace_command: &'a WorkspaceCommandHelper,
+    remote_name: &str,
+    revisions: &[RevisionArg],
+    use_default_revset: bool,
+) -> Result<Vec<(&'a str, LocalAndRemoteRef<'a>)>, CommandError> {
+    let mut revision_commit_ids = HashSet::new();
+    if use_default_revset {
+        let Some(wc_commit_id) = workspace_command.get_wc_commit_id().cloned() else {
+            return Err(user_error("Nothing checked out in this workspace"));
+        };
+        let current_branches_expression = RevsetExpression::remote_branches(
+            StringPattern::everything(),
+            StringPattern::Exact(remote_name.to_owned()),
+        )
+        .range(&RevsetExpression::commit(wc_commit_id))
+        .intersection(&RevsetExpression::branches(StringPattern::everything()));
+        let current_branches_revset =
+            current_branches_expression.evaluate_programmatic(workspace_command.repo().as_ref())?;
+        revision_commit_ids.extend(current_branches_revset.iter());
+        if revision_commit_ids.is_empty() {
+            writeln!(
+                ui.warning_default(),
+                "No branches found in the default push revset: \
+                 remote_branches(remote={remote_name})..@"
+            )?;
+        }
+    }
+    for rev_arg in revisions {
+        let mut expression = workspace_command.parse_revset(rev_arg)?;
+        expression.intersect_with(&RevsetExpression::branches(StringPattern::everything()));
+        let mut commit_ids = expression.evaluate_to_commit_ids()?.peekable();
+        if commit_ids.peek().is_none() {
+            writeln!(
+                ui.warning_default(),
+                "No branches point to the specified revisions: {rev_arg}"
+            )?;
+        }
+        revision_commit_ids.extend(commit_ids);
+    }
+    let branches_targeted = workspace_command
+        .repo()
+        .view()
+        .local_remote_branches(remote_name)
+        .filter(|(_, targets)| {
+            let mut local_ids = targets.local_target.added_ids();
+            local_ids.any(|id| revision_commit_ids.contains(id))
+        })
+        .collect_vec();
+    Ok(branches_targeted)
+}
+
 fn cmd_git_import(
     ui: &mut Ui,
     command: &CommandHelper,
@@ -1190,7 +1271,7 @@ fn cmd_git_import(
     // That's why cmd_git_export() doesn't export the HEAD ref.
     git::import_head(tx.mut_repo())?;
     let stats = git::import_refs(tx.mut_repo(), &command.settings().git_settings())?;
-    print_git_import_stats(ui, &stats)?;
+    print_git_import_stats(ui, tx.repo(), &stats, true)?;
     tx.finish(ui, "import git refs")?;
     Ok(())
 }
@@ -1215,12 +1296,12 @@ fn cmd_git_submodule_print_gitmodules(
 ) -> Result<(), CommandError> {
     let workspace_command = command.workspace_helper(ui)?;
     let repo = workspace_command.repo();
-    let commit = workspace_command.resolve_single_rev(&args.revisions, ui)?;
+    let commit = workspace_command.resolve_single_rev(&args.revisions)?;
     let tree = commit.tree()?;
     let gitmodules_path = RepoPath::from_internal_string(".gitmodules");
-    let mut gitmodules_file = match tree.path_value(gitmodules_path).into_resolved() {
+    let mut gitmodules_file = match tree.path_value(gitmodules_path)?.into_resolved() {
         Ok(None) => {
-            writeln!(ui.stderr(), "No submodules!")?;
+            writeln!(ui.status(), "No submodules!")?;
             return Ok(());
         }
         Ok(Some(TreeValue::File { id, .. })) => repo.store().read_file(gitmodules_path, &id)?,
