@@ -20,7 +20,8 @@ use jj_lib::workspace::Workspace;
 use tracing::instrument;
 
 use super::git;
-use crate::cli_util::{user_error_with_hint, user_error_with_message, CommandError, CommandHelper};
+use crate::cli_util::CommandHelper;
+use crate::command_error::{user_error_with_hint, user_error_with_message, CommandError};
 use crate::ui::Ui;
 
 /// Create a new repo in the given directory
@@ -49,7 +50,7 @@ pub(crate) fn cmd_init(
     command: &CommandHelper,
     args: &InitArgs,
 ) -> Result<(), CommandError> {
-    let cwd = command.cwd().canonicalize().unwrap();
+    let cwd = command.cwd();
     let wc_path = cwd.join(&args.destination);
     let wc_path = file_util::create_or_reuse_dir(&wc_path)
         .and_then(|_| wc_path.canonicalize())
@@ -61,8 +62,8 @@ pub(crate) fn cmd_init(
     if args.git || args.git_repo.is_some() {
         git::git_init(ui, command, &wc_path, colocate, args.git_repo.as_deref())?;
         writeln!(
-            ui.warning(),
-            "warning: `--git` and `--git-repo` are deprecated.
+            ui.warning_default(),
+            "`--git` and `--git-repo` are deprecated.
 Use `jj git init` instead"
         )?;
     } else {
@@ -76,9 +77,9 @@ Set `ui.allow-init-native` to allow initializing a repo with the native backend.
         Workspace::init_local(command.settings(), &wc_path)?;
     }
 
-    let relative_wc_path = file_util::relative_path(&cwd, &wc_path);
+    let relative_wc_path = file_util::relative_path(cwd, &wc_path);
     writeln!(
-        ui.stderr(),
+        ui.status(),
         "Initialized repo in \"{}\"",
         relative_wc_path.display()
     )?;
