@@ -13,17 +13,15 @@
 // limitations under the License.
 
 use jj_lib::backend::CommitId;
+use jj_lib::graph::{GraphEdgeType, ReverseGraphIterator, TopoGroupedGraphIterator};
 use jj_lib::repo::Repo;
-use jj_lib::revset::{self, RevsetExpression, RevsetFilterPredicate, RevsetIteratorExt};
-use jj_lib::revset_graph::{
-    ReverseRevsetGraphIterator, RevsetGraphEdgeType, TopoGroupedRevsetGraphIterator,
-};
+use jj_lib::revset::{RevsetExpression, RevsetFilterPredicate, RevsetIteratorExt};
 use tracing::instrument;
 
 use crate::cli_util::{format_template, CommandHelper, LogContentFormat, RevisionArg};
 use crate::command_error::CommandError;
 use crate::commit_templater::CommitTemplateLanguage;
-use crate::diff_util::{self, DiffFormatArgs};
+use crate::diff_util::DiffFormatArgs;
 use crate::graphlog::{get_graphlog, Edge};
 use crate::ui::Ui;
 
@@ -102,8 +100,7 @@ pub(crate) fn cmd_log(
     let revset = revset_expression.evaluate()?;
 
     let store = repo.store();
-    let diff_formats =
-        diff_util::diff_formats_for_log(command.settings(), &args.diff_format, args.patch)?;
+    let diff_renderer = workspace_command.diff_renderer_for_log(&args.diff_format, args.patch)?;
 
     let use_elided_nodes = command
         .settings()
@@ -142,9 +139,9 @@ pub(crate) fn cmd_log(
 
         if !args.no_graph {
             let mut graph = get_graphlog(command.settings(), formatter.raw());
-            let forward_iter = TopoGroupedRevsetGraphIterator::new(revset.iter_graph());
+            let forward_iter = TopoGroupedGraphIterator::new(revset.iter_graph());
             let iter: Box<dyn Iterator<Item = _>> = if args.reversed {
-                Box::new(ReverseRevsetGraphIterator::new(forward_iter))
+                Box::new(ReverseGraphIterator::new(forward_iter))
             } else {
                 Box::new(forward_iter)
             };
@@ -158,13 +155,13 @@ pub(crate) fn cmd_log(
                 let mut elided_targets = vec![];
                 for edge in edges {
                     match edge.edge_type {
-                        RevsetGraphEdgeType::Missing => {
+                        GraphEdgeType::Missing => {
                             has_missing = true;
                         }
-                        RevsetGraphEdgeType::Direct => {
+                        GraphEdgeType::Direct => {
                             graphlog_edges.push(Edge::Direct((edge.target, false)));
                         }
-                        RevsetGraphEdgeType::Indirect => {
+                        GraphEdgeType::Indirect => {
                             if use_elided_nodes {
                                 elided_targets.push(edge.target.clone());
                                 graphlog_edges.push(Edge::Direct((edge.target, true)));
@@ -188,16 +185,9 @@ pub(crate) fn cmd_log(
                 if !buffer.ends_with(b"\n") {
                     buffer.push(b'\n');
                 }
-                if !diff_formats.is_empty() {
+                if let Some(renderer) = &diff_renderer {
                     let mut formatter = ui.new_formatter(&mut buffer);
-                    diff_util::show_patch(
-                        ui,
-                        formatter.as_mut(),
-                        &workspace_command,
-                        &commit,
-                        matcher.as_ref(),
-                        &diff_formats,
-                    )?;
+                    renderer.show_patch(ui, formatter.as_mut(), &commit, matcher.as_ref())?;
                 }
 
                 let node_symbol = format_template(ui, &Some(commit), &node_template);
@@ -236,15 +226,8 @@ pub(crate) fn cmd_log(
                 let commit = commit_or_error?;
                 with_content_format
                     .write(formatter, |formatter| template.format(&commit, formatter))?;
-                if !diff_formats.is_empty() {
-                    diff_util::show_patch(
-                        ui,
-                        formatter,
-                        &workspace_command,
-                        &commit,
-                        matcher.as_ref(),
-                        &diff_formats,
-                    )?;
+                if let Some(renderer) = &diff_renderer {
+                    renderer.show_patch(ui, formatter, &commit, matcher.as_ref())?;
                 }
             }
         }
@@ -262,7 +245,9 @@ pub(crate) fn cmd_log(
                  working copy commit, pass -r '@' instead."
             )?;
         } else if revset.is_empty()
-            && revset::parse(only_path, &workspace_command.revset_parse_context()).is_ok()
+            && workspace_command
+                .parse_revset(&RevisionArg::from(only_path.to_owned()))
+                .is_ok()
         {
             writeln!(
                 ui.warning_default(),
