@@ -29,6 +29,7 @@ mod diffedit;
 mod duplicate;
 mod edit;
 mod files;
+mod fix;
 mod git;
 mod init;
 mod interdiff;
@@ -39,6 +40,7 @@ mod new;
 mod next;
 mod obslog;
 mod operation;
+mod parallelize;
 mod prev;
 mod rebase;
 mod resolve;
@@ -62,7 +64,8 @@ use std::fmt::Debug;
 use clap::{CommandFactory, FromArgMatches, Subcommand};
 use tracing::instrument;
 
-use crate::cli_util::{user_error_with_hint, Args, CommandError, CommandHelper};
+use crate::cli_util::{Args, CommandHelper};
+use crate::command_error::{user_error_with_hint, CommandError};
 use crate::ui::Ui;
 
 #[derive(clap::Parser, Clone, Debug)]
@@ -90,12 +93,13 @@ enum Command {
     Duplicate(duplicate::DuplicateArgs),
     Edit(edit::EditArgs),
     Files(files::FilesArgs),
+    Fix(fix::FixArgs),
     #[command(subcommand)]
     Git(git::GitCommand),
     Init(init::InitArgs),
     Interdiff(interdiff::InterdiffArgs),
     Log(log::LogArgs),
-    /// Merge work from multiple branches
+    /// Merge work from multiple branches (DEPRECATED, use `jj new`)
     ///
     /// Unlike most other VCSs, `jj merge` does not implicitly include the
     /// working copy revision's parent as one of the parents of the merge;
@@ -106,6 +110,7 @@ enum Command {
     /// arguments.
     #[command(hide = true)]
     Merge(new::NewArgs),
+    #[command(hide = true)]
     Move(r#move::MoveArgs),
     New(new::NewArgs),
     Next(next::NextArgs),
@@ -113,6 +118,7 @@ enum Command {
     #[command(subcommand)]
     #[command(visible_alias = "op")]
     Operation(operation::OperationCommand),
+    Parallelize(parallelize::ParallelizeArgs),
     Prev(prev::PrevArgs),
     Rebase(rebase::RebaseArgs),
     Resolve(resolve::ResolveArgs),
@@ -179,6 +185,9 @@ pub fn run_command(ui: &mut Ui, command_helper: &CommandHelper) -> Result<(), Co
         Command::Abandon(sub_args) => abandon::cmd_abandon(ui, command_helper, sub_args),
         Command::Edit(sub_args) => edit::cmd_edit(ui, command_helper, sub_args),
         Command::Next(sub_args) => next::cmd_next(ui, command_helper, sub_args),
+        Command::Parallelize(sub_args) => {
+            parallelize::cmd_parallelize(ui, command_helper, sub_args)
+        }
         Command::Prev(sub_args) => prev::cmd_prev(ui, command_helper, sub_args),
         Command::New(sub_args) => new::cmd_new(ui, command_helper, sub_args),
         Command::Move(sub_args) => r#move::cmd_move(ui, command_helper, sub_args),
@@ -193,6 +202,7 @@ pub fn run_command(ui: &mut Ui, command_helper: &CommandHelper) -> Result<(), Co
         Command::Merge(sub_args) => merge::cmd_merge(ui, command_helper, sub_args),
         Command::Rebase(sub_args) => rebase::cmd_rebase(ui, command_helper, sub_args),
         Command::Backout(sub_args) => backout::cmd_backout(ui, command_helper, sub_args),
+        Command::Fix(sub_args) => fix::cmd_fix(ui, command_helper, sub_args),
         Command::Resolve(sub_args) => resolve::cmd_resolve(ui, command_helper, sub_args),
         Command::Branch(sub_args) => branch::cmd_branch(ui, command_helper, sub_args),
         Command::Undo(sub_args) => operation::cmd_op_undo(ui, command_helper, sub_args),
