@@ -16,12 +16,12 @@ use clap::ArgGroup;
 use jj_lib::object_id::ObjectId;
 use tracing::instrument;
 
-use super::squash::move_diff;
+use super::squash::{move_diff, SquashedDescription};
 use crate::cli_util::{CommandHelper, RevisionArg};
 use crate::command_error::{user_error, CommandError};
 use crate::ui::Ui;
 
-/// Move changes from one revision into another
+/// Move changes from one revision into another (DEPRECATED, use `jj squash`)
 ///
 /// Use `--interactive` to move only part of the source revision into the
 /// destination. The selected changes (or all the changes in the source revision
@@ -78,7 +78,9 @@ pub(crate) fn cmd_move(
     if source.id() == destination.id() {
         return Err(user_error("Source and destination cannot be the same."));
     }
-    let matcher = workspace_command.matcher_from_values(&args.paths)?;
+    let matcher = workspace_command
+        .parse_file_patterns(&args.paths)?
+        .to_matcher();
     let diff_selector =
         workspace_command.diff_selector(ui, args.tool.as_deref(), args.interactive)?;
     let mut tx = workspace_command.start_transaction();
@@ -95,7 +97,7 @@ pub(crate) fn cmd_move(
         &destination,
         matcher.as_ref(),
         &diff_selector,
-        None,
+        SquashedDescription::Combine,
         false,
         &args.paths,
     )?;
