@@ -15,13 +15,13 @@
 use itertools::Itertools;
 use jj_lib::repo::Repo;
 use jj_lib::revset::{RevsetExpression, RevsetFilterPredicate};
-use jj_lib::rewrite::merge_commit_trees;
 use tracing::instrument;
 
 use crate::cli_util::{print_conflicted_paths, CommandHelper};
 use crate::command_error::CommandError;
+use crate::diff_util::DiffFormat;
+use crate::revset_util;
 use crate::ui::Ui;
-use crate::{diff_util, revset_util};
 
 /// Show high-level repo status
 ///
@@ -59,17 +59,14 @@ pub(crate) fn cmd_status(
     let formatter = formatter.as_mut();
 
     if let Some(wc_commit) = &maybe_wc_commit {
-        let parent_tree = merge_commit_trees(repo.as_ref(), &wc_commit.parents())?;
+        let parent_tree = wc_commit.parent_tree(repo.as_ref())?;
         let tree = wc_commit.tree()?;
         if tree.id() == parent_tree.id() {
             writeln!(formatter, "The working copy is clean")?;
         } else {
             writeln!(formatter, "Working copy changes:")?;
-            diff_util::show_diff_summary(
-                formatter,
-                &workspace_command,
-                parent_tree.diff_stream(&tree, matcher.as_ref()),
-            )?;
+            let diff_renderer = workspace_command.diff_renderer(vec![DiffFormat::Summary]);
+            diff_renderer.show_diff(ui, formatter, &parent_tree, &tree, &matcher)?;
         }
 
         // TODO: Conflicts should also be filtered by the `matcher`. See the related
@@ -88,6 +85,7 @@ pub(crate) fn cmd_status(
         formatter.with_label("working_copy", |fmt| template.format(wc_commit, fmt))?;
         writeln!(formatter)?;
         for parent in wc_commit.parents() {
+            let parent = parent?;
             write!(formatter, "Parent commit: ")?;
             template.format(&parent, formatter)?;
             writeln!(formatter)?;
