@@ -16,12 +16,12 @@ use std::io::Write;
 use std::slice;
 use std::time::{Duration, SystemTime};
 
-use clap::Subcommand;
-use clap_complete::Shell;
+use clap::{Command, Subcommand};
 use jj_lib::repo::Repo;
 use tracing::instrument;
 
-use crate::cli_util::{user_error, CommandError, CommandHelper};
+use crate::cli_util::CommandHelper;
+use crate::command_error::{user_error, CommandError};
 use crate::ui::Ui;
 
 /// Infrequently used commands such as for generating shell completions
@@ -40,19 +40,24 @@ pub(crate) enum UtilCommand {
 
 Apply it by running one of these:
 
-- **bash**: `source <(jj util completion)`
-- **fish**: `jj util completion --fish | source`
-- **zsh**:
+- Bash: `source <(jj util completion bash)`
+- Fish: `jj util completion fish | source`
+- Nushell:
+     ```nu
+     jj util completion nushell | save "completions-jj.nu"
+     use "completions-jj.nu" *  # Or `source "completions-jj.nu"`
+     ```
+- Zsh:
      ```shell
      autoload -U compinit
      compinit
-     source <(jj util completion --zsh)
+     source <(jj util completion zsh)
      ```
 "#]
 #[derive(clap::Args, Clone, Debug)]
 #[command(verbatim_doc_comment)]
 pub(crate) struct UtilCompletionArgs {
-    shell: Option<Shell>,
+    shell: Option<ShellCompletion>,
     /// Deprecated. Use the SHELL positional argument instead.
     #[arg(long, hide = true)]
     bash: bool,
@@ -91,6 +96,17 @@ pub(crate) struct UtilMarkdownHelp {}
 #[derive(clap::Args, Clone, Debug)]
 pub(crate) struct UtilConfigSchemaArgs {}
 
+/// Available shell completions
+#[derive(clap::ValueEnum, Clone, Copy, Debug, Eq, Hash, PartialEq)]
+enum ShellCompletion {
+    Bash,
+    Elvish,
+    Fish,
+    Nushell,
+    PowerShell,
+    Zsh,
+}
+
 #[instrument(skip_all)]
 pub(crate) fn cmd_util(
     ui: &mut Ui,
@@ -112,30 +128,32 @@ fn cmd_util_completion(
     args: &UtilCompletionArgs,
 ) -> Result<(), CommandError> {
     let mut app = command.app().clone();
-    let warn = |shell| {
+    let warn = |shell| -> std::io::Result<()> {
         writeln!(
-            ui.warning(),
-            "Warning: `jj util completion --{shell}` will be removed in a future version, and \
-             this will be a hard error"
+            ui.warning_default(),
+            "`jj util completion --{shell}` will be removed in a future version, and this will be \
+             a hard error"
         )?;
-        writeln!(ui.hint(), "Hint: Use `jj util completion {shell}` instead")
+        if let Some(mut writer) = ui.hint_default() {
+            writeln!(writer, "Use `jj util completion {shell}` instead")?;
+        }
+        Ok(())
     };
-    let mut buf = vec![];
     let shell = match (args.shell, args.fish, args.zsh, args.bash) {
         (Some(s), false, false, false) => s,
         // allow `--fish` and `--zsh` for back-compat, but don't allow them to be combined
         (None, true, false, false) => {
             warn("fish")?;
-            Shell::Fish
+            ShellCompletion::Fish
         }
         (None, false, true, false) => {
             warn("zsh")?;
-            Shell::Zsh
+            ShellCompletion::Zsh
         }
         // default to bash for back-compat. TODO: consider making `shell` a required argument
         (None, false, false, _) => {
             warn("bash")?;
-            Shell::Bash
+            ShellCompletion::Bash
         }
         _ => {
             return Err(user_error(
@@ -143,7 +161,8 @@ fn cmd_util_completion(
             ))
         }
     };
-    clap_complete::generate(shell, &mut app, "jj", &mut buf);
+
+    let buf = shell.generate(&mut app);
     ui.stdout_formatter().write_all(&buf)?;
     Ok(())
 }
@@ -206,3 +225,25 @@ fn cmd_util_config_schema(
     ui.stdout_formatter().write_all(buf)?;
     Ok(())
 }
+
+impl ShellCompletion {
+    fn generate(&self, cmd: &mut Command) -> Vec<u8> {
+        use clap_complete::{generate, Shell};
+        use clap_complete_nushell::Nushell;
+
+        let mut buf = Vec::new();
+
+        let bin_name = "jj";
+
+        match self {
+            Self::Bash => generate(Shell::Bash, cmd, bin_name, &mut buf),
+            Self::Elvish => generate(Shell::Elvish, cmd, bin_name, &mut buf),
+            Self::Fish => generate(Shell::Fish, cmd, bin_name, &mut buf),
+            Self::Nushell => generate(Nushell, cmd, bin_name, &mut buf),
+            Self::PowerShell => generate(Shell::PowerShell, cmd, bin_name, &mut buf),
+            Self::Zsh => generate(Shell::Zsh, cmd, bin_name, &mut buf),
+        }
+
+        buf
+    }
+}
