@@ -22,13 +22,13 @@ use itertools::Itertools as _;
 use jj_lib::backend::{ChangeId, CommitId};
 use jj_lib::commit::Commit;
 use jj_lib::extensions_map::ExtensionsMap;
+use jj_lib::git;
 use jj_lib::hex_util::to_reverse_hex;
 use jj_lib::id_prefix::IdPrefixContext;
 use jj_lib::object_id::ObjectId as _;
-use jj_lib::op_store::{RefTarget, WorkspaceId};
+use jj_lib::op_store::{RefTarget, RemoteRef, WorkspaceId};
 use jj_lib::repo::Repo;
-use jj_lib::revset::{self, Revset, RevsetExpression, RevsetParseContext};
-use jj_lib::{git, rewrite};
+use jj_lib::revset::{self, Revset, RevsetExpression, RevsetModifier, RevsetParseContext};
 use once_cell::unsync::OnceCell;
 
 use crate::template_builder::{
@@ -37,7 +37,7 @@ use crate::template_builder::{
 };
 use crate::template_parser::{self, FunctionCallNode, TemplateParseError, TemplateParseResult};
 use crate::templater::{
-    self, IntoTemplate, PlainTextFormattedProperty, Template, TemplateFormatter, TemplateProperty,
+    self, PlainTextFormattedProperty, SizeHint, Template, TemplateFormatter, TemplateProperty,
     TemplatePropertyError, TemplatePropertyExt as _,
 };
 use crate::{revset_util, text_util};
@@ -130,9 +130,7 @@ impl<'repo> TemplateLanguage<'repo> for CommitTemplateLanguage<'repo> {
                 let type_name = "Commit";
                 let table = &self.build_fn_table.commit_methods;
                 let build = template_parser::lookup_method(type_name, table, function)?;
-                let inner_property = property.and_then(|opt| {
-                    opt.ok_or_else(|| TemplatePropertyError("No commit available".into()))
-                });
+                let inner_property = property.try_unwrap(type_name);
                 build(self, build_ctx, Box::new(inner_property), function)
             }
             CommitTemplatePropertyKind::CommitList(property) => {
@@ -154,9 +152,7 @@ impl<'repo> TemplateLanguage<'repo> for CommitTemplateLanguage<'repo> {
                 let type_name = "RefName";
                 let table = &self.build_fn_table.ref_name_methods;
                 let build = template_parser::lookup_method(type_name, table, function)?;
-                let inner_property = property.and_then(|opt| {
-                    opt.ok_or_else(|| TemplatePropertyError("No RefName available".into()))
-                });
+                let inner_property = property.try_unwrap(type_name);
                 build(self, build_ctx, Box::new(inner_property), function)
             }
             CommitTemplatePropertyKind::RefNameList(property) => {
@@ -221,19 +217,19 @@ impl<'repo> CommitTemplateLanguage<'repo> {
     }
 
     pub fn wrap_ref_name(
-        property: impl TemplateProperty<Output = RefName> + 'repo,
+        property: impl TemplateProperty<Output = Rc<RefName>> + 'repo,
     ) -> CommitTemplatePropertyKind<'repo> {
         CommitTemplatePropertyKind::RefName(Box::new(property))
     }
 
     pub fn wrap_ref_name_opt(
-        property: impl TemplateProperty<Output = Option<RefName>> + 'repo,
+        property: impl TemplateProperty<Output = Option<Rc<RefName>>> + 'repo,
     ) -> CommitTemplatePropertyKind<'repo> {
         CommitTemplatePropertyKind::RefNameOpt(Box::new(property))
     }
 
     pub fn wrap_ref_name_list(
-        property: impl TemplateProperty<Output = Vec<RefName>> + 'repo,
+        property: impl TemplateProperty<Output = Vec<Rc<RefName>>> + 'repo,
     ) -> CommitTemplatePropertyKind<'repo> {
         CommitTemplatePropertyKind::RefNameList(Box::new(property))
     }
@@ -256,9 +252,9 @@ pub enum CommitTemplatePropertyKind<'repo> {
     Commit(Box<dyn TemplateProperty<Output = Commit> + 'repo>),
     CommitOpt(Box<dyn TemplateProperty<Output = Option<Commit>> + 'repo>),
     CommitList(Box<dyn TemplateProperty<Output = Vec<Commit>> + 'repo>),
-    RefName(Box<dyn TemplateProperty<Output = RefName> + 'repo>),
-    RefNameOpt(Box<dyn TemplateProperty<Output = Option<RefName>> + 'repo>),
-    RefNameList(Box<dyn TemplateProperty<Output = Vec<RefName>> + 'repo>),
+    RefName(Box<dyn TemplateProperty<Output = Rc<RefName>> + 'repo>),
+    RefNameOpt(Box<dyn TemplateProperty<Output = Option<Rc<RefName>>> + 'repo>),
+    RefNameList(Box<dyn TemplateProperty<Output = Vec<Rc<RefName>>> + 'repo>),
     CommitOrChangeId(Box<dyn TemplateProperty<Output = CommitOrChangeId> + 'repo>),
     ShortestIdPrefix(Box<dyn TemplateProperty<Output = ShortestIdPrefix> + 'repo>),
 }
@@ -344,7 +340,7 @@ pub type CommitTemplateBuildMethodFnMap<'repo, T> =
 pub struct CommitTemplateBuildFnTable<'repo> {
     pub core: CoreTemplateBuildFnTable<'repo, CommitTemplateLanguage<'repo>>,
     pub commit_methods: CommitTemplateBuildMethodFnMap<'repo, Commit>,
-    pub ref_name_methods: CommitTemplateBuildMethodFnMap<'repo, RefName>,
+    pub ref_name_methods: CommitTemplateBuildMethodFnMap<'repo, Rc<RefName>>,
     pub commit_or_change_id_methods: CommitTemplateBuildMethodFnMap<'repo, CommitOrChangeId>,
     pub shortest_id_prefix_methods: CommitTemplateBuildMethodFnMap<'repo, ShortestIdPrefix>,
 }
@@ -439,7 +435,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "description",
         |_language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let out_property =
                 self_property.map(|commit| text_util::complete_newline(commit.description()));
             Ok(L::wrap_string(out_property))
@@ -448,7 +444,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "change_id",
         |_language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let out_property =
                 self_property.map(|commit| CommitOrChangeId::Change(commit.change_id().to_owned()));
             Ok(L::wrap_commit_or_change_id(out_property))
@@ -457,7 +453,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "commit_id",
         |_language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let out_property =
                 self_property.map(|commit| CommitOrChangeId::Commit(commit.id().to_owned()));
             Ok(L::wrap_commit_or_change_id(out_property))
@@ -466,15 +462,16 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "parents",
         |_language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
-            let out_property = self_property.map(|commit| commit.parents());
+            function.expect_no_arguments()?;
+            let out_property =
+                self_property.and_then(|commit| Ok(commit.parents().try_collect()?));
             Ok(L::wrap_commit_list(out_property))
         },
     );
     map.insert(
         "author",
         |_language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let out_property = self_property.map(|commit| commit.author().clone());
             Ok(L::wrap_signature(out_property))
         },
@@ -482,21 +479,21 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "committer",
         |_language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let out_property = self_property.map(|commit| commit.committer().clone());
             Ok(L::wrap_signature(out_property))
         },
     );
     map.insert("mine", |language, _build_ctx, self_property, function| {
-        template_parser::expect_no_arguments(function)?;
-        let user_email = language.revset_parse_context.user_email.clone();
+        function.expect_no_arguments()?;
+        let user_email = language.revset_parse_context.user_email().to_owned();
         let out_property = self_property.map(move |commit| commit.author().email == user_email);
         Ok(L::wrap_boolean(out_property))
     });
     map.insert(
         "working_copies",
         |language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let repo = language.repo;
             let out_property = self_property.map(|commit| extract_working_copies(repo, &commit));
             Ok(L::wrap_string(out_property))
@@ -505,7 +502,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "current_working_copy",
         |language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let repo = language.repo;
             let workspace_id = language.workspace_id.clone();
             let out_property = self_property.map(move |commit| {
@@ -517,7 +514,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "branches",
         |language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let index = language.keyword_cache.branches_index(language.repo).clone();
             let out_property = self_property.map(move |commit| {
                 index
@@ -533,7 +530,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "local_branches",
         |language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let index = language.keyword_cache.branches_index(language.repo).clone();
             let out_property = self_property.map(move |commit| {
                 index
@@ -549,7 +546,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "remote_branches",
         |language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let index = language.keyword_cache.branches_index(language.repo).clone();
             let out_property = self_property.map(move |commit| {
                 index
@@ -563,7 +560,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
         },
     );
     map.insert("tags", |language, _build_ctx, self_property, function| {
-        template_parser::expect_no_arguments(function)?;
+        function.expect_no_arguments()?;
         let index = language.keyword_cache.tags_index(language.repo).clone();
         let out_property = self_property.map(move |commit| index.get(commit.id()).to_vec());
         Ok(L::wrap_ref_name_list(out_property))
@@ -571,7 +568,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "git_refs",
         |language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let index = language.keyword_cache.git_refs_index(language.repo).clone();
             let out_property = self_property.map(move |commit| index.get(commit.id()).to_vec());
             Ok(L::wrap_ref_name_list(out_property))
@@ -580,7 +577,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "git_head",
         |language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let repo = language.repo;
             let out_property = self_property.map(|commit| extract_git_head(repo, &commit));
             Ok(L::wrap_ref_name_opt(out_property))
@@ -589,7 +586,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "divergent",
         |language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let repo = language.repo;
             let out_property = self_property.map(|commit| {
                 // The given commit could be hidden in e.g. obslog.
@@ -600,7 +597,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
         },
     );
     map.insert("hidden", |language, _build_ctx, self_property, function| {
-        template_parser::expect_no_arguments(function)?;
+        function.expect_no_arguments()?;
         let repo = language.repo;
         let out_property = self_property.map(|commit| {
             let maybe_entries = repo.resolve_change_id(commit.change_id());
@@ -611,7 +608,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "immutable",
         |language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let is_immutable = language
                 .keyword_cache
                 .is_immutable_fn(language, function.name_span)?
@@ -623,7 +620,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "contained_in",
         |language, _build_ctx, self_property, function| {
-            let [revset_node] = template_parser::expect_exact_arguments(function)?;
+            let [revset_node] = function.expect_exact_arguments()?;
 
             let is_contained =
                 template_parser::expect_string_literal_with(revset_node, |revset, span| {
@@ -637,25 +634,19 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
     map.insert(
         "conflict",
         |_language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let out_property = self_property.and_then(|commit| Ok(commit.has_conflict()?));
             Ok(L::wrap_boolean(out_property))
         },
     );
     map.insert("empty", |language, _build_ctx, self_property, function| {
-        template_parser::expect_no_arguments(function)?;
+        function.expect_no_arguments()?;
         let repo = language.repo;
-        let out_property = self_property.and_then(|commit| {
-            if let [parent] = &commit.parents()[..] {
-                return Ok(parent.tree_id() == commit.tree_id());
-            }
-            let parent_tree = rewrite::merge_commit_trees(repo, &commit.parents())?;
-            Ok(*commit.tree_id() == parent_tree.id())
-        });
+        let out_property = self_property.and_then(|commit| Ok(commit.is_empty(repo)?));
         Ok(L::wrap_boolean(out_property))
     });
     map.insert("root", |language, _build_ctx, self_property, function| {
-        template_parser::expect_no_arguments(function)?;
+        function.expect_no_arguments()?;
         let repo = language.repo;
         let out_property = self_property.map(|commit| commit.id() == repo.store().root_commit_id());
         Ok(L::wrap_boolean(out_property))
@@ -687,7 +678,7 @@ fn evaluate_revset_expression<'repo>(
 ) -> Result<Box<dyn Revset + 'repo>, TemplateParseError> {
     let symbol_resolver = revset_util::default_symbol_resolver(
         language.repo,
-        language.revset_parse_context.extensions.symbol_resolvers(),
+        language.revset_parse_context.symbol_resolvers(),
         language.id_prefix_context,
     );
     let revset =
@@ -717,28 +708,114 @@ fn evaluate_user_revset<'repo>(
     span: pest::Span<'_>,
     revset: &str,
 ) -> Result<Box<dyn Revset + 'repo>, TemplateParseError> {
-    let expression = revset::parse(revset, &language.revset_parse_context).map_err(|err| {
-        TemplateParseError::expression("Failed to parse revset", span).with_source(err)
-    })?;
+    let (expression, modifier) =
+        revset::parse_with_modifier(revset, &language.revset_parse_context).map_err(|err| {
+            TemplateParseError::expression("Failed to parse revset", span).with_source(err)
+        })?;
+    let (None | Some(RevsetModifier::All)) = modifier;
 
     evaluate_revset_expression(language, span, expression)
 }
 
 /// Branch or tag name with metadata.
-#[derive(Clone, Debug, Eq, PartialEq)]
+#[derive(Debug)]
 pub struct RefName {
     /// Local name.
     name: String,
     /// Remote name if this is a remote or Git-tracking ref.
     remote: Option<String>,
-    /// Ref target has conflicts.
-    conflict: bool,
+    /// Target commit ids.
+    target: RefTarget,
+    /// Local ref metadata which tracks this remote ref.
+    tracking_ref: Option<TrackingRef>,
     /// Local ref is synchronized with all tracking remotes, or tracking remote
     /// ref is synchronized with the local.
     synced: bool,
 }
 
+#[derive(Debug)]
+struct TrackingRef {
+    /// Local ref target which tracks the other remote ref.
+    target: RefTarget,
+    /// Number of commits ahead of the tracking `target`.
+    ahead_count: OnceCell<SizeHint>,
+    /// Number of commits behind of the tracking `target`.
+    behind_count: OnceCell<SizeHint>,
+}
+
 impl RefName {
+    // RefName is wrapped by Rc<T> to make it cheaply cloned and share
+    // lazy-evaluation results across clones.
+
+    /// Creates local ref representation which might track some of the
+    /// `remote_refs`.
+    pub fn local<'a>(
+        name: impl Into<String>,
+        target: RefTarget,
+        remote_refs: impl IntoIterator<Item = &'a RemoteRef>,
+    ) -> Rc<Self> {
+        let synced = remote_refs
+            .into_iter()
+            .all(|remote_ref| !remote_ref.is_tracking() || remote_ref.target == target);
+        Rc::new(RefName {
+            name: name.into(),
+            remote: None,
+            target,
+            tracking_ref: None,
+            synced,
+        })
+    }
+
+    /// Creates local ref representation which doesn't track any remote refs.
+    pub fn local_only(name: impl Into<String>, target: RefTarget) -> Rc<Self> {
+        Self::local(name, target, [])
+    }
+
+    /// Creates remote ref representation which might be tracked by a local ref
+    /// pointing to the `local_target`.
+    pub fn remote(
+        name: impl Into<String>,
+        remote_name: impl Into<String>,
+        remote_ref: RemoteRef,
+        local_target: &RefTarget,
+    ) -> Rc<Self> {
+        let synced = remote_ref.is_tracking() && remote_ref.target == *local_target;
+        let tracking_ref = remote_ref.is_tracking().then(|| {
+            let count = if synced {
+                OnceCell::from((0, Some(0))) // fast path for synced remotes
+            } else {
+                OnceCell::new()
+            };
+            TrackingRef {
+                target: local_target.clone(),
+                ahead_count: count.clone(),
+                behind_count: count,
+            }
+        });
+        Rc::new(RefName {
+            name: name.into(),
+            remote: Some(remote_name.into()),
+            target: remote_ref.target,
+            tracking_ref,
+            synced,
+        })
+    }
+
+    /// Creates remote ref representation which isn't tracked by a local ref.
+    pub fn remote_only(
+        name: impl Into<String>,
+        remote_name: impl Into<String>,
+        target: RefTarget,
+    ) -> Rc<Self> {
+        Rc::new(RefName {
+            name: name.into(),
+            remote: Some(remote_name.into()),
+            target,
+            tracking_ref: None,
+            synced: false, // has no local counterpart
+        })
+    }
+
     fn is_local(&self) -> bool {
         self.remote.is_none()
     }
@@ -746,9 +823,63 @@ impl RefName {
     fn is_remote(&self) -> bool {
         self.remote.is_some()
     }
+
+    fn is_present(&self) -> bool {
+        self.target.is_present()
+    }
+
+    /// Whether the ref target has conflicts.
+    fn has_conflict(&self) -> bool {
+        self.target.has_conflict()
+    }
+
+    /// Returns true if this ref is tracked by a local ref. The local ref might
+    /// have been deleted (but not pushed yet.)
+    fn is_tracked(&self) -> bool {
+        self.tracking_ref.is_some()
+    }
+
+    /// Returns true if this ref is tracked by a local ref, and if the local ref
+    /// is present.
+    fn is_tracking_present(&self) -> bool {
+        self.tracking_ref
+            .as_ref()
+            .map_or(false, |tracking| tracking.target.is_present())
+    }
+
+    /// Number of commits ahead of the tracking local ref.
+    fn tracking_ahead_count(&self, repo: &dyn Repo) -> Result<SizeHint, TemplatePropertyError> {
+        let Some(tracking) = &self.tracking_ref else {
+            return Err(TemplatePropertyError("Not a tracked remote ref".into()));
+        };
+        tracking
+            .ahead_count
+            .get_or_try_init(|| {
+                let self_ids = self.target.added_ids().cloned().collect_vec();
+                let other_ids = tracking.target.added_ids().cloned().collect_vec();
+                Ok(revset::walk_revs(repo, &self_ids, &other_ids)?.count_estimate())
+            })
+            .copied()
+    }
+
+    /// Number of commits behind of the tracking local ref.
+    fn tracking_behind_count(&self, repo: &dyn Repo) -> Result<SizeHint, TemplatePropertyError> {
+        let Some(tracking) = &self.tracking_ref else {
+            return Err(TemplatePropertyError("Not a tracked remote ref".into()));
+        };
+        tracking
+            .behind_count
+            .get_or_try_init(|| {
+                let self_ids = self.target.added_ids().cloned().collect_vec();
+                let other_ids = tracking.target.added_ids().cloned().collect_vec();
+                Ok(revset::walk_revs(repo, &other_ids, &self_ids)?.count_estimate())
+            })
+            .copied()
+    }
 }
 
-impl Template for RefName {
+// If wrapping with Rc<T> becomes common, add generic impl for Rc<T>.
+impl Template for Rc<RefName> {
     fn format(&self, formatter: &mut TemplateFormatter) -> io::Result<()> {
         write!(formatter.labeled("name"), "{}", self.name)?;
         if let Some(remote) = &self.remote {
@@ -757,7 +888,7 @@ impl Template for RefName {
         }
         // Don't show both conflict and unsynced sigils as conflicted ref wouldn't
         // be pushed.
-        if self.conflict {
+        if self.has_conflict() {
             write!(formatter, "??")?;
         } else if self.is_local() && !self.synced {
             write!(formatter, "*")?;
@@ -766,41 +897,130 @@ impl Template for RefName {
     }
 }
 
-impl Template for Vec<RefName> {
+impl Template for Vec<Rc<RefName>> {
     fn format(&self, formatter: &mut TemplateFormatter) -> io::Result<()> {
         templater::format_joined(formatter, self, " ")
     }
 }
 
-fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, RefName> {
+fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Rc<RefName>> {
     type L<'repo> = CommitTemplateLanguage<'repo>;
     // Not using maplit::hashmap!{} or custom declarative macro here because
     // code completion inside macro is quite restricted.
-    let mut map = CommitTemplateBuildMethodFnMap::<RefName>::new();
+    let mut map = CommitTemplateBuildMethodFnMap::<Rc<RefName>>::new();
     map.insert("name", |_language, _build_ctx, self_property, function| {
-        template_parser::expect_no_arguments(function)?;
-        let out_property = self_property.map(|ref_name| ref_name.name);
+        function.expect_no_arguments()?;
+        let out_property = self_property.map(|ref_name| ref_name.name.clone());
         Ok(L::wrap_string(out_property))
     });
     map.insert(
         "remote",
         |_language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
-            let out_property = self_property.map(|ref_name| ref_name.remote.unwrap_or_default());
+            function.expect_no_arguments()?;
+            let out_property =
+                self_property.map(|ref_name| ref_name.remote.clone().unwrap_or_default());
             Ok(L::wrap_string(out_property))
         },
     );
+    map.insert(
+        "present",
+        |_language, _build_ctx, self_property, function| {
+            function.expect_no_arguments()?;
+            let out_property = self_property.map(|ref_name| ref_name.is_present());
+            Ok(L::wrap_boolean(out_property))
+        },
+    );
+    map.insert(
+        "conflict",
+        |_language, _build_ctx, self_property, function| {
+            function.expect_no_arguments()?;
+            let out_property = self_property.map(|ref_name| ref_name.has_conflict());
+            Ok(L::wrap_boolean(out_property))
+        },
+    );
+    map.insert(
+        "normal_target",
+        |language, _build_ctx, self_property, function| {
+            function.expect_no_arguments()?;
+            let repo = language.repo;
+            let out_property = self_property.and_then(|ref_name| {
+                let maybe_id = ref_name.target.as_normal();
+                Ok(maybe_id.map(|id| repo.store().get_commit(id)).transpose()?)
+            });
+            Ok(L::wrap_commit_opt(out_property))
+        },
+    );
+    map.insert(
+        "removed_targets",
+        |language, _build_ctx, self_property, function| {
+            function.expect_no_arguments()?;
+            let repo = language.repo;
+            let out_property = self_property.and_then(|ref_name| {
+                let ids = ref_name.target.removed_ids();
+                Ok(ids.map(|id| repo.store().get_commit(id)).try_collect()?)
+            });
+            Ok(L::wrap_commit_list(out_property))
+        },
+    );
+    map.insert(
+        "added_targets",
+        |language, _build_ctx, self_property, function| {
+            function.expect_no_arguments()?;
+            let repo = language.repo;
+            let out_property = self_property.and_then(|ref_name| {
+                let ids = ref_name.target.added_ids();
+                Ok(ids.map(|id| repo.store().get_commit(id)).try_collect()?)
+            });
+            Ok(L::wrap_commit_list(out_property))
+        },
+    );
+    map.insert(
+        "tracked",
+        |_language, _build_ctx, self_property, function| {
+            function.expect_no_arguments()?;
+            let out_property = self_property.map(|ref_name| ref_name.is_tracked());
+            Ok(L::wrap_boolean(out_property))
+        },
+    );
+    map.insert(
+        "tracking_present",
+        |_language, _build_ctx, self_property, function| {
+            function.expect_no_arguments()?;
+            let out_property = self_property.map(|ref_name| ref_name.is_tracking_present());
+            Ok(L::wrap_boolean(out_property))
+        },
+    );
+    map.insert(
+        "tracking_ahead_count",
+        |language, _build_ctx, self_property, function| {
+            function.expect_no_arguments()?;
+            let repo = language.repo;
+            let out_property =
+                self_property.and_then(|ref_name| ref_name.tracking_ahead_count(repo));
+            Ok(L::wrap_size_hint(out_property))
+        },
+    );
+    map.insert(
+        "tracking_behind_count",
+        |language, _build_ctx, self_property, function| {
+            function.expect_no_arguments()?;
+            let repo = language.repo;
+            let out_property =
+                self_property.and_then(|ref_name| ref_name.tracking_behind_count(repo));
+            Ok(L::wrap_size_hint(out_property))
+        },
+    );
     map
 }
 
 /// Cache for reverse lookup refs.
 #[derive(Clone, Debug, Default)]
 pub struct RefNamesIndex {
-    index: HashMap<CommitId, Vec<RefName>>,
+    index: HashMap<CommitId, Vec<Rc<RefName>>>,
 }
 
 impl RefNamesIndex {
-    fn insert<'a>(&mut self, ids: impl IntoIterator<Item = &'a CommitId>, name: RefName) {
+    fn insert<'a>(&mut self, ids: impl IntoIterator<Item = &'a CommitId>, name: Rc<RefName>) {
         for id in ids {
             let ref_names = self.index.entry(id.clone()).or_default();
             ref_names.push(name.clone());
@@ -809,7 +1029,7 @@ impl RefNamesIndex {
 
     #[allow(unknown_lints)] // XXX FIXME (aseipp): nightly bogons; re-test this occasionally
     #[allow(clippy::manual_unwrap_or_default)]
-    pub fn get(&self, id: &CommitId) -> &[RefName] {
+    pub fn get(&self, id: &CommitId) -> &[Rc<RefName>] {
         if let Some(names) = self.index.get(id) {
             names
         } else {
@@ -824,23 +1044,16 @@ fn build_branches_index(repo: &dyn Repo) -> RefNamesIndex {
         let local_target = branch_target.local_target;
         let remote_refs = branch_target.remote_refs;
         if local_target.is_present() {
-            let ref_name = RefName {
-                name: branch_name.to_owned(),
-                remote: None,
-                conflict: local_target.has_conflict(),
-                synced: remote_refs.iter().all(|&(_, remote_ref)| {
-                    !remote_ref.is_tracking() || remote_ref.target == *local_target
-                }),
-            };
+            let ref_name = RefName::local(
+                branch_name,
+                local_target.clone(),
+                remote_refs.iter().map(|&(_, remote_ref)| remote_ref),
+            );
             index.insert(local_target.added_ids(), ref_name);
         }
         for &(remote_name, remote_ref) in &remote_refs {
-            let ref_name = RefName {
-                name: branch_name.to_owned(),
-                remote: Some(remote_name.to_owned()),
-                conflict: remote_ref.target.has_conflict(),
-                synced: remote_ref.is_tracking() && remote_ref.target == *local_target,
-            };
+            let ref_name =
+                RefName::remote(branch_name, remote_name, remote_ref.clone(), local_target);
             index.insert(remote_ref.target.added_ids(), ref_name);
         }
     }
@@ -852,27 +1065,18 @@ fn build_ref_names_index<'a>(
 ) -> RefNamesIndex {
     let mut index = RefNamesIndex::default();
     for (name, target) in ref_pairs {
-        let ref_name = RefName {
-            name: name.to_owned(),
-            remote: None,
-            conflict: target.has_conflict(),
-            synced: true, // has no tracking remotes
-        };
+        let ref_name = RefName::local_only(name, target.clone());
         index.insert(target.added_ids(), ref_name);
     }
     index
 }
 
-fn extract_git_head(repo: &dyn Repo, commit: &Commit) -> Option<RefName> {
+fn extract_git_head(repo: &dyn Repo, commit: &Commit) -> Option<Rc<RefName>> {
     let target = repo.view().git_head();
-    target.added_ids().contains(commit.id()).then(|| {
-        RefName {
-            name: "HEAD".to_owned(),
-            remote: Some(git::REMOTE_NAME_FOR_LOCAL_GIT_REPO.to_owned()),
-            conflict: target.has_conflict(),
-            synced: false, // has no local counterpart
-        }
-    })
+    target
+        .added_ids()
+        .contains(commit.id())
+        .then(|| RefName::remote_only("HEAD", git::REMOTE_NAME_FOR_LOCAL_GIT_REPO, target.clone()))
 }
 
 #[derive(Clone, Debug, Eq, PartialEq)]
@@ -931,7 +1135,7 @@ fn builtin_commit_or_change_id_methods<'repo>(
     // code completion inside macro is quite restricted.
     let mut map = CommitTemplateBuildMethodFnMap::<CommitOrChangeId>::new();
     map.insert("short", |language, build_ctx, self_property, function| {
-        let ([], [len_node]) = template_parser::expect_arguments(function)?;
+        let ([], [len_node]) = function.expect_arguments()?;
         let len_property = len_node
             .map(|node| template_builder::expect_usize_expression(language, build_ctx, node))
             .transpose()?;
@@ -943,7 +1147,7 @@ fn builtin_commit_or_change_id_methods<'repo>(
         "shortest",
         |language, build_ctx, self_property, function| {
             let id_prefix_context = &language.id_prefix_context;
-            let ([], [len_node]) = template_parser::expect_arguments(function)?;
+            let ([], [len_node]) = function.expect_arguments()?;
             let len_property = len_node
                 .map(|node| template_builder::expect_usize_expression(language, build_ctx, node))
                 .transpose()?;
@@ -992,23 +1196,23 @@ fn builtin_shortest_id_prefix_methods<'repo>(
     map.insert(
         "prefix",
         |_language, _build_ctx, self_property, function| {
-            template_parser::expect_no_arguments(function)?;
+            function.expect_no_arguments()?;
             let out_property = self_property.map(|id| id.prefix);
             Ok(L::wrap_string(out_property))
         },
     );
     map.insert("rest", |_language, _build_ctx, self_property, function| {
-        template_parser::expect_no_arguments(function)?;
+        function.expect_no_arguments()?;
         let out_property = self_property.map(|id| id.rest);
         Ok(L::wrap_string(out_property))
     });
     map.insert("upper", |_language, _build_ctx, self_property, function| {
-        template_parser::expect_no_arguments(function)?;
+        function.expect_no_arguments()?;
         let out_property = self_property.map(|id| id.to_upper());
         Ok(L::wrap_shortest_id_prefix(out_property))
     });
     map.insert("lower", |_language, _build_ctx, self_property, function| {
-        template_parser::expect_no_arguments(function)?;
+        function.expect_no_arguments()?;
         let out_property = self_property.map(|id| id.to_lower());
         Ok(L::wrap_shortest_id_prefix(out_property))
     });
