@@ -16,14 +16,44 @@ use std::borrow::Cow;
 use std::collections::{HashMap, HashSet};
 use std::path::{Path, PathBuf};
 use std::process::Command;
-use std::{env, fmt};
+use std::str::FromStr;
+use std::{env, fmt, slice};
 
 use config::Source;
 use itertools::Itertools;
 use jj_lib::settings::ConfigResultExt as _;
+use regex::{Captures, Regex};
 use thiserror::Error;
 use tracing::instrument;
 
+use crate::command_error::{user_error, user_error_with_message, CommandError};
+
+pub fn to_toml_value(value: &config::Value) -> Result<toml_edit::Value, config::ConfigError> {
+    fn type_error<T: fmt::Display>(message: T) -> config::ConfigError {
+        config::ConfigError::Message(message.to_string())
+    }
+    // It's unlikely that the config object contained unsupported values, but
+    // there's no guarantee. For example, values coming from environment
+    // variables might be big int.
+    match value.kind {
+        config::ValueKind::Nil => Err(type_error(format!("Unexpected value: {value}"))),
+        config::ValueKind::Boolean(v) => Ok(v.into()),
+        config::ValueKind::I64(v) => Ok(v.into()),
+        config::ValueKind::I128(v) => Ok(i64::try_from(v).map_err(type_error)?.into()),
+        config::ValueKind::U64(v) => Ok(i64::try_from(v).map_err(type_error)?.into()),
+        config::ValueKind::U128(v) => Ok(i64::try_from(v).map_err(type_error)?.into()),
+        config::ValueKind::Float(v) => Ok(v.into()),
+        config::ValueKind::String(ref v) => Ok(v.into()),
+        // TODO: Remove sorting when config crate maintains deterministic ordering.
+        config::ValueKind::Table(ref table) => table
+            .iter()
+            .sorted_by_key(|(k, _)| *k)
+            .map(|(k, v)| Ok((k, to_toml_value(v)?)))
+            .collect(),
+        config::ValueKind::Array(ref array) => array.iter().map(to_toml_value).collect(),
+    }
+}
+
 #[derive(Error, Debug)]
 pub enum ConfigError {
     #[error(transparent)]
@@ -34,6 +64,111 @@ pub enum ConfigError {
     ConfigCreateError(#[from] std::io::Error),
 }
 
+/// Dotted config name path.
+#[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+pub struct ConfigNamePathBuf(Vec<toml_edit::Key>);
+
+impl ConfigNamePathBuf {
+    /// Creates an empty path pointing to the root table.
+    ///
+    /// This isn't a valid TOML key expression, but provided for convenience.
+    pub fn root() -> Self {
+        ConfigNamePathBuf(vec![])
+    }
+
+    /// Returns true if the path is empty (i.e. pointing to the root table.)
+    pub fn is_root(&self) -> bool {
+        self.0.is_empty()
+    }
+
+    /// Returns iterator of path components (or keys.)
+    pub fn components(&self) -> slice::Iter<'_, toml_edit::Key> {
+        self.0.iter()
+    }
+
+    /// Appends the given `key` component.
+    pub fn push(&mut self, key: impl Into<toml_edit::Key>) {
+        self.0.push(key.into());
+    }
+
+    /// Looks up value in the given `config`.
+    ///
+    /// This is a workaround for the `config.get()` API, which doesn't support
+    /// literal path expression. If we implement our own config abstraction,
+    /// this method should be moved there.
+    pub fn lookup_value(
+        &self,
+        config: &config::Config,
+    ) -> Result<config::Value, config::ConfigError> {
+        // Use config.get() if the TOML keys can be converted to config path
+        // syntax. This should be cheaper than cloning the whole config map.
+        let (key_prefix, components) = self.split_safe_prefix();
+        let value: config::Value = match &key_prefix {
+            Some(key) => config.get(key)?,
+            None => config.collect()?.into(),
+        };
+        components
+            .iter()
+            .try_fold(value, |value, key| {
+                let mut table = value.into_table().ok()?;
+                table.remove(key.get())
+            })
+            .ok_or_else(|| config::ConfigError::NotFound(self.to_string()))
+    }
+
+    /// Splits path to dotted literal expression and remainder.
+    ///
+    /// The literal expression part doesn't contain meta characters other than
+    /// ".", therefore it can be passed in to `config.get()`.
+    /// https://github.com/mehcode/config-rs/issues/110
+    fn split_safe_prefix(&self) -> (Option<Cow<'_, str>>, &[toml_edit::Key]) {
+        // https://github.com/mehcode/config-rs/blob/v0.13.4/src/path/parser.rs#L15
+        let is_ident = |key: &str| {
+            key.chars()
+                .all(|c| c.is_ascii_alphanumeric() || c == '_' || c == '-')
+        };
+        let pos = self.0.iter().take_while(|&k| is_ident(k)).count();
+        let safe_key = match pos {
+            0 => None,
+            1 => Some(Cow::Borrowed(self.0[0].get())),
+            _ => Some(Cow::Owned(self.0[..pos].iter().join("."))),
+        };
+        (safe_key, &self.0[pos..])
+    }
+}
+
+impl<K: Into<toml_edit::Key>> FromIterator<K> for ConfigNamePathBuf {
+    fn from_iter<I: IntoIterator<Item = K>>(iter: I) -> Self {
+        let keys = iter.into_iter().map(|k| k.into()).collect();
+        ConfigNamePathBuf(keys)
+    }
+}
+
+impl FromStr for ConfigNamePathBuf {
+    type Err = toml_edit::TomlError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        // TOML parser ensures that the returned vec is not empty.
+        toml_edit::Key::parse(s).map(ConfigNamePathBuf)
+    }
+}
+
+impl AsRef<[toml_edit::Key]> for ConfigNamePathBuf {
+    fn as_ref(&self) -> &[toml_edit::Key] {
+        &self.0
+    }
+}
+
+impl fmt::Display for ConfigNamePathBuf {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let mut components = self.0.iter().fuse();
+        if let Some(key) = components.next() {
+            write!(f, "{key}")?;
+        }
+        components.try_for_each(|key| write!(f, ".{key}"))
+    }
+}
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum ConfigSource {
     Default,
@@ -46,7 +181,7 @@ pub enum ConfigSource {
 
 #[derive(Clone, Debug, PartialEq)]
 pub struct AnnotatedValue {
-    pub path: Vec<String>,
+    pub path: ConfigNamePathBuf,
     pub value: config::Value,
     pub source: ConfigSource,
     pub is_overridden: bool,
@@ -139,41 +274,28 @@ impl LayeredConfigs {
 
     pub fn resolved_config_values(
         &self,
-        filter_prefix: &[&str],
+        filter_prefix: &ConfigNamePathBuf,
     ) -> Result<Vec<AnnotatedValue>, ConfigError> {
         // Collect annotated values from each config.
         let mut config_vals = vec![];
-
-        let prefix_key = match filter_prefix {
-            &[] => None,
-            _ => Some(filter_prefix.join(".")),
-        };
         for (source, config) in self.sources() {
-            let top_value = match prefix_key {
-                Some(ref key) => {
-                    if let Some(val) = config.get(key).optional()? {
-                        val
-                    } else {
-                        continue;
-                    }
-                }
-                None => config.collect()?.into(),
+            let Some(top_value) = filter_prefix.lookup_value(config).optional()? else {
+                continue;
             };
-            let mut config_stack: Vec<(Vec<&str>, &config::Value)> =
-                vec![(filter_prefix.to_vec(), &top_value)];
+            let mut config_stack = vec![(filter_prefix.clone(), &top_value)];
             while let Some((path, value)) = config_stack.pop() {
                 match &value.kind {
                     config::ValueKind::Table(table) => {
                         // TODO: Remove sorting when config crate maintains deterministic ordering.
                         for (k, v) in table.iter().sorted_by_key(|(k, _)| *k).rev() {
-                            let mut key_path = path.to_vec();
+                            let mut key_path = path.clone();
                             key_path.push(k);
                             config_stack.push((key_path, v));
                         }
                     }
                     _ => {
                         config_vals.push(AnnotatedValue {
-                            path: path.iter().map(|&s| s.to_owned()).collect_vec(),
+                            path,
                             value: value.to_owned(),
                             source: source.to_owned(),
                             // Note: Value updated below.
@@ -434,6 +556,71 @@ fn read_config_path(config_path: &Path) -> Result<config::Config, config::Config
         .build()
 }
 
+pub fn write_config_value_to_file(
+    key: &ConfigNamePathBuf,
+    value_str: &str,
+    path: &Path,
+) -> Result<(), CommandError> {
+    // Read config
+    let config_toml = std::fs::read_to_string(path).or_else(|err| {
+        match err.kind() {
+            // If config doesn't exist yet, read as empty and we'll write one.
+            std::io::ErrorKind::NotFound => Ok("".to_string()),
+            _ => Err(user_error_with_message(
+                format!("Failed to read file {path}", path = path.display()),
+                err,
+            )),
+        }
+    })?;
+    let mut doc: toml_edit::Document = config_toml.parse().map_err(|err| {
+        user_error_with_message(
+            format!("Failed to parse file {path}", path = path.display()),
+            err,
+        )
+    })?;
+
+    // Apply config value
+    // Interpret value as string if it can't be parsed as a TOML value.
+    // TODO(#531): Infer types based on schema (w/ --type arg to override).
+    let item = match value_str.parse() {
+        Ok(value) => toml_edit::Item::Value(value),
+        _ => toml_edit::value(value_str),
+    };
+    let mut target_table = doc.as_table_mut();
+    let mut key_parts_iter = key.components();
+    let last_key_part = key_parts_iter.next_back().expect("key must not be empty");
+    for key_part in key_parts_iter {
+        target_table = target_table
+            .entry(key_part)
+            .or_insert_with(|| toml_edit::Item::Table(toml_edit::Table::new()))
+            .as_table_mut()
+            .ok_or_else(|| {
+                user_error(format!(
+                    "Failed to set {key}: would overwrite non-table value with parent table"
+                ))
+            })?;
+    }
+    // Error out if overwriting non-scalar value for key (table or array) with
+    // scalar.
+    match target_table.get(last_key_part) {
+        None | Some(toml_edit::Item::None | toml_edit::Item::Value(_)) => {}
+        Some(toml_edit::Item::Table(_) | toml_edit::Item::ArrayOfTables(_)) => {
+            return Err(user_error(format!(
+                "Failed to set {key}: would overwrite entire table"
+            )));
+        }
+    }
+    target_table[last_key_part] = item;
+
+    // Write config back
+    std::fs::write(path, doc.to_string()).map_err(|err| {
+        user_error_with_message(
+            format!("Failed to write file {path}", path = path.display()),
+            err,
+        )
+    })
+}
+
 /// Command name and arguments specified by config.
 #[derive(Clone, Debug, Eq, PartialEq, serde::Deserialize)]
 #[serde(untagged)]
@@ -475,12 +662,22 @@ impl CommandNameAndArgs {
 
     /// Returns process builder configured with this.
     pub fn to_command(&self) -> Command {
+        let empty: HashMap<&str, &str> = HashMap::new();
+        self.to_command_with_variables(&empty)
+    }
+
+    /// Returns process builder configured with this after interpolating
+    /// variables into the arguments.
+    pub fn to_command_with_variables<V: AsRef<str>>(
+        &self,
+        variables: &HashMap<&str, V>,
+    ) -> Command {
         let (name, args) = self.split_name_and_args();
         let mut cmd = Command::new(name.as_ref());
         if let CommandNameAndArgs::Structured { env, .. } = self {
             cmd.envs(env);
         }
-        cmd.args(args.as_ref());
+        cmd.args(interpolate_variables(&args, variables));
         cmd
     }
 }
@@ -507,6 +704,41 @@ impl fmt::Display for CommandNameAndArgs {
     }
 }
 
+// Not interested in $UPPER_CASE_VARIABLES
+static VARIABLE_REGEX: once_cell::sync::Lazy<Regex> =
+    once_cell::sync::Lazy::new(|| Regex::new(r"\$([a-z0-9_]+)\b").unwrap());
+
+pub fn interpolate_variables<V: AsRef<str>>(
+    args: &[String],
+    variables: &HashMap<&str, V>,
+) -> Vec<String> {
+    args.iter()
+        .map(|arg| {
+            VARIABLE_REGEX
+                .replace_all(arg, |caps: &Captures| {
+                    let name = &caps[1];
+                    if let Some(subst) = variables.get(name) {
+                        subst.as_ref().to_owned()
+                    } else {
+                        caps[0].to_owned()
+                    }
+                })
+                .into_owned()
+        })
+        .collect()
+}
+
+/// Return all variable names found in the args, without the dollar sign
+pub fn find_all_variables(args: &[String]) -> impl Iterator<Item = &str> {
+    let regex = &*VARIABLE_REGEX;
+    args.iter()
+        .flat_map(|arg| regex.find_iter(arg))
+        .map(|single_match| {
+            let s = single_match.as_str();
+            &s[1..]
+        })
+}
+
 /// Wrapper to reject an array without command name.
 // Based on https://github.com/serde-rs/serde/issues/939
 #[derive(Clone, Debug, Eq, Hash, PartialEq, serde::Deserialize)]
@@ -531,6 +763,43 @@ mod tests {
 
     use super::*;
 
+    #[test]
+    fn test_split_safe_config_name_path() {
+        let parse = |s| ConfigNamePathBuf::from_str(s).unwrap();
+        let key = |s: &str| toml_edit::Key::new(s);
+
+        // Empty (or root) path isn't recognized by config::Config::get()
+        assert_eq!(
+            ConfigNamePathBuf::root().split_safe_prefix(),
+            (None, [].as_slice())
+        );
+
+        assert_eq!(
+            parse("Foo-bar_1").split_safe_prefix(),
+            (Some("Foo-bar_1".into()), [].as_slice())
+        );
+        assert_eq!(
+            parse("'foo()'").split_safe_prefix(),
+            (None, [key("foo()")].as_slice())
+        );
+        assert_eq!(
+            parse("foo.'bar()'").split_safe_prefix(),
+            (Some("foo".into()), [key("bar()")].as_slice())
+        );
+        assert_eq!(
+            parse("foo.'bar()'.baz").split_safe_prefix(),
+            (Some("foo".into()), [key("bar()"), key("baz")].as_slice())
+        );
+        assert_eq!(
+            parse("foo.bar").split_safe_prefix(),
+            (Some("foo.bar".into()), [].as_slice())
+        );
+        assert_eq!(
+            parse("foo.bar.'baz()'").split_safe_prefix(),
+            (Some("foo.bar".into()), [key("baz()")].as_slice())
+        );
+    }
+
     #[test]
     fn test_command_args() {
         let config = config::Config::builder()
@@ -606,7 +875,12 @@ mod tests {
             env_overrides: empty_config,
             arg_overrides: None,
         };
-        assert_eq!(layered_configs.resolved_config_values(&[]).unwrap(), []);
+        assert_eq!(
+            layered_configs
+                .resolved_config_values(&ConfigNamePathBuf::root())
+                .unwrap(),
+            []
+        );
     }
 
     #[test]
@@ -634,14 +908,30 @@ mod tests {
         };
         // Note: "email" is alphabetized, before "name" from same layer.
         insta::assert_debug_snapshot!(
-            layered_configs.resolved_config_values(&[]).unwrap(),
+            layered_configs.resolved_config_values(&ConfigNamePathBuf::root()).unwrap(),
             @r###"
         [
             AnnotatedValue {
-                path: [
-                    "user",
-                    "email",
-                ],
+                path: ConfigNamePathBuf(
+                    [
+                        Key {
+                            key: "user",
+                            repr: None,
+                            decor: Decor {
+                                prefix: "default",
+                                suffix: "default",
+                            },
+                        },
+                        Key {
+                            key: "email",
+                            repr: None,
+                            decor: Decor {
+                                prefix: "default",
+                                suffix: "default",
+                            },
+                        },
+                    ],
+                ),
                 value: Value {
                     origin: None,
                     kind: String(
@@ -652,10 +942,26 @@ mod tests {
                 is_overridden: true,
             },
             AnnotatedValue {
-                path: [
-                    "user",
-                    "name",
-                ],
+                path: ConfigNamePathBuf(
+                    [
+                        Key {
+                            key: "user",
+                            repr: None,
+                            decor: Decor {
+                                prefix: "default",
+                                suffix: "default",
+                            },
+                        },
+                        Key {
+                            key: "name",
+                            repr: None,
+                            decor: Decor {
+                                prefix: "default",
+                                suffix: "default",
+                            },
+                        },
+                    ],
+                ),
                 value: Value {
                     origin: None,
                     kind: String(
@@ -666,10 +972,26 @@ mod tests {
                 is_overridden: false,
             },
             AnnotatedValue {
-                path: [
-                    "user",
-                    "email",
-                ],
+                path: ConfigNamePathBuf(
+                    [
+                        Key {
+                            key: "user",
+                            repr: None,
+                            decor: Decor {
+                                prefix: "default",
+                                suffix: "default",
+                            },
+                        },
+                        Key {
+                            key: "email",
+                            repr: None,
+                            decor: Decor {
+                                prefix: "default",
+                                suffix: "default",
+                            },
+                        },
+                    ],
+                ),
                 value: Value {
                     origin: None,
                     kind: String(
@@ -709,15 +1031,31 @@ mod tests {
         };
         insta::assert_debug_snapshot!(
             layered_configs
-                .resolved_config_values(&["test-table1"])
+                .resolved_config_values(&ConfigNamePathBuf::from_iter(["test-table1"]))
                 .unwrap(),
             @r###"
         [
             AnnotatedValue {
-                path: [
-                    "test-table1",
-                    "foo",
-                ],
+                path: ConfigNamePathBuf(
+                    [
+                        Key {
+                            key: "test-table1",
+                            repr: None,
+                            decor: Decor {
+                                prefix: "default",
+                                suffix: "default",
+                            },
+                        },
+                        Key {
+                            key: "foo",
+                            repr: None,
+                            decor: Decor {
+                                prefix: "default",
+                                suffix: "default",
+                            },
+                        },
+                    ],
+                ),
                 value: Value {
                     origin: None,
                     kind: String(
@@ -728,10 +1066,26 @@ mod tests {
                 is_overridden: false,
             },
             AnnotatedValue {
-                path: [
-                    "test-table1",
-                    "bar",
-                ],
+                path: ConfigNamePathBuf(
+                    [
+                        Key {
+                            key: "test-table1",
+                            repr: None,
+                            decor: Decor {
+                                prefix: "default",
+                                suffix: "default",
+                            },
+                        },
+                        Key {
+                            key: "bar",
+                            repr: None,
+                            decor: Decor {
+                                prefix: "default",
+                                suffix: "default",
+                            },
+                        },
+                    ],
+                ),
                 value: Value {
                     origin: None,
                     kind: String(
