@@ -19,7 +19,7 @@ use std::ops::Range;
 
 use futures::{try_join, Stream, StreamExt};
 use itertools::Itertools;
-use jj_lib::backend::{BackendResult, TreeValue};
+use jj_lib::backend::{BackendError, BackendResult, TreeValue};
 use jj_lib::commit::Commit;
 use jj_lib::conflicts::{materialize_tree_value, MaterializedTreeValue};
 use jj_lib::diff::{Diff, DiffHunk};
@@ -29,19 +29,18 @@ use jj_lib::merge::MergedTreeValue;
 use jj_lib::merged_tree::{MergedTree, TreeDiffStream};
 use jj_lib::object_id::ObjectId;
 use jj_lib::repo::Repo;
-use jj_lib::repo_path::{RepoPath, RepoPathBuf};
+use jj_lib::repo_path::{RepoPath, RepoPathBuf, RepoPathUiConverter};
 use jj_lib::settings::{ConfigResultExt as _, UserSettings};
 use jj_lib::store::Store;
-use jj_lib::{diff, files, rewrite};
+use jj_lib::{diff, files};
 use pollster::FutureExt;
+use thiserror::Error;
 use tracing::instrument;
 use unicode_width::UnicodeWidthStr as _;
 
-use crate::cli_util::WorkspaceCommandHelper;
-use crate::command_error::CommandError;
 use crate::config::CommandNameAndArgs;
 use crate::formatter::Formatter;
-use crate::merge_tools::{self, ExternalMergeTool};
+use crate::merge_tools::{self, DiffGenerateError, ExternalMergeTool};
 use crate::text_util;
 use crate::ui::Ui;
 
@@ -190,65 +189,104 @@ fn default_diff_format(
     }
 }
 
-pub fn show_diff(
-    ui: &Ui,
-    formatter: &mut dyn Formatter,
-    workspace_command: &WorkspaceCommandHelper,
-    from_tree: &MergedTree,
-    to_tree: &MergedTree,
-    matcher: &dyn Matcher,
-    formats: &[DiffFormat],
-) -> Result<(), CommandError> {
-    for format in formats {
-        match format {
-            DiffFormat::Summary => {
-                let tree_diff = from_tree.diff_stream(to_tree, matcher);
-                show_diff_summary(formatter, workspace_command, tree_diff)?;
-            }
-            DiffFormat::Stat => {
-                let tree_diff = from_tree.diff_stream(to_tree, matcher);
-                show_diff_stat(ui, formatter, workspace_command, tree_diff)?;
-            }
-            DiffFormat::Types => {
-                let tree_diff = from_tree.diff_stream(to_tree, matcher);
-                show_types(formatter, workspace_command, tree_diff)?;
-            }
-            DiffFormat::Git { context } => {
-                let tree_diff = from_tree.diff_stream(to_tree, matcher);
-                show_git_diff(formatter, workspace_command, *context, tree_diff)?;
-            }
-            DiffFormat::ColorWords { context } => {
-                let tree_diff = from_tree.diff_stream(to_tree, matcher);
-                show_color_words_diff(formatter, workspace_command, *context, tree_diff)?;
-            }
-            DiffFormat::Tool(tool) => {
-                merge_tools::generate_diff(ui, formatter.raw(), from_tree, to_tree, matcher, tool)?;
-            }
-        }
-    }
-    Ok(())
+#[derive(Debug, Error)]
+pub enum DiffRenderError {
+    #[error("Failed to generate diff")]
+    DiffGenerate(#[source] DiffGenerateError),
+    #[error(transparent)]
+    Backend(#[from] BackendError),
+    #[error("Access denied to {path}: {source}")]
+    AccessDenied {
+        path: String,
+        source: Box<dyn std::error::Error + Send + Sync>,
+    },
+    #[error(transparent)]
+    Io(#[from] io::Error),
 }
 
-pub fn show_patch(
-    ui: &Ui,
-    formatter: &mut dyn Formatter,
-    workspace_command: &WorkspaceCommandHelper,
-    commit: &Commit,
-    matcher: &dyn Matcher,
-    formats: &[DiffFormat],
-) -> Result<(), CommandError> {
-    let parents = commit.parents();
-    let from_tree = rewrite::merge_commit_trees(workspace_command.repo().as_ref(), &parents)?;
-    let to_tree = commit.tree()?;
-    show_diff(
-        ui,
-        formatter,
-        workspace_command,
-        &from_tree,
-        &to_tree,
-        matcher,
-        formats,
-    )
+/// Configuration and environment to render textual diff.
+pub struct DiffRenderer<'a> {
+    repo: &'a dyn Repo,
+    path_converter: &'a RepoPathUiConverter,
+    formats: Vec<DiffFormat>,
+}
+
+impl<'a> DiffRenderer<'a> {
+    pub fn new(
+        repo: &'a dyn Repo,
+        path_converter: &'a RepoPathUiConverter,
+        formats: Vec<DiffFormat>,
+    ) -> Self {
+        DiffRenderer {
+            repo,
+            formats,
+            path_converter,
+        }
+    }
+
+    /// Generates diff between `from_tree` and `to_tree`.
+    pub fn show_diff(
+        &self,
+        ui: &Ui, // TODO: remove Ui dependency if possible
+        formatter: &mut dyn Formatter,
+        from_tree: &MergedTree,
+        to_tree: &MergedTree,
+        matcher: &dyn Matcher,
+    ) -> Result<(), DiffRenderError> {
+        let repo = self.repo;
+        let path_converter = self.path_converter;
+        for format in &self.formats {
+            match format {
+                DiffFormat::Summary => {
+                    let tree_diff = from_tree.diff_stream(to_tree, matcher);
+                    show_diff_summary(formatter, tree_diff, path_converter)?;
+                }
+                DiffFormat::Stat => {
+                    let tree_diff = from_tree.diff_stream(to_tree, matcher);
+                    // TODO: In graph log, graph width should be subtracted
+                    let width = usize::from(ui.term_width().unwrap_or(80));
+                    show_diff_stat(repo, formatter, tree_diff, path_converter, width)?;
+                }
+                DiffFormat::Types => {
+                    let tree_diff = from_tree.diff_stream(to_tree, matcher);
+                    show_types(formatter, tree_diff, path_converter)?;
+                }
+                DiffFormat::Git { context } => {
+                    let tree_diff = from_tree.diff_stream(to_tree, matcher);
+                    show_git_diff(repo, formatter, *context, tree_diff)?;
+                }
+                DiffFormat::ColorWords { context } => {
+                    let tree_diff = from_tree.diff_stream(to_tree, matcher);
+                    show_color_words_diff(repo, formatter, *context, tree_diff, path_converter)?;
+                }
+                DiffFormat::Tool(tool) => {
+                    merge_tools::generate_diff(
+                        ui,
+                        formatter.raw(),
+                        from_tree,
+                        to_tree,
+                        matcher,
+                        tool,
+                    )
+                    .map_err(DiffRenderError::DiffGenerate)?;
+                }
+            }
+        }
+        Ok(())
+    }
+
+    /// Generates diff of the given `commit` compared to its parents.
+    pub fn show_patch(
+        &self,
+        ui: &Ui,
+        formatter: &mut dyn Formatter,
+        commit: &Commit,
+        matcher: &dyn Matcher,
+    ) -> Result<(), DiffRenderError> {
+        let from_tree = commit.parent_tree(self.repo)?;
+        let to_tree = commit.tree()?;
+        self.show_diff(ui, formatter, &from_tree, &to_tree, matcher)
+    }
 }
 
 fn show_color_words_diff_hunks(
@@ -401,12 +439,13 @@ fn file_content_for_diff(reader: &mut dyn io::Read) -> io::Result<FileContent> {
     })
 }
 
-fn diff_content(
-    path: &RepoPath,
-    value: MaterializedTreeValue,
-) -> Result<FileContent, CommandError> {
+fn diff_content(path: &RepoPath, value: MaterializedTreeValue) -> io::Result<FileContent> {
     match value {
         MaterializedTreeValue::Absent => Ok(FileContent::empty()),
+        MaterializedTreeValue::AccessDenied(err) => Ok(FileContent {
+            is_binary: false,
+            contents: format!("Access denied: {err}").into_bytes(),
+        }),
         MaterializedTreeValue::File { mut reader, .. } => {
             file_content_for_diff(&mut reader).map_err(Into::into)
         }
@@ -420,7 +459,11 @@ fn diff_content(
             contents: format!("Git submodule checked out at {}", id.hex()).into_bytes(),
         }),
         // TODO: are we sure this is never binary?
-        MaterializedTreeValue::Conflict { id: _, contents } => Ok(FileContent {
+        MaterializedTreeValue::Conflict {
+            id: _,
+            contents,
+            executable: _,
+        } => Ok(FileContent {
             is_binary: false,
             contents,
         }),
@@ -435,6 +478,7 @@ fn basic_diff_file_type(value: &MaterializedTreeValue) -> &'static str {
         MaterializedTreeValue::Absent => {
             panic!("absent path in diff");
         }
+        MaterializedTreeValue::AccessDenied(_) => "access denied",
         MaterializedTreeValue::File { executable, .. } => {
             if *executable {
                 "executable file"
@@ -450,17 +494,31 @@ fn basic_diff_file_type(value: &MaterializedTreeValue) -> &'static str {
 }
 
 pub fn show_color_words_diff(
+    repo: &dyn Repo,
     formatter: &mut dyn Formatter,
-    workspace_command: &WorkspaceCommandHelper,
     num_context_lines: usize,
     tree_diff: TreeDiffStream,
-) -> Result<(), CommandError> {
+    path_converter: &RepoPathUiConverter,
+) -> Result<(), DiffRenderError> {
     formatter.push_label("diff")?;
-    let mut diff_stream = materialized_diff_stream(workspace_command.repo().store(), tree_diff);
+    let mut diff_stream = materialized_diff_stream(repo.store(), tree_diff);
     async {
         while let Some((path, diff)) = diff_stream.next().await {
-            let ui_path = workspace_command.format_file_path(&path);
+            let ui_path = path_converter.format_file_path(&path);
             let (left_value, right_value) = diff?;
+
+            match (&left_value, &right_value) {
+                (_, MaterializedTreeValue::AccessDenied(source))
+                | (MaterializedTreeValue::AccessDenied(source), _) => {
+                    write!(
+                        formatter.labeled("access-denied"),
+                        "Access denied to {ui_path}:"
+                    )?;
+                    writeln!(formatter, " {source}")?;
+                    continue;
+                }
+                _ => {}
+            }
             if left_value.is_absent() {
                 let description = basic_diff_file_type(&right_value);
                 writeln!(
@@ -562,7 +620,7 @@ pub fn show_color_words_diff(
                 }
             }
         }
-        Ok::<(), CommandError>(())
+        Ok::<(), DiffRenderError>(())
     }
     .block_on()?;
     formatter.pop_label()?;
@@ -578,7 +636,7 @@ struct GitDiffPart {
 fn git_diff_part(
     path: &RepoPath,
     value: MaterializedTreeValue,
-) -> Result<GitDiffPart, CommandError> {
+) -> Result<GitDiffPart, DiffRenderError> {
     let mode;
     let hash;
     let mut contents: Vec<u8>;
@@ -586,6 +644,12 @@ fn git_diff_part(
         MaterializedTreeValue::Absent => {
             panic!("Absent path {path:?} in diff should have been handled by caller");
         }
+        MaterializedTreeValue::AccessDenied(err) => {
+            return Err(DiffRenderError::AccessDenied {
+                path: path.as_internal_file_string().to_owned(),
+                source: err,
+            });
+        }
         MaterializedTreeValue::File {
             id,
             executable,
@@ -615,8 +679,13 @@ fn git_diff_part(
         MaterializedTreeValue::Conflict {
             id: _,
             contents: conflict_data,
+            executable,
         } => {
-            mode = "100644".to_string();
+            mode = if executable {
+                "100755".to_string()
+            } else {
+                "100644".to_string()
+            };
             hash = "0000000000".to_string();
             contents = conflict_data
         }
@@ -730,7 +799,7 @@ fn show_unified_diff_hunks(
     left_content: &[u8],
     right_content: &[u8],
     num_context_lines: usize,
-) -> Result<(), CommandError> {
+) -> io::Result<()> {
     for hunk in unified_diff_hunks(left_content, right_content, num_context_lines) {
         writeln!(
             formatter.labeled("hunk_header"),
@@ -794,14 +863,14 @@ fn materialized_diff_stream<'a>(
 }
 
 pub fn show_git_diff(
+    repo: &dyn Repo,
     formatter: &mut dyn Formatter,
-    workspace_command: &WorkspaceCommandHelper,
     num_context_lines: usize,
     tree_diff: TreeDiffStream,
-) -> Result<(), CommandError> {
+) -> Result<(), DiffRenderError> {
     formatter.push_label("diff")?;
 
-    let mut diff_stream = materialized_diff_stream(workspace_command.repo().store(), tree_diff);
+    let mut diff_stream = materialized_diff_stream(repo.store(), tree_diff);
     async {
         while let Some((path, diff)) = diff_stream.next().await {
             let path_string = path.as_internal_file_string();
@@ -858,7 +927,7 @@ pub fn show_git_diff(
                 show_unified_diff_hunks(formatter, &left_part.content, &[], num_context_lines)?;
             }
         }
-        Ok::<(), CommandError>(())
+        Ok::<(), DiffRenderError>(())
     }
     .block_on()?;
     formatter.pop_label()?;
@@ -868,31 +937,21 @@ pub fn show_git_diff(
 #[instrument(skip_all)]
 pub fn show_diff_summary(
     formatter: &mut dyn Formatter,
-    workspace_command: &WorkspaceCommandHelper,
     mut tree_diff: TreeDiffStream,
+    path_converter: &RepoPathUiConverter,
 ) -> io::Result<()> {
     formatter.with_label("diff", |formatter| -> io::Result<()> {
         async {
             while let Some((repo_path, diff)) = tree_diff.next().await {
                 let (before, after) = diff.unwrap();
+                let ui_path = path_converter.format_file_path(&repo_path);
                 if before.is_present() && after.is_present() {
-                    writeln!(
-                        formatter.labeled("modified"),
-                        "M {}",
-                        workspace_command.format_file_path(&repo_path)
-                    )?;
+                    writeln!(formatter.labeled("modified"), "M {ui_path}")?;
                 } else if before.is_absent() {
-                    writeln!(
-                        formatter.labeled("added"),
-                        "A {}",
-                        workspace_command.format_file_path(&repo_path)
-                    )?;
+                    writeln!(formatter.labeled("added"), "A {ui_path}")?;
                 } else {
-                    writeln!(
-                        formatter.labeled("removed"),
-                        "D {}", // `R` could be interpreted as "renamed"
-                        workspace_command.format_file_path(&repo_path)
-                    )?;
+                    // `R` could be interpreted as "renamed"
+                    writeln!(formatter.labeled("removed"), "D {ui_path}")?;
                 }
             }
             Ok(())
@@ -935,20 +994,21 @@ fn get_diff_stat(
 }
 
 pub fn show_diff_stat(
-    ui: &Ui,
+    repo: &dyn Repo,
     formatter: &mut dyn Formatter,
-    workspace_command: &WorkspaceCommandHelper,
     tree_diff: TreeDiffStream,
-) -> Result<(), CommandError> {
+    path_converter: &RepoPathUiConverter,
+    display_width: usize,
+) -> Result<(), DiffRenderError> {
     let mut stats: Vec<DiffStat> = vec![];
     let mut max_path_width = 0;
     let mut max_diffs = 0;
 
-    let mut diff_stream = materialized_diff_stream(workspace_command.repo().store(), tree_diff);
+    let mut diff_stream = materialized_diff_stream(repo.store(), tree_diff);
     async {
         while let Some((repo_path, diff)) = diff_stream.next().await {
             let (left, right) = diff?;
-            let path = workspace_command.format_file_path(&repo_path);
+            let path = path_converter.format_file_path(&repo_path);
             let left_content = diff_content(&repo_path, left)?;
             let right_content = diff_content(&repo_path, right)?;
             max_path_width = max(max_path_width, path.width());
@@ -956,14 +1016,13 @@ pub fn show_diff_stat(
             max_diffs = max(max_diffs, stat.added + stat.removed);
             stats.push(stat);
         }
-        Ok::<(), CommandError>(())
+        Ok::<(), DiffRenderError>(())
     }
     .block_on()?;
 
     let number_padding = max_diffs.to_string().len();
     // 4 characters padding for the graph
-    let available_width =
-        usize::from(ui.term_width().unwrap_or(80)).saturating_sub(4 + " | ".len() + number_padding);
+    let available_width = display_width.saturating_sub(4 + " | ".len() + number_padding);
     // Always give at least a tiny bit of room
     let available_width = max(available_width, 5);
     let max_path_width = max_path_width.clamp(3, (0.7 * available_width as f64) as usize);
@@ -1013,8 +1072,8 @@ pub fn show_diff_stat(
 
 pub fn show_types(
     formatter: &mut dyn Formatter,
-    workspace_command: &WorkspaceCommandHelper,
     mut tree_diff: TreeDiffStream,
+    path_converter: &RepoPathUiConverter,
 ) -> io::Result<()> {
     formatter.with_label("diff", |formatter| {
         async {
@@ -1025,7 +1084,7 @@ pub fn show_types(
                     "{}{} {}",
                     diff_summary_char(&before),
                     diff_summary_char(&after),
-                    workspace_command.format_file_path(&repo_path)
+                    path_converter.format_file_path(&repo_path)
                 )?;
             }
             Ok(())
