@@ -4,7 +4,7 @@ use std::sync::Arc;
 
 use futures::{StreamExt, TryFutureExt, TryStreamExt};
 use itertools::Itertools;
-use jj_lib::backend::{BackendError, FileId, MergedTreeId, TreeValue};
+use jj_lib::backend::{BackendError, BackendResult, FileId, MergedTreeId, TreeValue};
 use jj_lib::conflicts::{materialize_tree_value, MaterializedTreeValue};
 use jj_lib::diff::{find_line_ranges, Diff, DiffHunk};
 use jj_lib::files::{self, ContentHunk, MergeResult};
@@ -128,7 +128,7 @@ fn read_file_contents(
     tree: &MergedTree,
     path: &RepoPath,
 ) -> Result<FileInfo, BuiltinToolError> {
-    let value = tree.path_value(path);
+    let value = tree.path_value(path)?;
     let materialized_value = materialize_tree_value(store, path, value)
         .map_err(BuiltinToolError::BackendError)
         .block_on()?;
@@ -137,6 +137,14 @@ fn read_file_contents(
             file_mode: scm_record::FileMode::absent(),
             contents: FileContents::Absent,
         }),
+        MaterializedTreeValue::AccessDenied(err) => Ok(FileInfo {
+            file_mode: scm_record::FileMode(mode::NORMAL),
+            contents: FileContents::Text {
+                contents: format!("Access denied: {err}"),
+                hash: None,
+                num_bytes: 0,
+            },
+        }),
 
         MaterializedTreeValue::File {
             id,
@@ -184,7 +192,11 @@ fn read_file_contents(
             item: "git submodule",
             id: id.hex(),
         }),
-        MaterializedTreeValue::Conflict { id: _, contents } => {
+        MaterializedTreeValue::Conflict {
+            id: _,
+            contents,
+            executable: _,
+        } => {
             // TODO: Render the ID somehow?
             let contents = buf_to_file_contents(None, contents);
             Ok(FileInfo {
@@ -412,7 +424,7 @@ pub fn apply_diff_builtin(
     right_tree: &MergedTree,
     changed_files: Vec<RepoPathBuf>,
     files: &[scm_record::File],
-) -> Result<MergedTreeId, BackendError> {
+) -> BackendResult<MergedTreeId> {
     let mut tree_builder = MergedTreeBuilder::new(left_tree.id().clone());
     assert_eq!(
         changed_files.len(),
@@ -444,7 +456,7 @@ pub fn apply_diff_builtin(
                 let file_deleted = file_existed_previously && !file_exists_now;
 
                 if new_empty_file {
-                    let value = right_tree.path_value(&path);
+                    let value = right_tree.path_value(&path)?;
                     tree_builder.set_or_remove(path, value);
                 } else if file_deleted {
                     tree_builder.set_or_remove(path, Merge::absent());
@@ -458,7 +470,7 @@ pub fn apply_diff_builtin(
                 old_description: _,
                 new_description: _,
             } => {
-                let value = right_tree.path_value(&path);
+                let value = right_tree.path_value(&path)?;
                 tree_builder.set_or_remove(path, value);
             }
             scm_record::SelectedContents::Present { contents } => {
@@ -1028,9 +1040,9 @@ mod tests {
             }
         }
         let merge = Merge::from_vec(vec![
-            to_file_id(left_tree.path_value(path)),
-            to_file_id(base_tree.path_value(path)),
-            to_file_id(right_tree.path_value(path)),
+            to_file_id(left_tree.path_value(path).unwrap()),
+            to_file_id(base_tree.path_value(path).unwrap()),
+            to_file_id(right_tree.path_value(path).unwrap()),
         ]);
         let content = extract_as_single_hunk(&merge, store, path).block_on();
         let slices = content.map(|ContentHunk(buf)| buf.as_slice());
