@@ -91,10 +91,7 @@ pub enum ConflictResolveError {
 pub enum MergeToolConfigError {
     #[error(transparent)]
     Config(#[from] ConfigError),
-    #[error(
-        "To use `{tool_name}` as a merge tool, the config `merge-tools.{tool_name}.merge-args` \
-         must be defined (see docs for details)"
-    )]
+    #[error("The tool `{tool_name}` cannot be used as a merge tool with `jj resolve`")]
     MergeArgsNotConfigured { tool_name: String },
 }
 
@@ -286,7 +283,7 @@ impl MergeEditor {
         tree: &MergedTree,
         repo_path: &RepoPath,
     ) -> Result<MergedTreeId, ConflictResolveError> {
-        let conflict = match tree.path_value(repo_path).into_resolved() {
+        let conflict = match tree.path_value(repo_path)?.into_resolved() {
             Err(conflict) => conflict,
             Ok(Some(_)) => return Err(ConflictResolveError::NotAConflict(repo_path.to_owned())),
             Ok(None) => return Err(ConflictResolveError::PathNotFound(repo_path.to_owned())),
