@@ -15,6 +15,7 @@
 //! Utility for parsing and evaluating user-provided revset expressions.
 
 use std::rc::Rc;
+use std::sync::Arc;
 
 use itertools::Itertools as _;
 use jj_lib::backend::{BackendResult, CommitId};
@@ -23,8 +24,8 @@ use jj_lib::id_prefix::IdPrefixContext;
 use jj_lib::repo::Repo;
 use jj_lib::revset::{
     self, DefaultSymbolResolver, Revset, RevsetAliasesMap, RevsetCommitRef, RevsetEvaluationError,
-    RevsetExpression, RevsetIteratorExt as _, RevsetParseContext, RevsetParseError,
-    RevsetResolutionError,
+    RevsetExpression, RevsetExtensions, RevsetIteratorExt as _, RevsetParseContext,
+    RevsetParseError, RevsetResolutionError, SymbolResolverExtension,
 };
 use jj_lib::settings::ConfigResultExt as _;
 use thiserror::Error;
@@ -48,6 +49,7 @@ pub enum UserRevsetEvaluationError {
 /// Wrapper around `RevsetExpression` to provide convenient methods.
 pub struct RevsetExpressionEvaluator<'repo> {
     repo: &'repo dyn Repo,
+    extensions: Arc<RevsetExtensions>,
     id_prefix_context: &'repo IdPrefixContext,
     expression: Rc<RevsetExpression>,
 }
@@ -55,11 +57,13 @@ pub struct RevsetExpressionEvaluator<'repo> {
 impl<'repo> RevsetExpressionEvaluator<'repo> {
     pub fn new(
         repo: &'repo dyn Repo,
+        extensions: Arc<RevsetExtensions>,
         id_prefix_context: &'repo IdPrefixContext,
         expression: Rc<RevsetExpression>,
     ) -> Self {
         RevsetExpressionEvaluator {
             repo,
+            extensions,
             id_prefix_context,
             expression,
         }
@@ -77,7 +81,11 @@ impl<'repo> RevsetExpressionEvaluator<'repo> {
 
     /// Evaluates the expression.
     pub fn evaluate(&self) -> Result<Box<dyn Revset + 'repo>, UserRevsetEvaluationError> {
-        let symbol_resolver = default_symbol_resolver(self.repo, self.id_prefix_context);
+        let symbol_resolver = default_symbol_resolver(
+            self.repo,
+            self.extensions.symbol_resolvers(),
+            self.id_prefix_context,
+        );
         evaluate(self.repo, &symbol_resolver, self.expression.clone())
     }
 
@@ -129,7 +137,7 @@ pub fn load_revset_aliases(
 
     // TODO: If we add support for function overloading (#2966), this check can
     // be removed.
-    let (params, _) = aliases_map.get_function(BUILTIN_IMMUTABLE_HEADS).unwrap();
+    let (_, params, _) = aliases_map.get_function(BUILTIN_IMMUTABLE_HEADS).unwrap();
     if !params.is_empty() {
         return Err(user_error(format!(
             "The `revset-aliases.{name}()` function must be declared without arguments",
@@ -157,23 +165,18 @@ pub fn evaluate<'a>(
 /// `evaluate()`.
 pub fn default_symbol_resolver<'a>(
     repo: &'a dyn Repo,
+    extensions: &[impl AsRef<dyn SymbolResolverExtension>],
     id_prefix_context: &'a IdPrefixContext,
 ) -> DefaultSymbolResolver<'a> {
-    let commit_id_resolver: revset::PrefixResolver<CommitId> =
-        Box::new(|repo, prefix| id_prefix_context.resolve_commit_prefix(repo, prefix));
-    let change_id_resolver: revset::PrefixResolver<Vec<CommitId>> =
-        Box::new(|repo, prefix| id_prefix_context.resolve_change_prefix(repo, prefix));
-    DefaultSymbolResolver::new(repo)
-        .with_commit_id_resolver(commit_id_resolver)
-        .with_change_id_resolver(change_id_resolver)
+    DefaultSymbolResolver::new(repo, extensions).with_id_prefix_context(id_prefix_context)
 }
 
 /// Parses user-configured expression defining the immutable set.
 pub fn parse_immutable_expression(
     context: &RevsetParseContext,
 ) -> Result<Rc<RevsetExpression>, RevsetParseError> {
-    let (params, immutable_heads_str) = context
-        .aliases_map
+    let (_, params, immutable_heads_str) = context
+        .aliases_map()
         .get_function(BUILTIN_IMMUTABLE_HEADS)
         .unwrap();
     assert!(
