@@ -9,6 +9,8 @@ fn test_no_forgotten_test_files() {
 }
 
 mod test_abandon_command;
+mod test_acls;
+mod test_advance_branches;
 mod test_alias;
 mod test_branch_command;
 mod test_builtin_aliases;
@@ -25,6 +27,7 @@ mod test_diff_command;
 mod test_diffedit_command;
 mod test_duplicate_command;
 mod test_edit_command;
+mod test_fix_command;
 mod test_generate_md_cli_help;
 mod test_git_clone;
 mod test_git_colocated;
@@ -45,6 +48,7 @@ mod test_new_command;
 mod test_next_prev_commands;
 mod test_obslog_command;
 mod test_operations;
+mod test_parallelize_command;
 mod test_rebase_command;
 mod test_repo_change_report;
 mod test_resolve_command;
@@ -59,7 +63,6 @@ mod test_squash_command;
 mod test_status_command;
 mod test_tag_command;
 mod test_templater;
-mod test_tree_level_conflicts;
 mod test_undo;
 mod test_unsquash_command;
 mod test_untrack_command;
