@@ -18,12 +18,10 @@ use itertools::Itertools as _;
 
 use crate::common::TestEnvironment;
 
-pub mod common;
-
 #[test]
 fn test_alias_basic() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.add_config(r#"aliases.b = ["log", "-r", "@", "-T", "branches"]"#);
@@ -39,7 +37,7 @@ fn test_alias_basic() {
 #[test]
 fn test_alias_legacy_section() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     // Can define aliases in [alias] section
@@ -64,7 +62,7 @@ fn test_alias_legacy_section() {
 #[test]
 fn test_alias_bad_name() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let stderr = test_env.jj_cmd_cli_error(&repo_path, &["foo."]);
@@ -80,7 +78,7 @@ fn test_alias_bad_name() {
 #[test]
 fn test_alias_calls_empty_command() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.add_config(
@@ -108,7 +106,7 @@ fn test_alias_calls_empty_command() {
 #[test]
 fn test_alias_calls_unknown_command() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.add_config(r#"aliases.foo = ["nonexistent"]"#);
@@ -127,7 +125,7 @@ fn test_alias_calls_unknown_command() {
 #[test]
 fn test_alias_calls_command_with_invalid_option() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.add_config(r#"aliases.foo = ["log", "--nonexistent"]"#);
@@ -146,7 +144,7 @@ fn test_alias_calls_command_with_invalid_option() {
 #[test]
 fn test_alias_calls_help() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     test_env.add_config(r#"aliases.h = ["--help"]"#);
     let stdout = test_env.jj_cmd_success(&repo_path, &["h"]);
@@ -162,21 +160,20 @@ fn test_alias_calls_help() {
 #[test]
 fn test_alias_cannot_override_builtin() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.add_config(r#"aliases.log = ["rebase"]"#);
-    // Alias should be ignored
-    let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-r", "root()"]);
-    insta::assert_snapshot!(stdout, @r###"
-    ◉  zzzzzzzz root() 00000000
-    "###);
+    // Alias should give a warning
+    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["log", "-r", "root()"]);
+    insta::assert_snapshot!(stdout, @"◉  zzzzzzzz root() 00000000\n");
+    insta::assert_snapshot!(stderr, @"Warning: Cannot define an alias that overrides the built-in command 'log'\n");
 }
 
 #[test]
 fn test_alias_recursive() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.add_config(
@@ -201,7 +198,7 @@ fn test_alias_recursive() {
 #[test]
 fn test_alias_global_args_before_and_after() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     test_env.add_config(r#"aliases.l = ["log", "-T", "commit_id", "-r", "all()"]"#);
     // Test the setup
@@ -236,7 +233,7 @@ fn test_alias_global_args_before_and_after() {
 #[test]
 fn test_alias_global_args_in_definition() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     test_env.add_config(
         r#"aliases.l = ["log", "-T", "commit_id", "--at-op", "@-", "-r", "all()", "--color=always"]"#,
@@ -272,10 +269,10 @@ fn test_alias_invalid_definition() {
 #[test]
 fn test_alias_in_repo_config() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo1", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo1"]);
     let repo1_path = test_env.env_root().join("repo1");
     fs::create_dir(repo1_path.join("sub")).unwrap();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo2", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo2"]);
     let repo2_path = test_env.env_root().join("repo2");
     fs::create_dir(repo2_path.join("sub")).unwrap();
 
@@ -305,7 +302,7 @@ fn test_alias_in_repo_config() {
     user alias
     "###);
     insta::assert_snapshot!(stderr, @r###"
-    Command aliases cannot be loaded from -R/--repository path
+    Warning: Command aliases cannot be loaded from -R/--repository path
     "###);
 
     // Aliases are loaded from the cwd-relative workspace even with -R.
@@ -315,7 +312,7 @@ fn test_alias_in_repo_config() {
     repo1 alias
     "###);
     insta::assert_snapshot!(stderr, @r###"
-    Command aliases cannot be loaded from -R/--repository path
+    Warning: Command aliases cannot be loaded from -R/--repository path
     "###);
 
     // No warning if the expanded command is identical.
@@ -337,6 +334,6 @@ fn test_alias_in_repo_config() {
         ],
     );
     insta::assert_snapshot!(stdout, @r###"
-    aliases.l=["log", "-r@", "--no-graph", "-T\"user alias\\n\""]
+    aliases.l = ["log", "-r@", "--no-graph", '-T"user alias\n"']
     "###);
 }
