@@ -19,7 +19,7 @@ use crate::common::TestEnvironment;
 #[test]
 fn test_branch_multiple_names() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "create", "foo", "bar"]);
@@ -60,7 +60,7 @@ fn test_branch_multiple_names() {
 #[test]
 fn test_branch_at_root() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let (stdout, stderr) =
@@ -79,7 +79,7 @@ fn test_branch_at_root() {
 #[test]
 fn test_branch_empty_name() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let stderr = test_env.jj_cmd_cli_error(&repo_path, &["branch", "create", ""]);
@@ -93,7 +93,7 @@ fn test_branch_empty_name() {
 #[test]
 fn test_branch_move() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let stderr = test_env.jj_cmd_failure(&repo_path, &["branch", "set", "foo"]);
@@ -131,7 +131,7 @@ fn test_branch_move() {
 #[test]
 fn test_branch_move_conflicting() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let get_log = || {
         let template = r#"separate(" ", description.first_line(), branches)"#;
@@ -190,7 +190,7 @@ fn test_branch_move_conflicting() {
 #[test]
 fn test_branch_rename() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     // Set up remote
@@ -235,7 +235,7 @@ fn test_branch_rename() {
 #[test]
 fn test_branch_forget_glob() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(&repo_path, &["branch", "create", "foo-1"]);
@@ -303,7 +303,7 @@ fn test_branch_forget_glob() {
 fn test_branch_delete_glob() {
     // Set up a git repo with a branch and a jj repo that has it as a remote.
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let git_repo_path = test_env.env_root().join("git-repo");
     let git_repo = git2::Repository::init_bare(git_repo_path).unwrap();
@@ -376,13 +376,10 @@ fn test_branch_delete_glob() {
       @origin: qpvuntsm 6fbf398c (empty) commit
     foo-1 (deleted)
       @origin: qpvuntsm 6fbf398c (empty) commit
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     foo-3 (deleted)
       @origin: qpvuntsm 6fbf398c (empty) commit
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     foo-4 (deleted)
       @origin: qpvuntsm 6fbf398c (empty) commit
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     "###);
 
     // Malformed glob
@@ -406,7 +403,7 @@ fn test_branch_delete_glob() {
 #[test]
 fn test_branch_delete_export() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(&repo_path, &["new"]);
@@ -414,29 +411,29 @@ fn test_branch_delete_export() {
     test_env.jj_cmd_ok(&repo_path, &["git", "export"]);
 
     test_env.jj_cmd_ok(&repo_path, &["branch", "delete", "foo"]);
-    let stdout = test_env.jj_cmd_success(&repo_path, &["branch", "list", "--all-remotes"]);
+    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "list", "--all-remotes"]);
     insta::assert_snapshot!(stdout, @r###"
     foo (deleted)
       @git: rlvkpnrz 65b6b74e (empty) (no description set)
-      (this branch will be deleted from the underlying Git repo on the next `jj git export`)
+    "###);
+    insta::assert_snapshot!(stderr, @r###"
+    Hint: Branches marked as deleted will be deleted from the underlying Git repo on the next `jj git export`.
     "###);
 
     test_env.jj_cmd_ok(&repo_path, &["git", "export"]);
-    let stdout = test_env.jj_cmd_success(&repo_path, &["branch", "list", "--all-remotes"]);
-    insta::assert_snapshot!(stdout, @r###"
+    insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
     "###);
 }
 
 #[test]
 fn test_branch_forget_export() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(&repo_path, &["new"]);
     test_env.jj_cmd_ok(&repo_path, &["branch", "create", "foo"]);
-    let stdout = test_env.jj_cmd_success(&repo_path, &["branch", "list", "--all-remotes"]);
-    insta::assert_snapshot!(stdout, @r###"
+    insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
     foo: rlvkpnrz 65b6b74e (empty) (no description set)
     "###);
 
@@ -449,8 +446,7 @@ fn test_branch_forget_export() {
     insta::assert_snapshot!(stderr, @"");
     // Forgetting a branch deletes local and remote-tracking branches including
     // the corresponding git-tracking branch.
-    let stdout = test_env.jj_cmd_success(&repo_path, &["branch", "list", "--all-remotes"]);
-    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @"");
     let stderr = test_env.jj_cmd_failure(&repo_path, &["log", "-r=foo", "--no-graph"]);
     insta::assert_snapshot!(stderr, @r###"
     Error: Revision "foo" doesn't exist
@@ -463,8 +459,7 @@ fn test_branch_forget_export() {
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["git", "export"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @"");
-    let stdout = test_env.jj_cmd_success(&repo_path, &["branch", "list", "--all-remotes"]);
-    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @"");
 }
 
 #[test]
@@ -475,7 +470,7 @@ fn test_branch_forget_fetched_branch() {
     // Set up a git repo with a branch and a jj repo that has it as a remote.
     let test_env = TestEnvironment::default();
     test_env.add_config("git.auto-local-branch = true");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let git_repo_path = test_env.env_root().join("git-repo");
     let git_repo = git2::Repository::init_bare(git_repo_path).unwrap();
@@ -597,7 +592,7 @@ fn test_branch_forget_deleted_or_nonexistent_branch() {
     // Set up a git repo with a branch and a jj repo that has it as a remote.
     let test_env = TestEnvironment::default();
     test_env.add_config("git.auto-local-branch = true");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let git_repo_path = test_env.env_root().join("git-repo");
     let git_repo = git2::Repository::init_bare(git_repo_path).unwrap();
@@ -632,7 +627,6 @@ fn test_branch_forget_deleted_or_nonexistent_branch() {
     insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
     feature1 (deleted)
       @origin: mzyxwzks 9f01a0e0 message
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     "###);
 
     // ============ End of test setup ============
@@ -651,7 +645,7 @@ fn test_branch_forget_deleted_or_nonexistent_branch() {
 #[test]
 fn test_branch_track_untrack() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     // Set up remote
@@ -833,10 +827,39 @@ fn test_branch_track_untrack() {
     "###);
 }
 
+#[test]
+fn test_branch_track_conflict() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    let git_repo_path = test_env.env_root().join("git-repo");
+    git2::Repository::init_bare(git_repo_path).unwrap();
+    test_env.jj_cmd_ok(
+        &repo_path,
+        &["git", "remote", "add", "origin", "../git-repo"],
+    );
+    test_env.jj_cmd_ok(&repo_path, &["branch", "create", "main"]);
+    test_env.jj_cmd_ok(&repo_path, &["describe", "-m", "a"]);
+    test_env.jj_cmd_ok(&repo_path, &["git", "push", "-b", "main"]);
+    test_env.jj_cmd_ok(&repo_path, &["branch", "untrack", "main@origin"]);
+    test_env.jj_cmd_ok(
+        &repo_path,
+        &["describe", "-m", "b", "-r", "main", "--ignore-immutable"],
+    );
+    let (_, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "track", "main@origin"]);
+    insta::assert_snapshot!(stderr, @r###"
+main (conflicted):
+  + qpvuntsm b4a6b8c5 (empty) b
+  + qpvuntsm hidden 4bfd80cd (empty) a
+  @origin (behind by 1 commits): qpvuntsm hidden 4bfd80cd (empty) a
+"###);
+}
+
 #[test]
 fn test_branch_track_untrack_patterns() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     // Set up remote
@@ -973,7 +996,7 @@ fn test_branch_list() {
     test_env.add_config("git.auto-local-branch = true");
 
     // Initialize remote refs
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "remote", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "remote"]);
     let remote_path = test_env.env_root().join("remote");
     for branch in [
         "remote-sync",
@@ -1009,29 +1032,137 @@ fn test_branch_list() {
 
     // Synchronized tracking remotes and non-tracking remotes aren't listed by
     // default
-    insta::assert_snapshot!(
-        test_env.jj_cmd_success(&local_path, &["branch", "list"]), @r###"
+    let (stdout, stderr) = test_env.jj_cmd_ok(&local_path, &["branch", "list"]);
+    insta::assert_snapshot!(stdout, @r###"
     local-only: wqnwkozp 4e887f78 (empty) local-only
     remote-delete (deleted)
       @origin: mnmymoky 203e60eb (empty) remote-delete
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     remote-sync: zwtyzrop c761c7ea (empty) remote-sync
     remote-unsync: wqnwkozp 4e887f78 (empty) local-only
       @origin (ahead by 1 commits, behind by 1 commits): qpsqxpyq 38ef8af7 (empty) remote-unsync
     "###);
+    insta::assert_snapshot!(stderr, @r###"
+    Hint: Branches marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this.
+    "###);
 
-    insta::assert_snapshot!(
-        test_env.jj_cmd_success(&local_path, &["branch", "list", "--all-remotes"]), @r###"
+    let (stdout, stderr) = test_env.jj_cmd_ok(&local_path, &["branch", "list", "--all-remotes"]);
+    insta::assert_snapshot!(stdout, @r###"
     local-only: wqnwkozp 4e887f78 (empty) local-only
     remote-delete (deleted)
       @origin: mnmymoky 203e60eb (empty) remote-delete
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     remote-sync: zwtyzrop c761c7ea (empty) remote-sync
       @origin: zwtyzrop c761c7ea (empty) remote-sync
     remote-unsync: wqnwkozp 4e887f78 (empty) local-only
       @origin (ahead by 1 commits, behind by 1 commits): qpsqxpyq 38ef8af7 (empty) remote-unsync
     remote-untrack@origin: vmortlor 71a16b05 (empty) remote-untrack
     "###);
+    insta::assert_snapshot!(stderr, @r###"
+    Hint: Branches marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this.
+    "###);
+
+    let template = r#"
+    concat(
+      "[" ++ name ++ if(remote, "@" ++ remote) ++ "]\n",
+      separate(" ", "present:", present) ++ "\n",
+      separate(" ", "conflict:", conflict) ++ "\n",
+      separate(" ", "normal_target:", normal_target.description().first_line()) ++ "\n",
+      separate(" ", "removed_targets:", removed_targets.map(|c| c.description().first_line())) ++ "\n",
+      separate(" ", "added_targets:", added_targets.map(|c| c.description().first_line())) ++ "\n",
+      separate(" ", "tracked:", tracked) ++ "\n",
+      separate(" ", "tracking_present:", tracking_present) ++ "\n",
+      separate(" ", "tracking_ahead_count:", tracking_ahead_count.lower()) ++ "\n",
+      separate(" ", "tracking_behind_count:", tracking_behind_count.lower()) ++ "\n",
+    )
+    "#;
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &local_path,
+        &["branch", "list", "--all-remotes", "-T", template],
+    );
+    insta::assert_snapshot!(stdout, @r###"
+    [local-only]
+    present: true
+    conflict: false
+    normal_target: local-only
+    removed_targets:
+    added_targets: local-only
+    tracked: false
+    tracking_present: false
+    tracking_ahead_count: <Error: Not a tracked remote ref>
+    tracking_behind_count: <Error: Not a tracked remote ref>
+    [remote-delete]
+    present: false
+    conflict: false
+    normal_target: <Error: No Commit available>
+    removed_targets:
+    added_targets:
+    tracked: false
+    tracking_present: false
+    tracking_ahead_count: <Error: Not a tracked remote ref>
+    tracking_behind_count: <Error: Not a tracked remote ref>
+    [remote-delete@origin]
+    present: true
+    conflict: false
+    normal_target: remote-delete
+    removed_targets:
+    added_targets: remote-delete
+    tracked: true
+    tracking_present: false
+    tracking_ahead_count: 2
+    tracking_behind_count: 0
+    [remote-sync]
+    present: true
+    conflict: false
+    normal_target: remote-sync
+    removed_targets:
+    added_targets: remote-sync
+    tracked: false
+    tracking_present: false
+    tracking_ahead_count: <Error: Not a tracked remote ref>
+    tracking_behind_count: <Error: Not a tracked remote ref>
+    [remote-sync@origin]
+    present: true
+    conflict: false
+    normal_target: remote-sync
+    removed_targets:
+    added_targets: remote-sync
+    tracked: true
+    tracking_present: true
+    tracking_ahead_count: 0
+    tracking_behind_count: 0
+    [remote-unsync]
+    present: true
+    conflict: false
+    normal_target: local-only
+    removed_targets:
+    added_targets: local-only
+    tracked: false
+    tracking_present: false
+    tracking_ahead_count: <Error: Not a tracked remote ref>
+    tracking_behind_count: <Error: Not a tracked remote ref>
+    [remote-unsync@origin]
+    present: true
+    conflict: false
+    normal_target: remote-unsync
+    removed_targets:
+    added_targets: remote-unsync
+    tracked: true
+    tracking_present: true
+    tracking_ahead_count: 1
+    tracking_behind_count: 1
+    [remote-untrack@origin]
+    present: true
+    conflict: false
+    normal_target: remote-untrack
+    removed_targets:
+    added_targets: remote-untrack
+    tracked: false
+    tracking_present: false
+    tracking_ahead_count: <Error: Not a tracked remote ref>
+    tracking_behind_count: <Error: Not a tracked remote ref>
+    "###);
+    insta::assert_snapshot!(stderr, @r###"
+    Hint: Branches marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this.
+    "###);
 }
 
 #[test]
@@ -1041,7 +1172,7 @@ fn test_branch_list_filtered() {
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
 
     // Initialize remote refs
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "remote", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "remote"]);
     let remote_path = test_env.env_root().join("remote");
     for branch in ["remote-keep", "remote-delete", "remote-rewrite"] {
         test_env.jj_cmd_ok(&remote_path, &["new", "root()", "-m", branch]);
@@ -1085,95 +1216,122 @@ fn test_branch_list_filtered() {
     "###);
 
     // All branches are listed by default.
-    insta::assert_snapshot!(test_env.jj_cmd_success(&local_path, &["branch", "list"]), @r###"
+    let (stdout, stderr) = test_env.jj_cmd_ok(&local_path, &["branch", "list"]);
+    insta::assert_snapshot!(stdout, @r###"
     local-keep: kpqxywon c7b4c09c (empty) local-keep
     remote-delete (deleted)
       @origin: yxusvupt dad5f298 (empty) remote-delete
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     remote-keep: nlwprzpn 911e9120 (empty) remote-keep
     remote-rewrite: xyxluytn e31634b6 (empty) rewritten
       @origin (ahead by 1 commits, behind by 1 commits): xyxluytn hidden 3e9a5af6 (empty) remote-rewrite
     "###);
+    insta::assert_snapshot!(stderr, @r###"
+    Hint: Branches marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this.
+    "###);
 
     let query =
-        |args: &[&str]| test_env.jj_cmd_success(&local_path, &[&["branch", "list"], args].concat());
+        |args: &[&str]| test_env.jj_cmd_ok(&local_path, &[&["branch", "list"], args].concat());
     let query_error =
         |args: &[&str]| test_env.jj_cmd_failure(&local_path, &[&["branch", "list"], args].concat());
 
     // "all()" doesn't include deleted branches since they have no local targets.
     // So "all()" is identical to "branches()".
-    insta::assert_snapshot!(query(&["-rall()"]), @r###"
+    let (stdout, stderr) = query(&["-rall()"]);
+    insta::assert_snapshot!(stdout, @r###"
     local-keep: kpqxywon c7b4c09c (empty) local-keep
     remote-keep: nlwprzpn 911e9120 (empty) remote-keep
     remote-rewrite: xyxluytn e31634b6 (empty) rewritten
       @origin (ahead by 1 commits, behind by 1 commits): xyxluytn hidden 3e9a5af6 (empty) remote-rewrite
     "###);
+    insta::assert_snapshot!(stderr, @"");
 
     // Exclude remote-only branches. "remote-rewrite@origin" is included since
     // local "remote-rewrite" target matches.
-    insta::assert_snapshot!(query(&["-rbranches()"]), @r###"
+    let (stdout, stderr) = query(&["-rbranches()"]);
+    insta::assert_snapshot!(stdout, @r###"
     local-keep: kpqxywon c7b4c09c (empty) local-keep
     remote-keep: nlwprzpn 911e9120 (empty) remote-keep
     remote-rewrite: xyxluytn e31634b6 (empty) rewritten
       @origin (ahead by 1 commits, behind by 1 commits): xyxluytn hidden 3e9a5af6 (empty) remote-rewrite
     "###);
+    insta::assert_snapshot!(stderr, @"");
 
     // Select branches by name.
-    insta::assert_snapshot!(query(&["remote-rewrite"]), @r###"
+    let (stdout, stderr) = query(&["remote-rewrite"]);
+    insta::assert_snapshot!(stdout, @r###"
     remote-rewrite: xyxluytn e31634b6 (empty) rewritten
       @origin (ahead by 1 commits, behind by 1 commits): xyxluytn hidden 3e9a5af6 (empty) remote-rewrite
     "###);
-    insta::assert_snapshot!(query(&["-rbranches(remote-rewrite)"]), @r###"
+    insta::assert_snapshot!(stderr, @"");
+    let (stdout, stderr) = query(&["-rbranches(remote-rewrite)"]);
+    insta::assert_snapshot!(stdout, @r###"
     remote-rewrite: xyxluytn e31634b6 (empty) rewritten
       @origin (ahead by 1 commits, behind by 1 commits): xyxluytn hidden 3e9a5af6 (empty) remote-rewrite
     "###);
+    insta::assert_snapshot!(stderr, @"");
 
     // Select branches by name, combined with --all-remotes
     test_env.jj_cmd_ok(&local_path, &["git", "export"]);
-    insta::assert_snapshot!(query(&["--all-remotes", "remote-rewrite"]), @r###"
+    let (stdout, stderr) = query(&["--all-remotes", "remote-rewrite"]);
+    insta::assert_snapshot!(stdout, @r###"
     remote-rewrite: xyxluytn e31634b6 (empty) rewritten
       @git: xyxluytn e31634b6 (empty) rewritten
       @origin (ahead by 1 commits, behind by 1 commits): xyxluytn hidden 3e9a5af6 (empty) remote-rewrite
     "###);
-    insta::assert_snapshot!(query(&["--all-remotes", "-rbranches(remote-rewrite)"]), @r###"
+    insta::assert_snapshot!(stderr, @"");
+    let (stdout, stderr) = query(&["--all-remotes", "-rbranches(remote-rewrite)"]);
+    insta::assert_snapshot!(stdout, @r###"
     remote-rewrite: xyxluytn e31634b6 (empty) rewritten
       @git: xyxluytn e31634b6 (empty) rewritten
       @origin (ahead by 1 commits, behind by 1 commits): xyxluytn hidden 3e9a5af6 (empty) remote-rewrite
     "###);
+    insta::assert_snapshot!(stderr, @"");
 
     // Can select deleted branch by name pattern, but not by revset.
-    insta::assert_snapshot!(query(&["remote-delete"]), @r###"
+    let (stdout, stderr) = query(&["remote-delete"]);
+    insta::assert_snapshot!(stdout, @r###"
     remote-delete (deleted)
       @origin: yxusvupt dad5f298 (empty) remote-delete
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     "###);
-    insta::assert_snapshot!(query(&["-rbranches(remote-delete)"]), @r###"
+    insta::assert_snapshot!(stderr, @r###"
+    Hint: Branches marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this.
+    "###);
+    let (stdout, stderr) = query(&["-rbranches(remote-delete)"]);
+    insta::assert_snapshot!(stdout, @r###"
     "###);
     insta::assert_snapshot!(query_error(&["-rremote-delete"]), @r###"
     Error: Revision "remote-delete" doesn't exist
     Hint: Did you mean "remote-delete@origin", "remote-keep", "remote-rewrite", "remote-rewrite@origin"?
     "###);
+    insta::assert_snapshot!(stderr, @"");
 
     // Name patterns are OR-ed.
-    insta::assert_snapshot!(query(&["glob:*-keep", "remote-delete"]), @r###"
+    let (stdout, stderr) = query(&["glob:*-keep", "remote-delete"]);
+    insta::assert_snapshot!(stdout, @r###"
     local-keep: kpqxywon c7b4c09c (empty) local-keep
     remote-delete (deleted)
       @origin: yxusvupt dad5f298 (empty) remote-delete
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     remote-keep: nlwprzpn 911e9120 (empty) remote-keep
     "###);
+    insta::assert_snapshot!(stderr, @r###"
+    Hint: Branches marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this.
+    "###);
 
     // Unmatched name pattern shouldn't be an error. A warning can be added later.
-    insta::assert_snapshot!(query(&["local-keep", "glob:push-*"]), @r###"
+    let (stdout, stderr) = query(&["local-keep", "glob:push-*"]);
+    insta::assert_snapshot!(stdout, @r###"
     local-keep: kpqxywon c7b4c09c (empty) local-keep
     "###);
+    insta::assert_snapshot!(stderr, @"");
 
     // Name pattern and revset are OR-ed.
-    insta::assert_snapshot!(query(&["local-keep", "-rbranches(remote-rewrite)"]), @r###"
+    let (stdout, stderr) = query(&["local-keep", "-rbranches(remote-rewrite)"]);
+    insta::assert_snapshot!(stdout, @r###"
     local-keep: kpqxywon c7b4c09c (empty) local-keep
     remote-rewrite: xyxluytn e31634b6 (empty) rewritten
       @origin (ahead by 1 commits, behind by 1 commits): xyxluytn hidden 3e9a5af6 (empty) remote-rewrite
     "###);
+    insta::assert_snapshot!(stderr, @"");
 }
 
 #[test]
@@ -1182,7 +1340,7 @@ fn test_branch_list_much_remote_divergence() {
     test_env.add_config("git.auto-local-branch = true");
 
     // Initialize remote refs
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "remote", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "remote"]);
     let remote_path = test_env.env_root().join("remote");
     test_env.jj_cmd_ok(&remote_path, &["new", "root()", "-m", "remote-unsync"]);
     for _ in 0..15 {
@@ -1212,12 +1370,13 @@ fn test_branch_list_much_remote_divergence() {
         &["branch", "set", "--allow-backwards", "remote-unsync"],
     );
 
-    insta::assert_snapshot!(
-        test_env.jj_cmd_success(&local_path, &["branch", "list"]), @r###"
+    let (stdout, stderr) = test_env.jj_cmd_ok(&local_path, &["branch", "list"]);
+    insta::assert_snapshot!(stdout, @r###"
     local-only: zkyosouw 4ab3f751 (empty) local-only
     remote-unsync: zkyosouw 4ab3f751 (empty) local-only
       @origin (ahead by at least 10 commits, behind by at least 10 commits): lxyktnks 19582022 (empty) remote-unsync
     "###);
+    insta::assert_snapshot!(stderr, @"");
 }
 
 #[test]
@@ -1226,7 +1385,7 @@ fn test_branch_list_tracked() {
     test_env.add_config("git.auto-local-branch = true");
 
     // Initialize remote refs
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "remote", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "remote"]);
     let remote_path = test_env.env_root().join("remote");
     for branch in [
         "remote-sync",
@@ -1254,7 +1413,7 @@ fn test_branch_list_tracked() {
         ],
     );
 
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "upstream", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "upstream"]);
 
     // Initialize a second remote
     let mut upstream_git_path = test_env.env_root().join("upstream");
@@ -1305,13 +1464,12 @@ fn test_branch_list_tracked() {
         &["branch", "set", "--allow-backwards", "remote-unsync"],
     );
 
-    insta::assert_snapshot!(
-        test_env.jj_cmd_success(&local_path, &["branch", "list", "--all-remotes"]), @r###"
+    let (stdout, stderr) = test_env.jj_cmd_ok(&local_path, &["branch", "list", "--all-remotes"]);
+    insta::assert_snapshot!(stdout, @r###"
     local-only: nmzmmopx e1da745b (empty) local-only
       @git: nmzmmopx e1da745b (empty) local-only
     remote-delete (deleted)
       @origin: mnmymoky 203e60eb (empty) remote-delete
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     remote-sync: zwtyzrop c761c7ea (empty) remote-sync
       @git: zwtyzrop c761c7ea (empty) remote-sync
       @origin: zwtyzrop c761c7ea (empty) remote-sync
@@ -1324,12 +1482,14 @@ fn test_branch_list_tracked() {
       @git: lolpmnqw 32fa6da0 (empty) upstream-sync
       @upstream: lolpmnqw 32fa6da0 (empty) upstream-sync
     "###);
+    insta::assert_snapshot!(stderr, @r###"
+    Hint: Branches marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this.
+    "###);
 
-    insta::assert_snapshot!(
-        test_env.jj_cmd_success(&local_path, &["branch", "list", "--tracked"]), @r###"
+    let (stdout, stderr) = test_env.jj_cmd_ok(&local_path, &["branch", "list", "--tracked"]);
+    insta::assert_snapshot!(stdout, @r###"
     remote-delete (deleted)
       @origin: mnmymoky 203e60eb (empty) remote-delete
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     remote-sync: zwtyzrop c761c7ea (empty) remote-sync
       @origin: zwtyzrop c761c7ea (empty) remote-sync
     remote-unsync: nmzmmopx e1da745b (empty) local-only
@@ -1339,33 +1499,48 @@ fn test_branch_list_tracked() {
       @upstream: lolpmnqw 32fa6da0 (empty) upstream-sync
     "###
     );
+    insta::assert_snapshot!(stderr, @r###"
+    Hint: Branches marked as deleted will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this.
+    "###);
 
-    insta::assert_snapshot!(
-        test_env.jj_cmd_success(&local_path, &["branch", "list", "--tracked", "remote-unsync"]), @r###"
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &local_path,
+        &["branch", "list", "--tracked", "remote-unsync"],
+    );
+    insta::assert_snapshot!(stdout, @r###"
     remote-unsync: nmzmmopx e1da745b (empty) local-only
       @origin (ahead by 1 commits, behind by 1 commits): qpsqxpyq 38ef8af7 (empty) remote-unsync
       @upstream (ahead by 1 commits, behind by 1 commits): qpsqxpyq 38ef8af7 (empty) remote-unsync
     "###);
+    insta::assert_snapshot!(stderr, @"");
 
-    insta::assert_snapshot!(
-        test_env.jj_cmd_success(&local_path, &["branch", "list", "--tracked", "remote-untrack"]), @"");
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &local_path,
+        &["branch", "list", "--tracked", "remote-untrack"],
+    );
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @"");
 
     test_env.jj_cmd_ok(
         &local_path,
         &["branch", "untrack", "remote-unsync@upstream"],
     );
 
-    insta::assert_snapshot!(
-        test_env.jj_cmd_success(&local_path, &["branch", "list", "--tracked", "remote-unsync"]), @r###"
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &local_path,
+        &["branch", "list", "--tracked", "remote-unsync"],
+    );
+    insta::assert_snapshot!(stdout, @r###"
     remote-unsync: nmzmmopx e1da745b (empty) local-only
       @origin (ahead by 1 commits, behind by 1 commits): qpsqxpyq 38ef8af7 (empty) remote-unsync
     "###);
+    insta::assert_snapshot!(stderr, @"");
 }
 
 #[test]
 fn test_branch_list_conflicted() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     // Track existing branch. Local branch should result in conflict.
@@ -1407,5 +1582,6 @@ fn get_log_output(test_env: &TestEnvironment, cwd: &Path) -> String {
 }
 
 fn get_branch_output(test_env: &TestEnvironment, repo_path: &Path) -> String {
-    test_env.jj_cmd_success(repo_path, &["branch", "list", "--all-remotes"])
+    // --quiet to suppress deleted branches hint
+    test_env.jj_cmd_success(repo_path, &["branch", "list", "--all-remotes", "--quiet"])
 }
