@@ -17,7 +17,7 @@ use crate::common::TestEnvironment;
 #[test]
 fn test_cat() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "a\n").unwrap();
@@ -89,13 +89,13 @@ fn test_cat() {
     test_env.jj_cmd_ok(&repo_path, &["rebase", "-r", "@", "-d", "@--"]);
     let stdout = test_env.jj_cmd_success(&repo_path, &["cat", "file1"]);
     insta::assert_snapshot!(stdout, @r###"
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
     -b
     +a
-    +++++++
+    +++++++ Contents of side #2
     c
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     "###);
 }
 
@@ -118,6 +118,6 @@ fn test_cat_symlink() {
     a
     "###);
     insta::assert_snapshot!(stderr, @r###"
-    Warning: Path exists but is not a file: symlink1
+    Warning: Path 'symlink1' exists but is not a file
     "###);
 }
