@@ -19,7 +19,7 @@ use crate::common::TestEnvironment;
 #[test]
 fn test_commit_with_description_from_cli() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let workspace_path = test_env.env_root().join("repo");
 
     // Description applies to the current working-copy (not the new one)
@@ -34,7 +34,7 @@ fn test_commit_with_description_from_cli() {
 #[test]
 fn test_commit_with_editor() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let workspace_path = test_env.env_root().join("repo");
 
     // Check that the text file gets initialized with the current description and
@@ -76,7 +76,7 @@ fn test_commit_with_editor() {
 #[test]
 fn test_commit_interactive() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let workspace_path = test_env.env_root().join("repo");
 
     std::fs::write(workspace_path.join("file1"), "foo\n").unwrap();
@@ -100,12 +100,33 @@ fn test_commit_interactive() {
 
     JJ: Lines starting with "JJ: " (like this one) will be removed.
     "###);
+
+    // Try again with --tool=<name>, which implies --interactive
+    test_env.jj_cmd_ok(&workspace_path, &["undo"]);
+    test_env.jj_cmd_ok(
+        &workspace_path,
+        &[
+            "commit",
+            "--config-toml=ui.diff-editor='false'",
+            "--tool=fake-diff-editor",
+        ],
+    );
+
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor")).unwrap(), @r###"
+    add files
+
+    JJ: This commit contains the following changes:
+    JJ:     A file1
+
+    JJ: Lines starting with "JJ: " (like this one) will be removed.
+    "###);
 }
 
 #[test]
 fn test_commit_with_default_description() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     test_env.add_config(r#"ui.default-description = "\n\nTESTED=TODO""#);
     let workspace_path = test_env.env_root().join("repo");
 
@@ -137,7 +158,7 @@ JJ: Lines starting with "JJ: " (like this one) will be removed.
 #[test]
 fn test_commit_without_working_copy() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let workspace_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(&workspace_path, &["workspace", "forget"]);
@@ -150,7 +171,7 @@ fn test_commit_without_working_copy() {
 #[test]
 fn test_commit_paths() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let workspace_path = test_env.env_root().join("repo");
 
     std::fs::write(workspace_path.join("file1"), "foo\n").unwrap();
@@ -173,7 +194,7 @@ fn test_commit_paths() {
 #[test]
 fn test_commit_paths_warning() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let workspace_path = test_env.env_root().join("repo");
 
     std::fs::write(workspace_path.join("file1"), "foo\n").unwrap();
@@ -181,7 +202,7 @@ fn test_commit_paths_warning() {
 
     let (stdout, stderr) = test_env.jj_cmd_ok(&workspace_path, &["commit", "-m=first", "file3"]);
     insta::assert_snapshot!(stderr, @r###"
-    The given paths do not match any file: file3
+    Warning: The given paths do not match any file: file3
     Working copy now at: rlvkpnrz 67872820 (no description set)
     Parent commit      : qpvuntsm 69542c19 (empty) first
     "###);
