@@ -33,7 +33,7 @@ fn test_config_list_single() {
         &["config", "list", "test-table.somekey"],
     );
     insta::assert_snapshot!(stdout, @r###"
-    test-table.somekey="some value"
+    test-table.somekey = "some value"
     "###);
 
     let stdout = test_env.jj_cmd_success(
@@ -67,15 +67,17 @@ fn test_config_list_table() {
     x = true
     y.foo = "abc"
     y.bar = 123
+    "z"."with space"."function()" = 5
     "#,
     );
     let stdout = test_env.jj_cmd_success(test_env.env_root(), &["config", "list", "test-table"]);
     insta::assert_snapshot!(
         stdout,
         @r###"
-    test-table.x=true
-    test-table.y.bar=123
-    test-table.y.foo="abc"
+    test-table.x = true
+    test-table.y.bar = 123
+    test-table.y.foo = "abc"
+    test-table.z."with space"."function()" = 5
     "###);
 }
 
@@ -89,7 +91,7 @@ fn test_config_list_array() {
     );
     let stdout = test_env.jj_cmd_success(test_env.env_root(), &["config", "list", "test-array"]);
     insta::assert_snapshot!(stdout, @r###"
-    test-array=[1, "b", 3.4]
+    test-array = [1, "b", 3.4]
     "###);
 }
 
@@ -102,11 +104,12 @@ fn test_config_list_inline_table() {
         x = 1
         [[test-table]]
         y = ["z"]
+        z."key=with whitespace" = []
     "#,
     );
     let stdout = test_env.jj_cmd_success(test_env.env_root(), &["config", "list", "test-table"]);
     insta::assert_snapshot!(stdout, @r###"
-    test-table=[{x=1}, {y=["z"]}]
+    test-table = [{ x = 1 }, { y = ["z"], z = { "key=with whitespace" = [] } }]
     "###);
 }
 
@@ -127,17 +130,56 @@ fn test_config_list_all() {
     insta::assert_snapshot!(
         find_stdout_lines(r"(test-val|test-table\b[^=]*)", &stdout),
         @r###"
-    test-table.x=true
-    test-table.y.bar=123
-    test-table.y.foo="abc"
-    test-val=[1, 2, 3]
+    test-table.x = true
+    test-table.y.bar = 123
+    test-table.y.foo = "abc"
+    test-val = [1, 2, 3]
+    "###);
+}
+
+#[test]
+fn test_config_list_multiline_string() {
+    let test_env = TestEnvironment::default();
+    test_env.add_config(
+        r#"
+    multiline = '''
+foo
+bar
+'''
+    "#,
+    );
+
+    let stdout = test_env.jj_cmd_success(test_env.env_root(), &["config", "list", "multiline"]);
+    insta::assert_snapshot!(stdout, @r###"
+    multiline = """
+    foo
+    bar
+    """
+    "###);
+
+    let stdout = test_env.jj_cmd_success(
+        test_env.env_root(),
+        &[
+            "config",
+            "list",
+            "multiline",
+            "--include-overridden",
+            "--config-toml=multiline='single'",
+        ],
+    );
+    insta::assert_snapshot!(stdout, @r###"
+    # multiline = """
+    # foo
+    # bar
+    # """
+    multiline = "single"
     "###);
 }
 
 #[test]
 fn test_config_list_layer() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let user_config_path = test_env.config_path().join("config.toml");
     test_env.set_config_path(user_config_path.to_owned());
     let repo_path = test_env.env_root().join("repo");
@@ -161,8 +203,8 @@ fn test_config_list_layer() {
 
     let stdout = test_env.jj_cmd_success(&repo_path, &["config", "list", "--user"]);
     insta::assert_snapshot!(stdout, @r###"
-    test-key="test-val"
-    test-layered-key="test-original-val"
+    test-key = "test-val"
+    test-layered-key = "test-original-val"
     "###);
 
     // Repo
@@ -179,19 +221,19 @@ fn test_config_list_layer() {
 
     let stdout = test_env.jj_cmd_success(&repo_path, &["config", "list", "--user"]);
     insta::assert_snapshot!(stdout, @r###"
-    test-key="test-val"
+    test-key = "test-val"
     "###);
 
     let stdout = test_env.jj_cmd_success(&repo_path, &["config", "list", "--repo"]);
     insta::assert_snapshot!(stdout, @r###"
-    test-layered-key="test-layered-val"
+    test-layered-key = "test-layered-val"
     "###);
 }
 
 #[test]
 fn test_config_layer_override_default() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let config_key = "merge-tools.vimdiff.program";
 
@@ -201,14 +243,14 @@ fn test_config_layer_override_default() {
         &["config", "list", config_key, "--include-defaults"],
     );
     insta::assert_snapshot!(stdout, @r###"
-    merge-tools.vimdiff.program="vim"
+    merge-tools.vimdiff.program = "vim"
     "###);
 
     // User
     test_env.add_config(&format!("{config_key} = {value:?}\n", value = "user"));
     let stdout = test_env.jj_cmd_success(&repo_path, &["config", "list", config_key]);
     insta::assert_snapshot!(stdout, @r###"
-    merge-tools.vimdiff.program="user"
+    merge-tools.vimdiff.program = "user"
     "###);
 
     // Repo
@@ -219,7 +261,7 @@ fn test_config_layer_override_default() {
     .unwrap();
     let stdout = test_env.jj_cmd_success(&repo_path, &["config", "list", config_key]);
     insta::assert_snapshot!(stdout, @r###"
-    merge-tools.vimdiff.program="repo"
+    merge-tools.vimdiff.program = "repo"
     "###);
 
     // Command argument
@@ -234,7 +276,7 @@ fn test_config_layer_override_default() {
         ],
     );
     insta::assert_snapshot!(stdout, @r###"
-    merge-tools.vimdiff.program="command-arg"
+    merge-tools.vimdiff.program = "command-arg"
     "###);
 
     // Allow printing overridden values
@@ -250,9 +292,9 @@ fn test_config_layer_override_default() {
         ],
     );
     insta::assert_snapshot!(stdout, @r###"
-    # merge-tools.vimdiff.program="user"
-    # merge-tools.vimdiff.program="repo"
-    merge-tools.vimdiff.program="command-arg"
+    # merge-tools.vimdiff.program = "user"
+    # merge-tools.vimdiff.program = "repo"
+    merge-tools.vimdiff.program = "command-arg"
     "###);
 
     let stdout = test_env.jj_cmd_success(
@@ -266,15 +308,15 @@ fn test_config_layer_override_default() {
         ],
     );
     insta::assert_snapshot!(stdout, @r###"
-    [38;5;8m# merge-tools.vimdiff.program="user"[39m
-    [38;5;2mmerge-tools.vimdiff.program[39m=[38;5;3m"repo"[39m
+    [38;5;8m# merge-tools.vimdiff.program = "user"[39m
+    [38;5;2mmerge-tools.vimdiff.program[39m = [38;5;3m"repo"[39m
     "###);
 }
 
 #[test]
 fn test_config_layer_override_env() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let config_key = "ui.editor";
 
@@ -282,14 +324,14 @@ fn test_config_layer_override_env() {
     test_env.add_env_var("EDITOR", "env-base");
     let stdout = test_env.jj_cmd_success(&repo_path, &["config", "list", config_key]);
     insta::assert_snapshot!(stdout, @r###"
-    ui.editor="env-base"
+    ui.editor = "env-base"
     "###);
 
     // User
     test_env.add_config(&format!("{config_key} = {value:?}\n", value = "user"));
     let stdout = test_env.jj_cmd_success(&repo_path, &["config", "list", config_key]);
     insta::assert_snapshot!(stdout, @r###"
-    ui.editor="user"
+    ui.editor = "user"
     "###);
 
     // Repo
@@ -300,14 +342,14 @@ fn test_config_layer_override_env() {
     .unwrap();
     let stdout = test_env.jj_cmd_success(&repo_path, &["config", "list", config_key]);
     insta::assert_snapshot!(stdout, @r###"
-    ui.editor="repo"
+    ui.editor = "repo"
     "###);
 
     // Environment override
     test_env.add_env_var("JJ_EDITOR", "env-override");
     let stdout = test_env.jj_cmd_success(&repo_path, &["config", "list", config_key]);
     insta::assert_snapshot!(stdout, @r###"
-    ui.editor="env-override"
+    ui.editor = "env-override"
     "###);
 
     // Command argument
@@ -322,7 +364,7 @@ fn test_config_layer_override_env() {
         ],
     );
     insta::assert_snapshot!(stdout, @r###"
-    ui.editor="command-arg"
+    ui.editor = "command-arg"
     "###);
 
     // Allow printing overridden values
@@ -338,18 +380,18 @@ fn test_config_layer_override_env() {
         ],
     );
     insta::assert_snapshot!(stdout, @r###"
-    # ui.editor="env-base"
-    # ui.editor="user"
-    # ui.editor="repo"
-    # ui.editor="env-override"
-    ui.editor="command-arg"
+    # ui.editor = "env-base"
+    # ui.editor = "user"
+    # ui.editor = "repo"
+    # ui.editor = "env-override"
+    ui.editor = "command-arg"
     "###);
 }
 
 #[test]
 fn test_config_layer_workspace() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "--git", "main"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "main"]);
     let main_path = test_env.env_root().join("main");
     let secondary_path = test_env.env_root().join("secondary");
     let config_key = "ui.editor";
@@ -369,16 +411,16 @@ fn test_config_layer_workspace() {
     .unwrap();
     let stdout = test_env.jj_cmd_success(&main_path, &["config", "list", config_key]);
     insta::assert_snapshot!(stdout, @r###"
-    ui.editor="main-repo"
+    ui.editor = "main-repo"
     "###);
     let stdout = test_env.jj_cmd_success(&secondary_path, &["config", "list", config_key]);
     insta::assert_snapshot!(stdout, @r###"
-    ui.editor="main-repo"
+    ui.editor = "main-repo"
     "###);
 }
 
 #[test]
-fn test_config_set_missing_opts() {
+fn test_config_set_bad_opts() {
     let test_env = TestEnvironment::default();
     let stderr = test_env.jj_cmd_cli_error(test_env.env_root(), &["config", "set"]);
     insta::assert_snapshot!(stderr, @r###"
@@ -389,6 +431,19 @@ fn test_config_set_missing_opts() {
 
     Usage: jj config set <--user|--repo> <NAME> <VALUE>
 
+    For more information, try '--help'.
+    "###);
+
+    let stderr =
+        test_env.jj_cmd_cli_error(test_env.env_root(), &["config", "set", "--user", "", "x"]);
+    insta::assert_snapshot!(stderr, @r###"
+    error: invalid value '' for '<NAME>': TOML parse error at line 1, column 1
+      |
+    1 | 
+      | ^
+    invalid key
+
+
     For more information, try '--help'.
     "###);
 }
@@ -396,7 +451,7 @@ fn test_config_set_missing_opts() {
 #[test]
 fn test_config_set_for_user() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     // Point to a config file since `config set` can't handle directories.
     let user_config_path = test_env.config_path().join("config.toml");
     test_env.set_config_path(user_config_path.to_owned());
@@ -410,6 +465,10 @@ fn test_config_set_for_user() {
         &repo_path,
         &["config", "set", "--user", "test-table.foo", "true"],
     );
+    test_env.jj_cmd_ok(
+        &repo_path,
+        &["config", "set", "--user", "test-table.'bar()'", "0"],
+    );
 
     // Ensure test-key successfully written to user config.
     let user_config_toml = std::fs::read_to_string(&user_config_path)
@@ -419,13 +478,14 @@ fn test_config_set_for_user() {
 
     [test-table]
     foo = true
+    "bar()" = 0
     "###);
 }
 
 #[test]
 fn test_config_set_for_repo() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     test_env.jj_cmd_ok(
         &repo_path,
@@ -455,7 +515,7 @@ fn test_config_set_for_repo() {
 #[test]
 fn test_config_set_toml_types() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let user_config_path = test_env.config_path().join("config.toml");
     test_env.set_config_path(user_config_path.clone());
     let repo_path = test_env.env_root().join("repo");
@@ -483,7 +543,7 @@ fn test_config_set_toml_types() {
 #[test]
 fn test_config_set_type_mismatch() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let user_config_path = test_env.config_path().join("config.toml");
     test_env.set_config_path(user_config_path);
     let repo_path = test_env.env_root().join("repo");
@@ -522,7 +582,7 @@ fn test_config_set_type_mismatch() {
 #[test]
 fn test_config_set_nontable_parent() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let user_config_path = test_env.config_path().join("config.toml");
     test_env.set_config_path(user_config_path);
     let repo_path = test_env.env_root().join("repo");
@@ -557,7 +617,7 @@ fn test_config_edit_missing_opt() {
 #[test]
 fn test_config_edit_user() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let edit_script = test_env.set_up_fake_editor();
 
@@ -572,7 +632,7 @@ fn test_config_edit_user() {
 #[test]
 fn test_config_edit_repo() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let edit_script = test_env.set_up_fake_editor();
 
@@ -590,7 +650,7 @@ fn test_config_edit_repo() {
 #[test]
 fn test_config_path() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     assert_snapshot!(
@@ -668,8 +728,113 @@ fn test_config_get() {
     insta::assert_snapshot!(stdout, @"bar");
 }
 
+#[test]
+fn test_config_path_syntax() {
+    let test_env = TestEnvironment::default();
+    test_env.add_config(
+        r#"
+    a.'b()' = 0
+    'b c'.d = 1
+    'b c'.e.'f[]' = 2
+    - = 3
+    _ = 4
+    '.' = 5
+    "#,
+    );
+
+    let stdout = test_env.jj_cmd_success(test_env.env_root(), &["config", "list", "a.'b()'"]);
+    insta::assert_snapshot!(stdout, @r###"
+    a.'b()' = 0
+    "###);
+    let stdout = test_env.jj_cmd_success(test_env.env_root(), &["config", "list", "'b c'"]);
+    insta::assert_snapshot!(stdout, @r###"
+    'b c'.d = 1
+    'b c'.e."f[]" = 2
+    "###);
+    let stdout = test_env.jj_cmd_success(test_env.env_root(), &["config", "list", "'b c'.d"]);
+    insta::assert_snapshot!(stdout, @r###"
+    'b c'.d = 1
+    "###);
+    let stdout = test_env.jj_cmd_success(test_env.env_root(), &["config", "list", "'b c'.e.'f[]'"]);
+    insta::assert_snapshot!(stdout, @r###"
+    'b c'.e.'f[]' = 2
+    "###);
+    let stdout = test_env.jj_cmd_success(test_env.env_root(), &["config", "get", "'b c'.e.'f[]'"]);
+    insta::assert_snapshot!(stdout, @r###"
+    2
+    "###);
+
+    // Not a table
+    let (stdout, stderr) =
+        test_env.jj_cmd_ok(test_env.env_root(), &["config", "list", "a.'b()'.x"]);
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Warning: No matching config key for a.'b()'.x
+    "###);
+    let stderr = test_env.jj_cmd_failure(test_env.env_root(), &["config", "get", "a.'b()'.x"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Config error: configuration property "a.'b()'.x" not found
+    For help, see https://github.com/martinvonz/jj/blob/main/docs/config.md.
+    "###);
+
+    // "-" and "_" are valid TOML keys
+    let stdout = test_env.jj_cmd_success(test_env.env_root(), &["config", "list", "-"]);
+    insta::assert_snapshot!(stdout, @r###"
+    - = 3
+    "###);
+    let stdout = test_env.jj_cmd_success(test_env.env_root(), &["config", "list", "_"]);
+    insta::assert_snapshot!(stdout, @r###"
+    _ = 4
+    "###);
+
+    // "." requires quoting
+    let stdout = test_env.jj_cmd_success(test_env.env_root(), &["config", "list", "'.'"]);
+    insta::assert_snapshot!(stdout, @r###"
+    '.' = 5
+    "###);
+    let stdout = test_env.jj_cmd_success(test_env.env_root(), &["config", "get", "'.'"]);
+    insta::assert_snapshot!(stdout, @r###"
+    5
+    "###);
+    let stderr = test_env.jj_cmd_cli_error(test_env.env_root(), &["config", "get", "."]);
+    insta::assert_snapshot!(stderr, @r###"
+    error: invalid value '.' for '<NAME>': TOML parse error at line 1, column 1
+      |
+    1 | .
+      | ^
+    invalid key
+
+
+    For more information, try '--help'.
+    "###);
+
+    // Invalid TOML keys
+    let stderr = test_env.jj_cmd_cli_error(test_env.env_root(), &["config", "list", "b c"]);
+    insta::assert_snapshot!(stderr, @r###"
+    error: invalid value 'b c' for '[NAME]': TOML parse error at line 1, column 3
+      |
+    1 | b c
+      |   ^
+
+
+
+    For more information, try '--help'.
+    "###);
+    let stderr = test_env.jj_cmd_cli_error(test_env.env_root(), &["config", "list", ""]);
+    insta::assert_snapshot!(stderr, @r###"
+    error: invalid value '' for '[NAME]': TOML parse error at line 1, column 1
+      |
+    1 | 
+      | ^
+    invalid key
+
+
+    For more information, try '--help'.
+    "###);
+}
+
 fn find_stdout_lines(keyname_pattern: &str, stdout: &str) -> String {
-    let key_line_re = Regex::new(&format!(r"(?m)^{keyname_pattern}=.*$")).unwrap();
+    let key_line_re = Regex::new(&format!(r"(?m)^{keyname_pattern} = .*$")).unwrap();
     key_line_re
         .find_iter(stdout)
         .map(|m| m.as_str())
