@@ -12,15 +12,14 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use common::TestEnvironment;
 use itertools::Itertools;
 
-pub mod common;
+use crate::common::{escaped_fake_diff_editor_path, strip_last_line, TestEnvironment};
 
 #[test]
 fn test_diff_basic() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "foo\n").unwrap();
@@ -41,9 +40,20 @@ fn test_diff_basic() {
             1: foo
     "###);
 
+    let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "--context=0"]);
+    insta::assert_snapshot!(stdout, @r###"
+    Removed regular file file1:
+       1     : foo
+    Modified regular file file2:
+       1    1: foo
+            2: bar
+    Added regular file file3:
+            1: foo
+    "###);
+
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     M file2
     A file3
     "###);
@@ -80,9 +90,33 @@ fn test_diff_basic() {
     +foo
     "###);
 
+    let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "--git", "--context=0"]);
+    insta::assert_snapshot!(stdout, @r###"
+    diff --git a/file1 b/file1
+    deleted file mode 100644
+    index 257cc5642c..0000000000
+    --- a/file1
+    +++ /dev/null
+    @@ -1,1 +1,0 @@
+    -foo
+    diff --git a/file2 b/file2
+    index 257cc5642c...3bd1f0e297 100644
+    --- a/file2
+    +++ b/file2
+    @@ -2,0 +2,1 @@
+    +bar
+    diff --git a/file3 b/file3
+    new file mode 100644
+    index 0000000000..257cc5642c
+    --- /dev/null
+    +++ b/file3
+    @@ -1,0 +1,1 @@
+    +foo
+    "###);
+
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s", "--git"]);
     insta::assert_snapshot!(stdout, @r###"
-    R file1
+    D file1
     M file2
     A file3
     diff --git a/file1 b/file1
@@ -115,12 +149,54 @@ fn test_diff_basic() {
     file3 | 1 +
     3 files changed, 2 insertions(+), 1 deletion(-)
     "###);
+
+    // Filter by glob pattern
+    let stdout = test_env.jj_cmd_success(
+        &repo_path,
+        &[
+            "diff",
+            "--config-toml=ui.allow-filesets=true",
+            "-s",
+            r#"glob:"file[12]""#,
+        ],
+    );
+    insta::assert_snapshot!(stdout, @r###"
+    D file1
+    M file2
+    "###);
+
+    // Unmatched paths should generate warnings
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        test_env.env_root(),
+        &[
+            "diff",
+            "-Rrepo",
+            "-s",
+            "repo",       // matches directory
+            "repo/file1", // deleted in to_tree, but exists in from_tree
+            "repo/x",
+            "repo/y/z",
+        ],
+    );
+    insta::assert_snapshot!(stdout.replace('\\', "/"), @r###"
+    D repo/file1
+    M repo/file2
+    A repo/file3
+    "###);
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    Warning: No matching entries for paths: repo/x, repo/y/z
+    "###);
+
+    // Unmodified paths shouldn't generate warnings
+    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["diff", "-s", "--from=@", "file2"]);
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @"");
 }
 
 #[test]
 fn test_diff_empty() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "").unwrap();
@@ -148,7 +224,7 @@ fn test_diff_empty() {
 #[test]
 fn test_diff_types() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let file_path = repo_path.join("foo");
@@ -165,7 +241,7 @@ fn test_diff_types() {
     std::fs::write(&file_path, "foo").unwrap();
     test_env.jj_cmd_ok(&repo_path, &["new", "root()"]);
     std::fs::write(&file_path, "bar").unwrap();
-    test_env.jj_cmd_ok(&repo_path, &["move", r#"--to=description("conflict")"#]);
+    test_env.jj_cmd_ok(&repo_path, &["squash", r#"--into=description("conflict")"#]);
 
     #[cfg(unix)]
     {
@@ -217,7 +293,7 @@ fn test_diff_types() {
 #[test]
 fn test_diff_bad_args() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let stderr = test_env.jj_cmd_cli_error(&repo_path, &["diff", "-s", "--types"]);
@@ -242,7 +318,7 @@ fn test_diff_bad_args() {
 #[test]
 fn test_diff_relative_paths() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::create_dir_all(repo_path.join("dir1").join("subdir1")).unwrap();
@@ -375,7 +451,7 @@ fn test_diff_relative_paths() {
 #[test]
 fn test_diff_missing_newline() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "foo").unwrap();
@@ -429,7 +505,7 @@ fn test_diff_missing_newline() {
 #[test]
 fn test_color_words_diff_missing_newline() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "").unwrap();
@@ -530,7 +606,7 @@ fn test_color_words_diff_missing_newline() {
 #[test]
 fn test_diff_skipped_context() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "a\nb\nc\nd\ne\nf\ng\nh\ni\nj").unwrap();
@@ -638,10 +714,78 @@ fn test_diff_skipped_context() {
     "###);
 }
 
+#[test]
+fn test_diff_skipped_context_nondefault() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    std::fs::write(repo_path.join("file1"), "a\nb\nc\nd").unwrap();
+    test_env.jj_cmd_ok(&repo_path, &["describe", "-m", "=== Left side of diffs"]);
+
+    test_env.jj_cmd_ok(&repo_path, &["new", "@", "-m", "=== Must skip 2 lines"]);
+    std::fs::write(repo_path.join("file1"), "A\nb\nc\nD").unwrap();
+    test_env.jj_cmd_ok(&repo_path, &["new", "@-", "-m", "=== Don't skip 1 line"]);
+    std::fs::write(repo_path.join("file1"), "A\nb\nC\nd").unwrap();
+    test_env.jj_cmd_ok(&repo_path, &["new", "@-", "-m", "=== No gap to skip"]);
+    std::fs::write(repo_path.join("file1"), "a\nB\nC\nd").unwrap();
+    test_env.jj_cmd_ok(&repo_path, &["new", "@-", "-m", "=== 1 line at start"]);
+    std::fs::write(repo_path.join("file1"), "a\nB\nc\nd").unwrap();
+    test_env.jj_cmd_ok(&repo_path, &["new", "@-", "-m", "=== 1 line at end"]);
+    std::fs::write(repo_path.join("file1"), "a\nb\nC\nd").unwrap();
+
+    let stdout = test_env.jj_cmd_success(
+        &repo_path,
+        &[
+            "log",
+            "-Tdescription",
+            "-p",
+            "--no-graph",
+            "--reversed",
+            "--context=0",
+        ],
+    );
+    insta::assert_snapshot!(stdout, @r###"
+    === Left side of diffs
+    Added regular file file1:
+            1: a
+            2: b
+            3: c
+            4: d
+    === Must skip 2 lines
+    Modified regular file file1:
+       1    1: aA
+        ...
+       4    4: dD
+    === Don't skip 1 line
+    Modified regular file file1:
+       1    1: aA
+       2    2: b
+       3    3: cC
+       4    4: d
+    === No gap to skip
+    Modified regular file file1:
+       1    1: a
+       2    2: bB
+       3    3: cC
+       4    4: d
+    === 1 line at start
+    Modified regular file file1:
+       1    1: a
+       2    2: bB
+        ...
+    === 1 line at end
+    Modified regular file file1:
+        ...
+       3    3: cC
+       4    4: d
+    "###);
+}
+
 #[test]
 fn test_diff_external_tool() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "foo\n").unwrap();
@@ -677,14 +821,14 @@ fn test_diff_external_tool() {
 
     insta::assert_snapshot!(
         test_env.jj_cmd_success(&repo_path, &["log", "-p", "--tool=fake-diff-editor"]), @r###"
-    @  rlvkpnrz test.user@example.com 2001-02-03 04:05:09.000 +07:00 0cba70c7
+    @  rlvkpnrz test.user@example.com 2001-02-03 08:05:09 0cba70c7
     │  (no description set)
     │  file1
     │  file2
     │  --
     │  file2
     │  file3
-    ◉  qpvuntsm test.user@example.com 2001-02-03 04:05:08.000 +07:00 39b5a56f
+    ◉  qpvuntsm test.user@example.com 2001-02-03 08:05:08 39b5a56f
     │  (no description set)
     │  --
     │  file1
@@ -697,8 +841,8 @@ fn test_diff_external_tool() {
         test_env.jj_cmd_success(&repo_path, &["show", "--tool=fake-diff-editor"]), @r###"
     Commit ID: 0cba70c72186eabb5a2f91be63a8366b9f6da6c6
     Change ID: rlvkpnrzqnoowoytxnquwvuryrwnrmlp
-    Author: Test User <test.user@example.com> (2001-02-03 04:05:08.000 +07:00)
-    Committer: Test User <test.user@example.com> (2001-02-03 04:05:09.000 +07:00)
+    Author: Test User <test.user@example.com> (2001-02-03 08:05:08)
+    Committer: Test User <test.user@example.com> (2001-02-03 08:05:09)
 
         (no description set)
 
@@ -720,7 +864,7 @@ fn test_diff_external_tool() {
     "###);
 
     // Inlined command arguments
-    let command = common::escaped_fake_diff_editor_path();
+    let command = escaped_fake_diff_editor_path();
     let config = format!(r#"--config-toml=ui.diff.tool=["{command}", "$right", "$left"]"#);
     insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff", &config]), @r###"
     file2
@@ -744,15 +888,23 @@ fn test_diff_external_tool() {
     insta::assert_snapshot!(stdout, @r###"
     Commit ID: 0cba70c72186eabb5a2f91be63a8366b9f6da6c6
     Change ID: rlvkpnrzqnoowoytxnquwvuryrwnrmlp
-    Author: Test User <test.user@example.com> (2001-02-03 04:05:08.000 +07:00)
-    Committer: Test User <test.user@example.com> (2001-02-03 04:05:09.000 +07:00)
+    Author: Test User <test.user@example.com> (2001-02-03 08:05:08)
+    Committer: Test User <test.user@example.com> (2001-02-03 08:05:09)
 
         (no description set)
 
     diff
     "###);
-    insta::assert_snapshot!(stderr, @r###"
-    Tool exited with a non-zero code (run with --verbose to see the exact invocation). Exit code: 1.
+    insta::assert_snapshot!(stderr.replace("exit code:", "exit status:"), @r###"
+    Warning: Tool exited with exit status: 1 (run with --debug to see the exact invocation)
+    "###);
+
+    // --tool=:builtin shouldn't be ignored
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["diff", "--tool=:builtin"]);
+    insta::assert_snapshot!(strip_last_line(&stderr), @r###"
+    Error: Failed to generate diff
+    Caused by:
+    1: Error executing ':builtin' (run with --debug to see the exact invocation)
     "###);
 }
 
@@ -760,7 +912,7 @@ fn test_diff_external_tool() {
 #[test]
 fn test_diff_external_tool_symlink() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let external_file_path = test_env.env_root().join("external-file");
@@ -802,7 +954,7 @@ fn test_diff_external_tool_symlink() {
 #[test]
 fn test_diff_stat() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     std::fs::write(repo_path.join("file1"), "foo\n").unwrap();
 
@@ -832,7 +984,7 @@ fn test_diff_stat() {
 fn test_diff_stat_long_name_or_stat() {
     let mut test_env = TestEnvironment::default();
     test_env.add_env_var("COLUMNS", "30");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let get_stat = |test_env: &TestEnvironment, path_length: usize, stat_size: usize| {
@@ -941,3 +1093,40 @@ fn test_diff_stat_long_name_or_stat() {
     2 files changed, 20 insertions(+), 0 deletions(-)
     "###);
 }
+
+#[test]
+fn test_diff_binary() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    std::fs::write(repo_path.join("file1.png"), b"\x89PNG\r\n\x1a\nabcdefg\0").unwrap();
+    std::fs::write(repo_path.join("file2.png"), b"\x89PNG\r\n\x1a\n0123456\0").unwrap();
+    test_env.jj_cmd_ok(&repo_path, &["new"]);
+    std::fs::remove_file(repo_path.join("file1.png")).unwrap();
+    std::fs::write(repo_path.join("file2.png"), "foo\nbar\n").unwrap();
+    std::fs::write(repo_path.join("file3.png"), b"\x89PNG\r\n\x1a\nxyz\0").unwrap();
+    // try a file that's valid UTF-8 but contains control characters
+    std::fs::write(repo_path.join("file4.png"), b"\0\0\0").unwrap();
+
+    let stdout = test_env.jj_cmd_success(&repo_path, &["diff"]);
+    insta::assert_snapshot!(stdout, @r###"
+    Removed regular file file1.png:
+        (binary)
+    Modified regular file file2.png:
+        (binary)
+    Added regular file file3.png:
+        (binary)
+    Added regular file file4.png:
+        (binary)
+    "###);
+
+    let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "--stat"]);
+    insta::assert_snapshot!(stdout, @r###"
+    file1.png | 3 ---
+    file2.png | 5 ++---
+    file3.png | 3 +++
+    file4.png | 1 +
+    4 files changed, 6 insertions(+), 6 deletions(-)
+    "###);
+}
