@@ -17,7 +17,7 @@ use crate::common::{escaped_fake_diff_editor_path, TestEnvironment};
 #[test]
 fn test_diffedit() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "a\n").unwrap();
@@ -47,11 +47,52 @@ fn test_diffedit() {
     M file2
     "###);
 
+    // Try again with ui.diff-instructions=false
+    std::fs::write(&edit_script, "files-before file1 file2\0files-after file2").unwrap();
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &repo_path,
+        &["diffedit", "--config-toml=ui.diff-instructions=false"],
+    );
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Nothing changed.
+    "###);
+    let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
+    insta::assert_snapshot!(stdout, @r###"
+    D file1
+    M file2
+    "###);
+
+    // Try again with --tool=<name>
+    std::fs::write(
+        &edit_script,
+        "files-before file1 file2\0files-after JJ-INSTRUCTIONS file2",
+    )
+    .unwrap();
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &repo_path,
+        &[
+            "diffedit",
+            "--config-toml=ui.diff-editor='false'",
+            "--tool=fake-diff-editor",
+        ],
+    );
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Nothing changed.
+    "###);
+    let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
+    insta::assert_snapshot!(stdout, @r###"
+    D file1
+    M file2
+    "###);
+
     // Nothing happens if the diff-editor exits with an error
     std::fs::write(&edit_script, "rm file2\0fail").unwrap();
-    insta::assert_snapshot!(&test_env.jj_cmd_failure(&repo_path, &["diffedit"]), @r###"
+    let stderr = &test_env.jj_cmd_failure(&repo_path, &["diffedit"]);
+    insta::assert_snapshot!(stderr.replace("exit code:", "exit status:"), @r###"
     Error: Failed to edit diff
-    Caused by: Tool exited with a non-zero code (run with --verbose to see the exact invocation). Exit code: 1.
+    Caused by: Tool exited with exit status: 1 (run with --debug to see the exact invocation)
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
@@ -64,8 +105,8 @@ fn test_diffedit() {
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["diffedit"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
-    Created kkmpptxz 1930da4a (no description set)
-    Working copy now at: kkmpptxz 1930da4a (no description set)
+    Created kkmpptxz cc387f43 (no description set)
+    Working copy now at: kkmpptxz cc387f43 (no description set)
     Parent commit      : rlvkpnrz 613028a4 (no description set)
     Added 0 files, modified 1 files, removed 0 files
     "###);
@@ -80,10 +121,10 @@ fn test_diffedit() {
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["diffedit", "-r", "@-"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
-    Created rlvkpnrz c03ae967 (no description set)
+    Created rlvkpnrz d842b979 (no description set)
     Rebased 1 descendant commits
-    Working copy now at: kkmpptxz 2a4dc204 (no description set)
-    Parent commit      : rlvkpnrz c03ae967 (no description set)
+    Working copy now at: kkmpptxz bc2b2dd6 (no description set)
+    Parent commit      : rlvkpnrz d842b979 (no description set)
     Added 0 files, modified 1 files, removed 0 files
     "###);
     let contents = String::from_utf8(std::fs::read(repo_path.join("file3")).unwrap()).unwrap();
@@ -101,8 +142,8 @@ fn test_diffedit() {
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["diffedit", "--from", "@--"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
-    Created kkmpptxz 15f2c966 (no description set)
-    Working copy now at: kkmpptxz 15f2c966 (no description set)
+    Created kkmpptxz d78a207f (no description set)
+    Working copy now at: kkmpptxz d78a207f (no description set)
     Parent commit      : rlvkpnrz 613028a4 (no description set)
     Added 0 files, modified 0 files, removed 1 files
     "###);
@@ -116,7 +157,7 @@ fn test_diffedit() {
 #[test]
 fn test_diffedit_new_file() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "a\n").unwrap();
@@ -182,7 +223,7 @@ fn test_diffedit_new_file() {
 #[test]
 fn test_diffedit_3pane() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "a\n").unwrap();
@@ -303,7 +344,7 @@ fn test_diffedit_3pane() {
 #[test]
 fn test_diffedit_merge() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "a\n").unwrap();
@@ -312,7 +353,7 @@ fn test_diffedit_merge() {
     test_env.jj_cmd_ok(&repo_path, &["branch", "create", "b"]);
     std::fs::write(repo_path.join("file1"), "b\n").unwrap();
     std::fs::write(repo_path.join("file2"), "b\n").unwrap();
-    test_env.jj_cmd_ok(&repo_path, &["co", "@-"]);
+    test_env.jj_cmd_ok(&repo_path, &["new", "@-"]);
     test_env.jj_cmd_ok(&repo_path, &["new"]);
     std::fs::write(repo_path.join("file1"), "c\n").unwrap();
     std::fs::write(repo_path.join("file2"), "c\n").unwrap();
@@ -339,11 +380,13 @@ fn test_diffedit_merge() {
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["diffedit", "-r", "@-"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
-    Created royxmykx 2b5202ae (conflict) merge
+    Created royxmykx b9539d6e (conflict) merge
     Rebased 1 descendant commits
-    Working copy now at: yqosqzyt 23b1fe1b (conflict) (empty) (no description set)
-    Parent commit      : royxmykx 2b5202ae (conflict) merge
+    Working copy now at: yqosqzyt 0a24ed24 (conflict) (empty) (no description set)
+    Parent commit      : royxmykx b9539d6e (conflict) merge
     Added 0 files, modified 0 files, removed 1 files
+    There are unresolved conflicts at these paths:
+    file2    2-sided conflict
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s", "-r", "@-"]);
     insta::assert_snapshot!(stdout, @r###"
@@ -353,20 +396,20 @@ fn test_diffedit_merge() {
     assert!(!repo_path.join("file1").exists());
     let stdout = test_env.jj_cmd_success(&repo_path, &["print", "file2"]);
     insta::assert_snapshot!(stdout, @r###"
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
     -a
     +c
-    +++++++
+    +++++++ Contents of side #2
     b
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     "###);
 }
 
 #[test]
 fn test_diffedit_old_restore_interactive_tests() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "a\n").unwrap();
@@ -393,9 +436,10 @@ fn test_diffedit_old_restore_interactive_tests() {
 
     // Nothing happens if the diff-editor exits with an error
     std::fs::write(&edit_script, "rm file2\0fail").unwrap();
-    insta::assert_snapshot!(&test_env.jj_cmd_failure(&repo_path, &["diffedit", "--from", "@-"]), @r###"
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["diffedit", "--from", "@-"]);
+    insta::assert_snapshot!(stderr.replace("exit code:", "exit status:"), @r###"
     Error: Failed to edit diff
-    Caused by: Tool exited with a non-zero code (run with --verbose to see the exact invocation). Exit code: 1.
+    Caused by: Tool exited with exit status: 1 (run with --debug to see the exact invocation)
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s"]);
     insta::assert_snapshot!(stdout, @r###"
