@@ -55,7 +55,7 @@ fn test_git_colocated() {
     );
 
     // Import the repo
-    test_env.jj_cmd_ok(&workspace_root, &["init", "--git-repo", "."]);
+    test_env.jj_cmd_ok(&workspace_root, &["git", "init", "--git-repo", "."]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  3e9369cd54227eb88455e1834dbc08aad6a16ac4
     ◉  e61b6729ff4292870702f2f72b2a60165679ef37 master HEAD@git initial
@@ -112,7 +112,7 @@ fn test_git_colocated_unborn_branch() {
     };
 
     // Initially, HEAD isn't set.
-    test_env.jj_cmd_ok(&workspace_root, &["init", "--git-repo", "."]);
+    test_env.jj_cmd_ok(&workspace_root, &["git", "init", "--git-repo", "."]);
     assert!(git_repo.head().is_err());
     assert_eq!(
         git_repo.find_reference("HEAD").unwrap().symbolic_target(),
@@ -239,7 +239,7 @@ fn test_git_colocated_export_branches_on_snapshot() {
     let test_env = TestEnvironment::default();
     let workspace_root = test_env.env_root().join("repo");
     let git_repo = git2::Repository::init(&workspace_root).unwrap();
-    test_env.jj_cmd_ok(&workspace_root, &["init", "--git-repo", "."]);
+    test_env.jj_cmd_ok(&workspace_root, &["git", "init", "--git-repo", "."]);
 
     // Create branch pointing to the initial commit
     std::fs::write(workspace_root.join("file"), "initial").unwrap();
@@ -269,7 +269,7 @@ fn test_git_colocated_rebase_on_import() {
     let test_env = TestEnvironment::default();
     let workspace_root = test_env.env_root().join("repo");
     let git_repo = git2::Repository::init(&workspace_root).unwrap();
-    test_env.jj_cmd_ok(&workspace_root, &["init", "--git-repo", "."]);
+    test_env.jj_cmd_ok(&workspace_root, &["git", "init", "--git-repo", "."]);
 
     // Make some changes in jj and check that they're reflected in git
     std::fs::write(workspace_root.join("file"), "contents").unwrap();
@@ -311,7 +311,7 @@ fn test_git_colocated_branches() {
     let test_env = TestEnvironment::default();
     let workspace_root = test_env.env_root().join("repo");
     let git_repo = git2::Repository::init(&workspace_root).unwrap();
-    test_env.jj_cmd_ok(&workspace_root, &["init", "--git-repo", "."]);
+    test_env.jj_cmd_ok(&workspace_root, &["git", "init", "--git-repo", "."]);
     test_env.jj_cmd_ok(&workspace_root, &["new", "-m", "foo"]);
     test_env.jj_cmd_ok(&workspace_root, &["new", "@-", "-m", "bar"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
@@ -368,7 +368,7 @@ fn test_git_colocated_branch_forget() {
     let test_env = TestEnvironment::default();
     let workspace_root = test_env.env_root().join("repo");
     let _git_repo = git2::Repository::init(&workspace_root).unwrap();
-    test_env.jj_cmd_ok(&workspace_root, &["init", "--git-repo", "."]);
+    test_env.jj_cmd_ok(&workspace_root, &["git", "init", "--git-repo", "."]);
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
     test_env.jj_cmd_ok(&workspace_root, &["branch", "create", "foo"]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
@@ -376,8 +376,7 @@ fn test_git_colocated_branch_forget() {
     ◉  230dd059e1b059aefc0da06a2e5a7dbf22362f22 HEAD@git
     ◉  0000000000000000000000000000000000000000
     "###);
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["branch", "list", "--all-remotes"]);
-    insta::assert_snapshot!(stdout, @r###"
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
     foo: rlvkpnrz 65b6b74e (empty) (no description set)
       @git: rlvkpnrz 65b6b74e (empty) (no description set)
     "###);
@@ -387,8 +386,7 @@ fn test_git_colocated_branch_forget() {
     insta::assert_snapshot!(stderr, @"");
     // A forgotten branch is deleted in the git repo. For a detailed demo explaining
     // this, see `test_branch_forget_export` in `test_branch_command.rs`.
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["branch", "list", "--all-remotes"]);
-    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @"");
 }
 
 #[test]
@@ -396,14 +394,14 @@ fn test_git_colocated_conflicting_git_refs() {
     let test_env = TestEnvironment::default();
     let workspace_root = test_env.env_root().join("repo");
     git2::Repository::init(&workspace_root).unwrap();
-    test_env.jj_cmd_ok(&workspace_root, &["init", "--git-repo", "."]);
+    test_env.jj_cmd_ok(&workspace_root, &["git", "init", "--git-repo", "."]);
     test_env.jj_cmd_ok(&workspace_root, &["branch", "create", "main"]);
     let (stdout, stderr) = test_env.jj_cmd_ok(&workspace_root, &["branch", "create", "main/sub"]);
     insta::assert_snapshot!(stdout, @"");
-    insta::with_settings!({filters => vec![(": The lock for resource.*", ": ...")]}, {
+    insta::with_settings!({filters => vec![("Failed to set: .*", "Failed to set: ...")]}, {
         insta::assert_snapshot!(stderr, @r###"
         Warning: Failed to export some branches:
-          main/sub: Failed to set: A lock could not be obtained for reference "refs/heads/main/sub": ...
+          main/sub: Failed to set: ...
         Hint: Git doesn't allow a branch name that looks like a parent directory of
         another (e.g. `foo` and `foo/bar`). Try to rename the branches that failed to
         export or their "parent" branches.
@@ -416,7 +414,7 @@ fn test_git_colocated_checkout_non_empty_working_copy() {
     let test_env = TestEnvironment::default();
     let workspace_root = test_env.env_root().join("repo");
     let git_repo = git2::Repository::init(&workspace_root).unwrap();
-    test_env.jj_cmd_ok(&workspace_root, &["init", "--git-repo", "."]);
+    test_env.jj_cmd_ok(&workspace_root, &["git", "init", "--git-repo", "."]);
 
     // Create an initial commit in Git
     // We use this to set HEAD to master
@@ -479,7 +477,7 @@ fn test_git_colocated_fetch_deleted_or_moved_branch() {
     test_env.add_config("git.auto-local-branch = true");
     let origin_path = test_env.env_root().join("origin");
     git2::Repository::init(&origin_path).unwrap();
-    test_env.jj_cmd_ok(&origin_path, &["init", "--git-repo=."]);
+    test_env.jj_cmd_ok(&origin_path, &["git", "init", "--git-repo=."]);
     test_env.jj_cmd_ok(&origin_path, &["describe", "-m=A"]);
     test_env.jj_cmd_ok(&origin_path, &["branch", "create", "A"]);
     test_env.jj_cmd_ok(&origin_path, &["new", "-m=B_to_delete"]);
@@ -489,7 +487,7 @@ fn test_git_colocated_fetch_deleted_or_moved_branch() {
 
     let clone_path = test_env.env_root().join("clone");
     git2::Repository::clone(origin_path.to_str().unwrap(), &clone_path).unwrap();
-    test_env.jj_cmd_ok(&clone_path, &["init", "--git-repo=."]);
+    test_env.jj_cmd_ok(&clone_path, &["git", "init", "--git-repo=."]);
     test_env.jj_cmd_ok(&clone_path, &["new", "A"]);
     insta::assert_snapshot!(get_log_output(&test_env, &clone_path), @r###"
     @  0335878796213c3a701f1c9c34dcae242bee4131
@@ -527,7 +525,7 @@ fn test_git_colocated_rebase_dirty_working_copy() {
     let test_env = TestEnvironment::default();
     let repo_path = test_env.env_root().join("repo");
     let git_repo = git2::Repository::init(&repo_path).unwrap();
-    test_env.jj_cmd_ok(&repo_path, &["init", "--git-repo=."]);
+    test_env.jj_cmd_ok(&repo_path, &["git", "init", "--git-repo=."]);
 
     std::fs::write(repo_path.join("file"), "base").unwrap();
     test_env.jj_cmd_ok(&repo_path, &["new"]);
@@ -575,7 +573,13 @@ fn test_git_colocated_external_checkout() {
     let test_env = TestEnvironment::default();
     let repo_path = test_env.env_root().join("repo");
     let git_repo = git2::Repository::init(&repo_path).unwrap();
-    test_env.jj_cmd_ok(&repo_path, &["init", "--git-repo=."]);
+    let git_check_out_ref = |name| {
+        git_repo
+            .set_head_detached(git_repo.find_reference(name).unwrap().target().unwrap())
+            .unwrap()
+    };
+
+    test_env.jj_cmd_ok(&repo_path, &["git", "init", "--git-repo=."]);
     test_env.jj_cmd_ok(&repo_path, &["ci", "-m=A"]);
     test_env.jj_cmd_ok(&repo_path, &["branch", "create", "-r@-", "master"]);
     test_env.jj_cmd_ok(&repo_path, &["new", "-m=B", "root()"]);
@@ -591,15 +595,7 @@ fn test_git_colocated_external_checkout() {
     "###);
 
     // Check out another branch by external command
-    git_repo
-        .set_head_detached(
-            git_repo
-                .find_reference("refs/heads/master")
-                .unwrap()
-                .target()
-                .unwrap(),
-        )
-        .unwrap();
+    git_check_out_ref("refs/heads/master");
 
     // The old working-copy commit gets abandoned, but the whole branch should not
     // be abandoned. (#1042)
@@ -614,6 +610,36 @@ fn test_git_colocated_external_checkout() {
     insta::assert_snapshot!(stderr, @r###"
     Reset the working copy parent to the new Git HEAD.
     "###);
+
+    // Edit non-head commit
+    test_env.jj_cmd_ok(&repo_path, &["new", "description(B)"]);
+    test_env.jj_cmd_ok(&repo_path, &["new", "-m=C", "--no-edit"]);
+    insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
+    ◉  99a813753d6db988d8fc436b0d6b30a54d6b2707 C
+    @  81e086b7f9b1dd7fde252e28bdcf4ba4abd86ce5
+    ◉  eccedddfa5152d99fc8ddd1081b375387a8a382a HEAD@git B
+    │ ◉  a86754f975f953fa25da4265764adc0c62e9ce6b master A
+    ├─╯
+    ◉  0000000000000000000000000000000000000000
+    "###);
+
+    // Check out another branch by external command
+    git_check_out_ref("refs/heads/master");
+
+    // The old working-copy commit shouldn't be abandoned. (#3747)
+    let (stdout, stderr) = get_log_output_with_stderr(&test_env, &repo_path);
+    insta::assert_snapshot!(stdout, @r###"
+    @  f9f6929eae811496820623f44199a9e04ee402e8
+    ◉  a86754f975f953fa25da4265764adc0c62e9ce6b master HEAD@git A
+    │ ◉  99a813753d6db988d8fc436b0d6b30a54d6b2707 C
+    │ ◉  81e086b7f9b1dd7fde252e28bdcf4ba4abd86ce5
+    │ ◉  eccedddfa5152d99fc8ddd1081b375387a8a382a B
+    ├─╯
+    ◉  0000000000000000000000000000000000000000
+    "###);
+    insta::assert_snapshot!(stderr, @r###"
+    Reset the working copy parent to the new Git HEAD.
+    "###);
 }
 
 #[test]
@@ -621,7 +647,7 @@ fn test_git_colocated_squash_undo() {
     let test_env = TestEnvironment::default();
     let repo_path = test_env.env_root().join("repo");
     git2::Repository::init(&repo_path).unwrap();
-    test_env.jj_cmd_ok(&repo_path, &["init", "--git-repo=."]);
+    test_env.jj_cmd_ok(&repo_path, &["git", "init", "--git-repo=."]);
     test_env.jj_cmd_ok(&repo_path, &["ci", "-m=A"]);
     // Test the setup
     insta::assert_snapshot!(get_log_output_divergence(&test_env, &repo_path), @r###"
@@ -651,7 +677,7 @@ fn test_git_colocated_undo_head_move() {
     let test_env = TestEnvironment::default();
     let repo_path = test_env.env_root().join("repo");
     let git_repo = git2::Repository::init(&repo_path).unwrap();
-    test_env.jj_cmd_ok(&repo_path, &["init", "--git-repo=."]);
+    test_env.jj_cmd_ok(&repo_path, &["git", "init", "--git-repo=."]);
 
     // Create new HEAD
     test_env.jj_cmd_ok(&repo_path, &["new"]);
@@ -783,7 +809,7 @@ fn test_git_colocated_unreachable_commits() {
     );
 
     // Import the repo while there is no path to the second commit
-    test_env.jj_cmd_ok(&workspace_root, &["init", "--git-repo", "."]);
+    test_env.jj_cmd_ok(&workspace_root, &["git", "init", "--git-repo", "."]);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
     @  66ae47cee4f8c28ee8d7e4f5d9401b03c07e22f2
     ◉  2ee37513d2b5e549f7478c671a780053614bff19 master HEAD@git initial
@@ -800,3 +826,8 @@ fn test_git_colocated_unreachable_commits() {
     Error: Revision "8e713ff77b54928dd4a82aaabeca44b1ae91722c" doesn't exist
     "###);
 }
+
+fn get_branch_output(test_env: &TestEnvironment, repo_path: &Path) -> String {
+    // --quiet to suppress deleted branches hint
+    test_env.jj_cmd_success(repo_path, &["branch", "list", "--all-remotes", "--quiet"])
+}
