@@ -15,8 +15,6 @@ use std::path::Path;
 
 use crate::common::TestEnvironment;
 
-pub mod common;
-
 /// Creates a remote Git repo containing a branch with the same name
 fn init_git_remote(test_env: &TestEnvironment, remote: &str) {
     let git_repo_path = test_env.env_root().join(remote);
@@ -52,7 +50,8 @@ fn add_git_remote(test_env: &TestEnvironment, repo_path: &Path, remote: &str) {
 }
 
 fn get_branch_output(test_env: &TestEnvironment, repo_path: &Path) -> String {
-    test_env.jj_cmd_success(repo_path, &["branch", "list", "--all"])
+    // --quiet to suppress deleted branches hint
+    test_env.jj_cmd_success(repo_path, &["branch", "list", "--all-remotes", "--quiet"])
 }
 
 fn create_commit(test_env: &TestEnvironment, repo_path: &Path, name: &str, parents: &[&str]) {
@@ -76,7 +75,7 @@ fn get_log_output(test_env: &TestEnvironment, workspace_root: &Path) -> String {
 #[test]
 fn test_git_fetch_with_default_config() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "origin");
 
@@ -90,7 +89,7 @@ fn test_git_fetch_with_default_config() {
 fn test_git_fetch_default_remote() {
     let test_env = TestEnvironment::default();
     test_env.add_config("git.auto-local-branch = true");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "origin");
 
@@ -105,15 +104,15 @@ fn test_git_fetch_default_remote() {
 fn test_git_fetch_single_remote() {
     let test_env = TestEnvironment::default();
     test_env.add_config("git.auto-local-branch = true");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
 
-    test_env
-        .jj_cmd(&repo_path, &["git", "fetch"])
-        .assert()
-        .success()
-        .stderr("Fetching from the only existing remote: rem1\n");
+    let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["git", "fetch"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Hint: Fetching from the only existing remote: rem1
+    branch: rem1@rem1 [new] tracked
+    "###);
     insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
     rem1: qxosxrvv 6a211027 message
       @rem1: qxosxrvv 6a211027 message
@@ -124,7 +123,7 @@ fn test_git_fetch_single_remote() {
 fn test_git_fetch_single_remote_all_remotes_flag() {
     let test_env = TestEnvironment::default();
     test_env.add_config("git.auto-local-branch = true");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
 
@@ -142,7 +141,7 @@ fn test_git_fetch_single_remote_all_remotes_flag() {
 fn test_git_fetch_single_remote_from_arg() {
     let test_env = TestEnvironment::default();
     test_env.add_config("git.auto-local-branch = true");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
 
@@ -157,7 +156,7 @@ fn test_git_fetch_single_remote_from_arg() {
 fn test_git_fetch_single_remote_from_config() {
     let test_env = TestEnvironment::default();
     test_env.add_config("git.auto-local-branch = true");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
     test_env.add_config(r#"git.fetch = "rem1""#);
@@ -173,7 +172,7 @@ fn test_git_fetch_single_remote_from_config() {
 fn test_git_fetch_multiple_remotes() {
     let test_env = TestEnvironment::default();
     test_env.add_config("git.auto-local-branch = true");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
     add_git_remote(&test_env, &repo_path, "rem2");
@@ -194,7 +193,7 @@ fn test_git_fetch_multiple_remotes() {
 fn test_git_fetch_all_remotes() {
     let test_env = TestEnvironment::default();
     test_env.add_config("git.auto-local-branch = true");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
     add_git_remote(&test_env, &repo_path, "rem2");
@@ -212,7 +211,7 @@ fn test_git_fetch_all_remotes() {
 fn test_git_fetch_multiple_remotes_from_config() {
     let test_env = TestEnvironment::default();
     test_env.add_config("git.auto-local-branch = true");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
     add_git_remote(&test_env, &repo_path, "rem2");
@@ -230,7 +229,7 @@ fn test_git_fetch_multiple_remotes_from_config() {
 #[test]
 fn test_git_fetch_nonexistent_remote() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
 
@@ -239,6 +238,7 @@ fn test_git_fetch_nonexistent_remote() {
         &["git", "fetch", "--remote", "rem1", "--remote", "rem2"],
     );
     insta::assert_snapshot!(stderr, @r###"
+    branch: rem1@rem1 [new] untracked
     Error: No git remote named 'rem2'
     "###);
     // No remote should have been fetched as part of the failing transaction
@@ -248,13 +248,14 @@ fn test_git_fetch_nonexistent_remote() {
 #[test]
 fn test_git_fetch_nonexistent_remote_from_config() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
     test_env.add_config(r#"git.fetch = ["rem1", "rem2"]"#);
 
     let stderr = &test_env.jj_cmd_failure(&repo_path, &["git", "fetch"]);
     insta::assert_snapshot!(stderr, @r###"
+    branch: rem1@rem1 [new] untracked
     Error: No git remote named 'rem2'
     "###);
     // No remote should have been fetched as part of the failing transaction
@@ -271,30 +272,32 @@ fn test_git_fetch_from_remote_named_git() {
     git_repo.remote("git", "../git").unwrap();
 
     // Existing remote named 'git' shouldn't block the repo initialization.
-    test_env.jj_cmd_ok(&repo_path, &["init", "--git-repo=."]);
+    test_env.jj_cmd_ok(&repo_path, &["git", "init", "--git-repo=."]);
 
     // Try fetching from the remote named 'git'.
     let stderr = &test_env.jj_cmd_failure(&repo_path, &["git", "fetch", "--remote=git"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Failed to import refs from underlying Git repo: Git remote named 'git' is reserved for local Git repository
+    Error: Failed to import refs from underlying Git repo
+    Caused by: Git remote named 'git' is reserved for local Git repository
     Hint: Run `jj git remote rename` to give different name.
     "###);
 
     // Implicit import shouldn't fail because of the remote ref.
-    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "list", "--all"]);
+    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "list", "--all-remotes"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @"");
 
     // Explicit import is an error.
     // (This could be warning if we add mechanism to report ignored refs.)
     insta::assert_snapshot!(test_env.jj_cmd_failure(&repo_path, &["git", "import"]), @r###"
-    Error: Failed to import refs from underlying Git repo: Git remote named 'git' is reserved for local Git repository
+    Error: Failed to import refs from underlying Git repo
+    Caused by: Git remote named 'git' is reserved for local Git repository
     Hint: Run `jj git remote rename` to give different name.
     "###);
 
     // The remote can be renamed, and the ref can be imported.
     test_env.jj_cmd_ok(&repo_path, &["git", "remote", "rename", "git", "bar"]);
-    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "list", "--all"]);
+    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "list", "--all-remotes"]);
     insta::assert_snapshot!(stdout, @r###"
     git: mrylzrtu 76fc7466 message
       @bar: mrylzrtu 76fc7466 message
@@ -309,7 +312,7 @@ fn test_git_fetch_from_remote_named_git() {
 fn test_git_fetch_prune_before_updating_tips() {
     let test_env = TestEnvironment::default();
     test_env.add_config("git.auto-local-branch = true");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "origin");
     test_env.jj_cmd_ok(&repo_path, &["git", "fetch"]);
@@ -337,7 +340,7 @@ fn test_git_fetch_prune_before_updating_tips() {
 fn test_git_fetch_conflicting_branches() {
     let test_env = TestEnvironment::default();
     test_env.add_config("git.auto-local-branch = true");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
 
@@ -368,7 +371,7 @@ fn test_git_fetch_conflicting_branches_colocated() {
     let repo_path = test_env.env_root().join("repo");
     let _git_repo = git2::Repository::init(&repo_path).unwrap();
     // create_colocated_repo_and_branches_from_trunk1(&test_env, &repo_path);
-    test_env.jj_cmd_ok(&repo_path, &["init", "--git-repo", "."]);
+    test_env.jj_cmd_ok(&repo_path, &["git", "init", "--git-repo", "."]);
     add_git_remote(&test_env, &repo_path, "rem1");
     insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @"");
 
@@ -402,7 +405,7 @@ fn create_colocated_repo_and_branches_from_trunk1(
     repo_path: &Path,
 ) -> String {
     // Create a colocated repo in `source` to populate it more easily
-    test_env.jj_cmd_ok(repo_path, &["init", "--git-repo", "."]);
+    test_env.jj_cmd_ok(repo_path, &["git", "init", "--git-repo", "."]);
     create_commit(test_env, repo_path, "trunk1", &[]);
     create_commit(test_env, repo_path, "a1", &["trunk1"]);
     create_commit(test_env, repo_path, "a2", &["trunk1"]);
@@ -463,7 +466,12 @@ fn test_git_fetch_all() {
     insta::assert_snapshot!(get_branch_output(&test_env, &target_jj_repo_path), @"");
     let (stdout, stderr) = test_env.jj_cmd_ok(&target_jj_repo_path, &["git", "fetch"]);
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    branch: a1@origin     [new] tracked
+    branch: a2@origin     [new] tracked
+    branch: b@origin      [new] tracked
+    branch: trunk1@origin [new] tracked
+    "###);
     insta::assert_snapshot!(get_branch_output(&test_env, &target_jj_repo_path), @r###"
     a1: nknoxmzm 359a9a02 descr_for_a1
       @origin: nknoxmzm 359a9a02 descr_for_a1
@@ -531,6 +539,10 @@ fn test_git_fetch_all() {
     let (stdout, stderr) = test_env.jj_cmd_ok(&target_jj_repo_path, &["git", "fetch"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
+    branch: a1@origin     [updated] tracked
+    branch: a2@origin     [updated] tracked
+    branch: b@origin      [updated] tracked
+    branch: trunk2@origin [new] tracked
     Abandoned 2 commits that are no longer reachable.
     "###);
     insta::assert_snapshot!(get_branch_output(&test_env, &target_jj_repo_path), @r###"
@@ -618,7 +630,9 @@ fn test_git_fetch_some_of_many_branches() {
     let (stdout, stderr) =
         test_env.jj_cmd_ok(&target_jj_repo_path, &["git", "fetch", "--branch", "b"]);
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    branch: b@origin [new] tracked
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
     ◉  c7d4bdcbc215 descr_for_b b
     ◉  ff36dc55760e descr_for_trunk1
@@ -637,7 +651,10 @@ fn test_git_fetch_some_of_many_branches() {
         &["git", "fetch", "--branch", "glob:a*"],
     );
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    branch: a1@origin [new] tracked
+    branch: a2@origin [new] tracked
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
     ◉  decaa3966c83 descr_for_a2 a2
     │ ◉  359a9a02457d descr_for_a1 a1
@@ -706,6 +723,8 @@ fn test_git_fetch_some_of_many_branches() {
     );
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
+    branch: a1@origin [updated] tracked
+    branch: b@origin  [updated] tracked
     Abandoned 1 commits that are no longer reachable.
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
@@ -743,6 +762,7 @@ fn test_git_fetch_some_of_many_branches() {
     );
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
+    branch: a2@origin [updated] tracked
     Abandoned 1 commits that are no longer reachable.
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
@@ -810,7 +830,10 @@ fn test_git_fetch_undo() {
         &["git", "fetch", "--branch", "b", "--branch", "a1"],
     );
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    branch: a1@origin [new] tracked
+    branch: b@origin  [new] tracked
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
     ◉  c7d4bdcbc215 descr_for_b b
     │ ◉  359a9a02457d descr_for_a1 a1
@@ -832,7 +855,9 @@ fn test_git_fetch_undo() {
     let (stdout, stderr) =
         test_env.jj_cmd_ok(&target_jj_repo_path, &["git", "fetch", "--branch", "b"]);
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    branch: b@origin [new] tracked
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
     ◉  c7d4bdcbc215 descr_for_b b
     ◉  ff36dc55760e descr_for_trunk1
@@ -882,7 +907,9 @@ fn test_fetch_undo_what() {
     // Fetch a branch
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["git", "fetch", "--branch", "b"]);
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    branch: b@origin [new] tracked
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
     ◉  c7d4bdcbc215 descr_for_b b
     ◉  ff36dc55760e descr_for_trunk1
@@ -905,7 +932,6 @@ fn test_fetch_undo_what() {
     insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
     b (deleted)
       @origin: vpupmnsl hidden c7d4bdcb descr_for_b
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     "###);
 
     // Now, let's demo restoring just the remote-tracking branch. First, let's
@@ -914,7 +940,6 @@ fn test_fetch_undo_what() {
     insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
     b (deleted)
       @origin: vpupmnsl hidden c7d4bdcb descr_for_b
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     newbranch: qpvuntsm 230dd059 (empty) (no description set)
     "###);
     // Restoring just the remote-tracking state will not affect `newbranch`, but
@@ -940,7 +965,7 @@ fn test_fetch_undo_what() {
 fn test_git_fetch_remove_fetch() {
     let test_env = TestEnvironment::default();
     test_env.add_config("git.auto-local-branch = true");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "origin");
 
@@ -969,7 +994,9 @@ fn test_git_fetch_remove_fetch() {
     // Check that origin@origin is properly recreated
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["git", "fetch"]);
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    branch: origin@origin [new] tracked
+    "###);
     insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
     origin (conflicted):
       + qpvuntsm 230dd059 (empty) (no description set)
@@ -982,7 +1009,7 @@ fn test_git_fetch_remove_fetch() {
 fn test_git_fetch_rename_fetch() {
     let test_env = TestEnvironment::default();
     test_env.add_config("git.auto-local-branch = true");
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "origin");
 
@@ -1052,7 +1079,12 @@ fn test_git_fetch_removed_branch() {
     // Fetch all branches
     let (stdout, stderr) = test_env.jj_cmd_ok(&target_jj_repo_path, &["git", "fetch"]);
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    branch: a1@origin     [new] tracked
+    branch: a2@origin     [new] tracked
+    branch: b@origin      [new] tracked
+    branch: trunk1@origin [new] tracked
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
     ◉  c7d4bdcbc215 descr_for_b b
     │ ◉  decaa3966c83 descr_for_a2 a2
@@ -1092,6 +1124,7 @@ fn test_git_fetch_removed_branch() {
         test_env.jj_cmd_ok(&target_jj_repo_path, &["git", "fetch", "--branch", "a2"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
+    branch: a2@origin [deleted] untracked
     Abandoned 1 commits that are no longer reachable.
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
@@ -1138,7 +1171,12 @@ fn test_git_fetch_removed_parent_branch() {
     // Fetch all branches
     let (stdout, stderr) = test_env.jj_cmd_ok(&target_jj_repo_path, &["git", "fetch"]);
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    branch: a1@origin     [new] tracked
+    branch: a2@origin     [new] tracked
+    branch: b@origin      [new] tracked
+    branch: trunk1@origin [new] tracked
+    "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
     ◉  c7d4bdcbc215 descr_for_b b
     │ ◉  decaa3966c83 descr_for_a2 a2
@@ -1165,6 +1203,8 @@ fn test_git_fetch_removed_parent_branch() {
     );
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
+    branch: a1@origin     [deleted] untracked
+    branch: trunk1@origin [deleted] untracked
     Abandoned 1 commits that are no longer reachable.
     "###);
     insta::assert_snapshot!(get_log_output(&test_env, &target_jj_repo_path), @r###"
@@ -1181,7 +1221,7 @@ fn test_git_fetch_removed_parent_branch() {
 #[test]
 fn test_git_fetch_remote_only_branch() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     // Create non-empty git repo to add as a remote
