@@ -17,7 +17,7 @@ use std::path::{Path, PathBuf};
 
 use test_case::test_case;
 
-use crate::common::TestEnvironment;
+use crate::common::{strip_last_line, TestEnvironment};
 
 fn init_git_repo(git_repo_path: &Path, bare: bool) -> git2::Repository {
     init_git_repo_with_opts(git_repo_path, git2::RepositoryInitOptions::new().bare(bare))
@@ -58,7 +58,12 @@ fn init_git_repo_with_opts(
 }
 
 fn get_branch_output(test_env: &TestEnvironment, repo_path: &Path) -> String {
-    test_env.jj_cmd_success(repo_path, &["branch", "list", "--all"])
+    test_env.jj_cmd_success(repo_path, &["branch", "list", "--all-remotes"])
+}
+
+fn get_log_output(test_env: &TestEnvironment, workspace_root: &Path) -> String {
+    let template = r#"separate(" ", commit_id.short(), branches, git_head, description)"#;
+    test_env.jj_cmd_success(workspace_root, &["log", "-T", template, "-r=all()"])
 }
 
 fn read_git_target(workspace_root: &Path) -> String {
@@ -134,12 +139,11 @@ fn test_git_init_external(bare: bool) {
     }
 
     // Check that the Git repo's HEAD got checked out
-    let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-r", "@-"]);
     insta::allow_duplicates! {
-        insta::assert_snapshot!(stdout, @r###"
-        ◉  mwrttmos git.user@example.com 1970-01-01 01:02:03.000 +01:00 my-branch HEAD@git 8d698d4a
-        │  My commit message
-        ~
+        insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
+        @  f6950fc115ae
+        ◉  8d698d4a8ee1 my-branch HEAD@git My commit message
+        ◉  000000000000
         "###);
     }
 }
@@ -151,8 +155,10 @@ fn test_git_init_external_non_existent_directory() {
         test_env.env_root(),
         &["git", "init", "repo", "--git-repo", "non-existent"],
     );
-    insta::assert_snapshot!(stderr, @r###"
-    Error: $TEST_ENV/non-existent doesn't exist
+    insta::assert_snapshot!(strip_last_line(&stderr), @r###"
+    Error: Failed to access the repository
+    Caused by:
+    1: Cannot access $TEST_ENV/non-existent
     "###);
 }
 
@@ -201,20 +207,19 @@ fn test_git_init_colocated_via_git_repo_path() {
         .ends_with("../../../.git"));
 
     // Check that the Git repo's HEAD got checked out
-    let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-r", "@-"]);
-    insta::assert_snapshot!(stdout, @r###"
-    ◉  mwrttmos git.user@example.com 1970-01-01 01:02:03.000 +01:00 my-branch HEAD@git 8d698d4a
-    │  My commit message
-    ~
+    insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r###"
+    @  f61b77cd4bb5
+    ◉  8d698d4a8ee1 my-branch HEAD@git My commit message
+    ◉  000000000000
     "###);
 
     // Check that the Git repo's HEAD moves
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["log", "-r", "@-"]);
-    insta::assert_snapshot!(stdout, @r###"
-    ◉  sqpuoqvx test.user@example.com 2001-02-03 04:05:07.000 +07:00 HEAD@git f61b77cd
-    │  (no description set)
-    ~
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  f1c7aa7c62d8
+    ◉  f61b77cd4bb5 HEAD@git
+    ◉  8d698d4a8ee1 my-branch My commit message
+    ◉  000000000000
     "###);
 }
 
@@ -238,20 +243,19 @@ fn test_git_init_colocated_via_git_repo_path_gitlink() {
     insta::assert_snapshot!(read_git_target(&workspace_root), @"../../../.git");
 
     // Check that the Git repo's HEAD got checked out
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["log", "-r", "@-"]);
-    insta::assert_snapshot!(stdout, @r###"
-    ◉  mwrttmos git.user@example.com 1970-01-01 01:02:03.000 +01:00 my-branch HEAD@git 8d698d4a
-    │  My commit message
-    ~
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  f61b77cd4bb5
+    ◉  8d698d4a8ee1 my-branch HEAD@git My commit message
+    ◉  000000000000
     "###);
 
     // Check that the Git repo's HEAD moves
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["log", "-r", "@-"]);
-    insta::assert_snapshot!(stdout, @r###"
-    ◉  sqpuoqvx test.user@example.com 2001-02-03 04:05:07.000 +07:00 HEAD@git f61b77cd
-    │  (no description set)
-    ~
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  f1c7aa7c62d8
+    ◉  f61b77cd4bb5 HEAD@git
+    ◉  8d698d4a8ee1 my-branch My commit message
+    ◉  000000000000
     "###);
 }
 
@@ -274,20 +278,19 @@ fn test_git_init_colocated_via_git_repo_path_symlink_directory() {
     insta::assert_snapshot!(read_git_target(&workspace_root), @"../../../.git");
 
     // Check that the Git repo's HEAD got checked out
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["log", "-r", "@-"]);
-    insta::assert_snapshot!(stdout, @r###"
-    ◉  mwrttmos git.user@example.com 1970-01-01 01:02:03.000 +01:00 my-branch HEAD@git 8d698d4a
-    │  My commit message
-    ~
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  f61b77cd4bb5
+    ◉  8d698d4a8ee1 my-branch HEAD@git My commit message
+    ◉  000000000000
     "###);
 
     // Check that the Git repo's HEAD moves
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["log", "-r", "@-"]);
-    insta::assert_snapshot!(stdout, @r###"
-    ◉  sqpuoqvx test.user@example.com 2001-02-03 04:05:07.000 +07:00 HEAD@git f61b77cd
-    │  (no description set)
-    ~
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  f1c7aa7c62d8
+    ◉  f61b77cd4bb5 HEAD@git
+    ◉  8d698d4a8ee1 my-branch My commit message
+    ◉  000000000000
     "###);
 }
 
@@ -313,20 +316,19 @@ fn test_git_init_colocated_via_git_repo_path_symlink_directory_without_bare_conf
     insta::assert_snapshot!(read_git_target(&workspace_root), @"../../../.git");
 
     // Check that the Git repo's HEAD got checked out
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["log", "-r", "@-"]);
-    insta::assert_snapshot!(stdout, @r###"
-    ◉  mwrttmos git.user@example.com 1970-01-01 01:02:03.000 +01:00 my-branch HEAD@git 8d698d4a
-    │  My commit message
-    ~
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  f61b77cd4bb5
+    ◉  8d698d4a8ee1 my-branch HEAD@git My commit message
+    ◉  000000000000
     "###);
 
     // Check that the Git repo's HEAD moves
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["log", "-r", "@-"]);
-    insta::assert_snapshot!(stdout, @r###"
-    ◉  sqpuoqvx test.user@example.com 2001-02-03 04:05:07.000 +07:00 HEAD@git f61b77cd
-    │  (no description set)
-    ~
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  f1c7aa7c62d8
+    ◉  f61b77cd4bb5 HEAD@git
+    ◉  8d698d4a8ee1 my-branch My commit message
+    ◉  000000000000
     "###);
 }
 
@@ -354,20 +356,19 @@ fn test_git_init_colocated_via_git_repo_path_symlink_gitlink() {
     insta::assert_snapshot!(read_git_target(&workspace_root), @"../../../.git");
 
     // Check that the Git repo's HEAD got checked out
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["log", "-r", "@-"]);
-    insta::assert_snapshot!(stdout, @r###"
-    ◉  mwrttmos git.user@example.com 1970-01-01 01:02:03.000 +01:00 my-branch HEAD@git 8d698d4a
-    │  My commit message
-    ~
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  f61b77cd4bb5
+    ◉  8d698d4a8ee1 my-branch HEAD@git My commit message
+    ◉  000000000000
     "###);
 
     // Check that the Git repo's HEAD moves
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["log", "-r", "@-"]);
-    insta::assert_snapshot!(stdout, @r###"
-    ◉  sqpuoqvx test.user@example.com 2001-02-03 04:05:07.000 +07:00 HEAD@git f61b77cd
-    │  (no description set)
-    ~
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  f1c7aa7c62d8
+    ◉  f61b77cd4bb5 HEAD@git
+    ◉  8d698d4a8ee1 my-branch My commit message
+    ◉  000000000000
     "###);
 }
 
@@ -427,7 +428,7 @@ fn test_git_init_colocated_via_git_repo_path_imported_refs() {
     let (_stdout, stderr) = test_env.jj_cmd_ok(&local_path, &["git", "init", "--git-repo=."]);
     insta::assert_snapshot!(stderr, @r###"
     Done importing changes from the underlying Git repo.
-    The following remote branches aren't associated with the existing local branches:
+    Hint: The following remote branches aren't associated with the existing local branches:
       local-remote@origin
     Hint: Run `jj branch track local-remote@origin` to keep local branches updated on future pulls.
     Initialized repo in "."
@@ -479,12 +480,12 @@ fn test_git_init_colocated_dirty_working_copy() {
     // Working-copy changes should have been snapshotted.
     let stdout = test_env.jj_cmd_success(&workspace_root, &["log", "-s", "--ignore-working-copy"]);
     insta::assert_snapshot!(stdout, @r###"
-    @  sqpuoqvx test.user@example.com 2001-02-03 04:05:07.000 +07:00 cd1e144d
+    @  sqpuoqvx test.user@example.com 2001-02-03 08:05:07 cd1e144d
     │  (no description set)
     │  A new-staged-file
     │  M some-file
     │  A unstaged-file
-    ◉  mwrttmos git.user@example.com 1970-01-01 01:02:03.000 +01:00 my-branch HEAD@git 8d698d4a
+    ◉  mwrttmos git.user@example.com 1970-01-01 11:02:03 my-branch HEAD@git 8d698d4a
     │  My commit message
     │  A some-file
     ◉  zzzzzzzz root() 00000000
@@ -521,18 +522,17 @@ fn test_git_init_external_but_git_dir_exists() {
     "###);
 
     // The local ".git" repository is unrelated, so no commits should be imported
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["log", "-r", "@-"]);
-    insta::assert_snapshot!(stdout, @r###"
-    ◉  zzzzzzzz root() 00000000
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  230dd059e1b0
+    ◉  000000000000
     "###);
 
     // Check that Git HEAD is not set because this isn't a colocated repo
     test_env.jj_cmd_ok(&workspace_root, &["new"]);
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["log", "-r", "@-"]);
-    insta::assert_snapshot!(stdout, @r###"
-    ◉  qpvuntsm test.user@example.com 2001-02-03 04:05:07.000 +07:00 230dd059
-    │  (empty) (no description set)
-    ~
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  4db490c88528
+    ◉  230dd059e1b0
+    ◉  000000000000
     "###);
 }
 
@@ -542,24 +542,56 @@ fn test_git_init_colocated_via_flag_git_dir_exists() {
     let workspace_root = test_env.env_root().join("repo");
     init_git_repo(&workspace_root, false);
 
-    let stderr = test_env.jj_cmd_failure(&workspace_root, &["git", "init", "--colocate"]);
+    let (stdout, stderr) =
+        test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "--colocate", "repo"]);
+    insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
-    Error: Failed to access the repository
-    Caused by:
-    1: Failed to initialize git repository
-    2: Refusing to initialize the existing '$TEST_ENV/repo/.git' directory
+    Done importing changes from the underlying Git repo.
+    Initialized repo in "repo"
+    "###);
+
+    // Check that the Git repo's HEAD got checked out
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  f61b77cd4bb5
+    ◉  8d698d4a8ee1 my-branch HEAD@git My commit message
+    ◉  000000000000
+    "###);
+
+    // Check that the Git repo's HEAD moves
+    test_env.jj_cmd_ok(&workspace_root, &["new"]);
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  f1c7aa7c62d8
+    ◉  f61b77cd4bb5 HEAD@git
+    ◉  8d698d4a8ee1 my-branch My commit message
+    ◉  000000000000
     "###);
 }
 
 #[test]
 fn test_git_init_colocated_via_flag_git_dir_not_exists() {
     let test_env = TestEnvironment::default();
+    let workspace_root = test_env.env_root().join("repo");
     let (stdout, stderr) =
         test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "--colocate", "repo"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Initialized repo in "repo"
     "###);
+    // No HEAD@git ref is available yet
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  230dd059e1b0
+    ◉  000000000000
+    "###);
+
+    // Create the default branch (create both in case we change the default)
+    test_env.jj_cmd_ok(&workspace_root, &["branch", "create", "main", "master"]);
+
+    // If .git/HEAD pointed to the default branch, new working-copy commit would
+    // be created on top.
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_root), @r###"
+    @  230dd059e1b0 main master
+    ◉  000000000000
+    "###);
 }
 
 #[test]
