@@ -12,13 +12,13 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 
-use crate::common::{get_stderr_string, get_stdout_string, TestEnvironment};
+use crate::common::TestEnvironment;
 
 fn set_up() -> (TestEnvironment, PathBuf) {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "--git", "origin"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "origin"]);
     let origin_path = test_env.env_root().join("origin");
     let origin_git_repo_path = origin_path
         .join(".jj")
@@ -50,8 +50,7 @@ fn set_up() -> (TestEnvironment, PathBuf) {
 fn test_git_push_nothing() {
     let (test_env, workspace_root) = set_up();
     // Show the setup. `insta` has trouble if this is done inside `set_up()`
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["branch", "list", "--all-remotes"]);
-    insta::assert_snapshot!(stdout, @r###"
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
     branch1: lzmmnrxq 45a3aa29 (empty) description 1
       @origin: lzmmnrxq 45a3aa29 (empty) description 1
     branch2: rlzusymt 8476341e (empty) description 2
@@ -80,8 +79,7 @@ fn test_git_push_current_branch() {
     test_env.jj_cmd_ok(&workspace_root, &["branch", "create", "my-branch"]);
     test_env.jj_cmd_ok(&workspace_root, &["describe", "-m", "foo"]);
     // Check the setup
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["branch", "list", "--all-remotes"]);
-    insta::assert_snapshot!(stdout, @r###"
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
     branch1: lzmmnrxq 19e00bf6 (empty) modified branch1 commit
       @origin (ahead by 1 commits, behind by 1 commits): lzmmnrxq hidden 45a3aa29 (empty) description 1
     branch2: yostqsxw 10ee3363 (empty) foo
@@ -93,7 +91,7 @@ fn test_git_push_current_branch() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Branch changes to push to origin:
-      Move branch branch2 from 8476341eb395 to 10ee3363b259
+      Move forward branch branch2 from 8476341eb395 to 10ee3363b259
       Add branch my-branch to 10ee3363b259
     Dry-run requested, not pushing.
     "###);
@@ -101,11 +99,10 @@ fn test_git_push_current_branch() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Branch changes to push to origin:
-      Move branch branch2 from 8476341eb395 to 10ee3363b259
+      Move forward branch branch2 from 8476341eb395 to 10ee3363b259
       Add branch my-branch to 10ee3363b259
     "###);
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["branch", "list", "--all-remotes"]);
-    insta::assert_snapshot!(stdout, @r###"
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
     branch1: lzmmnrxq 19e00bf6 (empty) modified branch1 commit
       @origin (ahead by 1 commits, behind by 1 commits): lzmmnrxq hidden 45a3aa29 (empty) description 1
     branch2: yostqsxw 10ee3363 (empty) foo
@@ -113,6 +110,33 @@ fn test_git_push_current_branch() {
     my-branch: yostqsxw 10ee3363 (empty) foo
       @origin: yostqsxw 10ee3363 (empty) foo
     "###);
+
+    // Try pushing backwards
+    test_env.jj_cmd_success(
+        &workspace_root,
+        &[
+            "branch",
+            "set",
+            "branch2",
+            "-rbranch2-",
+            "--allow-backwards",
+        ],
+    );
+    // This behavior is a strangeness of our definition of the default push revset.
+    // We could consider changing it.
+    let (stdout, stderr) = test_env.jj_cmd_ok(&workspace_root, &["git", "push"]);
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Warning: No branches found in the default push revset: remote_branches(remote=origin)..@
+    Nothing changed.
+    "###);
+    // We can move a branch backwards
+    let (stdout, stderr) = test_env.jj_cmd_ok(&workspace_root, &["git", "push", "-bbranch2"]);
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Branch changes to push to origin:
+      Move backward branch branch2 from 10ee3363b259 to 8476341eb395
+    "###);
 }
 
 #[test]
@@ -130,7 +154,7 @@ fn test_git_push_parent_branch() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Branch changes to push to origin:
-      Force branch branch1 from 45a3aa29e907 to d47326d59ee1
+      Move sideways branch branch1 from 45a3aa29e907 to d47326d59ee1
     "###);
 }
 
@@ -190,7 +214,7 @@ fn test_git_push_other_remote_has_branch() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Branch changes to push to origin:
-      Force branch branch1 from 45a3aa29e907 to 50421a29358a
+      Move sideways branch branch1 from 45a3aa29e907 to 50421a29358a
     "###);
     // Since it's already pushed to origin, nothing will happen if push again
     let (stdout, stderr) = test_env.jj_cmd_ok(&workspace_root, &["git", "push"]);
@@ -199,7 +223,14 @@ fn test_git_push_other_remote_has_branch() {
     Warning: No branches found in the default push revset: remote_branches(remote=origin)..@
     Nothing changed.
     "###);
-    // But it will still get pushed to another remote
+    // The branch was moved on the "other" remote as well (since it's actually the
+    // same remote), but `jj` is not aware of that since it thinks this is a
+    // different remote. So, the push should fail.
+    //
+    // But it succeeds! That's because the branch is created at the same location as
+    // it is on the remote. This would also work for a descendant.
+    //
+    // TODO: Saner test?
     let (stdout, stderr) = test_env.jj_cmd_ok(&workspace_root, &["git", "push", "--remote=other"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
@@ -209,7 +240,7 @@ fn test_git_push_other_remote_has_branch() {
 }
 
 #[test]
-fn test_git_push_not_fast_forward() {
+fn test_git_push_forward_unexpectedly_moved() {
     let (test_env, workspace_root) = set_up();
 
     // Move branch1 forward on the remote
@@ -225,15 +256,169 @@ fn test_git_push_not_fast_forward() {
     test_env.jj_cmd_ok(&workspace_root, &["branch", "set", "branch1"]);
 
     // Pushing should fail
-    let assert = test_env
-        .jj_cmd(&workspace_root, &["git", "push"])
-        .assert()
-        .code(1);
-    insta::assert_snapshot!(get_stdout_string(&assert), @"");
-    insta::assert_snapshot!(get_stderr_string(&assert), @r###"
+    let stderr = test_env.jj_cmd_failure(&workspace_root, &["git", "push"]);
+    insta::assert_snapshot!(stderr, @r###"
     Branch changes to push to origin:
-      Move branch branch1 from 45a3aa29e907 to c35839cb8e8c
-    Error: The push conflicts with changes made on the remote (it is not fast-forwardable).
+      Move forward branch branch1 from 45a3aa29e907 to c35839cb8e8c
+    Error: Refusing to push a branch that unexpectedly moved on the remote. Affected refs: refs/heads/branch1
+    Hint: Try fetching from the remote, then make the branch point to where you want it to be, and push again.
+    "###);
+}
+
+#[test]
+fn test_git_push_sideways_unexpectedly_moved() {
+    let (test_env, workspace_root) = set_up();
+
+    // Move branch1 forward on the remote
+    let origin_path = test_env.env_root().join("origin");
+    test_env.jj_cmd_ok(&origin_path, &["new", "branch1", "-m=remote"]);
+    std::fs::write(origin_path.join("remote"), "remote").unwrap();
+    test_env.jj_cmd_ok(&origin_path, &["branch", "set", "branch1"]);
+    insta::assert_snapshot!(get_branch_output(&test_env, &origin_path), @r###"
+    branch1: vruxwmqv fb645b4b remote
+      @git (behind by 1 commits): qpvuntsm 45a3aa29 (empty) description 1
+    branch2: zsuskuln 8476341e (empty) description 2
+      @git: zsuskuln 8476341e (empty) description 2
+    "###);
+    test_env.jj_cmd_ok(&origin_path, &["git", "export"]);
+
+    // Move branch1 sideways to another commit locally
+    test_env.jj_cmd_ok(&workspace_root, &["new", "root()", "-m=local"]);
+    std::fs::write(workspace_root.join("local"), "local").unwrap();
+    test_env.jj_cmd_ok(
+        &workspace_root,
+        &["branch", "set", "branch1", "--allow-backwards"],
+    );
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
+    branch1: kmkuslsw 0f8bf988 local
+      @origin (ahead by 1 commits, behind by 1 commits): lzmmnrxq 45a3aa29 (empty) description 1
+    branch2: rlzusymt 8476341e (empty) description 2
+      @origin: rlzusymt 8476341e (empty) description 2
+    "###);
+
+    let stderr = test_env.jj_cmd_failure(&workspace_root, &["git", "push"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Branch changes to push to origin:
+      Move sideways branch branch1 from 45a3aa29e907 to 0f8bf988588e
+    Error: Refusing to push a branch that unexpectedly moved on the remote. Affected refs: refs/heads/branch1
+    Hint: Try fetching from the remote, then make the branch point to where you want it to be, and push again.
+    "###);
+}
+
+// This tests whether the push checks that the remote branches are in expected
+// positions.
+#[test]
+fn test_git_push_deletion_unexpectedly_moved() {
+    let (test_env, workspace_root) = set_up();
+
+    // Move branch1 forward on the remote
+    let origin_path = test_env.env_root().join("origin");
+    test_env.jj_cmd_ok(&origin_path, &["new", "branch1", "-m=remote"]);
+    std::fs::write(origin_path.join("remote"), "remote").unwrap();
+    test_env.jj_cmd_ok(&origin_path, &["branch", "set", "branch1"]);
+    insta::assert_snapshot!(get_branch_output(&test_env, &origin_path), @r###"
+    branch1: vruxwmqv fb645b4b remote
+      @git (behind by 1 commits): qpvuntsm 45a3aa29 (empty) description 1
+    branch2: zsuskuln 8476341e (empty) description 2
+      @git: zsuskuln 8476341e (empty) description 2
+    "###);
+    test_env.jj_cmd_ok(&origin_path, &["git", "export"]);
+
+    // Delete branch1 locally
+    test_env.jj_cmd_ok(&workspace_root, &["branch", "delete", "branch1"]);
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
+    branch1 (deleted)
+      @origin: lzmmnrxq 45a3aa29 (empty) description 1
+    branch2: rlzusymt 8476341e (empty) description 2
+      @origin: rlzusymt 8476341e (empty) description 2
+    "###);
+
+    let stderr = test_env.jj_cmd_failure(&workspace_root, &["git", "push", "--branch", "branch1"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Branch changes to push to origin:
+      Delete branch branch1 from 45a3aa29e907
+    Error: Refusing to push a branch that unexpectedly moved on the remote. Affected refs: refs/heads/branch1
+    Hint: Try fetching from the remote, then make the branch point to where you want it to be, and push again.
+    "###);
+}
+
+#[test]
+fn test_git_push_unexpectedly_deleted() {
+    let (test_env, workspace_root) = set_up();
+
+    // Delete branch1 forward on the remote
+    let origin_path = test_env.env_root().join("origin");
+    test_env.jj_cmd_ok(&origin_path, &["branch", "delete", "branch1"]);
+    insta::assert_snapshot!(get_branch_output(&test_env, &origin_path), @r###"
+    branch1 (deleted)
+      @git: qpvuntsm 45a3aa29 (empty) description 1
+    branch2: zsuskuln 8476341e (empty) description 2
+      @git: zsuskuln 8476341e (empty) description 2
+    "###);
+    test_env.jj_cmd_ok(&origin_path, &["git", "export"]);
+
+    // Move branch1 sideways to another commit locally
+    test_env.jj_cmd_ok(&workspace_root, &["new", "root()", "-m=local"]);
+    std::fs::write(workspace_root.join("local"), "local").unwrap();
+    test_env.jj_cmd_ok(
+        &workspace_root,
+        &["branch", "set", "branch1", "--allow-backwards"],
+    );
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
+    branch1: kpqxywon 1ebe27ba local
+      @origin (ahead by 1 commits, behind by 1 commits): lzmmnrxq 45a3aa29 (empty) description 1
+    branch2: rlzusymt 8476341e (empty) description 2
+      @origin: rlzusymt 8476341e (empty) description 2
+    "###);
+
+    // Pushing a moved branch fails if deleted on remote
+    let stderr = test_env.jj_cmd_failure(&workspace_root, &["git", "push"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Branch changes to push to origin:
+      Move sideways branch branch1 from 45a3aa29e907 to 1ebe27ba04bf
+    Error: Refusing to push a branch that unexpectedly moved on the remote. Affected refs: refs/heads/branch1
+    Hint: Try fetching from the remote, then make the branch point to where you want it to be, and push again.
+    "###);
+
+    test_env.jj_cmd_ok(&workspace_root, &["branch", "delete", "branch1"]);
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
+    branch1 (deleted)
+      @origin: lzmmnrxq 45a3aa29 (empty) description 1
+    branch2: rlzusymt 8476341e (empty) description 2
+      @origin: rlzusymt 8476341e (empty) description 2
+    "###);
+    // Pushing a *deleted* branch succeeds if deleted on remote, even if we expect
+    // branch1@origin to exist and point somewhere.
+    let (stdout, stderr) = test_env.jj_cmd_ok(&workspace_root, &["git", "push", "-bbranch1"]);
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Branch changes to push to origin:
+      Delete branch branch1 from 45a3aa29e907
+    "###);
+}
+
+#[test]
+fn test_git_push_creation_unexpectedly_already_exists() {
+    let (test_env, workspace_root) = set_up();
+
+    // Forget branch1 locally
+    test_env.jj_cmd_ok(&workspace_root, &["branch", "forget", "branch1"]);
+
+    // Create a new branh1
+    test_env.jj_cmd_ok(&workspace_root, &["new", "root()", "-m=new branch1"]);
+    std::fs::write(workspace_root.join("local"), "local").unwrap();
+    test_env.jj_cmd_ok(&workspace_root, &["branch", "create", "branch1"]);
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
+    branch1: yostqsxw 4c595cf9 new branch1
+    branch2: rlzusymt 8476341e (empty) description 2
+      @origin: rlzusymt 8476341e (empty) description 2
+    "###);
+
+    let stderr = test_env.jj_cmd_failure(&workspace_root, &["git", "push"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Branch changes to push to origin:
+      Add branch branch1 to 4c595cf9ac0a
+    Error: Refusing to push a branch that unexpectedly moved on the remote. Affected refs: refs/heads/branch1
     Hint: Try fetching from the remote, then make the branch point to where you want it to be, and push again.
     "###);
 }
@@ -256,8 +441,7 @@ fn test_git_push_locally_created_and_rewritten() {
     // Rewrite it and push again, which would fail if the pushed branch weren't
     // set to "tracking"
     test_env.jj_cmd_ok(&workspace_root, &["describe", "-mlocal 2"]);
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["branch", "list", "--all-remotes"]);
-    insta::assert_snapshot!(stdout, @r###"
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
     branch1: lzmmnrxq 45a3aa29 (empty) description 1
       @origin: lzmmnrxq 45a3aa29 (empty) description 1
     branch2: rlzusymt 8476341e (empty) description 2
@@ -268,7 +452,7 @@ fn test_git_push_locally_created_and_rewritten() {
     let (_stdout, stderr) = test_env.jj_cmd_ok(&workspace_root, &["git", "push"]);
     insta::assert_snapshot!(stderr, @r###"
     Branch changes to push to origin:
-      Force branch my from fcc999921ce9 to bde1d2e44b2a
+      Move sideways branch my from fcc999921ce9 to bde1d2e44b2a
     "###);
 }
 
@@ -283,11 +467,9 @@ fn test_git_push_multiple() {
     test_env.jj_cmd_ok(&workspace_root, &["branch", "create", "my-branch"]);
     test_env.jj_cmd_ok(&workspace_root, &["describe", "-m", "foo"]);
     // Check the setup
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["branch", "list", "--all-remotes"]);
-    insta::assert_snapshot!(stdout, @r###"
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
     branch1 (deleted)
       @origin: lzmmnrxq 45a3aa29 (empty) description 1
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     branch2: yqosqzyt 15dcdaa4 (empty) foo
       @origin (ahead by 1 commits, behind by 1 commits): rlzusymt 8476341e (empty) description 2
     my-branch: yqosqzyt 15dcdaa4 (empty) foo
@@ -299,7 +481,7 @@ fn test_git_push_multiple() {
     insta::assert_snapshot!(stderr, @r###"
     Branch changes to push to origin:
       Delete branch branch1 from 45a3aa29e907
-      Force branch branch2 from 8476341eb395 to 15dcdaa4f12f
+      Move sideways branch branch2 from 8476341eb395 to 15dcdaa4f12f
       Add branch my-branch to 15dcdaa4f12f
     Dry-run requested, not pushing.
     "###);
@@ -344,7 +526,7 @@ fn test_git_push_multiple() {
     insta::assert_snapshot!(stderr, @r###"
     Branch changes to push to origin:
       Delete branch branch1 from 45a3aa29e907
-      Force branch branch2 from 8476341eb395 to 15dcdaa4f12f
+      Move sideways branch branch2 from 8476341eb395 to 15dcdaa4f12f
     Dry-run requested, not pushing.
     "###);
 
@@ -366,11 +548,10 @@ fn test_git_push_multiple() {
     insta::assert_snapshot!(stderr, @r###"
     Branch changes to push to origin:
       Delete branch branch1 from 45a3aa29e907
-      Force branch branch2 from 8476341eb395 to 15dcdaa4f12f
+      Move sideways branch branch2 from 8476341eb395 to 15dcdaa4f12f
       Add branch my-branch to 15dcdaa4f12f
     "###);
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["branch", "list", "--all-remotes"]);
-    insta::assert_snapshot!(stdout, @r###"
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
     branch2: yqosqzyt 15dcdaa4 (empty) foo
       @origin: yqosqzyt 15dcdaa4 (empty) foo
     my-branch: yqosqzyt 15dcdaa4 (empty) foo
@@ -410,7 +591,7 @@ fn test_git_push_changes() {
     insta::assert_snapshot!(stderr, @r###"
     Creating branch push-yqosqzytrlsw for revision @-
     Branch changes to push to origin:
-      Force branch push-yostqsxwqrlt from 28d7620ea63a to 48d8c7948133
+      Move sideways branch push-yostqsxwqrlt from 28d7620ea63a to 48d8c7948133
       Add branch push-yqosqzytrlsw to fa16a14170fb
     "###);
     // specifying the same change twice doesn't break things
@@ -419,7 +600,7 @@ fn test_git_push_changes() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Branch changes to push to origin:
-      Force branch push-yostqsxwqrlt from 48d8c7948133 to b5f030322b1d
+      Move sideways branch push-yostqsxwqrlt from 48d8c7948133 to b5f030322b1d
     "###);
 
     // specifying the same branch with --change/--branch doesn't break things
@@ -431,7 +612,7 @@ fn test_git_push_changes() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Branch changes to push to origin:
-      Force branch push-yostqsxwqrlt from b5f030322b1d to 4df62cec2ee4
+      Move sideways branch push-yostqsxwqrlt from b5f030322b1d to 4df62cec2ee4
     "###);
 
     // try again with --change that moves the branch forward
@@ -460,7 +641,7 @@ fn test_git_push_changes() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Branch changes to push to origin:
-      Force branch push-yostqsxwqrlt from 4df62cec2ee4 to 3e2ce808759b
+      Move sideways branch push-yostqsxwqrlt from 4df62cec2ee4 to 3e2ce808759b
     "###);
     let stdout = test_env.jj_cmd_success(&workspace_root, &["status"]);
     insta::assert_snapshot!(stdout, @r###"
@@ -642,6 +823,16 @@ fn test_git_push_no_description() {
     insta::assert_snapshot!(stderr, @r###"
     Error: Won't push commit 5b36783cd11c since it has no description
     "###);
+    test_env.jj_cmd_ok(
+        &workspace_root,
+        &[
+            "git",
+            "push",
+            "--branch",
+            "my-branch",
+            "--allow-empty-description",
+        ],
+    );
 }
 
 #[test]
@@ -756,8 +947,7 @@ fn test_git_push_conflicting_branches() {
     test_env.jj_cmd_ok(&workspace_root, &["new", "root()", "-m=description 3"]);
     test_env.jj_cmd_ok(&workspace_root, &["branch", "create", "branch2"]);
     test_env.jj_cmd_ok(&workspace_root, &["git", "fetch"]);
-    insta::assert_snapshot!(
-        test_env.jj_cmd_success(&workspace_root, &["branch", "list", "--all-remotes"]), @r###"
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
     branch1: lzmmnrxq 45a3aa29 (empty) description 1
       @origin: lzmmnrxq 45a3aa29 (empty) description 1
     branch2 (conflicted):
@@ -795,7 +985,7 @@ fn test_git_push_conflicting_branches() {
     Warning: Branch branch2 is conflicted
     Hint: Run `jj branch list` to inspect, and use `jj branch set` to fix it up.
     Branch changes to push to origin:
-      Move branch branch1 from 45a3aa29e907 to fd1d63e031ea
+      Move forward branch branch1 from 45a3aa29e907 to fd1d63e031ea
     "###);
 
     // --revisions shouldn't be blocked by conflicting branch
@@ -806,7 +996,7 @@ fn test_git_push_conflicting_branches() {
     Warning: Branch branch2 is conflicted
     Hint: Run `jj branch list` to inspect, and use `jj branch set` to fix it up.
     Branch changes to push to origin:
-      Move branch branch1 from fd1d63e031ea to 8263cf992d33
+      Move forward branch branch1 from fd1d63e031ea to 8263cf992d33
     "###);
 }
 
@@ -837,13 +1027,11 @@ fn test_git_push_tracked_vs_all() {
     test_env.jj_cmd_ok(&workspace_root, &["branch", "delete", "branch2"]);
     test_env.jj_cmd_ok(&workspace_root, &["branch", "untrack", "branch1@origin"]);
     test_env.jj_cmd_ok(&workspace_root, &["branch", "create", "branch3"]);
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["branch", "list", "--all-remotes"]);
-    insta::assert_snapshot!(stdout, @r###"
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
     branch1: vruxwmqv a25f24af (empty) moved branch1
     branch1@origin: lzmmnrxq 45a3aa29 (empty) description 1
     branch2 (deleted)
       @origin: rlzusymt 8476341e (empty) description 2
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     branch3: znkkpsqq 998d6a78 (empty) moved branch2
     "###);
 
@@ -859,8 +1047,7 @@ fn test_git_push_tracked_vs_all() {
 
     // Untrack the last remaining tracked branch.
     test_env.jj_cmd_ok(&workspace_root, &["branch", "untrack", "branch2@origin"]);
-    let stdout = test_env.jj_cmd_success(&workspace_root, &["branch", "list", "--all-remotes"]);
-    insta::assert_snapshot!(stdout, @r###"
+    insta::assert_snapshot!(get_branch_output(&test_env, &workspace_root), @r###"
     branch1: vruxwmqv a25f24af (empty) moved branch1
     branch1@origin: lzmmnrxq 45a3aa29 (empty) description 1
     branch2@origin: rlzusymt 8476341e (empty) description 2
@@ -950,3 +1137,8 @@ fn test_git_push_to_remote_named_git() {
     Error: Git remote named 'git' is reserved for local Git repository
     "###);
 }
+
+fn get_branch_output(test_env: &TestEnvironment, repo_path: &Path) -> String {
+    // --quiet to suppress deleted branches hint
+    test_env.jj_cmd_success(repo_path, &["branch", "list", "--all-remotes", "--quiet"])
+}
