@@ -21,7 +21,7 @@ fn test_gitignores() {
     let test_env = TestEnvironment::default();
     let workspace_root = test_env.env_root().join("repo");
     git2::Repository::init(&workspace_root).unwrap();
-    test_env.jj_cmd_ok(&workspace_root, &["init", "--git-repo", "."]);
+    test_env.jj_cmd_ok(&workspace_root, &["git", "init", "--git-repo", "."]);
 
     // Say in core.excludesFiles that we don't want file1, file2, or file3
     let mut file = std::fs::OpenOptions::new()
@@ -69,7 +69,7 @@ fn test_gitignores_ignored_file_in_target_commit() {
     let test_env = TestEnvironment::default();
     let workspace_root = test_env.env_root().join("repo");
     git2::Repository::init(&workspace_root).unwrap();
-    test_env.jj_cmd_ok(&workspace_root, &["init", "--git-repo", "."]);
+    test_env.jj_cmd_ok(&workspace_root, &["git", "init", "--git-repo", "."]);
 
     // Create a commit with file "ignored" in it
     std::fs::write(workspace_root.join("ignored"), "committed contents\n").unwrap();
