@@ -14,7 +14,7 @@
 
 use std::ffi::OsString;
 
-use crate::common::{get_stderr_string, TestEnvironment};
+use crate::common::{get_stderr_string, strip_last_line, TestEnvironment};
 
 #[test]
 fn test_non_utf8_arg() {
@@ -59,7 +59,7 @@ fn test_version() {
 #[test]
 fn test_no_subcommand() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     // Outside of a repo.
@@ -92,21 +92,32 @@ fn test_no_subcommand() {
     test_env.jj_cmd_ok(&repo_path, &["branch", "create", "show"]);
     // TODO: test_env.jj_cmd_ok(&repo_path, &["-r", "help"])
     insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["-r", "log"]), @r###"
-    @  qpvuntsm test.user@example.com 2001-02-03 04:05:07.000 +07:00 help log show 230dd059
+    @  qpvuntsm test.user@example.com 2001-02-03 08:05:07 help log show 230dd059
     │  (empty) (no description set)
     ~
     "###);
     insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["-r", "show"]), @r###"
-    @  qpvuntsm test.user@example.com 2001-02-03 04:05:07.000 +07:00 help log show 230dd059
+    @  qpvuntsm test.user@example.com 2001-02-03 08:05:07 help log show 230dd059
     │  (empty) (no description set)
     ~
     "###);
+
+    // Multiple default command strings work.
+    test_env.add_config(r#"ui.default-command=["commit", "-m", "foo"]"#);
+    test_env.jj_cmd_ok(&repo_path, &["new"]);
+    std::fs::write(repo_path.join("file.txt"), "file").unwrap();
+    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &[]);
+    assert_eq!(stdout, "");
+    insta::assert_snapshot!(stderr, @r###"
+    Working copy now at: kxryzmor 70ac3df3 (empty) (no description set)
+    Parent commit      : lylxulpl 9dbbb452 foo
+    "###);
 }
 
 #[test]
 fn test_ignore_working_copy() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
 
     let repo_path = test_env.env_root().join("repo");
 
@@ -155,7 +166,7 @@ fn test_repo_arg_with_git_clone() {
 #[test]
 fn test_resolve_workspace_directory() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let subdir = repo_path.join("dir").join("subdir");
     std::fs::create_dir_all(&subdir).unwrap();
@@ -210,14 +221,80 @@ fn test_no_workspace_directory() {
     insta::assert_snapshot!(stderr, @r###"
     Error: There is no jj repo in "."
     Hint: It looks like this is a git repo. You can create a jj repo backed by it by running this:
-    jj git init --git-repo=.
+    jj git init --colocate
+    "###);
+}
+
+#[test]
+fn test_bad_path() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+    let subdir = repo_path.join("dir");
+    std::fs::create_dir_all(&subdir).unwrap();
+
+    test_env.add_config("ui.allow-filesets = true");
+
+    // cwd == workspace_root
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["cat", "../out"]);
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    Error: Failed to parse fileset: Invalid file pattern
+    Caused by:
+    1:  --> 1:1
+      |
+    1 | ../out
+      | ^----^
+      |
+      = Invalid file pattern
+    2: Path "../out" is not in the repo "."
+    3: Invalid component ".." in repo-relative path "../out"
+    "###);
+
+    // cwd != workspace_root, can't be parsed as repo-relative path
+    let stderr = test_env.jj_cmd_failure(&subdir, &["cat", "../.."]);
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    Error: Failed to parse fileset: Invalid file pattern
+    Caused by:
+    1:  --> 1:1
+      |
+    1 | ../..
+      | ^---^
+      |
+      = Invalid file pattern
+    2: Path "../.." is not in the repo "../"
+    3: Invalid component ".." in repo-relative path "../"
+    "###);
+
+    // cwd != workspace_root, can be parsed as repo-relative path
+    let stderr = test_env.jj_cmd_failure(test_env.env_root(), &["cat", "-Rrepo", "out"]);
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    Error: Failed to parse fileset: Invalid file pattern
+    Caused by:
+    1:  --> 1:1
+      |
+    1 | out
+      | ^-^
+      |
+      = Invalid file pattern
+    2: Path "out" is not in the repo "repo"
+    3: Invalid component ".." in repo-relative path "../out"
+    Hint: Consider using root:"out" to specify repo-relative path
+    "###);
+
+    test_env.add_config("ui.allow-filesets = false");
+
+    // If fileset/pattern syntax is disabled, no hint should be generated
+    let stderr = test_env.jj_cmd_failure(test_env.env_root(), &["cat", "-Rrepo", "out"]);
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    Error: Path "out" is not in the repo "repo"
+    Caused by: Invalid component ".." in repo-relative path "../out"
     "###);
 }
 
 #[test]
 fn test_broken_repo_structure() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let store_path = repo_path.join(".jj").join("repo").join("store");
     let store_type_path = store_path.join("type");
@@ -268,7 +345,7 @@ fn test_broken_repo_structure() {
 fn test_color_config() {
     let mut test_env = TestEnvironment::default();
 
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     // Test that --color=always is respected.
@@ -355,6 +432,83 @@ fn test_color_config() {
     "###);
 }
 
+#[test]
+fn test_color_ui_messages() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+    test_env.add_config("ui.color = 'always'");
+
+    // hint and error
+    let stderr = test_env.jj_cmd_failure(test_env.env_root(), &["-R."]);
+    insta::assert_snapshot!(stderr, @r###"
+    [1m[38;5;6mHint: [0m[39mUse `jj -h` for a list of available commands.[39m
+    [39mRun `jj config set --user ui.default-command log` to disable this message.[39m
+    [1m[38;5;1mError: [39mThere is no jj repo in "."[0m
+    "###);
+
+    // error source
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["log", ".."]);
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    [1m[38;5;1mError: [39mPath ".." is not in the repo "."[0m
+    [1m[39mCaused by: [0m[39mInvalid component ".." in repo-relative path "../"[39m
+    "###);
+
+    // warning
+    let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["log", "@"]);
+    insta::assert_snapshot!(stderr, @r###"
+    [1m[38;5;3mWarning: [39mThe argument "@" is being interpreted as a path. To specify a revset, pass -r "@" instead.[0m
+    "###);
+
+    // error inlined in template output
+    test_env.jj_cmd_ok(&repo_path, &["new"]);
+    let stdout = test_env.jj_cmd_success(
+        &repo_path,
+        &[
+            "log",
+            "-r@|@--",
+            "--config-toml=templates.log_node='commit_id'",
+            "-Tdescription",
+        ],
+    );
+    insta::assert_snapshot!(stdout, @r###"
+    [38;5;4m167f90e7600a50f85c4f909b53eaf546faa82879[39m
+    [1m[39m<[38;5;1mError: [39mNo Commit available>[0m  [38;5;8m(elided revisions)[39m
+    [38;5;4m0000000000000000000000000000000000000000[39m
+    "###);
+
+    // formatted hint
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["new", ".."]);
+    insta::assert_snapshot!(stderr, @r###"
+    [1m[38;5;1mError: [39mRevset ".." resolved to more than one revision[0m
+    [1m[38;5;6mHint: [0m[39mThe revset ".." resolved to these revisions:[39m
+    [39m  [1m[38;5;5mm[0m[38;5;8mzvwutvl[39m [1m[38;5;4m1[0m[38;5;8m67f90e7[39m [38;5;2m(empty)[39m [38;5;2m(no description set)[39m[39m
+    [39m  [1m[38;5;5mq[0m[38;5;8mpvuntsm[39m [1m[38;5;4m2[0m[38;5;8m30dd059[39m [38;5;2m(empty)[39m [38;5;2m(no description set)[39m[39m
+    [1m[38;5;6mHint: [0m[39mPrefix the expression with 'all:' to allow any number of revisions (i.e. 'all:..').[39m
+    "###);
+
+    // debugging colors
+    let (stdout, _stderr) = test_env.jj_cmd_ok(&repo_path, &["st", "--color", "debug"]);
+    insta::assert_snapshot!(stdout, @r###"
+    <<::The working copy is clean>>
+    <<::Working copy : >>[1m[38;5;13m<<working_copy change_id shortest prefix::m>>[38;5;8m<<working_copy change_id shortest rest::zvwutvl>>[39m<<working_copy:: >>[38;5;12m<<working_copy commit_id shortest prefix::1>>[38;5;8m<<working_copy commit_id shortest rest::67f90e7>>[39m<<working_copy:: >>[38;5;10m<<working_copy empty::(empty)>>[39m<<working_copy:: >>[38;5;10m<<working_copy empty description placeholder::(no description set)>>[0m<<::>>
+    <<::Parent commit: >>[1m[38;5;5m<<change_id shortest prefix::q>>[0m[38;5;8m<<change_id shortest rest::pvuntsm>>[39m<<:: >>[1m[38;5;4m<<commit_id shortest prefix::2>>[0m[38;5;8m<<commit_id shortest rest::30dd059>>[39m<<:: >>[38;5;2m<<empty::(empty)>>[39m<<:: >>[38;5;2m<<empty description placeholder::(no description set)>>[39m<<::>>
+    "###);
+}
+
+#[test]
+fn test_quiet() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    // Can skip message about new working copy with `--quiet`
+    std::fs::write(repo_path.join("file1"), "contents").unwrap();
+    let (_stdout, stderr) =
+        test_env.jj_cmd_ok(&repo_path, &["--quiet", "describe", "-m=new description"]);
+    insta::assert_snapshot!(stderr, @"");
+}
+
 #[test]
 fn test_early_args() {
     // Test that help output parses early args
@@ -395,7 +549,7 @@ fn test_invalid_config() {
 fn test_no_user_configured() {
     // Test that the user is reminded if they haven't configured their name or email
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let assert = test_env
@@ -406,7 +560,7 @@ fn test_no_user_configured() {
     insta::assert_snapshot!(get_stderr_string(&assert), @r###"
     Working copy now at: qpvuntsm 7a7d6016 (empty) without name
     Parent commit      : zzzzzzzz 00000000 (empty) (no description set)
-    Name and email not configured. Until configured, your commits will be created with the empty identity, and can't be pushed to remotes. To configure, run:
+    Warning: Name and email not configured. Until configured, your commits will be created with the empty identity, and can't be pushed to remotes. To configure, run:
       jj config set --user user.name "Some One"
       jj config set --user user.email "someone@example.com"
     "###);
@@ -418,7 +572,7 @@ fn test_no_user_configured() {
     insta::assert_snapshot!(get_stderr_string(&assert), @r###"
     Working copy now at: qpvuntsm 906f8b89 (empty) without email
     Parent commit      : zzzzzzzz 00000000 (empty) (no description set)
-    Name and email not configured. Until configured, your commits will be created with the empty identity, and can't be pushed to remotes. To configure, run:
+    Warning: Name and email not configured. Until configured, your commits will be created with the empty identity, and can't be pushed to remotes. To configure, run:
       jj config set --user user.name "Some One"
       jj config set --user user.email "someone@example.com"
     "###);
@@ -440,40 +594,37 @@ fn test_help() {
                                  specified
           --from <FROM>          Show changes from this revision. Defaults to @ if --to is specified
           --to <TO>              Edit changes in this revision. Defaults to @ if --from is specified
+          --tool <NAME>          Specify diff editor to be used
       -h, --help                 Print help (see more with '--help')
 
     Global Options:
       -R, --repository <REPOSITORY>      Path to repository to operate on
           --ignore-working-copy          Don't snapshot the working copy, and don't update it
+          --ignore-immutable             Allow rewriting immutable commits
           --at-operation <AT_OPERATION>  Operation to load the repo at [default: @] [aliases: at-op]
-      -v, --verbose                      Enable verbose logging
-          --color <WHEN>                 When to colorize output (always, never, auto)
+          --debug                        Enable debug logging
+          --color <WHEN>                 When to colorize output (always, never, debug, auto)
+          --quiet                        Silence non-primary command output
           --no-pager                     Disable the pager
           --config-toml <TOML>           Additional configuration options (can be repeated)
     "###);
 }
 
 #[test]
-fn test_verbose_logging_enabled() {
-    // Test that the verbose flag enabled verbose logging
+fn test_debug_logging_enabled() {
+    // Test that the debug flag enabled debug logging
     let test_env = TestEnvironment::default();
 
-    let (_stdout, stderr) = test_env.jj_cmd_ok(test_env.env_root(), &["version", "-v"]);
+    let (_stdout, stderr) = test_env.jj_cmd_ok(test_env.env_root(), &["version", "--debug"]);
     // Split the first log line into a timestamp and the rest.
     // The timestamp is constant sized so this is a robust operation.
     // Example timestamp: 2022-11-20T06:24:05.477703Z
     let (_timestamp, log_line) = stderr
         .lines()
         .next()
-        .expect("verbose logging on first line")
+        .expect("debug logging on first line")
         .split_at(36);
     // The log format is currently Pretty so we include the terminal markup.
     // Luckily, insta will print this in colour when reviewing.
-    insta::assert_snapshot!(log_line, @"[32m INFO[0m [2mjj_cli::cli_util[0m[2m:[0m verbose logging enabled");
-}
-
-fn strip_last_line(s: &str) -> &str {
-    s.trim_end_matches('\n')
-        .rsplit_once('\n')
-        .map_or(s, |(h, _)| &s[..h.len() + 1])
+    insta::assert_snapshot!(log_line, @"[32m INFO[0m [2mjj_cli::cli_util[0m[2m:[0m debug logging enabled");
 }
