@@ -17,7 +17,7 @@ use crate::common::TestEnvironment;
 #[test]
 fn test_interdiff_basic() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "foo\n").unwrap();
@@ -79,7 +79,7 @@ fn test_interdiff_basic() {
 #[test]
 fn test_interdiff_paths() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "foo\n").unwrap();
@@ -129,7 +129,7 @@ fn test_interdiff_paths() {
 #[test]
 fn test_interdiff_conflicting() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file"), "foo\n").unwrap();
@@ -153,13 +153,13 @@ fn test_interdiff_conflicting() {
     --- a/file
     +++ b/file
     @@ -1,7 +1,1 @@
-    -<<<<<<<
-    -%%%%%%%
+    -<<<<<<< Conflict 1 of 1
+    -%%%%%%% Changes from base to side #1
     --foo
     -+abc
-    -+++++++
+    -+++++++ Contents of side #2
     -bar
-    ->>>>>>>
+    ->>>>>>> Conflict 1 of 1 ends
     +def
     "###);
 }
