@@ -17,7 +17,7 @@ use crate::common::{get_stdout_string, TestEnvironment};
 #[test]
 fn test_log_with_empty_revision() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let stderr = test_env.jj_cmd_cli_error(&repo_path, &["log", "-r="]);
@@ -31,7 +31,7 @@ fn test_log_with_empty_revision() {
 #[test]
 fn test_log_with_no_template() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let stderr = test_env.jj_cmd_cli_error(&repo_path, &["log", "-T"]);
@@ -60,7 +60,7 @@ fn test_log_with_no_template() {
 #[test]
 fn test_log_with_or_without_diff() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "foo\n").unwrap();
@@ -297,7 +297,7 @@ fn test_log_with_or_without_diff() {
 #[test]
 fn test_log_null_terminate_multiline_descriptions() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(
@@ -333,7 +333,7 @@ fn test_log_null_terminate_multiline_descriptions() {
 #[test]
 fn test_log_shortest_accessors() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let render = |rev, template| {
         test_env.jj_cmd_success(
@@ -441,7 +441,7 @@ fn test_log_shortest_accessors() {
 #[test]
 fn test_log_bad_short_prefixes() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     // Error on bad config of short prefixes
     test_env.add_config(r#"revsets.short-prefixes = "!nval!d""#);
@@ -454,7 +454,7 @@ fn test_log_bad_short_prefixes() {
     1 | !nval!d
       | ^---
       |
-      = expected <expression>
+      = expected <identifier> or <expression>
     For help, see https://github.com/martinvonz/jj/blob/main/docs/config.md.
     "###);
 }
@@ -462,7 +462,7 @@ fn test_log_bad_short_prefixes() {
 #[test]
 fn test_log_prefix_highlight_styled() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     fn prefix_format(len: Option<usize>) -> String {
@@ -593,7 +593,7 @@ fn test_log_prefix_highlight_styled() {
 #[test]
 fn test_log_prefix_highlight_counts_hidden_commits() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     test_env.add_config(
         r#"
@@ -661,7 +661,7 @@ fn test_log_prefix_highlight_counts_hidden_commits() {
 #[test]
 fn test_log_short_shortest_length_parameter() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let render = |template| test_env.jj_cmd_success(&repo_path, &["log", "-T", template]);
 
@@ -690,7 +690,7 @@ fn test_log_short_shortest_length_parameter() {
 #[test]
 fn test_log_author_format() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     insta::assert_snapshot!(
@@ -724,7 +724,7 @@ fn test_log_author_format() {
 #[test]
 fn test_log_divergence() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let template = r#"description.first_line() ++ if(divergent, " !divergence!")"#;
 
@@ -757,7 +757,7 @@ fn test_log_divergence() {
 #[test]
 fn test_log_reversed() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(&repo_path, &["describe", "-m", "first"]);
@@ -783,7 +783,7 @@ fn test_log_reversed() {
 #[test]
 fn test_log_filtered_by_path() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "foo\n").unwrap();
@@ -890,7 +890,7 @@ fn test_log_filtered_by_path() {
 #[test]
 fn test_log_limit() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(&repo_path, &["describe", "-m", "a"]);
@@ -972,7 +972,7 @@ fn test_log_limit() {
 #[test]
 fn test_log_warn_path_might_be_revset() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "foo\n").unwrap();
@@ -1029,7 +1029,7 @@ fn test_log_warn_path_might_be_revset() {
 #[test]
 fn test_default_revset() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "foo\n").unwrap();
@@ -1061,7 +1061,7 @@ fn test_default_revset() {
 #[test]
 fn test_default_revset_per_repo() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "foo\n").unwrap();
@@ -1088,7 +1088,7 @@ fn test_default_revset_per_repo() {
 #[test]
 fn test_multiple_revsets() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     for name in ["foo", "bar", "baz"] {
         test_env.jj_cmd_ok(&repo_path, &["new", "-m", name]);
@@ -1127,7 +1127,7 @@ fn test_multiple_revsets() {
 fn test_graph_template_color() {
     // Test that color codes from a multi-line template don't span the graph lines.
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(
@@ -1161,13 +1161,21 @@ fn test_graph_template_color() {
     │  [38;5;1mthird line[39m
     ◉
     "###);
+    let stdout = test_env.jj_cmd_success(&repo_path, &["--color=debug", "log", "-T", template]);
+    insta::assert_snapshot!(stdout, @r###"
+    <<node::@>>  [1m[38;5;2m<<log working_copy description::single line>>[0m
+    <<node::◉>>  [38;5;1m<<log description::first line>>[39m
+    │  [38;5;1m<<log description::second line>>[39m
+    │  [38;5;1m<<log description::third line>>[39m
+    <<node::◉>>
+    "###);
 }
 
 #[test]
 fn test_graph_styles() {
     // Test that different graph styles are available.
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(&repo_path, &["commit", "-m", "initial"]);
@@ -1272,7 +1280,7 @@ fn test_graph_styles() {
 #[test]
 fn test_log_word_wrap() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let render = |args: &[&str], columns: u32, word_wrap: bool| {
         let mut args = args.to_vec();
@@ -1369,7 +1377,7 @@ fn test_log_word_wrap() {
 fn test_elided() {
     // Test that elided commits are shown as synthetic nodes.
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(&repo_path, &["describe", "-m", "initial"]);
@@ -1468,7 +1476,7 @@ fn test_elided() {
 fn test_log_with_custom_symbols() {
     // Test that elided commits are shown as synthetic nodes.
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(&repo_path, &["describe", "-m", "initial"]);
