@@ -16,12 +16,10 @@ use std::path::Path;
 
 use crate::common::TestEnvironment;
 
-pub mod common;
-
 #[test]
 fn test_new() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(&repo_path, &["describe", "-m", "add a file"]);
@@ -68,7 +66,7 @@ fn test_new() {
 #[test]
 fn test_new_merge() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(&repo_path, &["branch", "create", "main"]);
@@ -123,10 +121,14 @@ fn test_new_merge() {
     // `jj merge` with less than two arguments is an error
     let stderr = test_env.jj_cmd_cli_error(&repo_path, &["merge"]);
     insta::assert_snapshot!(stderr, @r###"
+    Warning: `jj merge` is deprecated; use `jj new` instead, which is equivalent
+    Warning: `jj merge` will be removed in a future version, and this will be a hard error
     Error: Merge requires at least two revisions
     "###);
     let stderr = test_env.jj_cmd_cli_error(&repo_path, &["merge", "main"]);
     insta::assert_snapshot!(stderr, @r###"
+    Warning: `jj merge` is deprecated; use `jj new` instead, which is equivalent
+    Warning: `jj merge` will be removed in a future version, and this will be a hard error
     Error: Merge requires at least two revisions
     "###);
 
@@ -135,18 +137,25 @@ fn test_new_merge() {
     insta::assert_snapshot!(stderr, @r###"
     Error: More than one revset resolved to revision 3a44e52b073c
     "###);
+    // if prefixed with all:, duplicates are allowed
+    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["new", "@", "all:visible_heads()"]);
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Working copy now at: xznxytkn dddeb489 (empty) (no description set)
+    Parent commit      : wqnwkozp 3a44e52b (empty) (no description set)
+    "###);
 
     // merge with root
     let stderr = test_env.jj_cmd_failure(&repo_path, &["new", "@", "root()"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Cannot merge with root revision
+    Error: The Git backend does not support creating merge commits with the root commit as one of the parents.
     "###);
 }
 
 #[test]
 fn test_new_insert_after() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     setup_before_insertion(&test_env, &repo_path);
     insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r###"
@@ -162,20 +171,10 @@ fn test_new_insert_after() {
     ◉  root
     "###);
 
-    // --insert-after can be repeated (this does not affect the outcome); --after is
-    // an alias
+    // --insert-after can be repeated; --after is an alias
     let (stdout, stderr) = test_env.jj_cmd_ok(
         &repo_path,
-        &[
-            "new",
-            "--insert-after",
-            "-m",
-            "G",
-            "--after",
-            "B",
-            "--after",
-            "D",
-        ],
+        &["new", "-m", "G", "--insert-after", "B", "--after", "D"],
     );
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
@@ -200,7 +199,7 @@ fn test_new_insert_after() {
     "###);
 
     let (stdout, stderr) =
-        test_env.jj_cmd_ok(&repo_path, &["new", "--insert-after", "-m", "H", "D"]);
+        test_env.jj_cmd_ok(&repo_path, &["new", "-m", "H", "--insert-after", "D"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Rebased 3 descendant commits
@@ -223,12 +222,12 @@ fn test_new_insert_after() {
     ◉  root
     "###);
 
-    // --after cannot be used with --before
-    let stderr = test_env.jj_cmd_cli_error(&repo_path, &["new", "--after", "B", "--before", "D"]);
+    // --after cannot be used with revisions
+    let stderr = test_env.jj_cmd_cli_error(&repo_path, &["new", "--after", "B", "D"]);
     insta::assert_snapshot!(stderr, @r###"
-    error: the argument '--insert-after' cannot be used with '--insert-before'
+    error: the argument '--insert-after <INSERT_AFTER>' cannot be used with '[REVISIONS]...'
 
-    Usage: jj new --insert-after <REVISIONS>...
+    Usage: jj new --insert-after <INSERT_AFTER> [REVISIONS]...
 
     For more information, try '--help'.
     "###);
@@ -237,7 +236,7 @@ fn test_new_insert_after() {
 #[test]
 fn test_new_insert_after_children() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     setup_before_insertion(&test_env, &repo_path);
     insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r###"
@@ -256,8 +255,18 @@ fn test_new_insert_after_children() {
     // Check that inserting G after A and C doesn't try to rebase B (which is
     // initially a child of A) onto G as that would create a cycle since B is
     // a parent of C which is a parent G.
-    let (stdout, stderr) =
-        test_env.jj_cmd_ok(&repo_path, &["new", "--insert-after", "-m", "G", "A", "C"]);
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &repo_path,
+        &[
+            "new",
+            "-m",
+            "G",
+            "--insert-after",
+            "A",
+            "--insert-after",
+            "C",
+        ],
+    );
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Working copy now at: kxryzmor b48d4d73 (empty) G
@@ -284,7 +293,7 @@ fn test_new_insert_after_children() {
 #[test]
 fn test_new_insert_before() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     setup_before_insertion(&test_env, &repo_path);
     insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r###"
@@ -300,15 +309,25 @@ fn test_new_insert_before() {
     ◉  root
     "###);
 
-    let (stdout, stderr) =
-        test_env.jj_cmd_ok(&repo_path, &["new", "--insert-before", "-m", "G", "C", "F"]);
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &repo_path,
+        &[
+            "new",
+            "-m",
+            "G",
+            "--insert-before",
+            "C",
+            "--insert-before",
+            "F",
+        ],
+    );
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Rebased 2 descendant commits
-    Working copy now at: kxryzmor ff6bbbc7 (empty) G
-    Parent commit      : znkkpsqq 41a89ffc E | (empty) E
-    Parent commit      : vruxwmqv c9257eff D | (empty) D
+    Working copy now at: kxryzmor 19e53931 (empty) G
     Parent commit      : kkmpptxz 6041917c B | (empty) B
+    Parent commit      : vruxwmqv c9257eff D | (empty) D
+    Parent commit      : znkkpsqq 41a89ffc E | (empty) E
     "###);
     insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r###"
     ◉  F
@@ -316,20 +335,30 @@ fn test_new_insert_before() {
     ├─╯
     @      G
     ├─┬─╮
-    │ │ ◉  B
-    │ │ ◉  A
+    │ │ ◉  E
     │ ◉ │  D
     │ ├─╯
-    ◉ │  E
+    ◉ │  B
+    ◉ │  A
     ├─╯
     ◉  root
     "###);
+
+    // --before cannot be used with revisions
+    let stderr = test_env.jj_cmd_cli_error(&repo_path, &["new", "--before", "B", "D"]);
+    insta::assert_snapshot!(stderr, @r###"
+    error: the argument '--insert-before <INSERT_BEFORE>' cannot be used with '[REVISIONS]...'
+
+    Usage: jj new --insert-before <INSERT_BEFORE> [REVISIONS]...
+
+    For more information, try '--help'.
+    "###);
 }
 
 #[test]
 fn test_new_insert_before_root_successors() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     setup_before_insertion(&test_env, &repo_path);
     insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r###"
@@ -345,8 +374,18 @@ fn test_new_insert_before_root_successors() {
     ◉  root
     "###);
 
-    let (stdout, stderr) =
-        test_env.jj_cmd_ok(&repo_path, &["new", "--insert-before", "-m", "G", "A", "D"]);
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &repo_path,
+        &[
+            "new",
+            "-m",
+            "G",
+            "--insert-before",
+            "A",
+            "--insert-before",
+            "D",
+        ],
+    );
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Rebased 5 descendant commits
@@ -371,7 +410,7 @@ fn test_new_insert_before_root_successors() {
 #[test]
 fn test_new_insert_before_no_loop() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     setup_before_insertion(&test_env, &repo_path);
     let template = r#"commit_id.short() ++ " " ++ if(description, description, "root")"#;
@@ -389,8 +428,18 @@ fn test_new_insert_before_no_loop() {
     ◉  000000000000 root
     "###);
 
-    let stderr =
-        test_env.jj_cmd_failure(&repo_path, &["new", "--insert-before", "-m", "G", "A", "C"]);
+    let stderr = test_env.jj_cmd_failure(
+        &repo_path,
+        &[
+            "new",
+            "-m",
+            "G",
+            "--insert-before",
+            "A",
+            "--insert-before",
+            "C",
+        ],
+    );
     insta::assert_snapshot!(stderr, @r###"
     Error: Refusing to create a loop: commit 6041917ceeb5 would be both an ancestor and a descendant of the new commit
     "###);
@@ -399,7 +448,7 @@ fn test_new_insert_before_no_loop() {
 #[test]
 fn test_new_insert_before_no_root_merge() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     setup_before_insertion(&test_env, &repo_path);
     insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r###"
@@ -415,33 +464,27 @@ fn test_new_insert_before_no_root_merge() {
     ◉  root
     "###);
 
-    let (stdout, stderr) =
-        test_env.jj_cmd_ok(&repo_path, &["new", "--insert-before", "-m", "G", "B", "D"]);
-    insta::assert_snapshot!(stdout, @"");
+    let stderr = test_env.jj_cmd_failure(
+        &repo_path,
+        &[
+            "new",
+            "-m",
+            "G",
+            "--insert-before",
+            "B",
+            "--insert-before",
+            "D",
+        ],
+    );
     insta::assert_snapshot!(stderr, @r###"
-    Rebased 4 descendant commits
-    Working copy now at: kxryzmor bf9fc493 (empty) G
-    Parent commit      : qpvuntsm 65b1ef43 A | (empty) A
-    "###);
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r###"
-    ◉    F
-    ├─╮
-    │ ◉  E
-    ◉ │  D
-    │ │ ◉  C
-    │ │ ◉  B
-    ├───╯
-    @ │  G
-    ◉ │  A
-    ├─╯
-    ◉  root
+    Error: The Git backend does not support creating merge commits with the root commit as one of the parents.
     "###);
 }
 
 #[test]
 fn test_new_insert_before_root() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     setup_before_insertion(&test_env, &repo_path);
     insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r###"
@@ -458,12 +501,193 @@ fn test_new_insert_before_root() {
     "###);
 
     let stderr =
-        test_env.jj_cmd_failure(&repo_path, &["new", "--insert-before", "-m", "G", "root()"]);
+        test_env.jj_cmd_failure(&repo_path, &["new", "-m", "G", "--insert-before", "root()"]);
     insta::assert_snapshot!(stderr, @r###"
     Error: The root commit 000000000000 is immutable
     "###);
 }
 
+#[test]
+fn test_new_insert_after_before() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+    setup_before_insertion(&test_env, &repo_path);
+    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r###"
+    @    F
+    ├─╮
+    │ ◉  E
+    ◉ │  D
+    ├─╯
+    │ ◉  C
+    │ ◉  B
+    │ ◉  A
+    ├─╯
+    ◉  root
+    "###);
+
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &repo_path,
+        &["new", "-m", "G", "--after", "C", "--before", "F"],
+    );
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Rebased 1 descendant commits
+    Working copy now at: kxryzmor 33be1218 (empty) G
+    Parent commit      : mzvwutvl ec18c57d C | (empty) C
+    "###);
+    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r###"
+    ◉      F
+    ├─┬─╮
+    │ │ @  G
+    │ │ ◉  C
+    │ │ ◉  B
+    │ │ ◉  A
+    │ ◉ │  E
+    │ ├─╯
+    ◉ │  D
+    ├─╯
+    ◉  root
+    "###);
+
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &repo_path,
+        &["new", "-m", "H", "--after", "D", "--before", "B"],
+    );
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Rebased 4 descendant commits
+    Working copy now at: uyznsvlq fcf8281b (empty) H
+    Parent commit      : vruxwmqv c9257eff D | (empty) D
+    "###);
+    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r###"
+    ◉      F
+    ├─┬─╮
+    │ │ ◉  G
+    │ │ ◉  C
+    │ │ ◉    B
+    │ │ ├─╮
+    │ │ │ @  H
+    ├─────╯
+    ◉ │ │  D
+    │ │ ◉  A
+    ├───╯
+    │ ◉  E
+    ├─╯
+    ◉  root
+    "###);
+}
+
+#[test]
+fn test_new_insert_after_before_no_loop() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+    setup_before_insertion(&test_env, &repo_path);
+    let template = r#"commit_id.short() ++ " " ++ if(description, description, "root")"#;
+    let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", template]);
+    insta::assert_snapshot!(stdout, @r###"
+    @    7705d353bf5d F
+    ├─╮
+    │ ◉  41a89ffcbba2 E
+    ◉ │  c9257eff5bf9 D
+    ├─╯
+    │ ◉  ec18c57d72d8 C
+    │ ◉  6041917ceeb5 B
+    │ ◉  65b1ef43c737 A
+    ├─╯
+    ◉  000000000000 root
+    "###);
+
+    let stderr = test_env.jj_cmd_failure(
+        &repo_path,
+        &[
+            "new",
+            "-m",
+            "G",
+            "--insert-before",
+            "A",
+            "--insert-after",
+            "C",
+        ],
+    );
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Refusing to create a loop: commit ec18c57d72d8 would be both an ancestor and a descendant of the new commit
+    "###);
+}
+
+#[test]
+fn test_new_conflicting_branches() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    test_env.jj_cmd_ok(&repo_path, &["describe", "-m", "one"]);
+    test_env.jj_cmd_ok(&repo_path, &["new", "-m", "two", "@-"]);
+    test_env.jj_cmd_ok(&repo_path, &["branch", "create", "foo"]);
+    test_env.jj_cmd_ok(
+        &repo_path,
+        &[
+            "--at-op=@-",
+            "branch",
+            "create",
+            "foo",
+            "-r",
+            r#"description("one")"#,
+        ],
+    );
+
+    // Trigger resolution of concurrent operations
+    test_env.jj_cmd_ok(&repo_path, &["st"]);
+
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["new", "foo"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Revset "foo" resolved to more than one revision
+    Hint: Branch foo resolved to multiple revisions because it's conflicted.
+    It resolved to these revisions:
+      kkmpptxz 66c6502d foo?? | (empty) two
+      qpvuntsm a9330854 foo?? | (empty) one
+    Hint: Set which revision the branch points to with `jj branch set foo -r <REVISION>`.
+    "###);
+}
+
+#[test]
+fn test_new_conflicting_change_ids() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    test_env.jj_cmd_ok(&repo_path, &["describe", "-m", "one"]);
+    test_env.jj_cmd_ok(&repo_path, &["--at-op=@-", "describe", "-m", "two"]);
+
+    // Trigger resolution of concurrent operations
+    test_env.jj_cmd_ok(&repo_path, &["st"]);
+
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["new", "qpvuntsm"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Revset "qpvuntsm" resolved to more than one revision
+    Hint: The revset "qpvuntsm" resolved to these revisions:
+      qpvuntsm?? d2ae6806 (empty) two
+      qpvuntsm?? a9330854 (empty) one
+    Hint: Some of these commits have the same change id. Abandon one of them with `jj abandon -r <REVISION>`.
+    "###);
+}
+
+#[test]
+fn test_new_error_revision_does_not_exist() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    test_env.jj_cmd_ok(&repo_path, &["describe", "-m", "one"]);
+    test_env.jj_cmd_ok(&repo_path, &["new", "-m", "two"]);
+
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["new", "this"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Revision "this" doesn't exist
+    "###);
+}
+
 fn setup_before_insertion(test_env: &TestEnvironment, repo_path: &Path) {
     test_env.jj_cmd_ok(repo_path, &["branch", "create", "A"]);
     test_env.jj_cmd_ok(repo_path, &["commit", "-m", "A"]);
