@@ -14,6 +14,8 @@
 
 use std::path::Path;
 
+use indoc::indoc;
+
 use crate::common::TestEnvironment;
 
 fn create_commit(
@@ -43,7 +45,7 @@ fn get_log_output(test_env: &TestEnvironment, repo_path: &Path) -> String {
 #[test]
 fn test_resolution() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     create_commit(&test_env, &repo_path, "base", &[], &[("file", "base\n")]);
@@ -67,14 +69,14 @@ fn test_resolution() {
     insta::assert_snapshot!(
     std::fs::read_to_string(repo_path.join("file")).unwrap()
         , @r###"
-            <<<<<<<
-            %%%%%%%
-            -base
-            +a
-            +++++++
-            b
-            >>>>>>>
-            "###);
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
+    -base
+    +a
+    +++++++ Contents of side #2
+    b
+    >>>>>>> Conflict 1 of 1 ends
+    "###);
 
     let editor_script = test_env.set_up_fake_editor();
     // Check that output file starts out empty and resolve the conflict
@@ -95,26 +97,71 @@ fn test_resolution() {
     insta::assert_snapshot!(
         std::fs::read_to_string(test_env.env_root().join("editor0")).unwrap(), @r###"
     "###);
-    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff"]), 
+    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff", "--git"]), 
     @r###"
-    Resolved conflict in file:
-       1    1: <<<<<<<resolution
-       2     : %%%%%%%
-       3     : -base
-       4     : +a
-       5     : +++++++
-       6     : b
-       7     : >>>>>>>
+    diff --git a/file b/file
+    index 0000000000...88425ec521 100644
+    --- a/file
+    +++ b/file
+    @@ -1,7 +1,1 @@
+    -<<<<<<< Conflict 1 of 1
+    -%%%%%%% Changes from base to side #1
+    --base
+    -+a
+    -+++++++ Contents of side #2
+    -b
+    ->>>>>>> Conflict 1 of 1 ends
+    +resolution
     "###);
     insta::assert_snapshot!(test_env.jj_cmd_cli_error(&repo_path, &["resolve", "--list"]), 
     @r###"
     Error: No conflicts found at this revision
     "###);
 
+    // Try again with --tool=<name>
+    test_env.jj_cmd_ok(&repo_path, &["undo"]);
+    std::fs::write(&editor_script, "write\nresolution\n").unwrap();
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &repo_path,
+        &[
+            "resolve",
+            "--config-toml=ui.merge-editor='false'",
+            "--tool=fake-editor",
+        ],
+    );
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Resolving conflicts in: file
+    Working copy now at: vruxwmqv 1a70c7c6 conflict | conflict
+    Parent commit      : zsuskuln aa493daf a | a
+    Parent commit      : royxmykx db6a4daf b | b
+    Added 0 files, modified 1 files, removed 0 files
+    "###);
+    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff", "--git"]),
+    @r###"
+    diff --git a/file b/file
+    index 0000000000...88425ec521 100644
+    --- a/file
+    +++ b/file
+    @@ -1,7 +1,1 @@
+    -<<<<<<< Conflict 1 of 1
+    -%%%%%%% Changes from base to side #1
+    --base
+    -+a
+    -+++++++ Contents of side #2
+    -b
+    ->>>>>>> Conflict 1 of 1 ends
+    +resolution
+    "###);
+    insta::assert_snapshot!(test_env.jj_cmd_cli_error(&repo_path, &["resolve", "--list"]),
+    @r###"
+    Error: No conflicts found at this revision
+    "###);
+
     // Check that the output file starts with conflict markers if
     // `merge-tool-edits-conflict-markers=true`
     test_env.jj_cmd_ok(&repo_path, &["undo"]);
-    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff"]), 
+    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff", "--git"]),
     @"");
     std::fs::write(
         &editor_script,
@@ -131,44 +178,50 @@ fn test_resolution() {
     );
     insta::assert_snapshot!(
         std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r###"
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
     -base
     +a
-    +++++++
+    +++++++ Contents of side #2
     b
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     "###);
-    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff"]), 
+    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff", "--git"]), 
     @r###"
-    Resolved conflict in file:
-       1    1: <<<<<<<resolution
-       2     : %%%%%%%
-       3     : -base
-       4     : +a
-       5     : +++++++
-       6     : b
-       7     : >>>>>>>
+    diff --git a/file b/file
+    index 0000000000...88425ec521 100644
+    --- a/file
+    +++ b/file
+    @@ -1,7 +1,1 @@
+    -<<<<<<< Conflict 1 of 1
+    -%%%%%%% Changes from base to side #1
+    --base
+    -+a
+    -+++++++ Contents of side #2
+    -b
+    ->>>>>>> Conflict 1 of 1 ends
+    +resolution
     "###);
 
     // Check that if merge tool leaves conflict markers in output file and
     // `merge-tool-edits-conflict-markers=true`, these markers are properly parsed.
     test_env.jj_cmd_ok(&repo_path, &["undo"]);
-    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff"]), 
+    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff", "--git"]), 
     @"");
     std::fs::write(
         &editor_script,
         [
             "dump editor2",
-            "write
-<<<<<<<
-%%%%%%%
--some
-+fake
-+++++++
-conflict
->>>>>>>
-",
+            indoc! {"
+                write
+                <<<<<<<
+                %%%%%%%
+                -some
+                +fake
+                +++++++
+                conflict
+                >>>>>>>
+            "},
         ]
         .join("\0"),
     )
@@ -185,40 +238,46 @@ conflict
     insta::assert_snapshot!(stderr, @r###"
     Resolving conflicts in: file
     New conflicts appeared in these commits:
-      vruxwmqv ff4e8c6b conflict | (conflict) conflict
+      vruxwmqv 7699b9c3 conflict | (conflict) conflict
     To resolve the conflicts, start by updating to it:
       jj new vruxwmqvtpmx
     Then use `jj resolve`, or edit the conflict markers in the file directly.
     Once the conflicts are resolved, you may want inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
-    Working copy now at: vruxwmqv ff4e8c6b conflict | (conflict) conflict
+    Working copy now at: vruxwmqv 7699b9c3 conflict | (conflict) conflict
     Parent commit      : zsuskuln aa493daf a | a
     Parent commit      : royxmykx db6a4daf b | b
     Added 0 files, modified 1 files, removed 0 files
-    After this operation, some files at this revision still have conflicts:
+    There are unresolved conflicts at these paths:
     file    2-sided conflict
     "###);
     insta::assert_snapshot!(
         std::fs::read_to_string(test_env.env_root().join("editor2")).unwrap(), @r###"
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
     -base
     +a
-    +++++++
+    +++++++ Contents of side #2
     b
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     "###);
     // Note the "Modified" below
-    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff"]), 
+    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff", "--git"]), 
     @r###"
-    Modified conflict in file:
-       1    1: <<<<<<<
-       2    2: %%%%%%%
-       3    3: -basesome
-       4    4: +afake
-       5    5: +++++++
-       6    6: bconflict
-       7    7: >>>>>>>
+    diff --git a/file b/file
+    --- a/file
+    +++ b/file
+    @@ -1,7 +1,7 @@
+     <<<<<<< Conflict 1 of 1
+     %%%%%%% Changes from base to side #1
+    --base
+    -+a
+    +-some
+    ++fake
+     +++++++ Contents of side #2
+    -b
+    +conflict
+     >>>>>>> Conflict 1 of 1 ends
     "###);
     insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["resolve", "--list"]), 
     @r###"
@@ -229,21 +288,22 @@ conflict
     // `merge-tool-edits-conflict-markers=false` or is not specified,
     // `jj` considers the conflict resolved.
     test_env.jj_cmd_ok(&repo_path, &["undo"]);
-    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff"]), 
+    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff", "--git"]), 
     @"");
     std::fs::write(
         &editor_script,
         [
             "dump editor3",
-            "write
-<<<<<<<
-%%%%%%%
--some
-+fake
-+++++++
-conflict
->>>>>>>
-",
+            indoc! {"
+                write
+                <<<<<<<
+                %%%%%%%
+                -some
+                +fake
+                +++++++
+                conflict
+                >>>>>>>
+            "},
         ]
         .join("\0"),
     )
@@ -252,7 +312,7 @@ conflict
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Resolving conflicts in: file
-    Working copy now at: vruxwmqv 95418cb8 conflict | conflict
+    Working copy now at: vruxwmqv 3166dfd2 conflict | conflict
     Parent commit      : zsuskuln aa493daf a | a
     Parent commit      : royxmykx db6a4daf b | b
     Added 0 files, modified 1 files, removed 0 files
@@ -261,16 +321,27 @@ conflict
         std::fs::read_to_string(test_env.env_root().join("editor3")).unwrap(), @r###"
     "###);
     // Note the "Resolved" below
-    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff"]), 
+    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff", "--git"]), 
     @r###"
-    Resolved conflict in file:
-       1    1: <<<<<<<
-       2    2: %%%%%%%
-       3    3: -basesome
-       4    4: +afake
-       5    5: +++++++
-       6    6: bconflict
-       7    7: >>>>>>>
+    diff --git a/file b/file
+    index 0000000000...0610716cc1 100644
+    --- a/file
+    +++ b/file
+    @@ -1,7 +1,7 @@
+    -<<<<<<< Conflict 1 of 1
+    -%%%%%%% Changes from base to side #1
+    --base
+    -+a
+    -+++++++ Contents of side #2
+    -b
+    ->>>>>>> Conflict 1 of 1 ends
+    +<<<<<<<
+    +%%%%%%%
+    +-some
+    ++fake
+    ++++++++
+    +conflict
+    +>>>>>>>
     "###);
     insta::assert_snapshot!(test_env.jj_cmd_cli_error(&repo_path, &["resolve", "--list"]), 
     @r###"
@@ -299,7 +370,7 @@ fn check_resolve_produces_input_file(
     assert_eq!(
         &test_env.jj_cmd_failure(repo_path, &["resolve", "--config-toml", &merge_arg_config]),
         "Resolving conflicts in: file\nError: Failed to resolve conflicts\nCaused by: The output \
-         file is either unchanged or empty after the editor quit (run with --verbose to see the \
+         file is either unchanged or empty after the editor quit (run with --debug to see the \
          exact invocation).\n"
     );
 }
@@ -307,7 +378,7 @@ fn check_resolve_produces_input_file(
 #[test]
 fn test_normal_conflict_input_files() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     create_commit(&test_env, &repo_path, "base", &[], &[("file", "base\n")]);
@@ -331,14 +402,14 @@ fn test_normal_conflict_input_files() {
     insta::assert_snapshot!(
     std::fs::read_to_string(repo_path.join("file")).unwrap()
         , @r###"
-            <<<<<<<
-            %%%%%%%
-            -base
-            +a
-            +++++++
-            b
-            >>>>>>>
-            "###);
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
+    -base
+    +a
+    +++++++ Contents of side #2
+    b
+    >>>>>>> Conflict 1 of 1 ends
+    "###);
 
     check_resolve_produces_input_file(&mut test_env, &repo_path, "base", "base\n");
     check_resolve_produces_input_file(&mut test_env, &repo_path, "left", "a\n");
@@ -348,7 +419,7 @@ fn test_normal_conflict_input_files() {
 #[test]
 fn test_baseless_conflict_input_files() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     create_commit(&test_env, &repo_path, "base", &[], &[]);
@@ -372,12 +443,12 @@ fn test_baseless_conflict_input_files() {
     insta::assert_snapshot!(
     std::fs::read_to_string(repo_path.join("file")).unwrap()
         , @r###"
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
     +a
-    +++++++
+    +++++++ Contents of side #2
     b
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     "###);
 
     check_resolve_produces_input_file(&mut test_env, &repo_path, "base", "");
@@ -388,7 +459,7 @@ fn test_baseless_conflict_input_files() {
 #[test]
 fn test_too_many_parents() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     create_commit(&test_env, &repo_path, "base", &[], &[("file", "base\n")]);
@@ -408,6 +479,7 @@ fn test_too_many_parents() {
 
     let error = test_env.jj_cmd_failure(&repo_path, &["resolve"]);
     insta::assert_snapshot!(error, @r###"
+    Hint: Using default editor ':builtin'; run `jj config set --user ui.merge-editor :builtin` to disable this message.
     Resolving conflicts in: file
     Error: Failed to resolve conflicts
     Caused by: The conflict at "file" has 3 sides. At most 2 sides are supported.
@@ -417,7 +489,7 @@ fn test_too_many_parents() {
 #[test]
 fn test_edit_delete_conflict_input_files() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     create_commit(&test_env, &repo_path, "base", &[], &[("file", "base\n")]);
@@ -442,12 +514,12 @@ fn test_edit_delete_conflict_input_files() {
     insta::assert_snapshot!(
     std::fs::read_to_string(repo_path.join("file")).unwrap()
         , @r###"
-    <<<<<<<
-    +++++++
+    <<<<<<< Conflict 1 of 1
+    +++++++ Contents of side #1
     a
-    %%%%%%%
+    %%%%%%% Changes from base to side #2
     -base
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     "###);
 
     check_resolve_produces_input_file(&mut test_env, &repo_path, "base", "base\n");
@@ -458,7 +530,7 @@ fn test_edit_delete_conflict_input_files() {
 #[test]
 fn test_file_vs_dir() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     create_commit(&test_env, &repo_path, "base", &[], &[("file", "base\n")]);
@@ -485,6 +557,7 @@ fn test_file_vs_dir() {
     "###);
     let error = test_env.jj_cmd_failure(&repo_path, &["resolve"]);
     insta::assert_snapshot!(error, @r###"
+    Hint: Using default editor ':builtin'; run `jj config set --user ui.merge-editor :builtin` to disable this message.
     Resolving conflicts in: file
     Error: Failed to resolve conflicts
     Caused by: Only conflicts that involve normal files (not symlinks, not executable, etc.) are supported. Conflict summary for "file":
@@ -499,7 +572,7 @@ fn test_file_vs_dir() {
 #[test]
 fn test_description_with_dir_and_deletion() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     create_commit(&test_env, &repo_path, "base", &[], &[("file", "base\n")]);
@@ -541,6 +614,7 @@ fn test_description_with_dir_and_deletion() {
     "###);
     let error = test_env.jj_cmd_failure(&repo_path, &["resolve"]);
     insta::assert_snapshot!(error, @r###"
+    Hint: Using default editor ':builtin'; run `jj config set --user ui.merge-editor :builtin` to disable this message.
     Resolving conflicts in: file
     Error: Failed to resolve conflicts
     Caused by: Only conflicts that involve normal files (not symlinks, not executable, etc.) are supported. Conflict summary for "file":
@@ -556,7 +630,7 @@ fn test_description_with_dir_and_deletion() {
 #[test]
 fn test_multiple_conflicts() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     create_commit(
@@ -612,24 +686,24 @@ fn test_multiple_conflicts() {
     insta::assert_snapshot!(
     std::fs::read_to_string(repo_path.join("this_file_has_a_very_long_name_to_test_padding")).unwrap()
         , @r###"
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
     -first base
     +first a
-    +++++++
+    +++++++ Contents of side #2
     first b
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     "###);
     insta::assert_snapshot!(
     std::fs::read_to_string(repo_path.join("another_file")).unwrap()
         , @r###"
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
     -second base
     +second a
-    +++++++
+    +++++++ Contents of side #2
     second b
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     "###);
     insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["resolve", "--list"]), 
     @r###"
@@ -652,29 +726,34 @@ fn test_multiple_conflicts() {
     insta::assert_snapshot!(stderr, @r###"
     Resolving conflicts in: another_file
     New conflicts appeared in these commits:
-      vruxwmqv c3c25bce conflict | (conflict) conflict
+      vruxwmqv 6a90e546 conflict | (conflict) conflict
     To resolve the conflicts, start by updating to it:
       jj new vruxwmqvtpmx
     Then use `jj resolve`, or edit the conflict markers in the file directly.
     Once the conflicts are resolved, you may want inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
-    Working copy now at: vruxwmqv c3c25bce conflict | (conflict) conflict
+    Working copy now at: vruxwmqv 6a90e546 conflict | (conflict) conflict
     Parent commit      : zsuskuln de7553ef a | a
     Parent commit      : royxmykx f68bc2f0 b | b
     Added 0 files, modified 1 files, removed 0 files
-    After this operation, some files at this revision still have conflicts:
+    There are unresolved conflicts at these paths:
     this_file_has_a_very_long_name_to_test_padding 2-sided conflict
     "###);
-    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff"]), 
+    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff", "--git"]), 
     @r###"
-    Resolved conflict in another_file:
-       1     : <<<<<<<
-       2     : %%%%%%%
-       3    1: -secondresolution baseanother_file
-       4     : +second a
-       5     : +++++++
-       6     : second b
-       7     : >>>>>>>
+    diff --git a/another_file b/another_file
+    index 0000000000...a9fcc7d486 100644
+    --- a/another_file
+    +++ b/another_file
+    @@ -1,7 +1,1 @@
+    -<<<<<<< Conflict 1 of 1
+    -%%%%%%% Changes from base to side #1
+    --second base
+    -+second a
+    -+++++++ Contents of side #2
+    -second b
+    ->>>>>>> Conflict 1 of 1 ends
+    +resolution another_file
     "###);
     insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["resolve", "--list"]), 
     @r###"
@@ -686,25 +765,12 @@ fn test_multiple_conflicts() {
     std::fs::write(&editor_script, "expect\n\0write\nresolution another_file\n").unwrap();
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["resolve", "--quiet", "another_file"]);
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @r###"
-    Resolving conflicts in: another_file
-    New conflicts appeared in these commits:
-      vruxwmqv fd3874cd conflict | (conflict) conflict
-    To resolve the conflicts, start by updating to it:
-      jj new vruxwmqvtpmx
-    Then use `jj resolve`, or edit the conflict markers in the file directly.
-    Once the conflicts are resolved, you may want inspect the result with `jj diff`.
-    Then run `jj squash` to move the resolution into the conflicted commit.
-    Working copy now at: vruxwmqv fd3874cd conflict | (conflict) conflict
-    Parent commit      : zsuskuln de7553ef a | a
-    Parent commit      : royxmykx f68bc2f0 b | b
-    Added 0 files, modified 1 files, removed 0 files
-    "###);
+    insta::assert_snapshot!(stderr, @"");
 
     // For the rest of the test, we call `jj resolve` several times in a row to
     // resolve each conflict in the order it chooses.
     test_env.jj_cmd_ok(&repo_path, &["undo"]);
-    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff"]), 
+    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff", "--git"]), 
     @"");
     std::fs::write(
         &editor_script,
@@ -712,16 +778,21 @@ fn test_multiple_conflicts() {
     )
     .unwrap();
     test_env.jj_cmd_ok(&repo_path, &["resolve"]);
-    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff"]), 
+    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff", "--git"]), 
     @r###"
-    Resolved conflict in another_file:
-       1     : <<<<<<<
-       2     : %%%%%%%
-       3    1: first resolution for auto-secondchosen basefile
-       4     : +second a
-       5     : +++++++
-       6     : second b
-       7     : >>>>>>>
+    diff --git a/another_file b/another_file
+    index 0000000000...7903e1c1c7 100644
+    --- a/another_file
+    +++ b/another_file
+    @@ -1,7 +1,1 @@
+    -<<<<<<< Conflict 1 of 1
+    -%%%%%%% Changes from base to side #1
+    --second base
+    -+second a
+    -+++++++ Contents of side #2
+    -second b
+    ->>>>>>> Conflict 1 of 1 ends
+    +first resolution for auto-chosen file
     "###);
     insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["resolve", "--list"]), 
     @r###"
@@ -734,24 +805,34 @@ fn test_multiple_conflicts() {
     .unwrap();
 
     test_env.jj_cmd_ok(&repo_path, &["resolve"]);
-    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff"]), 
+    insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff", "--git"]), 
     @r###"
-    Resolved conflict in another_file:
-       1     : <<<<<<<
-       2     : %%%%%%%
-       3    1: first resolution for auto-secondchosen basefile
-       4     : +second a
-       5     : +++++++
-       6     : second b
-       7     : >>>>>>>
-    Resolved conflict in this_file_has_a_very_long_name_to_test_padding:
-       1     : <<<<<<<
-       2     : %%%%%%%
-       3    1: second resolution for auto-firstchosen basefile
-       4     : +first a
-       5     : +++++++
-       6     : first b
-       7     : >>>>>>>
+    diff --git a/another_file b/another_file
+    index 0000000000...7903e1c1c7 100644
+    --- a/another_file
+    +++ b/another_file
+    @@ -1,7 +1,1 @@
+    -<<<<<<< Conflict 1 of 1
+    -%%%%%%% Changes from base to side #1
+    --second base
+    -+second a
+    -+++++++ Contents of side #2
+    -second b
+    ->>>>>>> Conflict 1 of 1 ends
+    +first resolution for auto-chosen file
+    diff --git a/this_file_has_a_very_long_name_to_test_padding b/this_file_has_a_very_long_name_to_test_padding
+    index 0000000000...f8c72adf17 100644
+    --- a/this_file_has_a_very_long_name_to_test_padding
+    +++ b/this_file_has_a_very_long_name_to_test_padding
+    @@ -1,7 +1,1 @@
+    -<<<<<<< Conflict 1 of 1
+    -%%%%%%% Changes from base to side #1
+    --first base
+    -+first a
+    -+++++++ Contents of side #2
+    -first b
+    ->>>>>>> Conflict 1 of 1 ends
+    +second resolution for auto-chosen file
     "###);
 
     insta::assert_snapshot!(test_env.jj_cmd_cli_error(&repo_path, &["resolve", "--list"]), 
