@@ -19,7 +19,7 @@ use crate::common::TestEnvironment;
 #[test]
 fn test_restore() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     std::fs::write(repo_path.join("file1"), "a\n").unwrap();
@@ -62,15 +62,17 @@ fn test_restore() {
     Created rlvkpnrz e25100af (empty) (no description set)
     Rebased 1 descendant commits
     New conflicts appeared in these commits:
-      kkmpptxz e301deb3 (conflict) (no description set)
+      kkmpptxz 4906178a (conflict) (no description set)
     To resolve the conflicts, start by updating to it:
       jj new kkmpptxzrspx
     Then use `jj resolve`, or edit the conflict markers in the file directly.
     Once the conflicts are resolved, you may want inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
-    Working copy now at: kkmpptxz e301deb3 (conflict) (no description set)
+    Working copy now at: kkmpptxz 4906178a (conflict) (no description set)
     Parent commit      : rlvkpnrz e25100af (empty) (no description set)
     Added 0 files, modified 1 files, removed 0 files
+    There are unresolved conflicts at these paths:
+    file2    2-sided conflict including 1 deletion
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff", "-s", "-r=@-"]);
     insta::assert_snapshot!(stdout, @"");
@@ -149,7 +151,7 @@ fn test_restore() {
 #[test]
 fn test_restore_conflicted_merge() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     create_commit(&test_env, &repo_path, "base", &[], &[("file", "base\n")]);
@@ -169,27 +171,27 @@ fn test_restore_conflicted_merge() {
     insta::assert_snapshot!(
     std::fs::read_to_string(repo_path.join("file")).unwrap()
         , @r###"
-            <<<<<<<
-            %%%%%%%
-            -base
-            +a
-            +++++++
-            b
-            >>>>>>>
-            "###);
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
+    -base
+    +a
+    +++++++ Contents of side #2
+    b
+    >>>>>>> Conflict 1 of 1 ends
+    "###);
 
     // Overwrite the file...
     std::fs::write(repo_path.join("file"), "resolution").unwrap();
     insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff"]), 
     @r###"
     Resolved conflict in file:
-       1     : <<<<<<<
-       2     : %%%%%%%
+       1     : <<<<<<< Conflict 1 of 1
+       2     : %%%%%%% Changes from base to side #1
        3     : -base
        4     : +a
-       5     : +++++++
+       5     : +++++++ Contents of side #2
        6     : b
-       7     : >>>>>>>
+       7     : >>>>>>> Conflict 1 of 1 ends
             1: resolution
     "###);
 
@@ -197,22 +199,24 @@ fn test_restore_conflicted_merge() {
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["restore", "file"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
-    Created vruxwmqv b2c9c888 conflict | (conflict) (empty) conflict
-    Working copy now at: vruxwmqv b2c9c888 conflict | (conflict) (empty) conflict
+    Created vruxwmqv 126facb5 conflict | (conflict) (empty) conflict
+    Working copy now at: vruxwmqv 126facb5 conflict | (conflict) (empty) conflict
     Parent commit      : zsuskuln aa493daf a | a
     Parent commit      : royxmykx db6a4daf b | b
     Added 0 files, modified 1 files, removed 0 files
+    There are unresolved conflicts at these paths:
+    file    2-sided conflict
     "###);
     insta::assert_snapshot!(
     std::fs::read_to_string(repo_path.join("file")).unwrap()
         , @r###"
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
     -base
     +a
-    +++++++
+    +++++++ Contents of side #2
     b
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     "###);
     let stdout = test_env.jj_cmd_success(&repo_path, &["diff"]);
     insta::assert_snapshot!(stdout, @"");
@@ -222,13 +226,13 @@ fn test_restore_conflicted_merge() {
     insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["diff"]), 
     @r###"
     Resolved conflict in file:
-       1     : <<<<<<<
-       2     : %%%%%%%
+       1     : <<<<<<< Conflict 1 of 1
+       2     : %%%%%%% Changes from base to side #1
        3     : -base
        4     : +a
-       5     : +++++++
+       5     : +++++++ Contents of side #2
        6     : b
-       7     : >>>>>>>
+       7     : >>>>>>> Conflict 1 of 1 ends
             1: resolution
     "###);
 
@@ -236,22 +240,24 @@ fn test_restore_conflicted_merge() {
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["restore"]);
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
-    Created vruxwmqv 4fc10820 conflict | (conflict) (empty) conflict
-    Working copy now at: vruxwmqv 4fc10820 conflict | (conflict) (empty) conflict
+    Created vruxwmqv b553ebcf conflict | (conflict) (empty) conflict
+    Working copy now at: vruxwmqv b553ebcf conflict | (conflict) (empty) conflict
     Parent commit      : zsuskuln aa493daf a | a
     Parent commit      : royxmykx db6a4daf b | b
     Added 0 files, modified 1 files, removed 0 files
+    There are unresolved conflicts at these paths:
+    file    2-sided conflict
     "###);
     insta::assert_snapshot!(
     std::fs::read_to_string(repo_path.join("file")).unwrap()
         , @r###"
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
     -base
     +a
-    +++++++
+    +++++++ Contents of side #2
     b
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     "###);
 }
 
