@@ -17,7 +17,7 @@ use crate::common::TestEnvironment;
 #[test]
 fn test_syntax_error() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let stderr = test_env.jj_cmd_failure(&repo_path, &["log", "-r", ":x"]);
@@ -66,24 +66,12 @@ fn test_syntax_error() {
       = '^' is not a postfix operator
     Hint: Did you mean '-' for parents?
     "###);
-
-    // "jj new" supports "all:" prefix
-    let stderr = test_env.jj_cmd_failure(&repo_path, &["new", "ale:x"]);
-    insta::assert_snapshot!(stderr, @r###"
-    Error: Failed to parse revset: Modifier "ale" doesn't exist
-    Caused by:  --> 1:1
-      |
-    1 | ale:x
-      | ^-^
-      |
-      = Modifier "ale" doesn't exist
-    "###);
 }
 
 #[test]
 fn test_bad_function_call() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let stderr = test_env.jj_cmd_failure(&repo_path, &["log", "-r", "all(or::nothing)"]);
@@ -132,74 +120,74 @@ fn test_bad_function_call() {
 
     let stderr = test_env.jj_cmd_failure(&repo_path, &["log", "-r", "latest(a, not_an_integer)"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Failed to parse revset: Function "latest": Expected function argument of type integer
+    Error: Failed to parse revset: Expected expression of type integer
     Caused by:  --> 1:11
       |
     1 | latest(a, not_an_integer)
       |           ^------------^
       |
-      = Function "latest": Expected function argument of type integer
+      = Expected expression of type integer
     "###);
 
     let stderr = test_env.jj_cmd_failure(&repo_path, &["log", "-r", "file()"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Failed to parse revset: Function "file": Expected at least 1 argument
+    Error: Failed to parse revset: Function "file": Expected at least 1 arguments
     Caused by:  --> 1:6
       |
     1 | file()
       |      ^
       |
-      = Function "file": Expected at least 1 argument
+      = Function "file": Expected at least 1 arguments
     "###);
 
     let stderr = test_env.jj_cmd_failure(&repo_path, &["log", "-r", "file(a, not@a-string)"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Failed to parse revset: Function "file": Expected function argument of file pattern
+    Error: Failed to parse revset: Expected expression of file pattern
     Caused by:  --> 1:9
       |
     1 | file(a, not@a-string)
       |         ^----------^
       |
-      = Function "file": Expected function argument of file pattern
+      = Expected expression of file pattern
     "###);
 
     let stderr = test_env.jj_cmd_failure(&repo_path, &["log", "-r", r#"file(foo:"bar")"#]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Failed to parse revset: Function "file": Invalid file pattern
+    Error: Failed to parse revset: Invalid file pattern
     Caused by:
     1:  --> 1:6
       |
     1 | file(foo:"bar")
       |      ^-------^
       |
-      = Function "file": Invalid file pattern
+      = Invalid file pattern
     2: Invalid file pattern kind "foo:"
     "###);
 
     let stderr = test_env.jj_cmd_failure(&repo_path, &["log", "-r", r#"file(a, "../out")"#]);
     insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
-    Error: Failed to parse revset: Function "file": Invalid file pattern
+    Error: Failed to parse revset: Invalid file pattern
     Caused by:
     1:  --> 1:9
       |
     1 | file(a, "../out")
       |         ^------^
       |
-      = Function "file": Invalid file pattern
+      = Invalid file pattern
     2: Path "../out" is not in the repo "."
     3: Invalid component ".." in repo-relative path "../out"
     "###);
 
     let stderr = test_env.jj_cmd_failure(&repo_path, &["log", "-r", "branches(bad:pattern)"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Failed to parse revset: Function "branches": Invalid string pattern
+    Error: Failed to parse revset: Invalid string pattern
     Caused by:
     1:  --> 1:10
       |
     1 | branches(bad:pattern)
       |          ^---------^
       |
-      = Function "branches": Invalid string pattern
+      = Invalid string pattern
     2: Invalid string pattern kind "bad:"
     Hint: Try prefixing with one of `exact:`, `glob:` or `substring:`
     "###);
@@ -267,7 +255,7 @@ fn test_bad_function_call() {
 #[test]
 fn test_function_name_hint() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let evaluate_err = |expr| test_env.jj_cmd_failure(&repo_path, &["log", "-r", expr]);
 
@@ -290,7 +278,7 @@ fn test_function_name_hint() {
       | ^----^
       |
       = Function "branch" doesn't exist
-    Hint: Did you mean "branches"?
+    Hint: Did you mean "branches", "reachable"?
     "###);
 
     // Both builtin function and function alias should be suggested
@@ -320,14 +308,14 @@ fn test_function_name_hint() {
       | ^----^
       |
       = Function "branch" doesn't exist
-    Hint: Did you mean "branches"?
+    Hint: Did you mean "branches", "reachable"?
     "###);
 }
 
 #[test]
 fn test_alias() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.add_config(
@@ -397,7 +385,13 @@ fn test_alias() {
     1 | author(x)
       |        ^
       |
-      = Function "author": Expected function argument of string pattern
+      = Function parameter "x" cannot be expanded
+    3:  --> 1:11
+      |
+    1 | my_author(none())
+      |           ^----^
+      |
+      = Expected expression of string pattern
     "###);
 
     let stderr = test_env.jj_cmd_failure(&repo_path, &["log", "-r", "root() & recurse"]);
@@ -434,7 +428,7 @@ fn test_alias() {
 #[test]
 fn test_alias_override() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.add_config(
@@ -464,7 +458,7 @@ fn test_alias_override() {
 #[test]
 fn test_bad_alias_decl() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.add_config(
@@ -496,3 +490,108 @@ fn test_bad_alias_decl() {
       = Redefinition of function parameter
     "###);
 }
+
+#[test]
+fn test_all_modifier() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    // Command that accepts single revision by default
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["new", "all()"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Revset "all()" resolved to more than one revision
+    Hint: The revset "all()" resolved to these revisions:
+      qpvuntsm 230dd059 (empty) (no description set)
+      zzzzzzzz 00000000 (empty) (no description set)
+    Hint: Prefix the expression with 'all:' to allow any number of revisions (i.e. 'all:all()').
+    "###);
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["new", "all:all()"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: The Git backend does not support creating merge commits with the root commit as one of the parents.
+    "###);
+
+    // Command that accepts multiple revisions by default
+    let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-rall:all()"]);
+    insta::assert_snapshot!(stdout, @r###"
+    @  qpvuntsm test.user@example.com 2001-02-03 08:05:07 230dd059
+    │  (empty) (no description set)
+    ◉  zzzzzzzz root() 00000000
+    "###);
+
+    // Command that accepts only single revision
+    let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "create", "-rall:@", "x"]);
+    insta::assert_snapshot!(stderr, @"");
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["branch", "set", "-rall:all()", "x"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Revset "all:all()" resolved to more than one revision
+    Hint: The revset "all:all()" resolved to these revisions:
+      qpvuntsm 230dd059 x | (empty) (no description set)
+      zzzzzzzz 00000000 (empty) (no description set)
+    "###);
+
+    // Template expression that accepts multiple revisions by default
+    let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-Tself.contained_in('all:all()')"]);
+    insta::assert_snapshot!(stdout, @r###"
+    @  true
+    ◉  true
+    "###);
+
+    // Typo
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["new", "ale:x"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Failed to parse revset: Modifier "ale" doesn't exist
+    Caused by:  --> 1:1
+      |
+    1 | ale:x
+      | ^-^
+      |
+      = Modifier "ale" doesn't exist
+    "###);
+
+    // Modifier shouldn't be allowed in aliases (This restriction might be
+    // lifted later. "all:" in alias could be allowed if it is expanded to the
+    // top-level expression.)
+    let stderr = test_env.jj_cmd_failure(
+        &repo_path,
+        &["new", "x", "--config-toml=revset-aliases.x='all:@'"],
+    );
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Failed to parse revset: Alias "x" cannot be expanded
+    Caused by:
+    1:  --> 1:1
+      |
+    1 | x
+      | ^
+      |
+      = Alias "x" cannot be expanded
+    2:  --> 1:4
+      |
+    1 | all:@
+      |    ^
+      |
+      = ':' is not an infix operator
+    Hint: Did you mean '::' for DAG range?
+    "###);
+
+    // immutable_heads() alias may be parsed as a top-level expression, but
+    // still, modifier shouldn't be allowed there.
+    let stderr = test_env.jj_cmd_failure(
+        &repo_path,
+        &[
+            "new",
+            "--config-toml=revset-aliases.'immutable_heads()'='all:@'",
+            "--config-toml=revsets.short-prefixes='none()'",
+        ],
+    );
+    insta::assert_snapshot!(stderr, @r###"
+    Config error: Invalid `revset-aliases.immutable_heads()`
+    Caused by:  --> 1:4
+      |
+    1 | all:@
+      |    ^
+      |
+      = ':' is not an infix operator
+    For help, see https://github.com/martinvonz/jj/blob/main/docs/config.md.
+    "###);
+}
