@@ -19,30 +19,112 @@ use crate::common::TestEnvironment;
 #[test]
 fn test_templater_parse_error() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let render_err = |template| test_env.jj_cmd_failure(&repo_path, &["log", "-T", template]);
 
     insta::assert_snapshot!(render_err(r#"description ()"#), @r###"
-    Error: Failed to parse template:  --> 1:13
+    Error: Failed to parse template: Syntax error
+    Caused by:  --> 1:13
       |
     1 | description ()
       |             ^---
       |
-      = expected EOI
+      = expected <EOI>, `++`, `||`, or `&&`
+    "###);
+
+    // Typo
+    test_env.add_config(
+        r###"
+    [template-aliases]
+    'conflicting' = ''
+    'shorted()' = ''
+    'socat(x)' = 'x'
+    'format_id(id)' = 'id.sort()'
+    "###,
+    );
+    insta::assert_snapshot!(render_err(r#"conflicts"#), @r###"
+    Error: Failed to parse template: Keyword "conflicts" doesn't exist
+    Caused by:  --> 1:1
+      |
+    1 | conflicts
+      | ^-------^
+      |
+      = Keyword "conflicts" doesn't exist
+    Hint: Did you mean "conflict", "conflicting"?
+    "###);
+    insta::assert_snapshot!(render_err(r#"commit_id.shorter()"#), @r###"
+    Error: Failed to parse template: Method "shorter" doesn't exist for type "CommitOrChangeId"
+    Caused by:  --> 1:11
+      |
+    1 | commit_id.shorter()
+      |           ^-----^
+      |
+      = Method "shorter" doesn't exist for type "CommitOrChangeId"
+    Hint: Did you mean "short", "shortest"?
+    "###);
+    insta::assert_snapshot!(render_err(r#"oncat()"#), @r###"
+    Error: Failed to parse template: Function "oncat" doesn't exist
+    Caused by:  --> 1:1
+      |
+    1 | oncat()
+      | ^---^
+      |
+      = Function "oncat" doesn't exist
+    Hint: Did you mean "concat", "socat"?
+    "###);
+    insta::assert_snapshot!(render_err(r#""".lines().map(|s| se)"#), @r###"
+    Error: Failed to parse template: Keyword "se" doesn't exist
+    Caused by:  --> 1:20
+      |
+    1 | "".lines().map(|s| se)
+      |                    ^^
+      |
+      = Keyword "se" doesn't exist
+    Hint: Did you mean "s", "self"?
+    "###);
+    insta::assert_snapshot!(render_err(r#"format_id(commit_id)"#), @r###"
+    Error: Failed to parse template: Alias "format_id()" cannot be expanded
+    Caused by:
+    1:  --> 1:1
+      |
+    1 | format_id(commit_id)
+      | ^------------------^
+      |
+      = Alias "format_id()" cannot be expanded
+    2:  --> 1:4
+      |
+    1 | id.sort()
+      |    ^--^
+      |
+      = Method "sort" doesn't exist for type "CommitOrChangeId"
+    Hint: Did you mean "short", "shortest"?
+    "###);
+
+    // -Tbuiltin shows the predefined builtin_* aliases. This isn't 100%
+    // guaranteed, but is nice.
+    insta::assert_snapshot!(render_err(r#"builtin"#), @r###"
+    Error: Failed to parse template: Keyword "builtin" doesn't exist
+    Caused by:  --> 1:1
+      |
+    1 | builtin
+      | ^-----^
+      |
+      = Keyword "builtin" doesn't exist
+    Hint: Did you mean "builtin_log_comfortable", "builtin_log_compact", "builtin_log_detailed", "builtin_log_node", "builtin_log_node_ascii", "builtin_log_oneline", "builtin_op_log_comfortable", "builtin_op_log_compact", "builtin_op_log_node", "builtin_op_log_node_ascii"?
     "###);
 }
 
 #[test]
 fn test_templater_upper_lower() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let render = |template| get_colored_template_output(&test_env, &repo_path, "@-", template);
 
     insta::assert_snapshot!(
       render(r#"change_id.shortest(4).upper() ++ change_id.shortest(4).upper().lower()"#),
-      @"[1m[38;5;5mZ[0m[38;5;8mZZZ[39m[1m[38;5;5mz[0m[38;5;8mzzz[39m");
+      @"[1m[38;5;5mZ[0m[38;5;8mZZZ[1m[38;5;5mz[0m[38;5;8mzzz[39m");
     insta::assert_snapshot!(
       render(r#""Hello".upper() ++ "Hello".lower()"#), @"HELLOhello");
 }
@@ -50,7 +132,7 @@ fn test_templater_upper_lower() {
 #[test]
 fn test_templater_alias() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
     let render = |template| get_template_output(&test_env, &repo_path, "@-", template);
     let render_err = |template| test_env.jj_cmd_failure(&repo_path, &["log", "-T", template]);
@@ -73,28 +155,32 @@ fn test_templater_alias() {
     insta::assert_snapshot!(render("identity(my_commit_id)"), @"000000000000");
 
     insta::assert_snapshot!(render_err("commit_id ++ syntax_error"), @r###"
-    Error: Failed to parse template:  --> 1:14
+    Error: Failed to parse template: Alias "syntax_error" cannot be expanded
+    Caused by:
+    1:  --> 1:14
       |
     1 | commit_id ++ syntax_error
       |              ^----------^
       |
       = Alias "syntax_error" cannot be expanded
-     --> 1:5
+    2:  --> 1:5
       |
     1 | foo.
       |     ^---
       |
-      = expected identifier
+      = expected <identifier>
     "###);
 
     insta::assert_snapshot!(render_err("commit_id ++ name_error"), @r###"
-    Error: Failed to parse template:  --> 1:14
+    Error: Failed to parse template: Alias "name_error" cannot be expanded
+    Caused by:
+    1:  --> 1:14
       |
     1 | commit_id ++ name_error
       |              ^--------^
       |
       = Alias "name_error" cannot be expanded
-     --> 1:1
+    2:  --> 1:1
       |
     1 | unknown_id
       | ^--------^
@@ -103,46 +189,62 @@ fn test_templater_alias() {
     "###);
 
     insta::assert_snapshot!(render_err(r#"identity(identity(commit_id.short("")))"#), @r###"
-    Error: Failed to parse template:  --> 1:1
+    Error: Failed to parse template: Alias "identity()" cannot be expanded
+    Caused by:
+    1:  --> 1:1
       |
     1 | identity(identity(commit_id.short("")))
       | ^-------------------------------------^
       |
       = Alias "identity()" cannot be expanded
-     --> 1:10
+    2:  --> 1:1
+      |
+    1 | x
+      | ^
+      |
+      = Function parameter "x" cannot be expanded
+    3:  --> 1:10
       |
     1 | identity(identity(commit_id.short("")))
       |          ^---------------------------^
       |
       = Alias "identity()" cannot be expanded
-     --> 1:35
+    4:  --> 1:1
+      |
+    1 | x
+      | ^
+      |
+      = Function parameter "x" cannot be expanded
+    5:  --> 1:35
       |
     1 | identity(identity(commit_id.short("")))
       |                                   ^^
       |
-      = Expected expression of type "Integer"
+      = Expected expression of type "Integer", but actual type is "String"
     "###);
 
     insta::assert_snapshot!(render_err("commit_id ++ recurse"), @r###"
-    Error: Failed to parse template:  --> 1:14
+    Error: Failed to parse template: Alias "recurse" cannot be expanded
+    Caused by:
+    1:  --> 1:14
       |
     1 | commit_id ++ recurse
       |              ^-----^
       |
       = Alias "recurse" cannot be expanded
-     --> 1:1
+    2:  --> 1:1
       |
     1 | recurse1
       | ^------^
       |
       = Alias "recurse1" cannot be expanded
-     --> 1:1
+    3:  --> 1:1
       |
     1 | recurse2()
       | ^--------^
       |
       = Alias "recurse2()" cannot be expanded
-     --> 1:1
+    4:  --> 1:1
       |
     1 | recurse
       | ^-----^
@@ -151,7 +253,8 @@ fn test_templater_alias() {
     "###);
 
     insta::assert_snapshot!(render_err("identity()"), @r###"
-    Error: Failed to parse template:  --> 1:10
+    Error: Failed to parse template: Function "identity": Expected 1 arguments
+    Caused by:  --> 1:10
       |
     1 | identity()
       |          ^
@@ -159,7 +262,8 @@ fn test_templater_alias() {
       = Function "identity": Expected 1 arguments
     "###);
     insta::assert_snapshot!(render_err("identity(commit_id, commit_id)"), @r###"
-    Error: Failed to parse template:  --> 1:10
+    Error: Failed to parse template: Function "identity": Expected 1 arguments
+    Caused by:  --> 1:10
       |
     1 | identity(commit_id, commit_id)
       |          ^------------------^
@@ -168,25 +272,50 @@ fn test_templater_alias() {
     "###);
 
     insta::assert_snapshot!(render_err(r#"coalesce(label("x", "not boolean"), "")"#), @r###"
-    Error: Failed to parse template:  --> 1:1
+    Error: Failed to parse template: Alias "coalesce()" cannot be expanded
+    Caused by:
+    1:  --> 1:1
       |
     1 | coalesce(label("x", "not boolean"), "")
       | ^-------------------------------------^
       |
       = Alias "coalesce()" cannot be expanded
-     --> 1:10
+    2:  --> 1:4
+      |
+    1 | if(x, x, y)
+      |    ^
+      |
+      = Function parameter "x" cannot be expanded
+    3:  --> 1:10
       |
     1 | coalesce(label("x", "not boolean"), "")
       |          ^-----------------------^
       |
-      = Expected expression of type "Boolean"
+      = Expected expression of type "Boolean", but actual type is "Template"
+    "###);
+
+    insta::assert_snapshot!(render_err("(-my_commit_id)"), @r###"
+    Error: Failed to parse template: Alias "my_commit_id" cannot be expanded
+    Caused by:
+    1:  --> 1:3
+      |
+    1 | (-my_commit_id)
+      |   ^----------^
+      |
+      = Alias "my_commit_id" cannot be expanded
+    2:  --> 1:1
+      |
+    1 | commit_id.short()
+      | ^---------------^
+      |
+      = Expected expression of type "Integer", but actual type is "String"
     "###);
 }
 
 #[test]
 fn test_templater_alias_override() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.add_config(
@@ -216,7 +345,7 @@ fn test_templater_alias_override() {
 #[test]
 fn test_templater_bad_alias_decl() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.add_config(
@@ -232,7 +361,7 @@ fn test_templater_bad_alias_decl() {
         test_env.jj_cmd_ok(&repo_path, &["log", "--no-graph", "-r@-", "-Tmy_commit_id"]);
     insta::assert_snapshot!(stdout, @"000000000000");
     insta::assert_snapshot!(stderr, @r###"
-    Failed to load "template-aliases.badfn(a, a)":  --> 1:7
+    Warning: Failed to load "template-aliases.badfn(a, a)":  --> 1:7
       |
     1 | badfn(a, a)
       |       ^--^
