@@ -20,7 +20,7 @@ fn test_undo_rewrite_with_child() {
     // Test that if we undo an operation that rewrote some commit, any descendants
     // after that will be rebased on top of the un-rewritten commit.
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(&repo_path, &["describe", "-m", "initial"]);
@@ -206,7 +206,7 @@ fn test_git_push_undo_colocated() {
     git2::Repository::init_bare(git_repo_path.clone()).unwrap();
     let repo_path = test_env.env_root().join("clone");
     git2::Repository::clone(git_repo_path.to_str().unwrap(), &repo_path).unwrap();
-    test_env.jj_cmd_ok(&repo_path, &["init", "--git-repo=."]);
+    test_env.jj_cmd_ok(&repo_path, &["git", "init", "--git-repo=."]);
 
     test_env.advance_test_rng_seed_to_multiple_of(100_000);
     test_env.jj_cmd_ok(&repo_path, &["branch", "create", "main"]);
@@ -342,7 +342,6 @@ fn test_branch_track_untrack_undo() {
       @origin: qpvuntsm 270721f5 (empty) commit
     feature2 (deleted)
       @origin: qpvuntsm 270721f5 (empty) commit
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     "###);
 
     // Track/untrack can be undone so long as states can be trivially merged.
@@ -362,7 +361,6 @@ fn test_branch_track_untrack_undo() {
       @origin: qpvuntsm 270721f5 (empty) commit
     feature2 (deleted)
       @origin: qpvuntsm 270721f5 (empty) commit
-      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     "###);
 
     test_env.jj_cmd_ok(&repo_path, &["undo"]);
@@ -388,5 +386,6 @@ fn test_branch_track_untrack_undo() {
 }
 
 fn get_branch_output(test_env: &TestEnvironment, repo_path: &Path) -> String {
-    test_env.jj_cmd_success(repo_path, &["branch", "list", "--all-remotes"])
+    // --quiet to suppress deleted branches hint
+    test_env.jj_cmd_success(repo_path, &["branch", "list", "--all-remotes", "--quiet"])
 }
