@@ -19,7 +19,7 @@ use crate::common::TestEnvironment;
 #[test]
 fn test_unsquash() {
     let test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(&repo_path, &["branch", "create", "a"]);
@@ -103,7 +103,7 @@ fn test_unsquash() {
     "###);
 
     // Can unsquash from a merge commit
-    test_env.jj_cmd_ok(&repo_path, &["co", "e"]);
+    test_env.jj_cmd_ok(&repo_path, &["new", "e"]);
     std::fs::write(repo_path.join("file1"), "e\n").unwrap();
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["unsquash"]);
     insta::assert_snapshot!(stdout, @"");
@@ -131,7 +131,7 @@ fn test_unsquash() {
 #[test]
 fn test_unsquash_partial() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     test_env.jj_cmd_ok(&repo_path, &["branch", "create", "a"]);
@@ -205,6 +205,38 @@ fn test_unsquash_partial() {
     insta::assert_snapshot!(stdout, @r###"
     c
     "###);
+
+    // Try again with --tool=<name>, which implies --interactive
+    test_env.jj_cmd_ok(&repo_path, &["undo"]);
+    let (stdout, stderr) = test_env.jj_cmd_ok(
+        &repo_path,
+        &[
+            "unsquash",
+            "--config-toml=ui.diff-editor='false'",
+            "--tool=fake-diff-editor",
+        ],
+    );
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Working copy now at: mzvwutvl 1c82d27c c | (no description set)
+    Parent commit      : kkmpptxz b9d23fd8 b | (no description set)
+    "###);
+    let stdout = test_env.jj_cmd_success(&repo_path, &["print", "file1", "-r", "b"]);
+    insta::assert_snapshot!(stdout, @r###"
+    a
+    "###);
+    let stdout = test_env.jj_cmd_success(&repo_path, &["print", "file2", "-r", "b"]);
+    insta::assert_snapshot!(stdout, @r###"
+    b
+    "###);
+    let stdout = test_env.jj_cmd_success(&repo_path, &["print", "file1", "-r", "c"]);
+    insta::assert_snapshot!(stdout, @r###"
+    c
+    "###);
+    let stdout = test_env.jj_cmd_success(&repo_path, &["print", "file2", "-r", "c"]);
+    insta::assert_snapshot!(stdout, @r###"
+    c
+    "###);
 }
 
 fn get_log_output(test_env: &TestEnvironment, repo_path: &Path) -> String {
@@ -215,7 +247,7 @@ fn get_log_output(test_env: &TestEnvironment, repo_path: &Path) -> String {
 #[test]
 fn test_unsquash_description() {
     let mut test_env = TestEnvironment::default();
-    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
     let repo_path = test_env.env_root().join("repo");
 
     let edit_script = test_env.set_up_fake_editor();
@@ -264,7 +296,7 @@ fn test_unsquash_description() {
     JJ: Description from the destination commit:
     destination
 
-    JJ: Description from the source commit:
+    JJ: Description from source commit:
     source
 
     JJ: Lines starting with "JJ: " (like this one) will be removed.
