@@ -18,102 +18,41 @@ use std::any::Any;
 use std::collections::BTreeMap;
 use std::fmt::Debug;
 use std::io::Read;
-use std::result::Result;
-use std::vec::Vec;
+use std::time::SystemTime;
 
 use async_trait::async_trait;
 use thiserror::Error;
 
 use crate::content_hash::ContentHash;
+use crate::index::Index;
 use crate::merge::Merge;
+use crate::object_id::{id_type, ObjectId};
 use crate::repo_path::{RepoPath, RepoPathComponent, RepoPathComponentBuf};
 use crate::signing::SignResult;
 
-pub trait ObjectId {
-    fn new(value: Vec<u8>) -> Self;
-    fn object_type(&self) -> String;
-    fn from_bytes(bytes: &[u8]) -> Self;
-    fn as_bytes(&self) -> &[u8];
-    fn to_bytes(&self) -> Vec<u8>;
-    fn from_hex(hex: &str) -> Self;
-    fn hex(&self) -> String;
-}
-
-macro_rules! id_type {
-    ($vis:vis $name:ident) => {
-        content_hash! {
-            #[derive(PartialEq, Eq, PartialOrd, Ord, Clone, Hash)]
-            $vis struct $name(Vec<u8>);
-        }
-        $crate::backend::impl_id_type!($name);
-    };
-}
-
-macro_rules! impl_id_type {
-    ($name:ident) => {
-        impl std::fmt::Debug for $name {
-            fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
-                f.debug_tuple(stringify!($name)).field(&self.hex()).finish()
-            }
-        }
-
-        impl crate::backend::ObjectId for $name {
-            fn new(value: Vec<u8>) -> Self {
-                Self(value)
-            }
-
-            fn object_type(&self) -> String {
-                stringify!($name)
-                    .strip_suffix("Id")
-                    .unwrap()
-                    .to_ascii_lowercase()
-                    .to_string()
-            }
-
-            fn from_bytes(bytes: &[u8]) -> Self {
-                Self(bytes.to_vec())
-            }
-
-            fn as_bytes(&self) -> &[u8] {
-                &self.0
-            }
-
-            fn to_bytes(&self) -> Vec<u8> {
-                self.0.clone()
-            }
-
-            fn from_hex(hex: &str) -> Self {
-                Self(hex::decode(hex).unwrap())
-            }
-
-            fn hex(&self) -> String {
-                hex::encode(&self.0)
-            }
-        }
-    };
-}
-
-pub(crate) use {id_type, impl_id_type};
-
-id_type!(pub CommitId);
-id_type!(pub ChangeId);
+id_type!(
+    /// Identifier for a [`Commit`] based on its content. When a commit is
+    /// rewritten, its `CommitId` changes.
+    pub CommitId
+);
+id_type!(
+    /// Stable identifier for a [`Commit`]. Unlike the `CommitId`, the `ChangeId`
+    /// follows the commit and is not updated when the commit is rewritten.
+    pub ChangeId
+);
 id_type!(pub TreeId);
 id_type!(pub FileId);
 id_type!(pub SymlinkId);
 id_type!(pub ConflictId);
 
-content_hash! {
-    #[derive(Debug, PartialEq, Eq, Clone, PartialOrd, Ord)]
-    pub struct MillisSinceEpoch(pub i64);
-}
+#[derive(ContentHash, Debug, PartialEq, Eq, Clone, Copy, PartialOrd, Ord)]
+pub struct MillisSinceEpoch(pub i64);
 
-content_hash! {
-    #[derive(Debug, PartialEq, Eq, Clone, PartialOrd, Ord)]
-    pub struct Timestamp {
-        pub timestamp: MillisSinceEpoch,
-        // time zone offset in minutes
-        pub tz_offset: i32,
-    }
+#[derive(ContentHash, Debug, PartialEq, Eq, Clone, PartialOrd, Ord)]
+pub struct Timestamp {
+    pub timestamp: MillisSinceEpoch,
+    // time zone offset in minutes
+    pub tz_offset: i32,
 }
 
 impl Timestamp {
@@ -131,21 +70,19 @@ impl Timestamp {
     }
 }
 
-content_hash! {
-    #[derive(Debug, PartialEq, Eq, Clone)]
-    pub struct Signature {
-        pub name: String,
-        pub email: String,
-        pub timestamp: Timestamp,
-    }
+/// Represents a [`Commit`] signature.
+#[derive(ContentHash, Debug, PartialEq, Eq, Clone)]
+pub struct Signature {
+    pub name: String,
+    pub email: String,
+    pub timestamp: Timestamp,
 }
 
-content_hash! {
-    #[derive(Debug, PartialEq, Eq, Clone)]
-    pub struct SecureSig {
-        pub data: Vec<u8>,
-        pub sig: Vec<u8>,
-    }
+/// Represents a cryptographically signed [`Commit`] signature.
+#[derive(ContentHash, Debug, PartialEq, Eq, Clone)]
+pub struct SecureSig {
+    pub data: Vec<u8>,
+    pub sig: Vec<u8>,
 }
 
 pub type SigningFn<'a> = dyn FnMut(&[u8]) -> SignResult<Vec<u8>> + 'a;
@@ -155,7 +92,7 @@ pub type SigningFn<'a> = dyn FnMut(&[u8]) -> SignResult<Vec<u8>> + 'a;
 // TODO(#1624): Delete this type at some point in the future, when we decide to drop
 // support for conflicts in older repos, or maybe after we have provided an upgrade
 // mechanism.
-#[derive(Debug, Clone)]
+#[derive(ContentHash, Debug, Clone)]
 pub enum MergedTreeId {
     /// The tree id of a legacy tree
     Legacy(TreeId),
@@ -173,21 +110,6 @@ impl PartialEq for MergedTreeId {
 
 impl Eq for MergedTreeId {}
 
-impl ContentHash for MergedTreeId {
-    fn hash(&self, state: &mut impl digest::Update) {
-        match self {
-            MergedTreeId::Legacy(tree_id) => {
-                state.update(b"0");
-                ContentHash::hash(tree_id, state);
-            }
-            MergedTreeId::Merge(tree_ids) => {
-                state.update(b"1");
-                ContentHash::hash(tree_ids, state);
-            }
-        }
-    }
-}
-
 impl MergedTreeId {
     /// Create a resolved `MergedTreeId` from a single regular tree.
     pub fn resolved(tree_id: TreeId) -> Self {
@@ -203,37 +125,31 @@ impl MergedTreeId {
     }
 }
 
-content_hash! {
-    #[derive(Debug, PartialEq, Eq, Clone)]
-    pub struct Commit {
-        pub parents: Vec<CommitId>,
-        pub predecessors: Vec<CommitId>,
-        pub root_tree: MergedTreeId,
-        pub change_id: ChangeId,
-        pub description: String,
-        pub author: Signature,
-        pub committer: Signature,
-        pub secure_sig: Option<SecureSig>,
-    }
+#[derive(ContentHash, Debug, PartialEq, Eq, Clone)]
+pub struct Commit {
+    pub parents: Vec<CommitId>,
+    pub predecessors: Vec<CommitId>,
+    pub root_tree: MergedTreeId,
+    pub change_id: ChangeId,
+    pub description: String,
+    pub author: Signature,
+    pub committer: Signature,
+    pub secure_sig: Option<SecureSig>,
 }
 
-content_hash! {
-    #[derive(Debug, PartialEq, Eq, Clone)]
-    pub struct ConflictTerm {
-        pub value: TreeValue,
-    }
+#[derive(ContentHash, Debug, PartialEq, Eq, Clone)]
+pub struct ConflictTerm {
+    pub value: TreeValue,
 }
 
-content_hash! {
-    #[derive(Default, Debug, PartialEq, Eq, Clone)]
-    pub struct Conflict {
-        // A conflict is represented by a list of positive and negative states that need to be applied.
-        // In a simple 3-way merge of B and C with merge base A, the conflict will be { add: [B, C],
-        // remove: [A] }. Also note that a conflict of the form { add: [A], remove: [] } is the
-        // same as non-conflict A.
-        pub removes: Vec<ConflictTerm>,
-        pub adds: Vec<ConflictTerm>,
-    }
+#[derive(ContentHash, Default, Debug, PartialEq, Eq, Clone)]
+pub struct Conflict {
+    // A conflict is represented by a list of positive and negative states that need to be applied.
+    // In a simple 3-way merge of B and C with merge base A, the conflict will be { add: [B, C],
+    // remove: [A] }. Also note that a conflict of the form { add: [A], remove: [] } is the
+    // same as non-conflict A.
+    pub removes: Vec<ConflictTerm>,
+    pub adds: Vec<ConflictTerm>,
 }
 
 /// Error that may occur during backend initialization.
@@ -259,36 +175,46 @@ pub enum BackendError {
         object_type: String,
         hash: String,
     },
-    #[error("Invalid UTF-8 for object {hash} of type {object_type}: {source}")]
+    #[error("Invalid UTF-8 for object {hash} of type {object_type}")]
     InvalidUtf8 {
         object_type: String,
         hash: String,
         source: std::str::Utf8Error,
     },
-    #[error("Object {hash} of type {object_type} not found: {source}")]
+    #[error("Object {hash} of type {object_type} not found")]
     ObjectNotFound {
         object_type: String,
         hash: String,
         source: Box<dyn std::error::Error + Send + Sync>,
     },
-    #[error("Error when reading object {hash} of type {object_type}: {source}")]
+    #[error("Error when reading object {hash} of type {object_type}")]
     ReadObject {
         object_type: String,
         hash: String,
         source: Box<dyn std::error::Error + Send + Sync>,
     },
-    #[error("Could not write object of type {object_type}: {source}")]
+    #[error("Access denied to read object {hash} of type {object_type}")]
+    ReadAccessDenied {
+        object_type: String,
+        hash: String,
+        source: Box<dyn std::error::Error + Send + Sync>,
+    },
+    #[error("Could not write object of type {object_type}")]
     WriteObject {
         object_type: &'static str,
         source: Box<dyn std::error::Error + Send + Sync>,
     },
-    #[error("Error: {0}")]
+    #[error(transparent)]
     Other(Box<dyn std::error::Error + Send + Sync>),
+    /// A valid operation attempted, but failed because it isn't supported by
+    /// the particular backend.
+    #[error("{0}")]
+    Unsupported(String),
 }
 
 pub type BackendResult<T> = Result<T, BackendError>;
 
-#[derive(Debug, PartialEq, Eq, Clone, Hash)]
+#[derive(ContentHash, Debug, PartialEq, Eq, Clone, Hash)]
 pub enum TreeValue {
     File { id: FileId, executable: bool },
     Symlink(SymlinkId),
@@ -309,35 +235,6 @@ impl TreeValue {
     }
 }
 
-impl ContentHash for TreeValue {
-    fn hash(&self, state: &mut impl digest::Update) {
-        use TreeValue::*;
-        match self {
-            File { id, executable } => {
-                state.update(&0u32.to_le_bytes());
-                id.hash(state);
-                executable.hash(state);
-            }
-            Symlink(id) => {
-                state.update(&1u32.to_le_bytes());
-                id.hash(state);
-            }
-            Tree(id) => {
-                state.update(&2u32.to_le_bytes());
-                id.hash(state);
-            }
-            GitSubmodule(id) => {
-                state.update(&3u32.to_le_bytes());
-                id.hash(state);
-            }
-            Conflict(id) => {
-                state.update(&4u32.to_le_bytes());
-                id.hash(state);
-            }
-        }
-    }
-}
-
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub struct TreeEntry<'a> {
     name: &'a RepoPathComponent,
@@ -372,11 +269,9 @@ impl<'a> Iterator for TreeEntriesNonRecursiveIterator<'a> {
     }
 }
 
-content_hash! {
-    #[derive(Default, PartialEq, Eq, Debug, Clone)]
-    pub struct Tree {
-        entries: BTreeMap<RepoPathComponentBuf, TreeValue>,
-    }
+#[derive(ContentHash, Default, PartialEq, Eq, Debug, Clone)]
+pub struct Tree {
+    entries: BTreeMap<RepoPathComponentBuf, TreeValue>,
 }
 
 impl Tree {
@@ -424,26 +319,6 @@ impl Tree {
     }
 }
 
-/// Calculates common prefix length of two bytes. The length to be returned is
-/// a number of hexadecimal digits.
-pub fn common_hex_len(bytes_a: &[u8], bytes_b: &[u8]) -> usize {
-    iter_half_bytes(bytes_a)
-        .zip(iter_half_bytes(bytes_b))
-        .take_while(|(a, b)| a == b)
-        .count()
-}
-
-fn iter_half_bytes(bytes: &[u8]) -> impl ExactSizeIterator<Item = u8> + '_ {
-    (0..bytes.len() * 2).map(|i| {
-        let v = bytes[i / 2];
-        if i & 1 == 0 {
-            v >> 4
-        } else {
-            v & 0xf
-        }
-    })
-}
-
 pub fn make_root_commit(root_change_id: ChangeId, empty_tree_id: TreeId) -> Commit {
     let timestamp = Timestamp {
         timestamp: MillisSinceEpoch(0),
@@ -466,6 +341,7 @@ pub fn make_root_commit(root_change_id: ChangeId, empty_tree_id: TreeId) -> Comm
     }
 }
 
+/// Defines the interface for commit backends.
 #[async_trait]
 pub trait Backend: Send + Sync + Debug {
     fn as_any(&self) -> &dyn Any;
@@ -535,6 +411,9 @@ pub trait Backend: Send + Sync + Debug {
     ) -> BackendResult<(CommitId, Commit)>;
 
     /// Perform garbage collection.
-    // TODO: pass in the set of commits to keep here
-    fn gc(&self) -> Result<(), Box<dyn std::error::Error + Send + Sync>>;
+    ///
+    /// All commits found in the `index` won't be removed. In addition to that,
+    /// objects created after `keep_newer` will be preserved. This mitigates a
+    /// risk of deleting new commits created concurrently by another process.
+    fn gc(&self, index: &dyn Index, keep_newer: SystemTime) -> BackendResult<()>;
 }
