@@ -32,7 +32,7 @@ pub struct CommitBuilder<'repo> {
 }
 
 impl CommitBuilder<'_> {
-    // Use MutRepo::new_commit() instead
+    /// Only called from [`MutRepo::new_commit`]. Use that function instead.
     pub(crate) fn for_new_commit<'repo>(
         mut_repo: &'repo mut MutableRepo,
         settings: &UserSettings,
@@ -62,7 +62,9 @@ impl CommitBuilder<'_> {
         }
     }
 
-    // Use MutRepo::rewrite_commit() instead
+    /// Only called from [`MutRepo::rewrite_commit`]. Use that function instead.
+    #[allow(unknown_lints)] // XXX FIXME (aseipp): nightly bogons; re-test this occasionally
+    #[allow(clippy::assigning_clones)]
     pub(crate) fn for_rewrite_from<'repo>(
         mut_repo: &'repo mut MutableRepo,
         settings: &UserSettings,
@@ -178,13 +180,6 @@ impl CommitBuilder<'_> {
     }
 
     pub fn write(mut self) -> BackendResult<Commit> {
-        let mut rewrite_source_id = None;
-        if let Some(rewrite_source) = self.rewrite_source {
-            if *rewrite_source.change_id() == self.commit.change_id {
-                rewrite_source_id.replace(rewrite_source.id().clone());
-            }
-        }
-
         let sign_settings = &self.sign_settings;
         let store = self.mut_repo.store();
 
@@ -202,9 +197,11 @@ impl CommitBuilder<'_> {
         let commit = self
             .mut_repo
             .write_commit(self.commit, signing_fn.as_deref_mut())?;
-        if let Some(rewrite_source_id) = rewrite_source_id {
-            self.mut_repo
-                .record_rewritten_commit(rewrite_source_id, commit.id().clone())
+        if let Some(rewrite_source) = self.rewrite_source {
+            if rewrite_source.change_id() == commit.change_id() {
+                self.mut_repo
+                    .set_rewritten_commit(rewrite_source.id().clone(), commit.id().clone());
+            }
         }
         Ok(commit)
     }
