@@ -14,13 +14,14 @@
 
 #![allow(missing_docs)]
 
-use std::io::Write;
+use std::io::{Read, Write};
 use std::iter::zip;
 
 use futures::StreamExt;
 use itertools::Itertools;
+use regex::bytes::Regex;
 
-use crate::backend::{BackendResult, FileId};
+use crate::backend::{BackendError, BackendResult, CommitId, FileId, SymlinkId, TreeId, TreeValue};
 use crate::diff::{find_line_ranges, Diff, DiffHunk};
 use crate::files;
 use crate::files::{ContentHunk, MergeResult};
@@ -28,11 +29,29 @@ use crate::merge::{Merge, MergeBuilder, MergedTreeValue};
 use crate::repo_path::RepoPath;
 use crate::store::Store;
 
-const CONFLICT_START_LINE: &[u8] = b"<<<<<<<\n";
-const CONFLICT_END_LINE: &[u8] = b">>>>>>>\n";
-const CONFLICT_DIFF_LINE: &[u8] = b"%%%%%%%\n";
-const CONFLICT_MINUS_LINE: &[u8] = b"-------\n";
-const CONFLICT_PLUS_LINE: &[u8] = b"+++++++\n";
+const CONFLICT_START_LINE: &[u8] = b"<<<<<<<";
+const CONFLICT_END_LINE: &[u8] = b">>>>>>>";
+const CONFLICT_DIFF_LINE: &[u8] = b"%%%%%%%";
+const CONFLICT_MINUS_LINE: &[u8] = b"-------";
+const CONFLICT_PLUS_LINE: &[u8] = b"+++++++";
+const CONFLICT_START_LINE_CHAR: u8 = CONFLICT_START_LINE[0];
+const CONFLICT_END_LINE_CHAR: u8 = CONFLICT_END_LINE[0];
+const CONFLICT_DIFF_LINE_CHAR: u8 = CONFLICT_DIFF_LINE[0];
+const CONFLICT_MINUS_LINE_CHAR: u8 = CONFLICT_MINUS_LINE[0];
+const CONFLICT_PLUS_LINE_CHAR: u8 = CONFLICT_PLUS_LINE[0];
+
+/// A conflict marker is one of the separators, optionally followed by a space
+/// and some text.
+// TODO: All the `{7}` could be replaced with `{7,}` to allow longer
+// separators. This could be useful to make it possible to allow conflict
+// markers inside the text of the conflicts.
+static CONFLICT_MARKER_REGEX: once_cell::sync::Lazy<Regex> = once_cell::sync::Lazy::new(|| {
+    Regex::new(
+        r"(<{7}|>{7}|%{7}|\-{7}|\+{7})( .*)?
+",
+    )
+    .unwrap()
+});
 
 fn write_diff_hunks(hunks: &[DiffHunk], file: &mut dyn Write) -> std::io::Result<()> {
     for hunk in hunks {
@@ -104,30 +123,139 @@ pub async fn materialize(
     }
 }
 
+/// A type similar to `MergedTreeValue` but with associated data to include in
+/// e.g. the working copy or in a diff.
+pub enum MaterializedTreeValue {
+    Absent,
+    AccessDenied(Box<dyn std::error::Error + Send + Sync>),
+    File {
+        id: FileId,
+        executable: bool,
+        reader: Box<dyn Read>,
+    },
+    Symlink {
+        id: SymlinkId,
+        target: String,
+    },
+    Conflict {
+        id: MergedTreeValue,
+        contents: Vec<u8>,
+        executable: bool,
+    },
+    GitSubmodule(CommitId),
+    Tree(TreeId),
+}
+
+impl MaterializedTreeValue {
+    pub fn is_absent(&self) -> bool {
+        matches!(self, MaterializedTreeValue::Absent)
+    }
+
+    pub fn is_present(&self) -> bool {
+        !self.is_absent()
+    }
+}
+
+/// Reads the data associated with a `MergedTreeValue` so it can be written to
+/// e.g. the working copy or diff.
+pub async fn materialize_tree_value(
+    store: &Store,
+    path: &RepoPath,
+    value: MergedTreeValue,
+) -> BackendResult<MaterializedTreeValue> {
+    match materialize_tree_value_no_access_denied(store, path, value).await {
+        Err(BackendError::ReadAccessDenied { source, .. }) => {
+            Ok(MaterializedTreeValue::AccessDenied(source))
+        }
+        result => result,
+    }
+}
+
+async fn materialize_tree_value_no_access_denied(
+    store: &Store,
+    path: &RepoPath,
+    value: MergedTreeValue,
+) -> BackendResult<MaterializedTreeValue> {
+    match value.into_resolved() {
+        Ok(None) => Ok(MaterializedTreeValue::Absent),
+        Ok(Some(TreeValue::File { id, executable })) => {
+            let reader = store.read_file_async(path, &id).await?;
+            Ok(MaterializedTreeValue::File {
+                id,
+                executable,
+                reader,
+            })
+        }
+        Ok(Some(TreeValue::Symlink(id))) => {
+            let target = store.read_symlink_async(path, &id).await?;
+            Ok(MaterializedTreeValue::Symlink { id, target })
+        }
+        Ok(Some(TreeValue::GitSubmodule(id))) => Ok(MaterializedTreeValue::GitSubmodule(id)),
+        Ok(Some(TreeValue::Tree(id))) => Ok(MaterializedTreeValue::Tree(id)),
+        Ok(Some(TreeValue::Conflict(_))) => {
+            panic!("cannot materialize legacy conflict object at path {path:?}");
+        }
+        Err(conflict) => {
+            let mut contents = vec![];
+            materialize(&conflict, store, path, &mut contents)
+                .await
+                .expect("Failed to materialize conflict to in-memory buffer");
+            let executable = if let Some(merge) = conflict.to_executable_merge() {
+                merge.resolve_trivial().copied().unwrap_or_default()
+            } else {
+                false
+            };
+            Ok(MaterializedTreeValue::Conflict {
+                id: conflict,
+                contents,
+                executable,
+            })
+        }
+    }
+}
+
 pub fn materialize_merge_result(
     single_hunk: &Merge<ContentHunk>,
     output: &mut dyn Write,
 ) -> std::io::Result<()> {
     let slices = single_hunk.map(|content| content.0.as_slice());
-    let merge_result = files::merge(slices);
+    let merge_result = files::merge(&slices);
     match merge_result {
         MergeResult::Resolved(content) => {
             output.write_all(&content.0)?;
         }
         MergeResult::Conflict(hunks) => {
+            let num_conflicts = hunks
+                .iter()
+                .filter(|hunk| hunk.as_resolved().is_none())
+                .count();
+            let mut conflict_index = 0;
             for hunk in hunks {
                 if let Some(content) = hunk.as_resolved() {
                     output.write_all(&content.0)?;
                 } else {
+                    conflict_index += 1;
                     output.write_all(CONFLICT_START_LINE)?;
+                    output.write_all(
+                        format!(" Conflict {conflict_index} of {num_conflicts}\n").as_bytes(),
+                    )?;
                     let mut add_index = 0;
-                    for left in hunk.removes() {
-                        let right1 = if let Some(right1) = hunk.adds().get(add_index) {
+                    for (base_index, left) in hunk.removes().enumerate() {
+                        // The vast majority of conflicts one actually tries to
+                        // resolve manually have 1 base.
+                        let base_str = if hunk.removes().len() == 1 {
+                            "base".to_string()
+                        } else {
+                            format!("base #{}", base_index + 1)
+                        };
+
+                        let right1 = if let Some(right1) = hunk.get_add(add_index) {
                             right1
                         } else {
                             // If we have no more positive terms, emit the remaining negative
                             // terms as snapshots.
                             output.write_all(CONFLICT_MINUS_LINE)?;
+                            output.write_all(format!(" Contents of {base_str}\n").as_bytes())?;
                             output.write_all(&left.0)?;
                             continue;
                         };
@@ -137,7 +265,7 @@ pub fn materialize_merge_result(
                         // Check if the diff against the next positive term is better. Since
                         // we want to preserve the order of the terms, we don't match against
                         // any later positive terms.
-                        if let Some(right2) = hunk.adds().get(add_index + 1) {
+                        if let Some(right2) = hunk.get_add(add_index + 1) {
                             let diff2 =
                                 Diff::for_tokenizer(&[&left.0, &right2.0], &find_line_ranges)
                                     .hunks()
@@ -147,8 +275,18 @@ pub fn materialize_merge_result(
                                 // the current positive term as a snapshot and the next
                                 // positive term as a diff.
                                 output.write_all(CONFLICT_PLUS_LINE)?;
+                                output.write_all(
+                                    format!(" Contents of side #{}\n", add_index + 1).as_bytes(),
+                                )?;
                                 output.write_all(&right1.0)?;
                                 output.write_all(CONFLICT_DIFF_LINE)?;
+                                output.write_all(
+                                    format!(
+                                        " Changes from {base_str} to side #{}\n",
+                                        add_index + 2
+                                    )
+                                    .as_bytes(),
+                                )?;
                                 write_diff_hunks(&diff2, output)?;
                                 add_index += 2;
                                 continue;
@@ -156,16 +294,26 @@ pub fn materialize_merge_result(
                         }
 
                         output.write_all(CONFLICT_DIFF_LINE)?;
+                        output.write_all(
+                            format!(" Changes from {base_str} to side #{}\n", add_index + 1)
+                                .as_bytes(),
+                        )?;
                         write_diff_hunks(&diff1, output)?;
                         add_index += 1;
                     }
 
                     //  Emit the remaining positive terms as snapshots.
-                    for slice in &hunk.adds()[add_index..] {
+                    for (add_index, slice) in hunk.adds().enumerate().skip(add_index) {
                         output.write_all(CONFLICT_PLUS_LINE)?;
+                        output.write_all(
+                            format!(" Contents of side #{}\n", add_index + 1).as_bytes(),
+                        )?;
                         output.write_all(&slice.0)?;
                     }
                     output.write_all(CONFLICT_END_LINE)?;
+                    output.write_all(
+                        format!(" Conflict {conflict_index} of {num_conflicts} ends\n").as_bytes(),
+                    )?;
                 }
             }
         }
@@ -197,21 +345,25 @@ pub fn parse_conflict(input: &[u8], num_sides: usize) -> Option<Vec<Merge<Conten
     let mut pos = 0;
     let mut resolved_start = 0;
     let mut conflict_start = None;
+    let mut conflict_start_len = 0;
     for line in input.split_inclusive(|b| *b == b'\n') {
-        if line == CONFLICT_START_LINE {
-            conflict_start = Some(pos);
-        } else if conflict_start.is_some() && line == CONFLICT_END_LINE {
-            let conflict_body = &input[conflict_start.unwrap() + CONFLICT_START_LINE.len()..pos];
-            let hunk = parse_conflict_hunk(conflict_body);
-            if hunk.num_sides() == num_sides {
-                let resolved_slice = &input[resolved_start..conflict_start.unwrap()];
-                if !resolved_slice.is_empty() {
-                    hunks.push(Merge::resolved(ContentHunk(resolved_slice.to_vec())));
+        if CONFLICT_MARKER_REGEX.is_match_at(line, 0) {
+            if line[0] == CONFLICT_START_LINE_CHAR {
+                conflict_start = Some(pos);
+                conflict_start_len = line.len();
+            } else if conflict_start.is_some() && line[0] == CONFLICT_END_LINE_CHAR {
+                let conflict_body = &input[conflict_start.unwrap() + conflict_start_len..pos];
+                let hunk = parse_conflict_hunk(conflict_body);
+                if hunk.num_sides() == num_sides {
+                    let resolved_slice = &input[resolved_start..conflict_start.unwrap()];
+                    if !resolved_slice.is_empty() {
+                        hunks.push(Merge::resolved(ContentHunk(resolved_slice.to_vec())));
+                    }
+                    hunks.push(hunk);
+                    resolved_start = pos + line.len();
                 }
-                hunks.push(hunk);
-                resolved_start = pos + line.len();
+                conflict_start = None;
             }
-            conflict_start = None;
         }
         pos += line.len();
     }
@@ -239,24 +391,26 @@ fn parse_conflict_hunk(input: &[u8]) -> Merge<ContentHunk> {
     let mut removes = vec![];
     let mut adds = vec![];
     for line in input.split_inclusive(|b| *b == b'\n') {
-        match line {
-            CONFLICT_DIFF_LINE => {
-                state = State::Diff;
-                removes.push(ContentHunk(vec![]));
-                adds.push(ContentHunk(vec![]));
-                continue;
+        if CONFLICT_MARKER_REGEX.is_match_at(line, 0) {
+            match line[0] {
+                CONFLICT_DIFF_LINE_CHAR => {
+                    state = State::Diff;
+                    removes.push(ContentHunk(vec![]));
+                    adds.push(ContentHunk(vec![]));
+                    continue;
+                }
+                CONFLICT_MINUS_LINE_CHAR => {
+                    state = State::Minus;
+                    removes.push(ContentHunk(vec![]));
+                    continue;
+                }
+                CONFLICT_PLUS_LINE_CHAR => {
+                    state = State::Plus;
+                    adds.push(ContentHunk(vec![]));
+                    continue;
+                }
+                _ => {}
             }
-            CONFLICT_MINUS_LINE => {
-                state = State::Minus;
-                removes.push(ContentHunk(vec![]));
-                continue;
-            }
-            CONFLICT_PLUS_LINE => {
-                state = State::Plus;
-                adds.push(ContentHunk(vec![]));
-                continue;
-            }
-            _ => {}
         };
         match state {
             State::Diff => {
@@ -285,7 +439,7 @@ fn parse_conflict_hunk(input: &[u8]) -> Merge<ContentHunk> {
         }
     }
 
-    Merge::new(removes, adds)
+    Merge::from_removes_adds(removes, adds)
 }
 
 /// Parses conflict markers in `content` and returns an updated version of
