@@ -12,6 +12,13 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+//! An on-disk index of the commits in a repository.
+//!
+//! Implements an index of the commits in a repository that conforms to the
+//! trains in the [index module](crate::index). The index is stored on local
+//! disk and contains an entry for every commit in the repository. See
+//! [`DefaultReadonlyIndex`] and [`DefaultMutableIndex`].
+
 #![allow(missing_docs)]
 
 mod composite;
@@ -20,54 +27,50 @@ mod mutable;
 mod readonly;
 mod rev_walk;
 pub mod revset_engine;
-pub mod revset_graph_iterator;
+mod revset_graph_iterator;
 mod store;
 
 pub use self::composite::{AsCompositeIndex, CompositeIndex, IndexLevelStats, IndexStats};
 pub use self::entry::{IndexEntry, IndexPosition};
 pub use self::mutable::DefaultMutableIndex;
 pub use self::readonly::{DefaultReadonlyIndex, ReadonlyIndexLoadError};
-pub use self::rev_walk::{
-    RevWalk, RevWalkDescendants, RevWalkDescendantsGenerationRange, RevWalkGenerationRange,
-};
 pub use self::store::{DefaultIndexStore, DefaultIndexStoreError, DefaultIndexStoreInitError};
 
 #[cfg(test)]
 mod tests {
-    use std::ops::Range;
     use std::sync::Arc;
 
     use itertools::Itertools;
     use smallvec::smallvec_inline;
     use test_case::test_case;
 
-    use super::composite::IndexSegment;
+    use super::composite::{DynIndexSegment, IndexSegment};
     use super::entry::SmallIndexPositionsVec;
     use super::mutable::MutableIndexSegment;
     use super::*;
     use crate::backend::{ChangeId, CommitId};
+    use crate::default_index::entry::{LocalPosition, SmallLocalPositionsVec};
     use crate::index::Index;
     use crate::object_id::{HexPrefix, ObjectId, PrefixResolution};
 
+    /// Generator of unique 16-byte CommitId excluding root id
+    fn commit_id_generator() -> impl FnMut() -> CommitId {
+        let mut iter = (1_u128..).map(|n| CommitId::new(n.to_le_bytes().into()));
+        move || iter.next().unwrap()
+    }
+
     /// Generator of unique 16-byte ChangeId excluding root id
     fn change_id_generator() -> impl FnMut() -> ChangeId {
         let mut iter = (1_u128..).map(|n| ChangeId::new(n.to_le_bytes().into()));
         move || iter.next().unwrap()
     }
 
-    fn to_positions_vec(index: CompositeIndex<'_>, commit_ids: &[CommitId]) -> Vec<IndexPosition> {
-        commit_ids
-            .iter()
-            .map(|id| index.commit_id_to_pos(id).unwrap())
-            .collect()
-    }
-
     #[test_case(false; "memory")]
     #[test_case(true; "file")]
     fn index_empty(on_disk: bool) {
         let temp_dir = testutils::new_temp_dir();
         let mutable_segment = MutableIndexSegment::full(3, 16);
-        let index_segment: Box<dyn IndexSegment> = if on_disk {
+        let index_segment: Box<DynIndexSegment> = if on_disk {
             let saved_index = mutable_segment.save_in(temp_dir.path()).unwrap();
             Box::new(Arc::try_unwrap(saved_index).unwrap())
         } else {
@@ -98,7 +101,7 @@ mod tests {
         let id_0 = CommitId::from_hex("000000");
         let change_id0 = new_change_id();
         mutable_segment.add_commit_data(id_0.clone(), change_id0.clone(), &[]);
-        let index_segment: Box<dyn IndexSegment> = if on_disk {
+        let index_segment: Box<DynIndexSegment> = if on_disk {
             let saved_index = mutable_segment.save_in(temp_dir.path()).unwrap();
             Box::new(Arc::try_unwrap(saved_index).unwrap())
         } else {
@@ -183,7 +186,7 @@ mod tests {
         mutable_segment.add_commit_data(id_3.clone(), change_id3.clone(), &[id_2.clone()]);
         mutable_segment.add_commit_data(id_4.clone(), change_id4, &[id_1.clone()]);
         mutable_segment.add_commit_data(id_5.clone(), change_id5, &[id_4.clone(), id_2.clone()]);
-        let index_segment: Box<dyn IndexSegment> = if on_disk {
+        let index_segment: Box<DynIndexSegment> = if on_disk {
             let saved_index = mutable_segment.save_in(temp_dir.path()).unwrap();
             Box::new(Arc::try_unwrap(saved_index).unwrap())
         } else {
@@ -295,7 +298,7 @@ mod tests {
             new_change_id(),
             &[id_1, id_2, id_3, id_4, id_5],
         );
-        let index_segment: Box<dyn IndexSegment> = if on_disk {
+        let index_segment: Box<DynIndexSegment> = if on_disk {
             let saved_index = mutable_segment.save_in(temp_dir.path()).unwrap();
             Box::new(Arc::try_unwrap(saved_index).unwrap())
         } else {
@@ -328,7 +331,7 @@ mod tests {
     }
 
     #[test]
-    fn resolve_prefix() {
+    fn resolve_commit_id_prefix() {
         let temp_dir = testutils::new_temp_dir();
         let mut new_change_id = change_id_generator();
         let mut mutable_segment = MutableIndexSegment::full(3, 16);
@@ -581,6 +584,382 @@ mod tests {
         );
     }
 
+    #[test]
+    fn resolve_change_id_prefix() {
+        let temp_dir = testutils::new_temp_dir();
+        let mut new_commit_id = commit_id_generator();
+        let local_positions_vec = |positions: &[u32]| -> SmallLocalPositionsVec {
+            positions.iter().copied().map(LocalPosition).collect()
+        };
+        let index_positions_vec = |positions: &[u32]| -> SmallIndexPositionsVec {
+            positions.iter().copied().map(IndexPosition).collect()
+        };
+
+        let id_0 = ChangeId::from_hex("00000001");
+        let id_1 = ChangeId::from_hex("00999999");
+        let id_2 = ChangeId::from_hex("05548888");
+        let id_3 = ChangeId::from_hex("05544444");
+        let id_4 = ChangeId::from_hex("05555555");
+        let id_5 = ChangeId::from_hex("05555333");
+
+        // Create some commits with different various common prefixes.
+        let mut mutable_segment = MutableIndexSegment::full(16, 4);
+        mutable_segment.add_commit_data(new_commit_id(), id_0.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_1.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_2.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_1.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_2.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_2.clone(), &[]);
+
+        // Write these commits to one file and build the remainder on top.
+        let initial_file = mutable_segment.save_in(temp_dir.path()).unwrap();
+        mutable_segment = MutableIndexSegment::incremental(initial_file.clone());
+
+        mutable_segment.add_commit_data(new_commit_id(), id_3.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_3.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_4.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_1.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_5.clone(), &[]);
+
+        // Local lookup in readonly index with the full hex digits
+        assert_eq!(
+            initial_file.resolve_change_id_prefix(&HexPrefix::new(&id_0.hex()).unwrap()),
+            PrefixResolution::SingleMatch((id_0.clone(), local_positions_vec(&[0])))
+        );
+        assert_eq!(
+            initial_file.resolve_change_id_prefix(&HexPrefix::new(&id_1.hex()).unwrap()),
+            PrefixResolution::SingleMatch((id_1.clone(), local_positions_vec(&[1, 3])))
+        );
+        assert_eq!(
+            initial_file.resolve_change_id_prefix(&HexPrefix::new(&id_2.hex()).unwrap()),
+            PrefixResolution::SingleMatch((id_2.clone(), local_positions_vec(&[2, 4, 5])))
+        );
+
+        // Local lookup in mutable index with the full hex digits
+        assert_eq!(
+            mutable_segment.resolve_change_id_prefix(&HexPrefix::new(&id_1.hex()).unwrap()),
+            PrefixResolution::SingleMatch((id_1.clone(), local_positions_vec(&[3])))
+        );
+        assert_eq!(
+            mutable_segment.resolve_change_id_prefix(&HexPrefix::new(&id_3.hex()).unwrap()),
+            PrefixResolution::SingleMatch((id_3.clone(), local_positions_vec(&[0, 1])))
+        );
+        assert_eq!(
+            mutable_segment.resolve_change_id_prefix(&HexPrefix::new(&id_4.hex()).unwrap()),
+            PrefixResolution::SingleMatch((id_4.clone(), local_positions_vec(&[2])))
+        );
+        assert_eq!(
+            mutable_segment.resolve_change_id_prefix(&HexPrefix::new(&id_5.hex()).unwrap()),
+            PrefixResolution::SingleMatch((id_5.clone(), local_positions_vec(&[4])))
+        );
+
+        // Local lookup with locally unknown prefix
+        assert_eq!(
+            initial_file.resolve_change_id_prefix(&HexPrefix::new("0555").unwrap()),
+            PrefixResolution::NoMatch
+        );
+        assert_eq!(
+            mutable_segment.resolve_change_id_prefix(&HexPrefix::new("000").unwrap()),
+            PrefixResolution::NoMatch
+        );
+
+        // Local lookup with locally unique prefix
+        assert_eq!(
+            initial_file.resolve_change_id_prefix(&HexPrefix::new("0554").unwrap()),
+            PrefixResolution::SingleMatch((id_2.clone(), local_positions_vec(&[2, 4, 5])))
+        );
+        assert_eq!(
+            mutable_segment.resolve_change_id_prefix(&HexPrefix::new("0554").unwrap()),
+            PrefixResolution::SingleMatch((id_3.clone(), local_positions_vec(&[0, 1])))
+        );
+
+        // Local lookup with locally ambiguous prefix
+        assert_eq!(
+            initial_file.resolve_change_id_prefix(&HexPrefix::new("00").unwrap()),
+            PrefixResolution::AmbiguousMatch
+        );
+        assert_eq!(
+            mutable_segment.resolve_change_id_prefix(&HexPrefix::new("05555").unwrap()),
+            PrefixResolution::AmbiguousMatch
+        );
+
+        let index = mutable_segment.as_composite();
+
+        // Global lookup with the full hex digits
+        assert_eq!(
+            index.resolve_change_id_prefix(&HexPrefix::new(&id_0.hex()).unwrap()),
+            PrefixResolution::SingleMatch((id_0.clone(), index_positions_vec(&[0])))
+        );
+        assert_eq!(
+            index.resolve_change_id_prefix(&HexPrefix::new(&id_1.hex()).unwrap()),
+            PrefixResolution::SingleMatch((id_1.clone(), index_positions_vec(&[1, 3, 9])))
+        );
+        assert_eq!(
+            index.resolve_change_id_prefix(&HexPrefix::new(&id_2.hex()).unwrap()),
+            PrefixResolution::SingleMatch((id_2.clone(), index_positions_vec(&[2, 4, 5])))
+        );
+        assert_eq!(
+            index.resolve_change_id_prefix(&HexPrefix::new(&id_3.hex()).unwrap()),
+            PrefixResolution::SingleMatch((id_3.clone(), index_positions_vec(&[6, 7])))
+        );
+        assert_eq!(
+            index.resolve_change_id_prefix(&HexPrefix::new(&id_4.hex()).unwrap()),
+            PrefixResolution::SingleMatch((id_4.clone(), index_positions_vec(&[8])))
+        );
+        assert_eq!(
+            index.resolve_change_id_prefix(&HexPrefix::new(&id_5.hex()).unwrap()),
+            PrefixResolution::SingleMatch((id_5.clone(), index_positions_vec(&[10])))
+        );
+
+        // Global lookup with unknown prefix
+        assert_eq!(
+            index.resolve_change_id_prefix(&HexPrefix::new("ffffffff").unwrap()),
+            PrefixResolution::NoMatch
+        );
+        assert_eq!(
+            index.resolve_change_id_prefix(&HexPrefix::new("00000002").unwrap()),
+            PrefixResolution::NoMatch
+        );
+
+        // Global lookup with globally unique prefix
+        assert_eq!(
+            index.resolve_change_id_prefix(&HexPrefix::new("000").unwrap()),
+            PrefixResolution::SingleMatch((id_0.clone(), index_positions_vec(&[0])))
+        );
+        assert_eq!(
+            index.resolve_change_id_prefix(&HexPrefix::new("055553").unwrap()),
+            PrefixResolution::SingleMatch((id_5.clone(), index_positions_vec(&[10])))
+        );
+
+        // Global lookup with globally unique prefix stored in both parts
+        assert_eq!(
+            index.resolve_change_id_prefix(&HexPrefix::new("009").unwrap()),
+            PrefixResolution::SingleMatch((id_1.clone(), index_positions_vec(&[1, 3, 9])))
+        );
+
+        // Global lookup with locally ambiguous prefix
+        assert_eq!(
+            index.resolve_change_id_prefix(&HexPrefix::new("00").unwrap()),
+            PrefixResolution::AmbiguousMatch
+        );
+        assert_eq!(
+            index.resolve_change_id_prefix(&HexPrefix::new("05555").unwrap()),
+            PrefixResolution::AmbiguousMatch
+        );
+
+        // Global lookup with locally unique but globally ambiguous prefix
+        assert_eq!(
+            index.resolve_change_id_prefix(&HexPrefix::new("0554").unwrap()),
+            PrefixResolution::AmbiguousMatch
+        );
+    }
+
+    #[test]
+    fn neighbor_change_ids() {
+        let temp_dir = testutils::new_temp_dir();
+        let mut new_commit_id = commit_id_generator();
+
+        let id_0 = ChangeId::from_hex("00000001");
+        let id_1 = ChangeId::from_hex("00999999");
+        let id_2 = ChangeId::from_hex("05548888");
+        let id_3 = ChangeId::from_hex("05544444");
+        let id_4 = ChangeId::from_hex("05555555");
+        let id_5 = ChangeId::from_hex("05555333");
+
+        // Create some commits with different various common prefixes.
+        let mut mutable_segment = MutableIndexSegment::full(16, 4);
+        mutable_segment.add_commit_data(new_commit_id(), id_0.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_1.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_2.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_1.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_2.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_2.clone(), &[]);
+
+        // Write these commits to one file and build the remainder on top.
+        let initial_file = mutable_segment.save_in(temp_dir.path()).unwrap();
+        mutable_segment = MutableIndexSegment::incremental(initial_file.clone());
+
+        mutable_segment.add_commit_data(new_commit_id(), id_3.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_3.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_4.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_1.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_5.clone(), &[]);
+
+        // Local lookup in readonly index, change_id exists.
+        assert_eq!(
+            initial_file.resolve_neighbor_change_ids(&id_0),
+            (None, Some(id_1.clone())),
+        );
+        assert_eq!(
+            initial_file.resolve_neighbor_change_ids(&id_1),
+            (Some(id_0.clone()), Some(id_2.clone())),
+        );
+        assert_eq!(
+            initial_file.resolve_neighbor_change_ids(&id_2),
+            (Some(id_1.clone()), None),
+        );
+
+        // Local lookup in readonly index, change_id does not exist.
+        assert_eq!(
+            initial_file.resolve_neighbor_change_ids(&ChangeId::from_hex("00000000")),
+            (None, Some(id_0.clone())),
+        );
+        assert_eq!(
+            initial_file.resolve_neighbor_change_ids(&ChangeId::from_hex("00000002")),
+            (Some(id_0.clone()), Some(id_1.clone())),
+        );
+        assert_eq!(
+            initial_file.resolve_neighbor_change_ids(&ChangeId::from_hex("ffffffff")),
+            (Some(id_2.clone()), None),
+        );
+
+        // Local lookup in mutable index, change_id exists.
+        // id_1 < id_3 < id_5 < id_4
+        assert_eq!(
+            mutable_segment.resolve_neighbor_change_ids(&id_1),
+            (None, Some(id_3.clone())),
+        );
+        assert_eq!(
+            mutable_segment.resolve_neighbor_change_ids(&id_3),
+            (Some(id_1.clone()), Some(id_5.clone())),
+        );
+        assert_eq!(
+            mutable_segment.resolve_neighbor_change_ids(&id_5),
+            (Some(id_3.clone()), Some(id_4.clone())),
+        );
+        assert_eq!(
+            mutable_segment.resolve_neighbor_change_ids(&id_4),
+            (Some(id_5.clone()), None),
+        );
+
+        // Local lookup in mutable index, change_id does not exist.
+        // id_1 < id_3 < id_5 < id_4
+        assert_eq!(
+            mutable_segment.resolve_neighbor_change_ids(&ChangeId::from_hex("00999998")),
+            (None, Some(id_1.clone())),
+        );
+        assert_eq!(
+            mutable_segment.resolve_neighbor_change_ids(&ChangeId::from_hex("01000000")),
+            (Some(id_1.clone()), Some(id_3.clone())),
+        );
+        assert_eq!(
+            mutable_segment.resolve_neighbor_change_ids(&ChangeId::from_hex("05555332")),
+            (Some(id_3.clone()), Some(id_5.clone())),
+        );
+        assert_eq!(
+            mutable_segment.resolve_neighbor_change_ids(&ChangeId::from_hex("ffffffff")),
+            (Some(id_4.clone()), None),
+        );
+
+        let index = mutable_segment.as_composite();
+
+        // Global lookup, change_id exists.
+        // id_0 < id_1 < id_3 < id_2 < id_5 < id_4
+        assert_eq!(
+            index.resolve_neighbor_change_ids(&id_0),
+            (None, Some(id_1.clone())),
+        );
+        assert_eq!(
+            index.resolve_neighbor_change_ids(&id_1),
+            (Some(id_0.clone()), Some(id_3.clone())),
+        );
+        assert_eq!(
+            index.resolve_neighbor_change_ids(&id_3),
+            (Some(id_1.clone()), Some(id_2.clone())),
+        );
+        assert_eq!(
+            index.resolve_neighbor_change_ids(&id_2),
+            (Some(id_3.clone()), Some(id_5.clone())),
+        );
+        assert_eq!(
+            index.resolve_neighbor_change_ids(&id_5),
+            (Some(id_2.clone()), Some(id_4.clone())),
+        );
+        assert_eq!(
+            index.resolve_neighbor_change_ids(&id_4),
+            (Some(id_5.clone()), None),
+        );
+
+        // Global lookup, change_id doesn't exist.
+        // id_0 < id_1 < id_3 < id_2 < id_5 < id_4
+        assert_eq!(
+            index.resolve_neighbor_change_ids(&ChangeId::from_hex("00000000")),
+            (None, Some(id_0.clone())),
+        );
+        assert_eq!(
+            index.resolve_neighbor_change_ids(&ChangeId::from_hex("01000000")),
+            (Some(id_1.clone()), Some(id_3.clone())),
+        );
+        assert_eq!(
+            index.resolve_neighbor_change_ids(&ChangeId::from_hex("05544555")),
+            (Some(id_3.clone()), Some(id_2.clone())),
+        );
+        assert_eq!(
+            index.resolve_neighbor_change_ids(&ChangeId::from_hex("ffffffff")),
+            (Some(id_4.clone()), None),
+        );
+    }
+
+    #[test]
+    fn shortest_unique_change_id_prefix() {
+        let temp_dir = testutils::new_temp_dir();
+        let mut new_commit_id = commit_id_generator();
+
+        let id_0 = ChangeId::from_hex("00000001");
+        let id_1 = ChangeId::from_hex("00999999");
+        let id_2 = ChangeId::from_hex("05548888");
+        let id_3 = ChangeId::from_hex("05544444");
+        let id_4 = ChangeId::from_hex("05555555");
+        let id_5 = ChangeId::from_hex("05555333");
+
+        // Create some commits with different various common prefixes.
+        let mut mutable_segment = MutableIndexSegment::full(16, 4);
+        mutable_segment.add_commit_data(new_commit_id(), id_0.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_1.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_2.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_1.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_2.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_2.clone(), &[]);
+
+        // Write these commits to one file and build the remainder on top.
+        let initial_file = mutable_segment.save_in(temp_dir.path()).unwrap();
+        mutable_segment = MutableIndexSegment::incremental(initial_file.clone());
+
+        mutable_segment.add_commit_data(new_commit_id(), id_3.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_3.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_4.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_1.clone(), &[]);
+        mutable_segment.add_commit_data(new_commit_id(), id_5.clone(), &[]);
+
+        let index = mutable_segment.as_composite();
+
+        // Calculate shortest unique prefix len with known change_id
+        assert_eq!(index.shortest_unique_change_id_prefix_len(&id_0), 3);
+        assert_eq!(index.shortest_unique_change_id_prefix_len(&id_1), 3);
+        assert_eq!(index.shortest_unique_change_id_prefix_len(&id_2), 5);
+        assert_eq!(index.shortest_unique_change_id_prefix_len(&id_3), 5);
+        assert_eq!(index.shortest_unique_change_id_prefix_len(&id_4), 6);
+        assert_eq!(index.shortest_unique_change_id_prefix_len(&id_5), 6);
+
+        // Calculate shortest unique prefix len with unknown change_id
+        assert_eq!(
+            index.shortest_unique_change_id_prefix_len(&ChangeId::from_hex("00000002")),
+            8
+        );
+        assert_eq!(
+            index.shortest_unique_change_id_prefix_len(&ChangeId::from_hex("01000000")),
+            2
+        );
+        assert_eq!(
+            index.shortest_unique_change_id_prefix_len(&ChangeId::from_hex("05555344")),
+            7
+        );
+        assert_eq!(
+            index.shortest_unique_change_id_prefix_len(&ChangeId::from_hex("ffffffff")),
+            1
+        );
+    }
+
     #[test]
     fn test_is_ancestor() {
         let mut new_change_id = change_id_generator();
@@ -636,12 +1015,14 @@ mod tests {
         let id_3 = CommitId::from_hex("333333");
         let id_4 = CommitId::from_hex("444444");
         let id_5 = CommitId::from_hex("555555");
+        let id_6 = CommitId::from_hex("666666");
         index.add_commit_data(id_0.clone(), new_change_id(), &[]);
         index.add_commit_data(id_1.clone(), new_change_id(), &[id_0.clone()]);
         index.add_commit_data(id_2.clone(), new_change_id(), &[id_0.clone()]);
         index.add_commit_data(id_3.clone(), new_change_id(), &[id_0.clone()]);
         index.add_commit_data(id_4.clone(), new_change_id(), &[id_1.clone()]);
         index.add_commit_data(id_5.clone(), new_change_id(), &[id_4.clone(), id_2.clone()]);
+        index.add_commit_data(id_6.clone(), new_change_id(), &[id_4.clone()]);
 
         assert_eq!(
             index.common_ancestors(&[id_0.clone()], &[id_0.clone()]),
@@ -675,6 +1056,10 @@ mod tests {
             index.common_ancestors(&[id_5.clone()], &[id_3.clone()]),
             vec![id_0.clone()]
         );
+        assert_eq!(
+            index.common_ancestors(&[id_2.clone()], &[id_6.clone()]),
+            vec![id_0.clone()]
+        );
 
         // With multiple commits in an input set
         assert_eq!(
@@ -693,6 +1078,12 @@ mod tests {
             index.common_ancestors(&[id_1.clone(), id_2.clone()], &[id_4]),
             vec![id_1.clone()]
         );
+        assert_eq!(
+            index.common_ancestors(&[id_5.clone(), id_6.clone()], &[id_2.clone()]),
+            &[id_2.clone()]
+        );
+        // Both 1 and 2 are returned since (5) expands to (2, 4), which expands
+        // to (1,2) and matches the (1,2) of the first input set.
         assert_eq!(
             index.common_ancestors(&[id_1.clone(), id_2.clone()], &[id_5]),
             vec![id_1.clone(), id_2.clone()]
@@ -752,346 +1143,6 @@ mod tests {
         assert_eq!(common_ancestors, vec![id_2]);
     }
 
-    #[test]
-    fn test_walk_revs() {
-        let mut new_change_id = change_id_generator();
-        let mut index = DefaultMutableIndex::full(3, 16);
-        // 5
-        // |\
-        // 4 | 3
-        // | |/
-        // 1 2
-        // |/
-        // 0
-        let id_0 = CommitId::from_hex("000000");
-        let id_1 = CommitId::from_hex("111111");
-        let id_2 = CommitId::from_hex("222222");
-        let id_3 = CommitId::from_hex("333333");
-        let id_4 = CommitId::from_hex("444444");
-        let id_5 = CommitId::from_hex("555555");
-        index.add_commit_data(id_0.clone(), new_change_id(), &[]);
-        index.add_commit_data(id_1.clone(), new_change_id(), &[id_0.clone()]);
-        index.add_commit_data(id_2.clone(), new_change_id(), &[id_0.clone()]);
-        index.add_commit_data(id_3.clone(), new_change_id(), &[id_2.clone()]);
-        index.add_commit_data(id_4.clone(), new_change_id(), &[id_1.clone()]);
-        index.add_commit_data(id_5.clone(), new_change_id(), &[id_4.clone(), id_2.clone()]);
-
-        let walk_commit_ids = |wanted: &[CommitId], unwanted: &[CommitId]| {
-            let index = index.as_composite();
-            let wanted_positions = to_positions_vec(index, wanted);
-            let unwanted_positions = to_positions_vec(index, unwanted);
-            index
-                .walk_revs(&wanted_positions, &unwanted_positions)
-                .map(|entry| entry.commit_id())
-                .collect_vec()
-        };
-
-        // No wanted commits
-        assert!(walk_commit_ids(&[], &[]).is_empty());
-        // Simple linear walk to roo
-        assert_eq!(
-            walk_commit_ids(&[id_4.clone()], &[]),
-            vec![id_4.clone(), id_1.clone(), id_0.clone()]
-        );
-        // Commits that are both wanted and unwanted are not walked
-        assert_eq!(walk_commit_ids(&[id_0.clone()], &[id_0.clone()]), vec![]);
-        // Commits that are listed twice are only walked once
-        assert_eq!(
-            walk_commit_ids(&[id_0.clone(), id_0.clone()], &[]),
-            vec![id_0.clone()]
-        );
-        // If a commit and its ancestor are both wanted, the ancestor still gets walked
-        // only once
-        assert_eq!(
-            walk_commit_ids(&[id_0.clone(), id_1.clone()], &[]),
-            vec![id_1.clone(), id_0.clone()]
-        );
-        // Ancestors of both wanted and unwanted commits are not walked
-        assert_eq!(
-            walk_commit_ids(&[id_2.clone()], &[id_1.clone()]),
-            vec![id_2.clone()]
-        );
-        // Same as above, but the opposite order, to make sure that order in index
-        // doesn't matter
-        assert_eq!(
-            walk_commit_ids(&[id_1.clone()], &[id_2.clone()]),
-            vec![id_1.clone()]
-        );
-        // Two wanted nodes
-        assert_eq!(
-            walk_commit_ids(&[id_1.clone(), id_2.clone()], &[]),
-            vec![id_2.clone(), id_1.clone(), id_0.clone()]
-        );
-        // Order of output doesn't depend on order of input
-        assert_eq!(
-            walk_commit_ids(&[id_2.clone(), id_1.clone()], &[]),
-            vec![id_2.clone(), id_1.clone(), id_0]
-        );
-        // Two wanted nodes that share an unwanted ancestor
-        assert_eq!(
-            walk_commit_ids(&[id_5.clone(), id_3.clone()], &[id_2]),
-            vec![id_5, id_4, id_3, id_1]
-        );
-    }
-
-    #[test]
-    fn test_walk_revs_filter_by_generation() {
-        let mut new_change_id = change_id_generator();
-        let mut index = DefaultMutableIndex::full(3, 16);
-        // 8 6
-        // | |
-        // 7 5
-        // |/|
-        // 4 |
-        // | 3
-        // 2 |
-        // |/
-        // 1
-        // |
-        // 0
-        let id_0 = CommitId::from_hex("000000");
-        let id_1 = CommitId::from_hex("111111");
-        let id_2 = CommitId::from_hex("222222");
-        let id_3 = CommitId::from_hex("333333");
-        let id_4 = CommitId::from_hex("444444");
-        let id_5 = CommitId::from_hex("555555");
-        let id_6 = CommitId::from_hex("666666");
-        let id_7 = CommitId::from_hex("777777");
-        let id_8 = CommitId::from_hex("888888");
-        index.add_commit_data(id_0.clone(), new_change_id(), &[]);
-        index.add_commit_data(id_1.clone(), new_change_id(), &[id_0.clone()]);
-        index.add_commit_data(id_2.clone(), new_change_id(), &[id_1.clone()]);
-        index.add_commit_data(id_3.clone(), new_change_id(), &[id_1.clone()]);
-        index.add_commit_data(id_4.clone(), new_change_id(), &[id_2.clone()]);
-        index.add_commit_data(id_5.clone(), new_change_id(), &[id_4.clone(), id_3.clone()]);
-        index.add_commit_data(id_6.clone(), new_change_id(), &[id_5.clone()]);
-        index.add_commit_data(id_7.clone(), new_change_id(), &[id_4.clone()]);
-        index.add_commit_data(id_8.clone(), new_change_id(), &[id_7.clone()]);
-
-        let walk_commit_ids = |wanted: &[CommitId], unwanted: &[CommitId], range: Range<u32>| {
-            let index = index.as_composite();
-            let wanted_positions = to_positions_vec(index, wanted);
-            let unwanted_positions = to_positions_vec(index, unwanted);
-            index
-                .walk_revs(&wanted_positions, &unwanted_positions)
-                .filter_by_generation(range)
-                .map(|entry| entry.commit_id())
-                .collect_vec()
-        };
-
-        // Empty generation bounds
-        assert_eq!(walk_commit_ids(&[&id_8].map(Clone::clone), &[], 0..0), []);
-        assert_eq!(
-            walk_commit_ids(&[&id_8].map(Clone::clone), &[], Range { start: 2, end: 1 }),
-            []
-        );
-
-        // Simple generation bounds
-        assert_eq!(
-            walk_commit_ids(&[&id_2].map(Clone::clone), &[], 0..3),
-            [&id_2, &id_1, &id_0].map(Clone::clone)
-        );
-
-        // Ancestors may be walked with different generations
-        assert_eq!(
-            walk_commit_ids(&[&id_6].map(Clone::clone), &[], 2..4),
-            [&id_4, &id_3, &id_2, &id_1].map(Clone::clone)
-        );
-        assert_eq!(
-            walk_commit_ids(&[&id_5].map(Clone::clone), &[], 2..3),
-            [&id_2, &id_1].map(Clone::clone)
-        );
-        assert_eq!(
-            walk_commit_ids(&[&id_5, &id_7].map(Clone::clone), &[], 2..3),
-            [&id_2, &id_1].map(Clone::clone)
-        );
-        assert_eq!(
-            walk_commit_ids(&[&id_7, &id_8].map(Clone::clone), &[], 0..2),
-            [&id_8, &id_7, &id_4].map(Clone::clone)
-        );
-        assert_eq!(
-            walk_commit_ids(&[&id_6, &id_7].map(Clone::clone), &[], 0..3),
-            [&id_7, &id_6, &id_5, &id_4, &id_3, &id_2].map(Clone::clone)
-        );
-        assert_eq!(
-            walk_commit_ids(&[&id_6, &id_7].map(Clone::clone), &[], 2..3),
-            [&id_4, &id_3, &id_2].map(Clone::clone)
-        );
-
-        // Ancestors of both wanted and unwanted commits are not walked
-        assert_eq!(
-            walk_commit_ids(&[&id_5].map(Clone::clone), &[&id_2].map(Clone::clone), 1..5),
-            [&id_4, &id_3].map(Clone::clone)
-        );
-    }
-
-    #[test]
-    #[allow(clippy::redundant_clone)] // allow id_n.clone()
-    fn test_walk_revs_filter_by_generation_range_merging() {
-        let mut new_change_id = change_id_generator();
-        let mut index = DefaultMutableIndex::full(3, 16);
-        // Long linear history with some short branches
-        let ids = (0..11)
-            .map(|n| CommitId::try_from_hex(&format!("{n:06x}")).unwrap())
-            .collect_vec();
-        index.add_commit_data(ids[0].clone(), new_change_id(), &[]);
-        for i in 1..ids.len() {
-            index.add_commit_data(ids[i].clone(), new_change_id(), &[ids[i - 1].clone()]);
-        }
-        let id_branch5_0 = CommitId::from_hex("050000");
-        let id_branch5_1 = CommitId::from_hex("050001");
-        index.add_commit_data(id_branch5_0.clone(), new_change_id(), &[ids[5].clone()]);
-        index.add_commit_data(
-            id_branch5_1.clone(),
-            new_change_id(),
-            &[id_branch5_0.clone()],
-        );
-
-        let walk_commit_ids = |wanted: &[CommitId], range: Range<u32>| {
-            let index = index.as_composite();
-            let wanted_positions = to_positions_vec(index, wanted);
-            index
-                .walk_revs(&wanted_positions, &[])
-                .filter_by_generation(range)
-                .map(|entry| entry.commit_id())
-                .collect_vec()
-        };
-
-        // Multiple non-overlapping generation ranges to track:
-        // 9->6: 3..5, 6: 0..2
-        assert_eq!(
-            walk_commit_ids(&[&ids[9], &ids[6]].map(Clone::clone), 4..6),
-            [&ids[5], &ids[4], &ids[2], &ids[1]].map(Clone::clone)
-        );
-
-        // Multiple non-overlapping generation ranges to track, and merged later:
-        // 10->7: 3..5, 7: 0..2
-        // 10->6: 4..6, 7->6, 1..3, 6: 0..2
-        assert_eq!(
-            walk_commit_ids(&[&ids[10], &ids[7], &ids[6]].map(Clone::clone), 5..7),
-            [&ids[5], &ids[4], &ids[2], &ids[1], &ids[0]].map(Clone::clone)
-        );
-
-        // Merge range with sub-range (1..4 + 2..3 should be 1..4, not 1..3):
-        // 8,7,6->5::1..4, B5_1->5::2..3
-        assert_eq!(
-            walk_commit_ids(
-                &[&ids[8], &ids[7], &ids[6], &id_branch5_1].map(Clone::clone),
-                5..6
-            ),
-            [&ids[3], &ids[2], &ids[1]].map(Clone::clone)
-        );
-    }
-
-    #[test]
-    fn test_walk_revs_descendants_filtered_by_generation() {
-        let mut new_change_id = change_id_generator();
-        let mut index = DefaultMutableIndex::full(3, 16);
-        // 8 6
-        // | |
-        // 7 5
-        // |/|
-        // 4 |
-        // | 3
-        // 2 |
-        // |/
-        // 1
-        // |
-        // 0
-        let id_0 = CommitId::from_hex("000000");
-        let id_1 = CommitId::from_hex("111111");
-        let id_2 = CommitId::from_hex("222222");
-        let id_3 = CommitId::from_hex("333333");
-        let id_4 = CommitId::from_hex("444444");
-        let id_5 = CommitId::from_hex("555555");
-        let id_6 = CommitId::from_hex("666666");
-        let id_7 = CommitId::from_hex("777777");
-        let id_8 = CommitId::from_hex("888888");
-        index.add_commit_data(id_0.clone(), new_change_id(), &[]);
-        index.add_commit_data(id_1.clone(), new_change_id(), &[id_0.clone()]);
-        index.add_commit_data(id_2.clone(), new_change_id(), &[id_1.clone()]);
-        index.add_commit_data(id_3.clone(), new_change_id(), &[id_1.clone()]);
-        index.add_commit_data(id_4.clone(), new_change_id(), &[id_2.clone()]);
-        index.add_commit_data(id_5.clone(), new_change_id(), &[id_4.clone(), id_3.clone()]);
-        index.add_commit_data(id_6.clone(), new_change_id(), &[id_5.clone()]);
-        index.add_commit_data(id_7.clone(), new_change_id(), &[id_4.clone()]);
-        index.add_commit_data(id_8.clone(), new_change_id(), &[id_7.clone()]);
-
-        let visible_heads = [&id_6, &id_8].map(Clone::clone);
-        let walk_commit_ids = |roots: &[CommitId], heads: &[CommitId], range: Range<u32>| {
-            let index = index.as_composite();
-            let root_positions = to_positions_vec(index, roots);
-            let head_positions = to_positions_vec(index, heads);
-            index
-                .walk_revs(&head_positions, &[])
-                .descendants_filtered_by_generation(&root_positions, range)
-                .map(|entry| entry.commit_id())
-                .collect_vec()
-        };
-
-        // Empty generation bounds
-        assert_eq!(
-            walk_commit_ids(&[&id_0].map(Clone::clone), &visible_heads, 0..0),
-            []
-        );
-        assert_eq!(
-            walk_commit_ids(
-                &[&id_8].map(Clone::clone),
-                &visible_heads,
-                Range { start: 2, end: 1 }
-            ),
-            []
-        );
-
-        // Full generation bounds
-        assert_eq!(
-            walk_commit_ids(&[&id_0].map(Clone::clone), &visible_heads, 0..u32::MAX),
-            [&id_0, &id_1, &id_2, &id_3, &id_4, &id_5, &id_6, &id_7, &id_8].map(Clone::clone)
-        );
-
-        // Simple generation bounds
-        assert_eq!(
-            walk_commit_ids(&[&id_3].map(Clone::clone), &visible_heads, 0..3),
-            [&id_3, &id_5, &id_6].map(Clone::clone)
-        );
-
-        // Descendants may be walked with different generations
-        assert_eq!(
-            walk_commit_ids(&[&id_0].map(Clone::clone), &visible_heads, 2..4),
-            [&id_2, &id_3, &id_4, &id_5].map(Clone::clone)
-        );
-        assert_eq!(
-            walk_commit_ids(&[&id_1].map(Clone::clone), &visible_heads, 2..3),
-            [&id_4, &id_5].map(Clone::clone)
-        );
-        assert_eq!(
-            walk_commit_ids(&[&id_2, &id_3].map(Clone::clone), &visible_heads, 2..3),
-            [&id_5, &id_6, &id_7].map(Clone::clone)
-        );
-        assert_eq!(
-            walk_commit_ids(&[&id_2, &id_4].map(Clone::clone), &visible_heads, 0..2),
-            [&id_2, &id_4, &id_5, &id_7].map(Clone::clone)
-        );
-        assert_eq!(
-            walk_commit_ids(&[&id_2, &id_3].map(Clone::clone), &visible_heads, 0..3),
-            [&id_2, &id_3, &id_4, &id_5, &id_6, &id_7].map(Clone::clone)
-        );
-        assert_eq!(
-            walk_commit_ids(&[&id_2, &id_3].map(Clone::clone), &visible_heads, 2..3),
-            [&id_5, &id_6, &id_7].map(Clone::clone)
-        );
-
-        // Roots set contains entries unreachable from heads
-        assert_eq!(
-            walk_commit_ids(
-                &[&id_2, &id_3].map(Clone::clone),
-                &[&id_8].map(Clone::clone),
-                0..3
-            ),
-            [&id_2, &id_4, &id_7].map(Clone::clone)
-        );
-    }
-
     #[test]
     fn test_heads() {
         let mut new_change_id = change_id_generator();
