@@ -29,9 +29,10 @@ use thiserror::Error;
 use crate::backend::{BackendError, CommitId};
 use crate::commit::Commit;
 use crate::git_backend::GitBackend;
+use crate::index::Index;
 use crate::object_id::ObjectId;
 use crate::op_store::{RefTarget, RefTargetOptionExt, RemoteRef, RemoteRefState};
-use crate::refs::BranchPushUpdate;
+use crate::refs::{self, BranchPushUpdate};
 use crate::repo::{MutableRepo, Repo};
 use crate::revset::RevsetExpression;
 use crate::settings::GitSettings;
@@ -669,23 +670,22 @@ pub fn export_some_refs(
             .and_then(parse_git_ref)
         {
             let old_target = head_ref.inner.target.clone();
-            if let Ok(current_git_commit_id) = head_ref.into_fully_peeled_id() {
-                let detach_head =
-                    if let Some((_old_oid, new_oid)) = branches_to_update.get(&parsed_ref) {
-                        *new_oid != current_git_commit_id
-                    } else {
-                        branches_to_delete.contains_key(&parsed_ref)
-                    };
-                if detach_head {
-                    git_repo
-                        .reference(
-                            "HEAD",
-                            current_git_commit_id,
-                            gix::refs::transaction::PreviousValue::MustExistAndMatch(old_target),
-                            "export from jj",
-                        )
-                        .map_err(GitExportError::from_git)?;
-                }
+            let current_oid = match head_ref.into_fully_peeled_id() {
+                Ok(id) => Some(id.detach()),
+                Err(gix::reference::peel::Error::ToId(gix::refs::peel::to_id::Error::Follow(
+                    gix::refs::file::find::existing::Error::NotFound { .. },
+                ))) => None, // Unborn ref should be considered absent
+                Err(err) => return Err(GitExportError::from_git(err)),
+            };
+            let new_oid = if let Some((_old_oid, new_oid)) = branches_to_update.get(&parsed_ref) {
+                Some(new_oid)
+            } else if branches_to_delete.contains_key(&parsed_ref) {
+                None
+            } else {
+                current_oid.as_ref()
+            };
+            if new_oid != current_oid.as_ref() {
+                update_git_head(&git_repo, old_target, current_oid)?;
             }
         }
     }
@@ -920,6 +920,49 @@ fn update_git_ref(
     Ok(())
 }
 
+/// Ensures `HEAD@git` is detached and pointing to the `new_oid`. If `new_oid`
+/// is `None` (meaning absent), dummy placeholder ref will be set.
+fn update_git_head(
+    git_repo: &gix::Repository,
+    old_target: gix::refs::Target,
+    new_oid: Option<gix::ObjectId>,
+) -> Result<(), GitExportError> {
+    let mut ref_edits = Vec::new();
+    let new_target = if let Some(oid) = new_oid {
+        gix::refs::Target::Peeled(oid)
+    } else {
+        // Can't detach HEAD without a commit. Use placeholder ref to nullify
+        // the HEAD. The placeholder ref isn't a normal branch ref. Git CLI
+        // appears to deal with that, and can move the placeholder ref. So we
+        // need to ensure that the ref doesn't exist.
+        ref_edits.push(gix::refs::transaction::RefEdit {
+            change: gix::refs::transaction::Change::Delete {
+                expected: gix::refs::transaction::PreviousValue::Any,
+                log: gix::refs::transaction::RefLog::AndReference,
+            },
+            name: UNBORN_ROOT_REF_NAME.try_into().unwrap(),
+            deref: false,
+        });
+        gix::refs::Target::Symbolic(UNBORN_ROOT_REF_NAME.try_into().unwrap())
+    };
+    ref_edits.push(gix::refs::transaction::RefEdit {
+        change: gix::refs::transaction::Change::Update {
+            log: gix::refs::transaction::LogChange {
+                message: "export from jj".into(),
+                ..Default::default()
+            },
+            expected: gix::refs::transaction::PreviousValue::MustExistAndMatch(old_target),
+            new: new_target,
+        },
+        name: "HEAD".try_into().unwrap(),
+        deref: false,
+    });
+    git_repo
+        .edit_references(ref_edits)
+        .map_err(GitExportError::from_git)?;
+    Ok(())
+}
+
 /// Sets `HEAD@git` to the parent of the given working-copy commit and resets
 /// the Git index.
 pub fn reset_head(
@@ -928,16 +971,47 @@ pub fn reset_head(
     wc_commit: &Commit,
 ) -> Result<(), git2::Error> {
     let first_parent_id = &wc_commit.parent_ids()[0];
-    if first_parent_id != mut_repo.store().root_commit_id() {
-        let first_parent = RefTarget::normal(first_parent_id.clone());
+    let first_parent = if first_parent_id != mut_repo.store().root_commit_id() {
+        RefTarget::normal(first_parent_id.clone())
+    } else {
+        RefTarget::absent()
+    };
+    if first_parent.is_present() {
         let git_head = mut_repo.view().git_head();
         let new_git_commit_id = Oid::from_bytes(first_parent_id.as_bytes()).unwrap();
         let new_git_commit = git_repo.find_commit(new_git_commit_id)?;
         if git_head != &first_parent {
             git_repo.set_head_detached(new_git_commit_id)?;
-            mut_repo.set_git_head_target(first_parent);
         }
-        git_repo.reset(new_git_commit.as_object(), git2::ResetType::Mixed, None)?;
+
+        let is_same_tree = if git_head == &first_parent {
+            true
+        } else if let Some(git_head_id) = git_head.as_normal() {
+            let git_head_oid = Oid::from_bytes(git_head_id.as_bytes()).unwrap();
+            let git_head_commit = git_repo.find_commit(git_head_oid)?;
+            new_git_commit.tree_id() == git_head_commit.tree_id()
+        } else {
+            false
+        };
+        let skip_reset = if is_same_tree {
+            // `HEAD@git` already points to a commit with the correct tree contents,
+            // so we only need to reset the Git index. We can skip the reset if
+            // the Git index is empty (i.e. `git add` was never used).
+            // In large repositories, this is around 2x faster if the Git index is empty
+            // (~0.89s to check the diff, vs. ~1.72s to reset), and around 8% slower if
+            // it isn't (~1.86s to check the diff AND reset).
+            let diff = git_repo.diff_tree_to_index(
+                Some(&new_git_commit.tree()?),
+                None,
+                Some(git2::DiffOptions::new().skip_binary_check(true)),
+            )?;
+            diff.deltas().len() == 0
+        } else {
+            false
+        };
+        if !skip_reset {
+            git_repo.reset(new_git_commit.as_object(), git2::ResetType::Mixed, None)?;
+        }
     } else {
         // Can't detach HEAD without a commit. Use placeholder ref to nullify the HEAD.
         // We can't set_head() an arbitrary unborn ref, so use reference_symbolic()
@@ -956,8 +1030,8 @@ pub fn reset_head(
         index.clear()?; // or read empty tree
         index.write()?;
         git_repo.cleanup_state()?;
-        mut_repo.set_git_head_target(RefTarget::absent());
     }
+    mut_repo.set_git_head_target(first_parent);
     Ok(())
 }
 
@@ -1208,8 +1282,8 @@ pub enum GitPushError {
         name = REMOTE_NAME_FOR_LOCAL_GIT_REPO
     )]
     RemoteReservedForLocalGitRepo,
-    #[error("Push is not fast-forwardable")]
-    NotFastForward,
+    #[error("Refs in unexpected location: {0:?}")]
+    RefInUnexpectedLocation(Vec<String>),
     #[error("Remote rejected the update of some refs (do you have permission to push to {0:?}?)")]
     RefUpdateRejected(Vec<String>),
     // TODO: I'm sure there are other errors possible, such as transport-level errors,
@@ -1221,15 +1295,15 @@ pub enum GitPushError {
 #[derive(Clone, Debug)]
 pub struct GitBranchPushTargets {
     pub branch_updates: Vec<(String, BranchPushUpdate)>,
-    pub force_pushed_branches: HashSet<String>,
 }
 
 pub struct GitRefUpdate {
     pub qualified_name: String,
-    // TODO: We want this to be a `current_target: Option<CommitId>` for the expected current
-    // commit on the remote. It's a blunt "force" option instead until git2-rs supports the
-    // "push negotiation" callback (https://github.com/rust-lang/git2-rs/issues/733).
-    pub force: bool,
+    /// Expected position on the remote or None if we expect the ref to not
+    /// exist on the remote
+    ///
+    /// This is sourced from the local remote-tracking branch.
+    pub expected_current_target: Option<CommitId>,
     pub new_target: Option<CommitId>,
 }
 
@@ -1246,11 +1320,11 @@ pub fn push_branches(
         .iter()
         .map(|(branch_name, update)| GitRefUpdate {
             qualified_name: format!("refs/heads/{branch_name}"),
-            force: targets.force_pushed_branches.contains(branch_name),
+            expected_current_target: update.old_target.clone(),
             new_target: update.new_target.clone(),
         })
         .collect_vec();
-    push_updates(git_repo, remote_name, &ref_updates, callbacks)?;
+    push_updates(mut_repo, git_repo, remote_name, &ref_updates, callbacks)?;
 
     // TODO: add support for partially pushed refs? we could update the view
     // excluding rejected refs, but the transaction would be aborted anyway
@@ -1270,42 +1344,48 @@ pub fn push_branches(
 
 /// Pushes the specified Git refs without updating the repo view.
 pub fn push_updates(
+    repo: &dyn Repo,
     git_repo: &git2::Repository,
     remote_name: &str,
     updates: &[GitRefUpdate],
     callbacks: RemoteCallbacks<'_>,
 ) -> Result<(), GitPushError> {
-    let mut qualified_remote_refs = vec![];
+    let mut qualified_remote_refs_expected_locations = HashMap::new();
     let mut refspecs = vec![];
     for update in updates {
-        qualified_remote_refs.push(update.qualified_name.as_str());
+        qualified_remote_refs_expected_locations.insert(
+            update.qualified_name.as_str(),
+            update.expected_current_target.as_ref(),
+        );
         if let Some(new_target) = &update.new_target {
-            refspecs.push(format!(
-                "{}{}:{}",
-                (if update.force { "+" } else { "" }),
-                new_target.hex(),
-                update.qualified_name
-            ));
+            // We always force-push. We use the push_negotiation callback in
+            // `push_refs` to check that the refs did not unexpectedly move on
+            // the remote.
+            refspecs.push(format!("+{}:{}", new_target.hex(), update.qualified_name));
         } else {
+            // Prefixing this with `+` to force-push or not should make no
+            // difference. The push negotiation happens regardless, and wouldn't
+            // allow creating a branch if it's not a fast-forward.
             refspecs.push(format!(":{}", update.qualified_name));
         }
     }
-    // TODO(ilyagr): this function will be reused to implement force-with-lease, so
-    // don't inline for now. If it's after 2024-06-01 or so, ilyagr may have
-    // forgotten to delete this comment.
+    // TODO(ilyagr): `push_refs`, or parts of it, should probably be inlined. This
+    // requires adjusting some tests.
     push_refs(
+        repo,
         git_repo,
         remote_name,
-        &qualified_remote_refs,
+        &qualified_remote_refs_expected_locations,
         &refspecs,
         callbacks,
     )
 }
 
 fn push_refs(
+    repo: &dyn Repo,
     git_repo: &git2::Repository,
     remote_name: &str,
-    qualified_remote_refs: &[&str],
+    qualified_remote_refs_expected_locations: &HashMap<&str, Option<&CommitId>>,
     refspecs: &[String],
     callbacks: RemoteCallbacks<'_>,
 ) -> Result<(), GitPushError> {
@@ -1319,39 +1399,170 @@ fn push_refs(
             GitPushError::InternalGitError(err)
         }
     })?;
-    let mut remaining_remote_refs: HashSet<_> = qualified_remote_refs.iter().copied().collect();
-    let mut push_options = git2::PushOptions::new();
-    let mut proxy_options = git2::ProxyOptions::new();
-    proxy_options.auto();
-    push_options.proxy_options(proxy_options);
-    let mut callbacks = callbacks.into_git();
-    callbacks.push_update_reference(|refname, status| {
-        // The status is Some if the ref update was rejected
-        if status.is_none() {
-            remaining_remote_refs.remove(refname);
-        }
-        Ok(())
-    });
-    push_options.remote_callbacks(callbacks);
-    remote
-        .push(refspecs, Some(&mut push_options))
-        .map_err(|err| match (err.class(), err.code()) {
-            (git2::ErrorClass::Reference, git2::ErrorCode::NotFastForward) => {
-                GitPushError::NotFastForward
+    let mut remaining_remote_refs: HashSet<_> = qualified_remote_refs_expected_locations
+        .keys()
+        .copied()
+        .collect();
+    let mut failed_push_negotiations = vec![];
+    let push_result = {
+        let mut push_options = git2::PushOptions::new();
+        let mut proxy_options = git2::ProxyOptions::new();
+        proxy_options.auto();
+        push_options.proxy_options(proxy_options);
+        let mut callbacks = callbacks.into_git();
+        callbacks.push_negotiation(|updates| {
+            for update in updates {
+                let dst_refname = update
+                    .dst_refname()
+                    .expect("Expect reference name to be valid UTF-8");
+                let expected_remote_location = *qualified_remote_refs_expected_locations
+                    .get(dst_refname)
+                    .expect("Push is trying to move a ref it wasn't asked to move");
+                let oid_to_maybe_commitid =
+                    |oid: git2::Oid| (!oid.is_zero()).then(|| CommitId::from_bytes(oid.as_bytes()));
+                let actual_remote_location = oid_to_maybe_commitid(update.src());
+                let local_location = oid_to_maybe_commitid(update.dst());
+
+                match allow_push(
+                    repo.index(),
+                    actual_remote_location.as_ref(),
+                    expected_remote_location,
+                    local_location.as_ref(),
+                ) {
+                    Ok(PushAllowReason::NormalMatch) => {}
+                    Ok(PushAllowReason::UnexpectedNoop) => {
+                        tracing::info!(
+                            "The push of {dst_refname} is unexpectedly a no-op, the remote branch \
+                             is already at {actual_remote_location:?}. We expected it to be at \
+                             {expected_remote_location:?}. We don't consider this an error.",
+                        );
+                    }
+                    Ok(PushAllowReason::ExceptionalFastforward) => {
+                        // TODO(ilyagr): We could consider printing a user-facing message at
+                        // this point.
+                        tracing::info!(
+                            "We allow the push of {dst_refname} to {local_location:?}, even \
+                             though it is unexpectedly at {actual_remote_location:?} on the \
+                             server rather than the expected {expected_remote_location:?}. The \
+                             desired location is a descendant of the actual location, and the \
+                             actual location is a descendant of the expected location.",
+                        );
+                    }
+                    Err(()) => {
+                        // While we show debug info in the message with `--debug`,
+                        // there's probably no need to show the detailed commit
+                        // locations to the user normally. They should do a `jj git
+                        // fetch`, and the resulting branch conflicts should contain
+                        // all the information they need.
+                        tracing::info!(
+                            "Cannot push {dst_refname} to {local_location:?}; it is at \
+                             unexpectedly at {actual_remote_location:?} on the server as opposed \
+                             to the expected {expected_remote_location:?}",
+                        );
+
+                        failed_push_negotiations.push(dst_refname.to_string());
+                    }
+                }
             }
-            _ => GitPushError::InternalGitError(err),
-        })?;
-    drop(push_options);
-    if remaining_remote_refs.is_empty() {
-        Ok(())
-    } else {
-        Err(GitPushError::RefUpdateRejected(
-            remaining_remote_refs
-                .iter()
-                .sorted()
-                .map(|name| name.to_string())
-                .collect(),
+            if failed_push_negotiations.is_empty() {
+                Ok(())
+            } else {
+                Err(git2::Error::from_str("failed push negotiation"))
+            }
+        });
+        callbacks.push_update_reference(|refname, status| {
+            // The status is Some if the ref update was rejected
+            if status.is_none() {
+                remaining_remote_refs.remove(refname);
+            }
+            Ok(())
+        });
+        push_options.remote_callbacks(callbacks);
+        remote.push(refspecs, Some(&mut push_options))
+    };
+    if !failed_push_negotiations.is_empty() {
+        // If the push negotiation returned an error, `remote.push` would not
+        // have pushed anything and would have returned an error, as expected.
+        // However, the error it returns is not necessarily the error we'd
+        // expect. It also depends on the exact versions of `libgit2` and
+        // `git2.rs`. So, we cannot rely on it containing any useful
+        // information. See https://github.com/rust-lang/git2-rs/issues/1042.
+        assert!(push_result.is_err());
+        failed_push_negotiations.sort();
+        Err(GitPushError::RefInUnexpectedLocation(
+            failed_push_negotiations,
         ))
+    } else {
+        push_result?;
+        if remaining_remote_refs.is_empty() {
+            Ok(())
+        } else {
+            Err(GitPushError::RefUpdateRejected(
+                remaining_remote_refs
+                    .iter()
+                    .sorted()
+                    .map(|name| name.to_string())
+                    .collect(),
+            ))
+        }
+    }
+}
+
+#[derive(Debug, Clone, PartialEq, Eq)]
+enum PushAllowReason {
+    NormalMatch,
+    ExceptionalFastforward,
+    UnexpectedNoop,
+}
+
+fn allow_push(
+    index: &dyn Index,
+    actual_remote_location: Option<&CommitId>,
+    expected_remote_location: Option<&CommitId>,
+    destination_location: Option<&CommitId>,
+) -> Result<PushAllowReason, ()> {
+    if actual_remote_location == expected_remote_location {
+        return Ok(PushAllowReason::NormalMatch);
+    }
+
+    // If the remote ref is in an unexpected location, we still allow some
+    // pushes, based on whether `jj git fetch` would result in a conflicted ref.
+    //
+    // For `merge_ref_targets` to work correctly, `actual_remote_location` must
+    // be a commit that we locally know about.
+    //
+    // This does not lose any generality since for `merge_ref_targets` to
+    // resolve to `local_target` below, it is conceptually necessary (but not
+    // sufficient) for the destination_location to be either a descendant of
+    // actual_remote_location or equal to it. Either way, we would know about that
+    // commit locally.
+    if !actual_remote_location.map_or(true, |id| index.has_id(id)) {
+        return Err(());
+    }
+    let remote_target = RefTarget::resolved(actual_remote_location.cloned());
+    let base_target = RefTarget::resolved(expected_remote_location.cloned());
+    // The push destination is the local position of the ref
+    let local_target = RefTarget::resolved(destination_location.cloned());
+    if refs::merge_ref_targets(index, &remote_target, &base_target, &local_target) == local_target {
+        // Fetch would not change the local branch, so the push is OK in spite of
+        // the discrepancy with the expected location. We return some debug info and
+        // verify some invariants before OKing the push.
+        Ok(if actual_remote_location == destination_location {
+            // This is the situation of what we call "A - B + A = A"
+            // conflicts, see also test_refs.rs and
+            // https://github.com/martinvonz/jj/blob/c9b44f382824301e6c0fdd6f4cbc52bb00c50995/lib/src/merge.rs#L92.
+            PushAllowReason::UnexpectedNoop
+        } else {
+            // Due to our ref merge rules, this case should happen if an only
+            // if:
+            //
+            // 1. This is a fast-forward.
+            // 2. The expected location is an ancestor of both the actual location and the
+            //    destination (local position).
+            PushAllowReason::ExceptionalFastforward
+        })
+    } else {
+        Err(())
     }
 }
 
