@@ -48,6 +48,7 @@ pub mod git;
 pub mod git_backend;
 pub mod gitignore;
 pub mod gpg_signing;
+pub mod graph;
 pub mod hex_util;
 pub mod id_prefix;
 pub mod index;
@@ -68,8 +69,10 @@ pub mod refs;
 pub mod repo;
 pub mod repo_path;
 pub mod revset;
-pub mod revset_graph;
+mod revset_parser;
 pub mod rewrite;
+#[cfg(feature = "testing")]
+pub mod secret_backend;
 pub mod settings;
 pub mod signing;
 pub mod simple_op_heads_store;
@@ -82,6 +85,7 @@ pub mod submodule_store;
 pub mod transaction;
 pub mod tree;
 pub mod tree_builder;
+pub mod union_find;
 pub mod view;
 pub mod working_copy;
 pub mod workspace;
