@@ -20,6 +20,7 @@ use std::fs;
 use std::fs::File;
 use std::io::{Read, Write};
 use std::path::{Path, PathBuf};
+use std::time::SystemTime;
 
 use async_trait::async_trait;
 use blake2::{Blake2b512, Digest};
@@ -28,12 +29,14 @@ use tempfile::NamedTempFile;
 
 use crate::backend::{
     make_root_commit, Backend, BackendError, BackendResult, ChangeId, Commit, CommitId, Conflict,
-    ConflictId, ConflictTerm, FileId, MergedTreeId, MillisSinceEpoch, ObjectId, SecureSig,
-    Signature, SigningFn, SymlinkId, Timestamp, Tree, TreeId, TreeValue,
+    ConflictId, ConflictTerm, FileId, MergedTreeId, MillisSinceEpoch, SecureSig, Signature,
+    SigningFn, SymlinkId, Timestamp, Tree, TreeId, TreeValue,
 };
 use crate::content_hash::blake2b_hash;
 use crate::file_util::persist_content_addressed_temp_file;
+use crate::index::Index;
 use crate::merge::MergeBuilder;
+use crate::object_id::ObjectId;
 use crate::repo_path::{RepoPath, RepoPathComponentBuf};
 
 const COMMIT_ID_LENGTH: usize = 64;
@@ -181,13 +184,13 @@ impl Backend for LocalBackend {
         Ok(id)
     }
 
-    async fn read_symlink(&self, _path: &RepoPath, id: &SymlinkId) -> Result<String, BackendError> {
+    async fn read_symlink(&self, _path: &RepoPath, id: &SymlinkId) -> BackendResult<String> {
         let path = self.symlink_path(id);
         let target = fs::read_to_string(path).map_err(|err| map_not_found_err(err, id))?;
         Ok(target)
     }
 
-    fn write_symlink(&self, _path: &RepoPath, target: &str) -> Result<SymlinkId, BackendError> {
+    fn write_symlink(&self, _path: &RepoPath, target: &str) -> BackendResult<SymlinkId> {
         let mut temp_file = NamedTempFile::new_in(&self.path).map_err(to_other_err)?;
         temp_file
             .write_all(target.as_bytes())
@@ -298,11 +301,13 @@ impl Backend for LocalBackend {
         Ok((id, commit))
     }
 
-    fn gc(&self) -> Result<(), Box<dyn std::error::Error + Send + Sync>> {
+    fn gc(&self, _index: &dyn Index, _keep_newer: SystemTime) -> BackendResult<()> {
         Ok(())
     }
 }
 
+#[allow(unknown_lints)] // XXX FIXME (aseipp): nightly bogons; re-test this occasionally
+#[allow(clippy::assigning_clones)]
 pub fn commit_to_proto(commit: &Commit) -> crate::protos::local_store::Commit {
     let mut proto = crate::protos::local_store::Commit::default();
     for parent in &commit.parents {
@@ -493,7 +498,6 @@ mod tests {
     use pollster::FutureExt;
 
     use super::*;
-    use crate::backend::MillisSinceEpoch;
 
     /// Test that parents get written correctly
     #[test]
