@@ -13,6 +13,7 @@
 // limitations under the License.
 
 #![allow(missing_docs)]
+#![allow(clippy::let_unit_value)]
 
 use std::any::Any;
 use std::collections::HashSet;
@@ -40,7 +41,8 @@ use thiserror::Error;
 use tracing::{instrument, trace_span};
 
 use crate::backend::{
-    BackendError, FileId, MergedTreeId, MillisSinceEpoch, SymlinkId, TreeId, TreeValue,
+    BackendError, BackendResult, FileId, MergedTreeId, MillisSinceEpoch, SymlinkId, TreeId,
+    TreeValue,
 };
 use crate::commit::Commit;
 use crate::conflicts::{self, materialize_tree_value, MaterializedTreeValue};
@@ -108,7 +110,6 @@ impl FileState {
         let executable = {
             // Windows doesn't support executable bit.
             let _ = executable;
-            ()
         };
         FileState {
             file_type: FileType::Normal { executable },
@@ -148,13 +149,17 @@ impl FileStatesMap {
         FileStatesMap { data: Vec::new() }
     }
 
-    // TODO: skip sorting if the data is known to be sorted?
-    fn from_proto_unsorted(mut data: Vec<crate::protos::working_copy::FileStateEntry>) -> Self {
-        data.sort_unstable_by(|entry1, entry2| {
-            let path1 = RepoPath::from_internal_string(&entry1.path);
-            let path2 = RepoPath::from_internal_string(&entry2.path);
-            path1.cmp(path2)
-        });
+    fn from_proto(
+        mut data: Vec<crate::protos::working_copy::FileStateEntry>,
+        is_sorted: bool,
+    ) -> Self {
+        if !is_sorted {
+            data.sort_unstable_by(|entry1, entry2| {
+                let path1 = RepoPath::from_internal_string(&entry1.path);
+                let path2 = RepoPath::from_internal_string(&entry2.path);
+                path1.cmp(path2)
+            });
+        }
         debug_assert!(is_file_state_entries_proto_unique_and_sorted(&data));
         FileStatesMap { data }
     }
@@ -609,12 +614,15 @@ impl TreeState {
                 .collect();
             self.tree_id = MergedTreeId::Merge(tree_ids_builder.build());
         }
-        self.file_states = FileStatesMap::from_proto_unsorted(proto.file_states);
+        self.file_states =
+            FileStatesMap::from_proto(proto.file_states, proto.is_file_states_sorted);
         self.sparse_patterns = sparse_patterns_from_proto(proto.sparse_patterns.as_ref());
         self.watchman_clock = proto.watchman_clock;
         Ok(())
     }
 
+    #[allow(unknown_lints)] // XXX FIXME (aseipp): nightly bogons; re-test this occasionally
+    #[allow(clippy::assigning_clones)]
     fn save(&mut self) -> Result<(), TreeStateError> {
         let mut proto: crate::protos::working_copy::TreeState = Default::default();
         match &self.tree_id {
@@ -627,6 +635,8 @@ impl TreeState {
         }
 
         proto.file_states = self.file_states.data.clone();
+        // `FileStatesMap` is guaranteed to be sorted.
+        proto.is_file_states_sorted = true;
         let mut sparse_patterns = crate::protos::working_copy::SparsePatterns::default();
         for path in &self.sparse_patterns {
             sparse_patterns
@@ -661,7 +671,7 @@ impl TreeState {
         Ok(())
     }
 
-    fn current_tree(&self) -> Result<MergedTree, BackendError> {
+    fn current_tree(&self) -> BackendResult<MergedTree> {
         self.store.get_root_tree(&self.tree_id)
     }
 
@@ -832,7 +842,7 @@ impl TreeState {
             let tree = self.current_tree().unwrap();
             let tree_paths: HashSet<_> = tree
                 .entries_matching(sparse_matcher.as_ref())
-                .map(|(path, _)| path)
+                .filter_map(|(path, result)| result.is_ok().then_some(path))
                 .collect();
             let file_states = self.file_states.all();
             let state_paths: HashSet<_> = file_states.paths().map(|path| path.to_owned()).collect();
@@ -1078,7 +1088,7 @@ impl TreeState {
         if clean {
             Ok(None)
         } else {
-            let current_tree_values = current_tree.path_value(repo_path);
+            let current_tree_values = current_tree.path_value(repo_path)?;
             let new_file_type = if !self.symlink_support {
                 let mut new_file_type = new_file_state.file_type.clone();
                 if matches!(new_file_type, FileType::Normal { .. })
@@ -1149,10 +1159,15 @@ impl TreeState {
             .block_on()?;
             match new_file_ids.into_resolved() {
                 Ok(file_id) => {
+                    // On Windows, we preserve the executable bit from the merged trees.
                     #[cfg(windows)]
                     let executable = {
                         let () = executable; // use the variable
-                        false
+                        if let Some(merge) = current_tree_values.to_executable_merge() {
+                            merge.resolve_trivial().copied().unwrap_or_default()
+                        } else {
+                            false
+                        }
                     };
                     Ok(Merge::normal(TreeValue::File {
                         id: file_id.unwrap(),
@@ -1221,6 +1236,7 @@ impl TreeState {
         &self,
         disk_path: &Path,
         conflict_data: Vec<u8>,
+        executable: bool,
     ) -> Result<FileState, CheckoutError> {
         let mut file = OpenOptions::new()
             .write(true)
@@ -1236,12 +1252,11 @@ impl TreeState {
                 err: err.into(),
             })?;
         let size = conflict_data.len() as u64;
-        // TODO: Set the executable bit correctly (when possible) and preserve that on
-        // Windows like we do with the executable bit for regular files.
+        self.set_executable(disk_path, executable)?;
         let metadata = file
             .metadata()
             .map_err(|err| checkout_error_for_stat_error(err, disk_path))?;
-        Ok(FileState::for_file(false, size, &metadata))
+        Ok(FileState::for_file(executable, size, &metadata))
     }
 
     #[cfg_attr(windows, allow(unused_variables))]
@@ -1360,7 +1375,7 @@ impl TreeState {
             }
             // TODO: Check that the file has not changed before overwriting/removing it.
             let file_state = match after {
-                MaterializedTreeValue::Absent => {
+                MaterializedTreeValue::Absent | MaterializedTreeValue::AccessDenied(_) => {
                     let mut parent_dir = disk_path.parent().unwrap();
                     loop {
                         if fs::remove_dir(parent_dir).is_err() {
@@ -1390,9 +1405,11 @@ impl TreeState {
                 MaterializedTreeValue::Tree(_) => {
                     panic!("unexpected tree entry in diff at {path:?}");
                 }
-                MaterializedTreeValue::Conflict { id: _, contents } => {
-                    self.write_conflict(&disk_path, contents)?
-                }
+                MaterializedTreeValue::Conflict {
+                    id: _,
+                    contents,
+                    executable,
+                } => self.write_conflict(&disk_path, contents, executable)?,
             };
             changed_file_states.push((path, file_state));
         }
@@ -1704,8 +1721,12 @@ impl WorkingCopyFactory for LocalWorkingCopyFactory {
         store: Arc<Store>,
         working_copy_path: PathBuf,
         state_path: PathBuf,
-    ) -> Box<dyn WorkingCopy> {
-        Box::new(LocalWorkingCopy::load(store, working_copy_path, state_path))
+    ) -> Result<Box<dyn WorkingCopy>, WorkingCopyStateError> {
+        Ok(Box::new(LocalWorkingCopy::load(
+            store,
+            working_copy_path,
+            state_path,
+        )))
     }
 }
 
@@ -1885,7 +1906,7 @@ mod tests {
             new_proto_entry("b/e", 3),
             new_proto_entry("bc", 5),
         ];
-        let mut file_states = FileStatesMap::from_proto_unsorted(data);
+        let mut file_states = FileStatesMap::from_proto(data, false);
 
         let changed_file_states = vec![
             new_owned_entry("aa", 10),    // change
