@@ -27,8 +27,7 @@ use std::sync::Arc;
 use itertools::Itertools;
 use smallvec::{smallvec_inline, SmallVec};
 
-use crate::backend;
-use crate::backend::{BackendError, FileId, TreeId, TreeValue};
+use crate::backend::{self, BackendResult, FileId, TreeId, TreeValue};
 use crate::content_hash::{ContentHash, DigestUpdate};
 use crate::object_id::ObjectId;
 use crate::repo_path::RepoPath;
@@ -513,6 +512,16 @@ impl MergedTreeValue {
         })
     }
 
+    /// If this merge contains only files or absent entries, returns a merge of
+    /// the files' executable bits.
+    pub fn to_executable_merge(&self) -> Option<Merge<bool>> {
+        self.maybe_map(|term| match term {
+            None => Some(false),
+            Some(TreeValue::File { id: _, executable }) => Some(*executable),
+            _ => None,
+        })
+    }
+
     /// Creates a new merge with the file ids from the given merge. In other
     /// words, only the executable bits from `self` will be preserved.
     pub fn with_new_file_ids(&self, file_ids: &Merge<Option<FileId>>) -> Self {
@@ -559,7 +568,7 @@ where
         &self,
         store: &Arc<Store>,
         dir: &RepoPath,
-    ) -> Result<Option<Merge<Tree>>, BackendError> {
+    ) -> BackendResult<Option<Merge<Tree>>> {
         let tree_id_merge = self.maybe_map(|term| match term {
             None => Some(None),
             Some(value) => {
@@ -571,7 +580,7 @@ where
             }
         });
         if let Some(tree_id_merge) = tree_id_merge {
-            let get_tree = |id: &Option<&TreeId>| -> Result<Tree, BackendError> {
+            let get_tree = |id: &Option<&TreeId>| -> BackendResult<Tree> {
                 if let Some(id) = id {
                     store.get_tree(dir, id)
                 } else {
