@@ -26,14 +26,14 @@ use futures::stream::StreamExt;
 use futures::{Future, Stream, TryStreamExt};
 use itertools::Itertools;
 
-use crate::backend::{BackendError, BackendResult, ConflictId, MergedTreeId, TreeId, TreeValue};
+use crate::backend;
+use crate::backend::{BackendResult, ConflictId, MergedTreeId, TreeId, TreeValue};
 use crate::matchers::{EverythingMatcher, Matcher};
 use crate::merge::{Merge, MergeBuilder, MergedTreeValue};
-use crate::repo_path::{RepoPath, RepoPathBuf, RepoPathComponent, RepoPathComponentsIter};
+use crate::repo_path::{RepoPath, RepoPathBuf, RepoPathComponent};
 use crate::store::Store;
 use crate::tree::{try_resolve_file_conflict, Tree};
 use crate::tree_builder::TreeBuilder;
-use crate::{backend, tree};
 
 /// Presents a view of a merged set of trees.
 #[derive(PartialEq, Eq, Clone, Debug)]
@@ -56,7 +56,8 @@ pub enum MergedTreeVal<'a> {
 }
 
 impl MergedTreeVal<'_> {
-    fn to_merge(&self) -> MergedTreeValue {
+    /// Converts to an owned value.
+    pub fn to_merge(&self) -> MergedTreeValue {
         match self {
             MergedTreeVal::Resolved(value) => Merge::resolved(value.cloned()),
             MergedTreeVal::Conflict(merge) => merge.clone(),
@@ -131,7 +132,7 @@ impl MergedTree {
         let new_trees: Vec<_> = tree_builders
             .into_iter()
             .map(|builder| {
-                let tree_id = builder.write_tree();
+                let tree_id = builder.write_tree()?;
                 store.get_tree(RepoPath::root(), &tree_id)
             })
             .try_collect()?;
@@ -211,30 +212,30 @@ impl MergedTree {
     /// Gets the `MergeTree` in a subdirectory of the current tree. If the path
     /// doesn't correspond to a tree in any of the inputs to the merge, then
     /// that entry will be replace by an empty tree in the result.
-    pub fn sub_tree(&self, name: &RepoPathComponent) -> Option<MergedTree> {
+    pub fn sub_tree(&self, name: &RepoPathComponent) -> BackendResult<Option<MergedTree>> {
         if let MergedTree::Legacy(tree) = self {
-            tree.sub_tree(name).map(MergedTree::Legacy)
+            Ok(tree.sub_tree(name)?.map(MergedTree::Legacy))
         } else {
             match self.value(name) {
                 MergedTreeVal::Resolved(Some(TreeValue::Tree(sub_tree_id))) => {
                     let subdir = self.dir().join(name);
-                    Some(MergedTree::resolved(
-                        self.store().get_tree(&subdir, sub_tree_id).unwrap(),
-                    ))
+                    Ok(Some(MergedTree::resolved(
+                        self.store().get_tree(&subdir, sub_tree_id)?,
+                    )))
                 }
-                MergedTreeVal::Resolved(_) => None,
+                MergedTreeVal::Resolved(_) => Ok(None),
                 MergedTreeVal::Conflict(merge) => {
-                    let merged_trees = merge.map(|value| match value {
+                    let merged_trees = merge.try_map(|value| match value {
                         Some(TreeValue::Tree(sub_tree_id)) => {
                             let subdir = self.dir().join(name);
-                            self.store().get_tree(&subdir, sub_tree_id).unwrap()
+                            self.store().get_tree(&subdir, sub_tree_id)
                         }
                         _ => {
                             let subdir = self.dir().join(name);
-                            Tree::null(self.store().clone(), subdir.clone())
+                            Ok(Tree::null(self.store().clone(), subdir.clone()))
                         }
-                    });
-                    Some(MergedTree::Merge(merged_trees))
+                    })?;
+                    Ok(Some(MergedTree::Merge(merged_trees)))
                 }
             }
         }
@@ -243,17 +244,17 @@ impl MergedTree {
     /// The value at the given path. The value can be `Resolved` even if
     /// `self` is a `Conflict`, which happens if the value at the path can be
     /// trivially merged.
-    pub fn path_value(&self, path: &RepoPath) -> MergedTreeValue {
+    pub fn path_value(&self, path: &RepoPath) -> BackendResult<MergedTreeValue> {
         assert_eq!(self.dir(), RepoPath::root());
         match path.split() {
-            Some((dir, basename)) => match self.sub_tree_recursive(dir.components()) {
-                None => Merge::absent(),
-                Some(tree) => tree.value(basename).to_merge(),
+            Some((dir, basename)) => match self.sub_tree_recursive(dir)? {
+                None => Ok(Merge::absent()),
+                Some(tree) => Ok(tree.value(basename).to_merge()),
             },
             None => match self {
-                MergedTree::Legacy(tree) => Merge::normal(TreeValue::Tree(tree.id().clone())),
+                MergedTree::Legacy(tree) => Ok(Merge::normal(TreeValue::Tree(tree.id().clone()))),
                 MergedTree::Merge(trees) => {
-                    trees.map(|tree| Some(TreeValue::Tree(tree.id().clone())))
+                    Ok(trees.map(|tree| Some(TreeValue::Tree(tree.id().clone()))))
                 }
             },
         }
@@ -267,12 +268,20 @@ impl MergedTree {
         }
     }
 
-    fn sub_tree_recursive(&self, mut components: RepoPathComponentsIter) -> Option<MergedTree> {
-        if let Some(first) = components.next() {
-            components.try_fold(self.sub_tree(first)?, |tree, name| tree.sub_tree(name))
-        } else {
-            Some(self.clone())
+    /// Look up the tree at the given path.
+    pub fn sub_tree_recursive(&self, path: &RepoPath) -> BackendResult<Option<MergedTree>> {
+        let mut current_tree = self.clone();
+        for name in path.components() {
+            match current_tree.sub_tree(name)? {
+                None => {
+                    return Ok(None);
+                }
+                Some(sub_tree) => {
+                    current_tree = sub_tree;
+                }
+            }
         }
+        Ok(Some(current_tree))
     }
 
     /// Iterator over the entries matching the given matcher. Subtrees are
@@ -363,41 +372,34 @@ impl MergedTree {
 
     /// Merges this tree with `other`, using `base` as base.
     pub fn merge(&self, base: &MergedTree, other: &MergedTree) -> BackendResult<MergedTree> {
-        if let (MergedTree::Legacy(this), MergedTree::Legacy(base), MergedTree::Legacy(other)) =
-            (self, base, other)
-        {
-            let merged_tree = tree::merge_trees(this, base, other)?;
-            Ok(MergedTree::legacy(merged_tree))
-        } else {
-            // Convert legacy trees to merged trees and unwrap to `Merge<Tree>`
-            let to_merge = |tree: &MergedTree| -> BackendResult<Merge<Tree>> {
-                match tree {
-                    MergedTree::Legacy(tree) => {
-                        let MergedTree::Merge(tree) = MergedTree::from_legacy_tree(tree.clone())?
-                        else {
-                            unreachable!();
-                        };
-                        Ok(tree)
-                    }
-                    MergedTree::Merge(conflict) => Ok(conflict.clone()),
+        // Convert legacy trees to merged trees and unwrap to `Merge<Tree>`
+        let to_merge = |tree: &MergedTree| -> BackendResult<Merge<Tree>> {
+            match tree {
+                MergedTree::Legacy(tree) => {
+                    let MergedTree::Merge(tree) = MergedTree::from_legacy_tree(tree.clone())?
+                    else {
+                        unreachable!();
+                    };
+                    Ok(tree)
                 }
-            };
-            let nested = Merge::from_vec(vec![to_merge(self)?, to_merge(base)?, to_merge(other)?]);
-            let tree = merge_trees(&nested.flatten().simplify())?;
-            // If the result can be resolved, then `merge_trees()` above would have returned
-            // a resolved merge. However, that function will always preserve the arity of
-            // conflicts it cannot resolve. So we simplify the conflict again
-            // here to possibly reduce a complex conflict to a simpler one.
-            let tree = tree.simplify();
-            // If debug assertions are enabled, check that the merge was idempotent. In
-            // particular,  that this last simplification doesn't enable further automatic
-            // resolutions
-            if cfg!(debug_assertions) {
-                let re_merged = merge_trees(&tree).unwrap();
-                debug_assert_eq!(re_merged, tree);
+                MergedTree::Merge(conflict) => Ok(conflict.clone()),
             }
-            Ok(MergedTree::Merge(tree))
+        };
+        let nested = Merge::from_vec(vec![to_merge(self)?, to_merge(base)?, to_merge(other)?]);
+        let tree = merge_trees(&nested.flatten().simplify())?;
+        // If the result can be resolved, then `merge_trees()` above would have returned
+        // a resolved merge. However, that function will always preserve the arity of
+        // conflicts it cannot resolve. So we simplify the conflict again
+        // here to possibly reduce a complex conflict to a simpler one.
+        let tree = tree.simplify();
+        // If debug assertions are enabled, check that the merge was idempotent. In
+        // particular,  that this last simplification doesn't enable further automatic
+        // resolutions
+        if cfg!(debug_assertions) {
+            let re_merged = merge_trees(&tree).unwrap();
+            debug_assert_eq!(re_merged, tree);
         }
+        Ok(MergedTree::Merge(tree))
     }
 }
 
@@ -1182,51 +1184,24 @@ impl MergedTreeBuilder {
     }
 
     /// Create new tree(s) from the base tree(s) and overrides.
-    ///
-    /// When the base tree was a legacy tree and the
-    /// `format.tree-level-conflicts` config is disabled, then the result will
-    /// be another legacy tree. Overrides with conflicts will result in
-    /// conflict objects being written to the store. If
-    /// `format.tree-tree-level-conflicts` is enabled, then a legacy tree will
-    /// still be written and immediately converted and returned as a merged
-    /// tree.
     pub fn write_tree(self, store: &Arc<Store>) -> BackendResult<MergedTreeId> {
-        match self.base_tree_id.clone() {
+        let base_tree_ids = match self.base_tree_id.clone() {
             MergedTreeId::Legacy(base_tree_id) => {
-                let mut tree_builder = TreeBuilder::new(store.clone(), base_tree_id);
-                for (path, values) in self.overrides {
-                    let values = values.simplify();
-                    match values.into_resolved() {
-                        Ok(value) => {
-                            tree_builder.set_or_remove(path, value);
-                        }
-                        Err(values) => {
-                            let conflict_id = store.write_conflict(&path, &values)?;
-                            tree_builder.set(path, TreeValue::Conflict(conflict_id));
-                        }
-                    }
-                }
-                let legacy_id = tree_builder.write_tree();
-                if store.use_tree_conflict_format() {
-                    let legacy_tree = store.get_tree(RepoPath::root(), &legacy_id)?;
-                    let merged_tree = MergedTree::from_legacy_tree(legacy_tree)?;
-                    Ok(merged_tree.id())
-                } else {
-                    Ok(MergedTreeId::Legacy(legacy_id))
-                }
+                let legacy_base_tree = store.get_tree(RepoPath::root(), &base_tree_id)?;
+                let base_tree = MergedTree::from_legacy_tree(legacy_base_tree)?;
+                base_tree.id().to_merge()
             }
-            MergedTreeId::Merge(base_tree_ids) => {
-                let new_tree_ids = self.write_merged_trees(base_tree_ids, store)?;
-                Ok(MergedTreeId::Merge(new_tree_ids.simplify()))
-            }
-        }
+            MergedTreeId::Merge(base_tree_ids) => base_tree_ids,
+        };
+        let new_tree_ids = self.write_merged_trees(base_tree_ids, store)?;
+        Ok(MergedTreeId::Merge(new_tree_ids.simplify()))
     }
 
     fn write_merged_trees(
         self,
         mut base_tree_ids: Merge<TreeId>,
         store: &Arc<Store>,
-    ) -> Result<Merge<TreeId>, BackendError> {
+    ) -> BackendResult<Merge<TreeId>> {
         let num_sides = self
             .overrides
             .values()
@@ -1262,7 +1237,7 @@ impl MergedTreeBuilder {
         let merge_builder: MergeBuilder<TreeId> = tree_builders
             .into_iter()
             .map(|builder| builder.write_tree())
-            .collect();
+            .try_collect()?;
         Ok(merge_builder.build())
     }
 }
