@@ -14,6 +14,7 @@
 
 #![allow(missing_docs)]
 
+use std::collections::hash_map::Entry;
 use std::collections::{HashMap, HashSet};
 use std::fmt::{Debug, Formatter};
 use std::io::ErrorKind;
@@ -34,7 +35,7 @@ use crate::backend::{
 };
 use crate::commit::{Commit, CommitByCommitterTimestamp};
 use crate::commit_builder::CommitBuilder;
-use crate::default_index::DefaultIndexStore;
+use crate::default_index::{DefaultIndexStore, DefaultMutableIndex};
 use crate::default_submodule_store::DefaultSubmoduleStore;
 use crate::file_util::{IoResultExt as _, PathError};
 use crate::git_backend::GitBackend;
@@ -49,7 +50,8 @@ use crate::operation::Operation;
 use crate::refs::{
     diff_named_ref_targets, diff_named_remote_refs, merge_ref_targets, merge_remote_refs,
 };
-use crate::rewrite::{DescendantRebaser, RebaseOptions};
+use crate::revset::{RevsetEvaluationError, RevsetExpression, RevsetIteratorExt};
+use crate::rewrite::{merge_commit_trees, CommitRewriter, DescendantRebaser, RebaseOptions};
 use crate::settings::{RepoSettings, UserSettings};
 use crate::signing::{SignInitError, Signer};
 use crate::simple_op_heads_store::SimpleOpHeadsStore;
@@ -347,6 +349,19 @@ type IndexStoreFactory =
     Box<dyn Fn(&UserSettings, &Path) -> Result<Box<dyn IndexStore>, BackendLoadError>>;
 type SubmoduleStoreFactory = Box<dyn Fn(&UserSettings, &Path) -> Box<dyn SubmoduleStore>>;
 
+pub fn merge_factories_map<F>(base: &mut HashMap<String, F>, ext: HashMap<String, F>) {
+    for (name, factory) in ext {
+        match base.entry(name) {
+            Entry::Vacant(v) => {
+                v.insert(factory);
+            }
+            Entry::Occupied(o) => {
+                panic!("Conflicting factory definitions for '{}' factory", o.key())
+            }
+        }
+    }
+}
+
 pub struct StoreFactories {
     backend_factories: HashMap<String, BackendFactory>,
     op_store_factories: HashMap<String, OpStoreFactory>,
@@ -368,6 +383,15 @@ impl Default for StoreFactories {
             GitBackend::name(),
             Box::new(|settings, store_path| Ok(Box::new(GitBackend::load(settings, store_path)?))),
         );
+        #[cfg(feature = "testing")]
+        factories.add_backend(
+            crate::secret_backend::SecretBackend::name(),
+            Box::new(|settings, store_path| {
+                Ok(Box::new(crate::secret_backend::SecretBackend::load(
+                    settings, store_path,
+                )?))
+            }),
+        );
 
         // OpStores
         factories.add_op_store(
@@ -426,6 +450,25 @@ impl StoreFactories {
         }
     }
 
+    pub fn merge(&mut self, ext: StoreFactories) {
+        let StoreFactories {
+            backend_factories,
+            op_store_factories,
+            op_heads_store_factories,
+            index_store_factories,
+            submodule_store_factories,
+        } = ext;
+
+        merge_factories_map(&mut self.backend_factories, backend_factories);
+        merge_factories_map(&mut self.op_store_factories, op_store_factories);
+        merge_factories_map(&mut self.op_heads_store_factories, op_heads_store_factories);
+        merge_factories_map(&mut self.index_store_factories, index_store_factories);
+        merge_factories_map(
+            &mut self.submodule_store_factories,
+            submodule_store_factories,
+        );
+    }
+
     pub fn add_backend(&mut self, name: &str, factory: BackendFactory) {
         self.backend_factories.insert(name.to_string(), factory);
     }
@@ -727,18 +770,33 @@ impl RepoLoader {
     }
 }
 
-#[derive(Clone, Copy, Debug, PartialEq, Eq)]
-pub(crate) enum RewriteType {
-    Rewritten,
-    Divergent,
-    Abandoned,
+#[derive(Clone, Debug, PartialEq, Eq)]
+enum Rewrite {
+    /// The old commit was rewritten as this new commit. Children should be
+    /// rebased onto the new commit.
+    Rewritten(CommitId),
+    /// The old commit was rewritten as multiple other commits. Children should
+    /// not be rebased.
+    Divergent(Vec<CommitId>),
+    /// The old commit was abandoned. Children should be rebased onto the given
+    /// commits (typically the parents of the old commit).
+    Abandoned(Vec<CommitId>),
+}
+
+impl Rewrite {
+    fn new_parent_ids(&self) -> &[CommitId] {
+        match self {
+            Rewrite::Rewritten(new_parent_id) => std::slice::from_ref(new_parent_id),
+            Rewrite::Divergent(new_parent_ids) => new_parent_ids.as_slice(),
+            Rewrite::Abandoned(new_parent_ids) => new_parent_ids.as_slice(),
+        }
+    }
 }
 
 pub struct MutableRepo {
     base_repo: Arc<ReadonlyRepo>,
     index: Box<dyn MutableIndex>,
     view: DirtyCell<View>,
-    // TODO: make these fields private again
     // The commit identified by the key has been replaced by all the ones in the value.
     // * Branches pointing to the old commit should be updated to the new commit, resulting in a
     //   conflict if there multiple new commits.
@@ -747,7 +805,7 @@ pub struct MutableRepo {
     // * Working copies pointing to the old commit should be updated to the first of the new
     //   commits. However, if the type is `Abandoned`, a new working-copy commit should be created
     //   on top of all of the new commits instead.
-    pub(crate) parent_mapping: HashMap<CommitId, (RewriteType, Vec<CommitId>)>,
+    parent_mapping: HashMap<CommitId, Rewrite>,
 }
 
 impl MutableRepo {
@@ -828,7 +886,7 @@ impl MutableRepo {
     pub fn set_rewritten_commit(&mut self, old_id: CommitId, new_id: CommitId) {
         assert_ne!(old_id, *self.store().root_commit_id());
         self.parent_mapping
-            .insert(old_id, (RewriteType::Rewritten, vec![new_id]));
+            .insert(old_id, Rewrite::Rewritten(new_id));
     }
 
     /// Record a commit as being rewritten into multiple other commits in this
@@ -846,7 +904,7 @@ impl MutableRepo {
         assert_ne!(old_id, *self.store().root_commit_id());
         self.parent_mapping.insert(
             old_id.clone(),
-            (RewriteType::Divergent, new_ids.into_iter().collect()),
+            Rewrite::Divergent(new_ids.into_iter().collect()),
         );
     }
 
@@ -880,7 +938,7 @@ impl MutableRepo {
         assert_ne!(old_id, *self.store().root_commit_id());
         self.parent_mapping.insert(
             old_id,
-            (RewriteType::Abandoned, new_parent_ids.into_iter().collect()),
+            Rewrite::Abandoned(new_parent_ids.into_iter().collect()),
         );
     }
 
@@ -893,9 +951,9 @@ impl MutableRepo {
     /// rewritten and abandoned.
     ///
     /// Panics if `parent_mapping` contains cycles
-    pub fn new_parents(&self, old_ids: &[CommitId]) -> Vec<CommitId> {
+    pub fn new_parents(&self, old_ids: Vec<CommitId>) -> Vec<CommitId> {
         fn single_substitution_round(
-            parent_mapping: &HashMap<CommitId, (RewriteType, Vec<CommitId>)>,
+            parent_mapping: &HashMap<CommitId, Rewrite>,
             ids: Vec<CommitId>,
         ) -> (Vec<CommitId>, bool) {
             let mut made_replacements = false;
@@ -905,10 +963,14 @@ impl MutableRepo {
             // that case, but it probably doesn't matter much while they are Vec<u8>-s.
             for id in ids.into_iter() {
                 match parent_mapping.get(&id) {
-                    None | Some((RewriteType::Divergent, _)) => {
+                    None | Some(Rewrite::Divergent(_)) => {
                         new_ids.push(id);
                     }
-                    Some((_, replacements)) => {
+                    Some(Rewrite::Rewritten(replacement)) => {
+                        made_replacements = true;
+                        new_ids.push(replacement.clone())
+                    }
+                    Some(Rewrite::Abandoned(replacements)) => {
                         assert!(
                             // Each commit must have a parent, so a parent can
                             // not just be mapped to nothing. This assertion
@@ -925,7 +987,7 @@ impl MutableRepo {
             (new_ids, made_replacements)
         }
 
-        let mut new_ids: Vec<CommitId> = old_ids.to_vec();
+        let mut new_ids = old_ids;
         // The longest possible non-cycle substitution sequence goes through each key of
         // parent_mapping once.
         let mut allowed_iterations = 0..self.parent_mapping.len();
@@ -948,6 +1010,223 @@ impl MutableRepo {
         }
     }
 
+    /// Updates branches, working copies, and anonymous heads after rewriting
+    /// and/or abandoning commits.
+    pub fn update_rewritten_references(&mut self, settings: &UserSettings) -> BackendResult<()> {
+        self.update_all_references(settings)?;
+        self.update_heads();
+        Ok(())
+    }
+
+    fn update_all_references(&mut self, settings: &UserSettings) -> BackendResult<()> {
+        for (old_parent_id, rewrite) in self.parent_mapping.clone() {
+            // Call `new_parents()` here since `parent_mapping` only contains direct
+            // mappings, not transitive ones.
+            // TODO: keep parent_mapping updated with transitive mappings so we don't need
+            // to call `new_parents()` here.
+            let new_parent_ids = self.new_parents(rewrite.new_parent_ids().to_vec());
+            self.update_references(settings, old_parent_id, new_parent_ids)?;
+        }
+        Ok(())
+    }
+
+    fn update_references(
+        &mut self,
+        settings: &UserSettings,
+        old_commit_id: CommitId,
+        new_commit_ids: Vec<CommitId>,
+    ) -> BackendResult<()> {
+        let abandoned_old_commit = matches!(
+            self.parent_mapping.get(&old_commit_id),
+            Some(Rewrite::Abandoned(_))
+        );
+        self.update_wc_commits(
+            settings,
+            &old_commit_id,
+            &new_commit_ids,
+            abandoned_old_commit,
+        )?;
+
+        // Build a map from commit to branches pointing to it, so we don't need to scan
+        // all branches each time we rebase a commit.
+        // TODO: We no longer need to do this now that we update branches for all
+        // commits at once.
+        let mut branches: HashMap<_, HashSet<_>> = HashMap::new();
+        for (branch_name, target) in self.view().local_branches() {
+            for commit in target.added_ids() {
+                branches
+                    .entry(commit.clone())
+                    .or_default()
+                    .insert(branch_name.to_owned());
+            }
+        }
+
+        if let Some(branch_names) = branches.get(&old_commit_id).cloned() {
+            let mut branch_updates = vec![];
+            for branch_name in &branch_names {
+                let local_target = self.get_local_branch(branch_name);
+                for old_add in local_target.added_ids() {
+                    if *old_add == old_commit_id {
+                        branch_updates.push(branch_name.clone());
+                    }
+                }
+            }
+
+            let old_target = RefTarget::normal(old_commit_id.clone());
+            assert!(!new_commit_ids.is_empty());
+            let new_target = RefTarget::from_legacy_form(
+                std::iter::repeat(old_commit_id).take(new_commit_ids.len() - 1),
+                new_commit_ids,
+            );
+            for branch_name in &branch_updates {
+                self.merge_local_branch(branch_name, &old_target, &new_target);
+            }
+        }
+
+        Ok(())
+    }
+
+    fn update_wc_commits(
+        &mut self,
+        settings: &UserSettings,
+        old_commit_id: &CommitId,
+        new_commit_ids: &[CommitId],
+        abandoned_old_commit: bool,
+    ) -> BackendResult<()> {
+        let workspaces_to_update = self.view().workspaces_for_wc_commit_id(old_commit_id);
+        if workspaces_to_update.is_empty() {
+            return Ok(());
+        }
+
+        let new_wc_commit = if !abandoned_old_commit {
+            // We arbitrarily pick a new working-copy commit among the candidates.
+            self.store().get_commit(&new_commit_ids[0])?
+        } else {
+            let new_commits: Vec<_> = new_commit_ids
+                .iter()
+                .map(|id| self.store().get_commit(id))
+                .try_collect()?;
+            let merged_parents_tree = merge_commit_trees(self, &new_commits)?;
+            self.new_commit(
+                settings,
+                new_commit_ids.to_vec(),
+                merged_parents_tree.id().clone(),
+            )
+            .write()?
+        };
+        for workspace_id in workspaces_to_update.into_iter() {
+            self.edit(workspace_id, &new_wc_commit).unwrap();
+        }
+        Ok(())
+    }
+
+    fn update_heads(&mut self) {
+        let old_commits_expression =
+            RevsetExpression::commits(self.parent_mapping.keys().cloned().collect());
+        let heads_to_add_expression = old_commits_expression
+            .parents()
+            .minus(&old_commits_expression);
+        let heads_to_add = heads_to_add_expression
+            .evaluate_programmatic(self)
+            .unwrap()
+            .iter();
+
+        let mut view = self.view().store_view().clone();
+        for commit_id in self.parent_mapping.keys() {
+            view.head_ids.remove(commit_id);
+        }
+        view.head_ids.extend(heads_to_add);
+        self.set_view(view);
+    }
+
+    /// Find descendants of `root`, unless they've already been rewritten
+    /// (according to `parent_mapping`), and then return them in
+    /// an order they should be rebased in. The result is in reverse order
+    /// so the next value can be removed from the end.
+    fn find_descendants_to_rebase(&self, roots: Vec<CommitId>) -> BackendResult<Vec<Commit>> {
+        let store = self.store();
+        let to_visit_expression =
+            RevsetExpression::commits(roots)
+                .descendants()
+                .minus(&RevsetExpression::commits(
+                    self.parent_mapping.keys().cloned().collect(),
+                ));
+        let to_visit_revset = to_visit_expression
+            .evaluate_programmatic(self)
+            .map_err(|err| match err {
+                RevsetEvaluationError::StoreError(err) => err,
+                RevsetEvaluationError::Other(_) => panic!("Unexpected revset error: {err}"),
+            })?;
+        let to_visit: Vec<_> = to_visit_revset.iter().commits(store).try_collect()?;
+        drop(to_visit_revset);
+        let to_visit_set: HashSet<CommitId> =
+            to_visit.iter().map(|commit| commit.id().clone()).collect();
+        let mut visited = HashSet::new();
+        // Calculate an order where we rebase parents first, but if the parents were
+        // rewritten, make sure we rebase the rewritten parent first.
+        dag_walk::topo_order_reverse_ok(
+            to_visit.into_iter().map(Ok),
+            |commit| commit.id().clone(),
+            |commit| -> Vec<BackendResult<Commit>> {
+                visited.insert(commit.id().clone());
+                let mut dependents = vec![];
+                for parent in commit.parents() {
+                    let Ok(parent) = parent else {
+                        dependents.push(parent);
+                        continue;
+                    };
+                    if let Some(rewrite) = self.parent_mapping.get(parent.id()) {
+                        for target in rewrite.new_parent_ids() {
+                            if to_visit_set.contains(target) && !visited.contains(target) {
+                                dependents.push(store.get_commit(target));
+                            }
+                        }
+                    }
+                    if to_visit_set.contains(parent.id()) {
+                        dependents.push(Ok(parent));
+                    }
+                }
+                dependents
+            },
+        )
+    }
+
+    /// Rewrite descendants of the given roots.
+    ///
+    /// The callback will be called for each commit with the new parents
+    /// prepopulated. The callback may change the parents and write the new
+    /// commit, or it may abandon the commit, or it may leave the old commit
+    /// unchanged.
+    ///
+    /// The set of commits to visit is determined at the start. If the callback
+    /// adds new descendants, then the callback will not be called for those.
+    /// Similarly, if the callback rewrites unrelated commits, then the callback
+    /// will not be called for descendants of those commits.
+    pub fn transform_descendants(
+        &mut self,
+        settings: &UserSettings,
+        roots: Vec<CommitId>,
+        mut callback: impl FnMut(CommitRewriter) -> BackendResult<()>,
+    ) -> BackendResult<()> {
+        let mut to_visit = self.find_descendants_to_rebase(roots)?;
+        while let Some(old_commit) = to_visit.pop() {
+            let new_parent_ids = self.new_parents(old_commit.parent_ids().to_vec());
+            let rewriter = CommitRewriter::new(self, old_commit, new_parent_ids);
+            callback(rewriter)?;
+        }
+        self.update_rewritten_references(settings)?;
+        // Since we didn't necessarily visit all descendants of rewritten commits (e.g.
+        // if they were rewritten in the callback), there can still be commits left to
+        // rebase, so we don't clear `parent_mapping` here.
+        // TODO: Should we make this stricter? We could check that there were no
+        // rewrites before this function was called, and we can check that only
+        // commits in the `to_visit` set were added by the callback. Then we
+        // could clear `parent_mapping` here and not have to scan it again at
+        // the end of the transaction when we call `rebase_descendants()`.
+
+        Ok(())
+    }
+
     /// After the rebaser returned by this function is dropped,
     /// self.parent_mapping needs to be cleared.
     fn rebase_descendants_return_rebaser<'settings, 'repo>(
@@ -959,7 +1238,10 @@ impl MutableRepo {
             // Optimization
             return Ok(None);
         }
-        let mut rebaser = DescendantRebaser::new(settings, self);
+
+        let to_visit =
+            self.find_descendants_to_rebase(self.parent_mapping.keys().cloned().collect())?;
+        let mut rebaser = DescendantRebaser::new(settings, self, to_visit);
         *rebaser.mut_options() = options;
         rebaser.rebase_all()?;
         Ok(Some(rebaser))
@@ -993,6 +1275,7 @@ impl MutableRepo {
     /// **its parent**. One can tell this case apart since the change ids of the
     /// key and the value will not match. The parent will inherit the
     /// descendants and the branches of the abandoned commit.
+    // TODO: Rewrite this using `transform_descendants()`
     pub fn rebase_descendants_with_options_return_map(
         &mut self,
         settings: &UserSettings,
@@ -1009,7 +1292,18 @@ impl MutableRepo {
     }
 
     pub fn rebase_descendants(&mut self, settings: &UserSettings) -> BackendResult<usize> {
-        self.rebase_descendants_with_options(settings, Default::default())
+        let roots = self.parent_mapping.keys().cloned().collect_vec();
+        let mut num_rebased = 0;
+        self.transform_descendants(settings, roots, |rewriter| {
+            if rewriter.parents_changed() {
+                let builder = rewriter.rebase(settings)?;
+                builder.write()?;
+                num_rebased += 1;
+            }
+            Ok(())
+        })?;
+        self.parent_mapping.clear();
+        Ok(num_rebased)
     }
 
     pub fn rebase_descendants_return_map(
@@ -1070,7 +1364,7 @@ impl MutableRepo {
                 .store()
                 .get_commit(&wc_commit_id)
                 .map_err(EditCommitError::WorkingCopyCommitNotFound)?;
-            if wc_commit.is_discardable()
+            if wc_commit.is_discardable(self)?
                 && self
                     .view
                     .with_ref(|v| local_branch_target_ids(v).all(|id| id != wc_commit.id()))
@@ -1335,14 +1629,26 @@ impl MutableRepo {
                     .set_wc_commit(workspace_id.clone(), other_wc_commit.clone());
             }
         }
-
         let base_heads = base.heads().iter().cloned().collect_vec();
         let own_heads = self.view().heads().iter().cloned().collect_vec();
         let other_heads = other.heads().iter().cloned().collect_vec();
-        self.record_rewrites(&base_heads, &own_heads);
-        self.record_rewrites(&base_heads, &other_heads);
-        // No need to remove heads removed by `other` because we already marked them
-        // abandoned or rewritten.
+
+        // HACK: Don't walk long ranges of commits to find rewrites when using other
+        // custom implementations. The only custom index implementation we're currently
+        // aware of is Google's. That repo has too high commit rate for it to be
+        // feasible to walk all added and removed commits.
+        // TODO: Fix this somehow. Maybe a method on `Index` to find rewritten commits
+        // given `base_heads`, `own_heads` and `other_heads`?
+        if self.index.as_any().is::<DefaultMutableIndex>() {
+            self.record_rewrites(&base_heads, &own_heads);
+            self.record_rewrites(&base_heads, &other_heads);
+            // No need to remove heads removed by `other` because we already
+            // marked them abandoned or rewritten.
+        } else {
+            for removed_head in base.heads().difference(other.heads()) {
+                self.view_mut().remove_head(removed_head);
+            }
+        }
         for added_head in other.heads().difference(base.heads()) {
             self.view_mut().add_head(added_head);
         }
@@ -1477,6 +1783,8 @@ pub enum EditCommitError {
     WorkingCopyCommitNotFound(#[source] BackendError),
     #[error("Cannot rewrite the root commit")]
     RewriteRootCommit,
+    #[error(transparent)]
+    BackendError(#[from] BackendError),
 }
 
 /// Error from attempts to check out a commit
