@@ -16,7 +16,7 @@
 
 use std::borrow::Borrow;
 use std::cmp::Ordering;
-use std::fmt::{Debug, Error, Formatter};
+use std::fmt::{self, Debug, Formatter};
 use std::iter::FusedIterator;
 use std::ops::Deref;
 use std::path::{Component, Path, PathBuf};
@@ -24,20 +24,19 @@ use std::path::{Component, Path, PathBuf};
 use ref_cast::{ref_cast_custom, RefCastCustom};
 use thiserror::Error;
 
+use crate::content_hash::ContentHash;
 use crate::file_util;
 
-content_hash! {
-    /// Owned `RepoPath` component.
-    #[derive(Clone, PartialEq, Eq, PartialOrd, Ord, Debug, Hash)]
-    pub struct RepoPathComponentBuf {
-        // Don't add more fields. Eq, Hash, and Ord must be compatible with the
-        // borrowed RepoPathComponent type.
-        value: String,
-    }
+/// Owned `RepoPath` component.
+#[derive(ContentHash, Clone, PartialEq, Eq, PartialOrd, Ord, Hash)]
+pub struct RepoPathComponentBuf {
+    // Don't add more fields. Eq, Hash, and Ord must be compatible with the
+    // borrowed RepoPathComponent type.
+    value: String,
 }
 
 /// Borrowed `RepoPath` component.
-#[derive(PartialEq, Eq, PartialOrd, Ord, Debug, Hash, RefCastCustom)]
+#[derive(PartialEq, Eq, PartialOrd, Ord, Hash, RefCastCustom)]
 #[repr(transparent)]
 pub struct RepoPathComponent {
     value: str,
@@ -60,6 +59,18 @@ impl RepoPathComponent {
     }
 }
 
+impl Debug for RepoPathComponent {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        write!(f, "{:?}", &self.value)
+    }
+}
+
+impl Debug for RepoPathComponentBuf {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        <RepoPathComponent as Debug>::fmt(self, f)
+    }
+}
+
 impl From<&str> for RepoPathComponentBuf {
     fn from(value: &str) -> Self {
         RepoPathComponentBuf::from(value.to_owned())
@@ -135,7 +146,7 @@ impl<'a> Iterator for RepoPathComponentsIter<'a> {
         let (name, remainder) = self
             .value
             .split_once('/')
-            .unwrap_or_else(|| (&self.value, &self.value[self.value.len()..]));
+            .unwrap_or_else(|| (self.value, &self.value[self.value.len()..]));
         self.value = remainder;
         Some(RepoPathComponent::new_unchecked(name))
     }
@@ -149,7 +160,7 @@ impl DoubleEndedIterator for RepoPathComponentsIter<'_> {
         let (remainder, name) = self
             .value
             .rsplit_once('/')
-            .unwrap_or_else(|| (&self.value[..0], &self.value));
+            .unwrap_or_else(|| (&self.value[..0], self.value));
         self.value = remainder;
         Some(RepoPathComponent::new_unchecked(name))
     }
@@ -173,13 +184,13 @@ pub struct RepoPath {
 }
 
 impl Debug for RepoPath {
-    fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), Error> {
-        f.write_fmt(format_args!("{:?}", &self.value))
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        write!(f, "{:?}", &self.value)
     }
 }
 
 impl Debug for RepoPathBuf {
-    fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), Error> {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         <RepoPath as Debug>::fmt(self, f)
     }
 }
@@ -204,15 +215,28 @@ impl RepoPathBuf {
 
     /// Converts repo-relative `Path` to `RepoPathBuf`.
     ///
-    /// The input path should not contain `.` or `..`.
-    pub fn from_relative_path(relative_path: impl AsRef<Path>) -> Option<Self> {
+    /// The input path should not contain redundant `.` or `..`.
+    pub fn from_relative_path(
+        relative_path: impl AsRef<Path>,
+    ) -> Result<Self, RelativePathParseError> {
         let relative_path = relative_path.as_ref();
+        if relative_path == Path::new(".") {
+            return Ok(Self::root());
+        }
+
         let mut components = relative_path
             .components()
             .map(|c| match c {
-                Component::Normal(name) => Some(name.to_str().unwrap()),
-                // TODO: better to return Err instead of None?
-                _ => None,
+                Component::Normal(name) => {
+                    name.to_str()
+                        .ok_or_else(|| RelativePathParseError::InvalidUtf8 {
+                            path: relative_path.into(),
+                        })
+                }
+                _ => Err(RelativePathParseError::InvalidComponent {
+                    component: c.as_os_str().to_string_lossy().into(),
+                    path: relative_path.into(),
+                }),
             })
             .fuse();
         let mut value = String::with_capacity(relative_path.as_os_str().len());
@@ -223,7 +247,7 @@ impl RepoPathBuf {
             value.push('/');
             value.push_str(name?);
         }
-        Some(RepoPathBuf { value })
+        Ok(RepoPathBuf { value })
     }
 
     /// Parses an `input` path into a `RepoPathBuf` relative to `base`.
@@ -239,11 +263,11 @@ impl RepoPathBuf {
         let input = input.as_ref();
         let abs_input_path = file_util::normalize_path(&cwd.join(input));
         let repo_relative_path = file_util::relative_path(base, &abs_input_path);
-        if repo_relative_path == Path::new(".") {
-            return Ok(Self::root());
-        }
-        Self::from_relative_path(repo_relative_path)
-            .ok_or_else(|| FsPathParseError::InputNotInRepo(input.to_owned()))
+        Self::from_relative_path(repo_relative_path).map_err(|source| FsPathParseError {
+            base: file_util::relative_path(cwd, base).into(),
+            input: input.into(),
+            source,
+        })
     }
 
     /// Consumes this and returns the underlying string representation.
@@ -292,6 +316,9 @@ impl RepoPath {
         let mut result = PathBuf::with_capacity(base.as_os_str().len() + self.value.len() + 1);
         result.push(base);
         result.extend(self.components().map(RepoPathComponent::as_str));
+        if result.as_os_str().is_empty() {
+            result.push(".");
+        }
         result
     }
 
@@ -412,9 +439,25 @@ impl PartialOrd for RepoPathBuf {
 }
 
 #[derive(Clone, Debug, Eq, Error, PartialEq)]
-pub enum FsPathParseError {
-    #[error(r#"Path "{}" is not in the repo"#, .0.display())]
-    InputNotInRepo(PathBuf),
+pub enum RelativePathParseError {
+    #[error(r#"Invalid component "{component}" in repo-relative path "{path}""#)]
+    InvalidComponent {
+        component: Box<str>,
+        path: Box<Path>,
+    },
+    #[error(r#"Not valid UTF-8 path "{path}""#)]
+    InvalidUtf8 { path: Box<Path> },
+}
+
+#[derive(Clone, Debug, Eq, Error, PartialEq)]
+#[error(r#"Path "{input}" is not in the repo "{base}""#)]
+pub struct FsPathParseError {
+    /// Repository or workspace root path relative to the `cwd`.
+    pub base: Box<Path>,
+    /// Input path without normalization.
+    pub input: Box<Path>,
+    /// Source error.
+    pub source: RelativePathParseError,
 }
 
 fn is_valid_repo_path_component_str(value: &str) -> bool {
@@ -425,10 +468,58 @@ fn is_valid_repo_path_str(value: &str) -> bool {
     !value.starts_with('/') && !value.ends_with('/') && !value.contains("//")
 }
 
+/// An error from `RepoPathUiConverter::parse_file_path`.
+#[derive(Debug, Error)]
+pub enum UiPathParseError {
+    #[error(transparent)]
+    Fs(FsPathParseError),
+}
+
+/// Converts `RepoPath`s to and from plain strings as displayed to the user
+/// (e.g. relative to CWD).
+#[derive(Debug)]
+pub enum RepoPathUiConverter {
+    /// Variant for a local file system. Paths are interpreted relative to `cwd`
+    /// with the repo rooted in `base`.
+    ///
+    /// The `cwd` and `base` paths are supposed to be absolute and normalized in
+    /// the same manner.
+    Fs { cwd: PathBuf, base: PathBuf },
+    // TODO: Add a no-op variant that uses the internal `RepoPath` representation. Can be useful
+    // on a server.
+}
+
+impl RepoPathUiConverter {
+    /// Format a path for display in the UI.
+    pub fn format_file_path(&self, file: &RepoPath) -> String {
+        match self {
+            RepoPathUiConverter::Fs { cwd, base } => {
+                file_util::relative_path(cwd, &file.to_fs_path(base))
+                    .to_str()
+                    .unwrap()
+                    .to_owned()
+            }
+        }
+    }
+
+    /// Parses a path from the UI.
+    ///
+    /// It's up to the implementation whether absolute paths are allowed, and
+    /// where relative paths are interpreted as relative to.
+    pub fn parse_file_path(&self, input: &str) -> Result<RepoPathBuf, UiPathParseError> {
+        match self {
+            RepoPathUiConverter::Fs { cwd, base } => {
+                RepoPathBuf::parse_fs_path(cwd, base, input).map_err(UiPathParseError::Fs)
+            }
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use std::panic;
 
+    use assert_matches::assert_matches;
     use itertools::Itertools as _;
 
     use super::*;
@@ -621,7 +712,7 @@ mod tests {
             repo_path("").to_fs_path(Path::new("base/dir")),
             Path::new("base/dir")
         );
-        assert_eq!(repo_path("").to_fs_path(Path::new("")), Path::new(""));
+        assert_eq!(repo_path("").to_fs_path(Path::new("")), Path::new("."));
         assert_eq!(
             repo_path("file").to_fs_path(Path::new("base/dir")),
             Path::new("base/dir/file")
@@ -668,9 +759,12 @@ mod tests {
             RepoPathBuf::parse_fs_path(&cwd_path, wc_path, "dir/file").as_deref(),
             Ok(repo_path("dir/file"))
         );
-        assert_eq!(
+        assert_matches!(
             RepoPathBuf::parse_fs_path(&cwd_path, wc_path, ".."),
-            Err(FsPathParseError::InputNotInRepo("..".into()))
+            Err(FsPathParseError {
+                source: RelativePathParseError::InvalidComponent { .. },
+                ..
+            })
         );
         assert_eq!(
             RepoPathBuf::parse_fs_path(&cwd_path, &cwd_path, "../repo").as_deref(),
@@ -718,9 +812,12 @@ mod tests {
             RepoPathBuf::parse_fs_path(&cwd_path, &wc_path, "..").as_deref(),
             Ok(RepoPath::root())
         );
-        assert_eq!(
+        assert_matches!(
             RepoPathBuf::parse_fs_path(&cwd_path, &wc_path, "../.."),
-            Err(FsPathParseError::InputNotInRepo("../..".into()))
+            Err(FsPathParseError {
+                source: RelativePathParseError::InvalidComponent { .. },
+                ..
+            })
         );
         assert_eq!(
             RepoPathBuf::parse_fs_path(&cwd_path, &wc_path, "../other-dir/file").as_deref(),
@@ -734,13 +831,19 @@ mod tests {
         let cwd_path = temp_dir.path().join("cwd");
         let wc_path = cwd_path.join("repo");
 
-        assert_eq!(
+        assert_matches!(
             RepoPathBuf::parse_fs_path(&cwd_path, &wc_path, ""),
-            Err(FsPathParseError::InputNotInRepo("".into()))
+            Err(FsPathParseError {
+                source: RelativePathParseError::InvalidComponent { .. },
+                ..
+            })
         );
-        assert_eq!(
+        assert_matches!(
             RepoPathBuf::parse_fs_path(&cwd_path, &wc_path, "not-repo"),
-            Err(FsPathParseError::InputNotInRepo("not-repo".into()))
+            Err(FsPathParseError {
+                source: RelativePathParseError::InvalidComponent { .. },
+                ..
+            })
         );
         assert_eq!(
             RepoPathBuf::parse_fs_path(&cwd_path, &wc_path, "repo").as_deref(),
