@@ -19,6 +19,7 @@ use std::collections::HashMap;
 use std::fmt::{Debug, Formatter};
 use std::io::Read;
 use std::sync::{Arc, RwLock};
+use std::time::SystemTime;
 
 use pollster::FutureExt;
 
@@ -27,6 +28,7 @@ use crate::backend::{
     SymlinkId, TreeId,
 };
 use crate::commit::Commit;
+use crate::index::Index;
 use crate::merge::{Merge, MergedTreeValue};
 use crate::merged_tree::MergedTree;
 use crate::repo_path::{RepoPath, RepoPathBuf};
@@ -105,7 +107,7 @@ impl Store {
     }
 
     pub fn empty_merged_tree_id(&self) -> MergedTreeId {
-        MergedTreeId::Legacy(self.backend.empty_tree_id().clone())
+        MergedTreeId::resolved(self.backend.empty_tree_id().clone())
     }
 
     pub fn root_commit(self: &Arc<Self>) -> Commit {
@@ -266,7 +268,7 @@ impl Store {
         TreeBuilder::new(self.clone(), base_tree_id)
     }
 
-    pub fn gc(&self) -> Result<(), Box<dyn std::error::Error + Send + Sync>> {
-        self.backend.gc()
+    pub fn gc(&self, index: &dyn Index, keep_newer: SystemTime) -> BackendResult<()> {
+        self.backend.gc(index, keep_newer)
     }
 }
