@@ -12,7 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use jj_lib::backend::{ChangeId, MillisSinceEpoch, ObjectId, Signature, Timestamp};
+use itertools::Itertools;
+use jj_lib::backend::{ChangeId, MillisSinceEpoch, Signature, Timestamp};
 use jj_lib::matchers::EverythingMatcher;
 use jj_lib::merged_tree::DiffSummary;
 use jj_lib::repo::Repo;
@@ -77,8 +78,9 @@ fn test_initial(backend: TestRepoBackend) {
     let commit = builder.write().unwrap();
     tx.commit("test");
 
-    assert_eq!(commit.parents(), vec![store.root_commit()]);
-    assert_eq!(commit.predecessors(), vec![]);
+    let parents: Vec<_> = commit.parents().try_collect().unwrap();
+    assert_eq!(parents, vec![store.root_commit()]);
+    assert!(commit.predecessors().next().is_none());
     assert_eq!(commit.description(), "description");
     assert_eq!(commit.author(), &author_signature);
     assert_eq!(commit.committer(), &committer_signature);
@@ -152,11 +154,10 @@ fn test_rewrite(backend: TestRepoBackend) {
         .unwrap();
     tx.mut_repo().rebase_descendants(&settings).unwrap();
     tx.commit("test");
-    assert_eq!(rewritten_commit.parents(), vec![store.root_commit()]);
-    assert_eq!(
-        rewritten_commit.predecessors(),
-        vec![initial_commit.clone()]
-    );
+    let parents: Vec<_> = rewritten_commit.parents().try_collect().unwrap();
+    assert_eq!(parents, vec![store.root_commit()]);
+    let predecessors: Vec<_> = rewritten_commit.predecessors().try_collect().unwrap();
+    assert_eq!(predecessors, vec![initial_commit.clone()]);
     assert_eq!(rewritten_commit.author().name, settings.user_name());
     assert_eq!(rewritten_commit.author().email, settings.user_email());
     assert_eq!(
