@@ -12,6 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use indoc::indoc;
 use jj_lib::backend::FileId;
 use jj_lib::conflicts::{
     extract_as_single_hunk, materialize_merge_result, parse_conflict, update_from_content,
@@ -32,34 +33,37 @@ fn test_materialize_conflict_basic() {
     let base_id = testutils::write_file(
         store,
         path,
-        "line 1
-line 2
-line 3
-line 4
-line 5
-",
+        indoc! {"
+            line 1
+            line 2
+            line 3
+            line 4
+            line 5
+        "},
     );
     let left_id = testutils::write_file(
         store,
         path,
-        "line 1
-line 2
-left 3.1
-left 3.2
-left 3.3
-line 4
-line 5
-",
+        indoc! {"
+            line 1
+            line 2
+            left 3.1
+            left 3.2
+            left 3.3
+            line 4
+            line 5
+        "},
     );
     let right_id = testutils::write_file(
         store,
         path,
-        "line 1
-line 2
-right 3.1
-line 4
-line 5
-",
+        indoc! {"
+            line 1
+            line 2
+            right 3.1
+            line 4
+            line 5
+        "},
     );
 
     // The left side should come first. The diff should be use the smaller (right)
@@ -73,15 +77,15 @@ line 5
         @r###"
     line 1
     line 2
-    <<<<<<<
-    +++++++
+    <<<<<<< Conflict 1 of 1
+    +++++++ Contents of side #1
     left 3.1
     left 3.2
     left 3.3
-    %%%%%%%
+    %%%%%%% Changes from base to side #2
     -line 3
     +right 3.1
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     line 4
     line 5
     "###
@@ -97,15 +101,15 @@ line 5
         @r###"
     line 1
     line 2
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
     -line 3
     +right 3.1
-    +++++++
+    +++++++ Contents of side #2
     left 3.1
     left 3.2
     left 3.3
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     line 4
     line 5
     "###
@@ -122,37 +126,41 @@ fn test_materialize_conflict_multi_rebase_conflicts() {
     let base_id = testutils::write_file(
         store,
         path,
-        "line 1
-line 2 base
-line 3
-",
+        indoc! {"
+            line 1
+            line 2 base
+            line 3
+        "},
     );
     let a_id = testutils::write_file(
         store,
         path,
-        "line 1
-line 2 a.1
-line 2 a.2
-line 2 a.3
-line 3
-",
+        indoc! {"
+            line 1
+            line 2 a.1
+            line 2 a.2
+            line 2 a.3
+            line 3
+        "},
     );
     let b_id = testutils::write_file(
         store,
         path,
-        "line 1
-line 2 b.1
-line 2 b.2
-line 3
-",
+        indoc! {"
+            line 1
+            line 2 b.1
+            line 2 b.2
+            line 3
+        "},
     );
     let c_id = testutils::write_file(
         store,
         path,
-        "line 1
-line 2 c.1
-line 3
-",
+        indoc! {"
+            line 1
+            line 2 c.1
+            line 3
+        "},
     );
 
     // The order of (a, b, c) should be preserved. For all cases, the "a" side
@@ -165,19 +173,19 @@ line 3
         &materialize_conflict_string(store, path, &conflict),
         @r###"
     line 1
-    <<<<<<<
-    +++++++
+    <<<<<<< Conflict 1 of 1
+    +++++++ Contents of side #1
     line 2 a.1
     line 2 a.2
     line 2 a.3
-    %%%%%%%
+    %%%%%%% Changes from base #1 to side #2
     -line 2 base
     +line 2 b.1
     +line 2 b.2
-    %%%%%%%
+    %%%%%%% Changes from base #2 to side #3
     -line 2 base
     +line 2 c.1
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     line 3
     "###
     );
@@ -189,19 +197,19 @@ line 3
         &materialize_conflict_string(store, path, &conflict),
         @r###"
     line 1
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base #1 to side #1
     -line 2 base
     +line 2 c.1
-    %%%%%%%
+    %%%%%%% Changes from base #2 to side #2
     -line 2 base
     +line 2 b.1
     +line 2 b.2
-    +++++++
+    +++++++ Contents of side #3
     line 2 a.1
     line 2 a.2
     line 2 a.3
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     line 3
     "###
     );
@@ -213,24 +221,25 @@ line 3
         &materialize_conflict_string(store, path, &conflict),
         @r###"
     line 1
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base #1 to side #1
     -line 2 base
     +line 2 c.1
-    +++++++
+    +++++++ Contents of side #2
     line 2 a.1
     line 2 a.2
     line 2 a.3
-    %%%%%%%
+    %%%%%%% Changes from base #2 to side #3
     -line 2 base
     +line 2 b.1
     +line 2 b.2
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     line 3
     "###
     );
 }
 
+//  TODO: With options
 #[test]
 fn test_materialize_parse_roundtrip() {
     let test_repo = TestRepo::init();
@@ -240,32 +249,35 @@ fn test_materialize_parse_roundtrip() {
     let base_id = testutils::write_file(
         store,
         path,
-        "line 1
-line 2
-line 3
-line 4
-line 5
-",
+        indoc! {"
+            line 1
+            line 2
+            line 3
+            line 4
+            line 5
+        "},
     );
     let left_id = testutils::write_file(
         store,
         path,
-        "line 1 left
-line 2 left
-line 3
-line 4
-line 5 left
-",
+        indoc! {"
+            line 1 left
+            line 2 left
+            line 3
+            line 4
+            line 5 left
+        "},
     );
     let right_id = testutils::write_file(
         store,
         path,
-        "line 1 right
-line 2
-line 3
-line 4 right
-line 5 right
-",
+        indoc! {"
+            line 1 right
+            line 2
+            line 3
+            line 4 right
+            line 5 right
+        "},
     );
 
     let conflict = Merge::from_removes_adds(
@@ -276,25 +288,25 @@ line 5 right
     insta::assert_snapshot!(
         materialized,
         @r###"
-    <<<<<<<
-    +++++++
+    <<<<<<< Conflict 1 of 2
+    +++++++ Contents of side #1
     line 1 left
     line 2 left
-    %%%%%%%
+    %%%%%%% Changes from base to side #2
     -line 1
     +line 1 right
      line 2
-    >>>>>>>
+    >>>>>>> Conflict 1 of 2 ends
     line 3
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 2 of 2
+    %%%%%%% Changes from base to side #1
      line 4
     -line 5
     +line 5 left
-    +++++++
+    +++++++ Contents of side #2
     line 4 right
     line 5 right
-    >>>>>>>
+    >>>>>>> Conflict 2 of 2 ends
     "###
     );
 
@@ -335,31 +347,34 @@ fn test_materialize_conflict_modify_delete() {
     let base_id = testutils::write_file(
         store,
         path,
-        "line 1
-line 2
-line 3
-line 4
-line 5
-",
+        indoc! {"
+            line 1
+            line 2
+            line 3
+            line 4
+            line 5
+        "},
     );
     let modified_id = testutils::write_file(
         store,
         path,
-        "line 1
-line 2
-modified
-line 4
-line 5
-",
+        indoc! {"
+            line 1
+            line 2
+            modified
+            line 4
+            line 5
+        "},
     );
     let deleted_id = testutils::write_file(
         store,
         path,
-        "line 1
-line 2
-line 4
-line 5
-",
+        indoc! {"
+            line 1
+            line 2
+            line 4
+            line 5
+        "},
     );
 
     // left modifies a line, right deletes the same line.
@@ -370,12 +385,12 @@ line 5
     insta::assert_snapshot!(&materialize_conflict_string(store, path, &conflict), @r###"
     line 1
     line 2
-    <<<<<<<
-    +++++++
+    <<<<<<< Conflict 1 of 1
+    +++++++ Contents of side #1
     modified
-    %%%%%%%
+    %%%%%%% Changes from base to side #2
     -line 3
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     line 4
     line 5
     "###
@@ -389,12 +404,12 @@ line 5
     insta::assert_snapshot!(&materialize_conflict_string(store, path, &conflict), @r###"
     line 1
     line 2
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
     -line 3
-    +++++++
+    +++++++ Contents of side #2
     modified
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     line 4
     line 5
     "###
@@ -406,16 +421,16 @@ line 5
         vec![Some(modified_id.clone()), None],
     );
     insta::assert_snapshot!(&materialize_conflict_string(store, path, &conflict), @r###"
-    <<<<<<<
-    %%%%%%%
+    <<<<<<< Conflict 1 of 1
+    %%%%%%% Changes from base to side #1
      line 1
      line 2
     -line 3
     +modified
      line 4
      line 5
-    +++++++
-    >>>>>>>
+    +++++++ Contents of side #2
+    >>>>>>> Conflict 1 of 1 ends
     "###
     );
 }
@@ -459,18 +474,18 @@ fn test_materialize_conflict_two_forward_diffs() {
     insta::assert_snapshot!(
         &materialize_conflict_string(store, path, &conflict),
         @r###"
-    <<<<<<<
-    +++++++
+    <<<<<<< Conflict 1 of 1
+    +++++++ Contents of side #1
     A
-    %%%%%%%
+    %%%%%%% Changes from base #1 to side #2
      B
-    +++++++
+    +++++++ Contents of side #3
     D
-    %%%%%%%
+    %%%%%%% Changes from base #2 to side #4
      C
-    -------
+    ------- Contents of base #3
     E
-    >>>>>>>
+    >>>>>>> Conflict 1 of 1 ends
     "###
     );
 }
@@ -479,12 +494,13 @@ fn test_materialize_conflict_two_forward_diffs() {
 fn test_parse_conflict_resolved() {
     assert_eq!(
         parse_conflict(
-            b"line 1
+            indoc! {b"
+            line 1
 line 2
 line 3
 line 4
 line 5
-",
+"},
             2
         ),
         None
@@ -494,19 +510,55 @@ line 5
 #[test]
 fn test_parse_conflict_simple() {
     insta::assert_debug_snapshot!(
-        parse_conflict(
-            b"line 1
-<<<<<<<
-%%%%%%%
- line 2
--line 3
-+left
- line 4
-+++++++
-right
->>>>>>>
-line 5
-",
+        parse_conflict(indoc! {b"
+            line 1
+            <<<<<<<
+            %%%%%%%
+             line 2
+            -line 3
+            +left
+             line 4
+            +++++++
+            right
+            >>>>>>>
+            line 5
+            "},
+            2
+        ),
+        @r###"
+    Some(
+        [
+            Resolved(
+                "line 1\n",
+            ),
+            Conflicted(
+                [
+                    "line 2\nleft\nline 4\n",
+                    "line 2\nline 3\nline 4\n",
+                    "right\n",
+                ],
+            ),
+            Resolved(
+                "line 5\n",
+            ),
+        ],
+    )
+    "###
+    );
+    insta::assert_debug_snapshot!(
+        parse_conflict(indoc! {b"
+            line 1
+            <<<<<<<<<<< Text
+            %%%%%%%%%%% Different text 
+             line 2
+            -line 3
+            +left
+             line 4
+            +++++++++++ Yet <><>< more text
+            right
+            >>>>>>>>>>> More and more text
+            line 5
+            "},
             2
         ),
         @r###"
@@ -535,23 +587,24 @@ line 5
 fn test_parse_conflict_multi_way() {
     insta::assert_debug_snapshot!(
         parse_conflict(
-            b"line 1
-<<<<<<<
-%%%%%%%
- line 2
--line 3
-+left
- line 4
-+++++++
-right
-%%%%%%%
- line 2
-+forward
- line 3
- line 4
->>>>>>>
-line 5
-",
+            indoc! {b"
+                line 1
+                <<<<<<<
+                %%%%%%%
+                 line 2
+                -line 3
+                +left
+                 line 4
+                +++++++
+                right
+                %%%%%%%
+                 line 2
+                +forward
+                 line 3
+                 line 4
+                >>>>>>>
+                line 5
+                "},
             3
         ),
         @r###"
@@ -575,25 +628,69 @@ line 5
         ],
     )
     "###
+    );
+    insta::assert_debug_snapshot!(
+        parse_conflict(indoc! {b"
+            line 1
+            <<<<<<< Random text
+            %%%%%%% Random text
+             line 2
+            -line 3
+            +left
+             line 4
+            +++++++ Random text
+            right
+            %%%%%%% Random text
+             line 2
+            +forward
+             line 3
+             line 4
+            >>>>>>> Random text
+            line 5
+            "},
+            3
+        ),
+        @r###"
+    Some(
+        [
+            Resolved(
+                "line 1\n",
+            ),
+            Conflicted(
+                [
+                    "line 2\nleft\nline 4\n",
+                    "line 2\nline 3\nline 4\n",
+                    "right\n",
+                    "line 2\nline 3\nline 4\n",
+                    "line 2\nforward\nline 3\nline 4\n",
+                ],
+            ),
+            Resolved(
+                "line 5\n",
+            ),
+        ],
     )
+    "###
+    );
 }
 
 #[test]
 fn test_parse_conflict_different_wrong_arity() {
     assert_eq!(
         parse_conflict(
-            b"line 1
-<<<<<<<
-%%%%%%%
- line 2
--line 3
-+left
- line 4
-+++++++
-right
->>>>>>>
-line 5
-",
+            indoc! {b"
+            line 1
+            <<<<<<<
+            %%%%%%%
+             line 2
+            -line 3
+            +left
+             line 4
+            +++++++
+            right
+            >>>>>>>
+            line 5
+            "},
             3
         ),
         None
@@ -605,17 +702,18 @@ fn test_parse_conflict_malformed_marker() {
     // The conflict marker is missing `%%%%%%%`
     assert_eq!(
         parse_conflict(
-            b"line 1
-<<<<<<<
- line 2
--line 3
-+left
- line 4
-+++++++
-right
->>>>>>>
-line 5
-",
+            indoc! {b"
+            line 1
+            <<<<<<<
+             line 2
+            -line 3
+            +left
+             line 4
+            +++++++
+            right
+            >>>>>>>
+            line 5
+            "},
             2
         ),
         None
@@ -627,18 +725,19 @@ fn test_parse_conflict_malformed_diff() {
     // The diff part is invalid (missing space before "line 4")
     assert_eq!(
         parse_conflict(
-            b"line 1
-<<<<<<<
-%%%%%%%
- line 2
--line 3
-+left
-line 4
-+++++++
-right
->>>>>>>
-line 5
-",
+            indoc! {b"
+            line 1
+            <<<<<<<
+            %%%%%%%
+             line 2
+            -line 3
+            +left
+            line 4
+            +++++++
+            right
+            >>>>>>>
+            line 5
+            "},
             2
         ),
         None
