@@ -13,12 +13,13 @@
 // limitations under the License.
 
 use itertools::Itertools;
+use jj_lib::backend::CommitId;
 use jj_lib::commit::Commit;
 use jj_lib::default_index::revset_engine::{evaluate, RevsetImpl};
 use jj_lib::default_index::DefaultReadonlyIndex;
+use jj_lib::graph::GraphEdge;
 use jj_lib::repo::{ReadonlyRepo, Repo as _};
 use jj_lib::revset::ResolvedExpression;
-use jj_lib::revset_graph::RevsetGraphEdge;
 use test_case::test_case;
 use testutils::{CommitGraphBuilder, TestRepo};
 
@@ -36,16 +37,16 @@ fn revset_for_commits(
     evaluate(&expression, repo.store(), index.clone()).unwrap()
 }
 
-fn direct(commit: &Commit) -> RevsetGraphEdge {
-    RevsetGraphEdge::direct(commit.id().clone())
+fn direct(commit: &Commit) -> GraphEdge<CommitId> {
+    GraphEdge::direct(commit.id().clone())
 }
 
-fn indirect(commit: &Commit) -> RevsetGraphEdge {
-    RevsetGraphEdge::indirect(commit.id().clone())
+fn indirect(commit: &Commit) -> GraphEdge<CommitId> {
+    GraphEdge::indirect(commit.id().clone())
 }
 
-fn missing(commit: &Commit) -> RevsetGraphEdge {
-    RevsetGraphEdge::missing(commit.id().clone())
+fn missing(commit: &Commit) -> GraphEdge<CommitId> {
+    GraphEdge::missing(commit.id().clone())
 }
 
 #[test_case(false ; "keep transitive edges")]
@@ -73,10 +74,7 @@ fn test_graph_iterator_linearized(skip_transitive_edges: bool) {
     let root_commit = repo.store().root_commit();
 
     let revset = revset_for_commits(repo.as_ref(), &[&commit_a, &commit_d]);
-    let commits = revset
-        .iter_graph_impl()
-        .set_skip_transitive_edges(skip_transitive_edges)
-        .collect_vec();
+    let commits = revset.iter_graph_impl(skip_transitive_edges).collect_vec();
     assert_eq!(commits.len(), 2);
     assert_eq!(commits[0].0, *commit_d.id());
     assert_eq!(commits[1].0, *commit_a.id());
@@ -113,10 +111,7 @@ fn test_graph_iterator_virtual_octopus(skip_transitive_edges: bool) {
     let root_commit = repo.store().root_commit();
 
     let revset = revset_for_commits(repo.as_ref(), &[&commit_a, &commit_b, &commit_c, &commit_f]);
-    let commits = revset
-        .iter_graph_impl()
-        .set_skip_transitive_edges(skip_transitive_edges)
-        .collect_vec();
+    let commits = revset.iter_graph_impl(skip_transitive_edges).collect_vec();
     assert_eq!(commits.len(), 4);
     assert_eq!(commits[0].0, *commit_f.id());
     assert_eq!(commits[1].0, *commit_c.id());
@@ -164,10 +159,7 @@ fn test_graph_iterator_simple_fork(skip_transitive_edges: bool) {
     let root_commit = repo.store().root_commit();
 
     let revset = revset_for_commits(repo.as_ref(), &[&commit_a, &commit_c, &commit_e]);
-    let commits = revset
-        .iter_graph_impl()
-        .set_skip_transitive_edges(skip_transitive_edges)
-        .collect_vec();
+    let commits = revset.iter_graph_impl(skip_transitive_edges).collect_vec();
     assert_eq!(commits.len(), 3);
     assert_eq!(commits[0].0, *commit_e.id());
     assert_eq!(commits[1].0, *commit_c.id());
@@ -205,10 +197,7 @@ fn test_graph_iterator_multiple_missing(skip_transitive_edges: bool) {
     let root_commit = repo.store().root_commit();
 
     let revset = revset_for_commits(repo.as_ref(), &[&commit_b, &commit_f]);
-    let commits = revset
-        .iter_graph_impl()
-        .set_skip_transitive_edges(skip_transitive_edges)
-        .collect_vec();
+    let commits = revset.iter_graph_impl(skip_transitive_edges).collect_vec();
     assert_eq!(commits.len(), 2);
     assert_eq!(commits[0].0, *commit_f.id());
     assert_eq!(commits[1].0, *commit_b.id());
@@ -249,10 +238,7 @@ fn test_graph_iterator_edge_to_ancestor(skip_transitive_edges: bool) {
     let repo = tx.commit("test");
 
     let revset = revset_for_commits(repo.as_ref(), &[&commit_c, &commit_d, &commit_f]);
-    let commits = revset
-        .iter_graph_impl()
-        .set_skip_transitive_edges(skip_transitive_edges)
-        .collect_vec();
+    let commits = revset.iter_graph_impl(skip_transitive_edges).collect_vec();
     assert_eq!(commits.len(), 3);
     assert_eq!(commits[0].0, *commit_f.id());
     assert_eq!(commits[1].0, *commit_d.id());
@@ -308,10 +294,7 @@ fn test_graph_iterator_edge_escapes_from_(skip_transitive_edges: bool) {
         repo.as_ref(),
         &[&commit_a, &commit_d, &commit_g, &commit_h, &commit_j],
     );
-    let commits = revset
-        .iter_graph_impl()
-        .set_skip_transitive_edges(skip_transitive_edges)
-        .collect_vec();
+    let commits = revset.iter_graph_impl(skip_transitive_edges).collect_vec();
     assert_eq!(commits.len(), 5);
     assert_eq!(commits[0].0, *commit_j.id());
     assert_eq!(commits[1].0, *commit_h.id());
