@@ -29,12 +29,13 @@ use jj_lib::merged_tree::{MergedTree, MergedTreeBuilder};
 use jj_lib::op_store::{OperationId, WorkspaceId};
 use jj_lib::repo::{ReadonlyRepo, Repo};
 use jj_lib::repo_path::{RepoPath, RepoPathBuf, RepoPathComponent};
+use jj_lib::secret_backend::SecretBackend;
 use jj_lib::settings::UserSettings;
 use jj_lib::working_copy::{CheckoutStats, SnapshotError, SnapshotOptions};
-use jj_lib::workspace::LockedWorkspace;
+use jj_lib::workspace::{default_working_copy_factories, LockedWorkspace, Workspace};
 use test_case::test_case;
 use testutils::{
-    commit_with_tree, create_tree, write_random_commit, TestRepoBackend, TestWorkspace,
+    commit_with_tree, create_tree, write_random_commit, TestRepo, TestRepoBackend, TestWorkspace,
 };
 
 fn to_owned_path_vec(paths: &[&RepoPath]) -> Vec<RepoPathBuf> {
@@ -87,6 +88,8 @@ fn test_checkout_file_transitions(backend: TestRepoBackend) {
         // Executable, but same content as Normal, to test transition where only the bit changed
         ExecutableNormalContent,
         Conflict,
+        // Same content as Executable, to test that transition preserves the executable bit
+        ConflictedExecutableContent,
         Symlink,
         Tree,
         GitSubmodule,
@@ -110,7 +113,8 @@ fn test_checkout_file_transitions(backend: TestRepoBackend) {
                 })
             }
             Kind::Executable => {
-                let id = testutils::write_file(store, path, "executable file contents");
+                let id: jj_lib::backend::FileId =
+                    testutils::write_file(store, path, "executable file contents");
                 Merge::normal(TreeValue::File {
                     id,
                     executable: true,
@@ -144,6 +148,29 @@ fn test_checkout_file_transitions(backend: TestRepoBackend) {
                     ],
                 )
             }
+            Kind::ConflictedExecutableContent => {
+                let base_file_id = testutils::write_file(store, path, "executable file contents");
+                let left_file_id =
+                    testutils::write_file(store, path, "left executable file contents");
+                let right_file_id =
+                    testutils::write_file(store, path, "right executable file contents");
+                Merge::from_removes_adds(
+                    vec![Some(TreeValue::File {
+                        id: base_file_id,
+                        executable: true,
+                    })],
+                    vec![
+                        Some(TreeValue::File {
+                            id: left_file_id,
+                            executable: true,
+                        }),
+                        Some(TreeValue::File {
+                            id: right_file_id,
+                            executable: true,
+                        }),
+                    ],
+                )
+            }
             Kind::Symlink => {
                 let id = store.write_symlink(path, "target").unwrap();
                 Merge::normal(TreeValue::Symlink(id))
@@ -174,6 +201,7 @@ fn test_checkout_file_transitions(backend: TestRepoBackend) {
         Kind::Executable,
         Kind::ExecutableNormalContent,
         Kind::Conflict,
+        Kind::ConflictedExecutableContent,
         Kind::Tree,
     ];
     kinds.push(Kind::Symlink);
@@ -246,6 +274,17 @@ fn test_checkout_file_transitions(backend: TestRepoBackend) {
                     "{path:?} should not be executable"
                 );
             }
+            Kind::ConflictedExecutableContent => {
+                assert!(maybe_metadata.is_ok(), "{path:?} should exist");
+                let metadata = maybe_metadata.unwrap();
+                assert!(metadata.is_file(), "{path:?} should be a file");
+                #[cfg(unix)]
+                assert_ne!(
+                    metadata.permissions().mode() & 0o111,
+                    0,
+                    "{path:?} should be executable"
+                );
+            }
             Kind::Symlink => {
                 assert!(maybe_metadata.is_ok(), "{path:?} should exist");
                 let metadata = maybe_metadata.unwrap();
@@ -290,6 +329,61 @@ fn test_conflict_subdirectory() {
         .unwrap();
 }
 
+#[test]
+fn test_acl() {
+    let settings = testutils::user_settings();
+    let test_workspace = TestWorkspace::init_with_backend(&settings, TestRepoBackend::Git);
+    let repo = &test_workspace.repo;
+    let workspace_root = test_workspace.workspace.workspace_root().to_owned();
+
+    let secret_modified_path = RepoPath::from_internal_string("secret/modified");
+    let secret_added_path = RepoPath::from_internal_string("secret/added");
+    let secret_deleted_path = RepoPath::from_internal_string("secret/deleted");
+    let became_secret_path = RepoPath::from_internal_string("file1");
+    let became_public_path = RepoPath::from_internal_string("file2");
+    let tree1 = create_tree(
+        repo,
+        &[
+            (secret_modified_path, "0"),
+            (secret_deleted_path, "0"),
+            (became_secret_path, "public"),
+            (became_public_path, "secret"),
+        ],
+    );
+    let tree2 = create_tree(
+        repo,
+        &[
+            (secret_modified_path, "1"),
+            (secret_added_path, "1"),
+            (became_secret_path, "secret"),
+            (became_public_path, "public"),
+        ],
+    );
+    let commit1 = commit_with_tree(repo.store(), tree1.id());
+    let commit2 = commit_with_tree(repo.store(), tree2.id());
+    SecretBackend::adopt_git_repo(&workspace_root);
+
+    let mut ws = Workspace::load(
+        &settings,
+        &workspace_root,
+        &TestRepo::default_store_factories(),
+        &default_working_copy_factories(),
+    )
+    .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit1).unwrap();
+    assert!(!secret_modified_path.to_fs_path(&workspace_root).is_file());
+    assert!(!secret_added_path.to_fs_path(&workspace_root).is_file());
+    assert!(!secret_deleted_path.to_fs_path(&workspace_root).is_file());
+    assert!(became_secret_path.to_fs_path(&workspace_root).is_file());
+    assert!(!became_public_path.to_fs_path(&workspace_root).is_file());
+    ws.check_out(repo.op_id().clone(), None, &commit2).unwrap();
+    assert!(!secret_modified_path.to_fs_path(&workspace_root).is_file());
+    assert!(!secret_added_path.to_fs_path(&workspace_root).is_file());
+    assert!(!secret_deleted_path.to_fs_path(&workspace_root).is_file());
+    assert!(!became_secret_path.to_fs_path(&workspace_root).is_file());
+    assert!(became_public_path.to_fs_path(&workspace_root).is_file());
+}
+
 #[test]
 fn test_tree_builder_file_directory_transition() {
     let settings = testutils::user_settings();
@@ -310,7 +404,7 @@ fn test_tree_builder_file_directory_transition() {
     // Add file at parent_path
     let mut tree_builder = store.tree_builder(store.empty_tree_id().clone());
     testutils::write_normal_file(&mut tree_builder, parent_path, "");
-    let tree_id = tree_builder.write_tree();
+    let tree_id = tree_builder.write_tree().unwrap();
     check_out_tree(&tree_id);
     assert!(parent_path.to_fs_path(&workspace_root).is_file());
     assert!(!child_path.to_fs_path(&workspace_root).exists());
@@ -319,7 +413,7 @@ fn test_tree_builder_file_directory_transition() {
     let mut tree_builder = store.tree_builder(tree_id);
     tree_builder.remove(parent_path.to_owned());
     testutils::write_normal_file(&mut tree_builder, child_path, "");
-    let tree_id = tree_builder.write_tree();
+    let tree_id = tree_builder.write_tree().unwrap();
     check_out_tree(&tree_id);
     assert!(parent_path.to_fs_path(&workspace_root).is_dir());
     assert!(child_path.to_fs_path(&workspace_root).is_file());
@@ -328,7 +422,7 @@ fn test_tree_builder_file_directory_transition() {
     let mut tree_builder = store.tree_builder(tree_id);
     tree_builder.remove(child_path.to_owned());
     testutils::write_normal_file(&mut tree_builder, parent_path, "");
-    let tree_id = tree_builder.write_tree();
+    let tree_id = tree_builder.write_tree().unwrap();
     check_out_tree(&tree_id);
     assert!(parent_path.to_fs_path(&workspace_root).is_file());
     assert!(!child_path.to_fs_path(&workspace_root).exists());
@@ -761,7 +855,7 @@ fn test_gitignores_ignored_directory_already_tracked() {
                 }
             }
         }
-        let id = tree_builder.write_tree();
+        let id = tree_builder.write_tree().unwrap();
         MergedTree::legacy(store.get_tree(RepoPath::root(), &id).unwrap())
     };
 
@@ -892,7 +986,7 @@ fn test_gitsubmodule() {
         TreeValue::GitSubmodule(submodule_id),
     );
 
-    let tree_id = MergedTreeId::Legacy(tree_builder.write_tree());
+    let tree_id = MergedTreeId::Legacy(tree_builder.write_tree().unwrap());
     let tree = store.get_root_tree(&tree_id).unwrap();
     let commit = commit_with_tree(repo.store(), tree.id());
     let ws = &mut test_workspace.workspace;
