@@ -142,7 +142,7 @@ fn test_from_legacy_tree() {
     let dir1_filename_id = write_file(store.as_ref(), dir1_filename, "file5_v2");
     tree_builder.set(dir1_filename.to_owned(), file_value(&dir1_filename_id));
 
-    let tree_id = tree_builder.write_tree();
+    let tree_id = tree_builder.write_tree().unwrap();
     let tree = store.get_tree(RepoPath::root(), &tree_id).unwrap();
 
     let merged_tree = MergedTree::from_legacy_tree(tree.clone()).unwrap();
@@ -211,15 +211,6 @@ fn test_from_legacy_tree() {
         MergedTreeVal::Resolved(tree.value(dir1_basename))
     );
 
-    // Also test that MergedTreeBuilder can create the same tree by starting from an
-    // empty legacy tree.
-    let mut tree_builder = MergedTreeBuilder::new(store.empty_merged_tree_id());
-    for (path, value) in tree.entries() {
-        tree_builder.set_or_remove(path, Merge::normal(value));
-    }
-    let recreated_legacy_id = tree_builder.write_tree(store).unwrap();
-    assert_eq!(recreated_legacy_id, MergedTreeId::Legacy(tree_id.clone()));
-
     // Create the merged tree by starting from an empty merged tree and adding
     // entries from the merged tree we created before
     let empty_merged_id_builder: MergeBuilder<_> = std::iter::repeat(store.empty_tree_id())
@@ -301,7 +292,7 @@ fn test_path_value_and_entries() {
 
     // Get the root tree
     assert_eq!(
-        merged_tree.path_value(RepoPath::root()),
+        merged_tree.path_value(RepoPath::root()).unwrap(),
         Merge::from_removes_adds(
             vec![Some(TreeValue::Tree(tree1.id().clone()))],
             vec![
@@ -312,40 +303,43 @@ fn test_path_value_and_entries() {
     );
     // Get file path without conflict
     assert_eq!(
-        merged_tree.path_value(resolved_file_path),
-        Merge::resolved(tree1.path_value(resolved_file_path)),
+        merged_tree.path_value(resolved_file_path).unwrap(),
+        Merge::resolved(tree1.path_value(resolved_file_path).unwrap()),
     );
     // Get directory path without conflict
     assert_eq!(
-        merged_tree.path_value(resolved_dir_path),
-        Merge::resolved(tree1.path_value(resolved_dir_path)),
+        merged_tree.path_value(resolved_dir_path).unwrap(),
+        Merge::resolved(tree1.path_value(resolved_dir_path).unwrap()),
     );
     // Get missing path
-    assert_eq!(merged_tree.path_value(missing_path), Merge::absent());
+    assert_eq!(
+        merged_tree.path_value(missing_path).unwrap(),
+        Merge::absent()
+    );
     // Get modify/delete conflict (some None values)
     assert_eq!(
-        merged_tree.path_value(modify_delete_path),
+        merged_tree.path_value(modify_delete_path).unwrap(),
         Merge::from_removes_adds(
-            vec![tree1.path_value(modify_delete_path)],
-            vec![tree2.path_value(modify_delete_path), None]
+            vec![tree1.path_value(modify_delete_path).unwrap()],
+            vec![tree2.path_value(modify_delete_path).unwrap(), None]
         ),
     );
     // Get file/dir conflict path
     assert_eq!(
-        merged_tree.path_value(file_dir_conflict_path),
+        merged_tree.path_value(file_dir_conflict_path).unwrap(),
         Merge::from_removes_adds(
-            vec![tree1.path_value(file_dir_conflict_path)],
+            vec![tree1.path_value(file_dir_conflict_path).unwrap()],
             vec![
-                tree2.path_value(file_dir_conflict_path),
-                tree3.path_value(file_dir_conflict_path)
+                tree2.path_value(file_dir_conflict_path).unwrap(),
+                tree3.path_value(file_dir_conflict_path).unwrap()
             ]
         ),
     );
     // Get file inside file/dir conflict
     // There is a conflict in the parent directory, but this file is still resolved
     assert_eq!(
-        merged_tree.path_value(file_dir_conflict_sub_path),
-        Merge::resolved(tree3.path_value(file_dir_conflict_sub_path)),
+        merged_tree.path_value(file_dir_conflict_sub_path).unwrap(),
+        Merge::resolved(tree3.path_value(file_dir_conflict_sub_path).unwrap()),
     );
 
     // Test entries()
@@ -363,7 +357,7 @@ fn test_path_value_and_entries() {
     ]
     .iter()
     .sorted()
-    .map(|&path| (path.to_owned(), merged_tree.path_value(path)))
+    .map(|&path| (path.to_owned(), merged_tree.path_value(path).unwrap()))
     .collect_vec();
     assert_eq!(actual_entries, expected_entries);
 
@@ -378,7 +372,7 @@ fn test_path_value_and_entries() {
     let expected_entries = [resolved_file_path, modify_delete_path]
         .iter()
         .sorted()
-        .map(|&path| (path.to_owned(), merged_tree.path_value(path)))
+        .map(|&path| (path.to_owned(), merged_tree.path_value(path).unwrap()))
         .collect_vec();
     assert_eq!(actual_entries, expected_entries);
 }
@@ -592,8 +586,11 @@ fn test_conflict_iterator() {
     let conflicts = tree.conflicts().collect_vec();
     let conflict_at = |path: &RepoPath| {
         Merge::from_removes_adds(
-            vec![base1.path_value(path)],
-            vec![side1.path_value(path), side2.path_value(path)],
+            vec![base1.path_value(path).unwrap()],
+            vec![
+                side1.path_value(path).unwrap(),
+                side2.path_value(path).unwrap(),
+            ],
         )
     };
     // We initially also get a conflict in trivial_hunk_path because we had
@@ -681,11 +678,14 @@ fn test_conflict_iterator_higher_arity() {
     let conflicts = tree.conflicts().collect_vec();
     let conflict_at = |path: &RepoPath| {
         Merge::from_removes_adds(
-            vec![base1.path_value(path), base2.path_value(path)],
             vec![
-                side1.path_value(path),
-                side2.path_value(path),
-                side3.path_value(path),
+                base1.path_value(path).unwrap(),
+                base2.path_value(path).unwrap(),
+            ],
+            vec![
+                side1.path_value(path).unwrap(),
+                side2.path_value(path).unwrap(),
+                side3.path_value(path).unwrap(),
             ],
         )
     };
@@ -710,10 +710,10 @@ fn test_conflict_iterator_higher_arity() {
             (
                 two_sided_path.to_owned(),
                 Merge::from_removes_adds(
-                    vec![base2.path_value(two_sided_path)],
+                    vec![base2.path_value(two_sided_path).unwrap()],
                     vec![
-                        side1.path_value(two_sided_path),
-                        side3.path_value(two_sided_path),
+                        side1.path_value(two_sided_path).unwrap(),
+                        side3.path_value(two_sided_path).unwrap(),
                     ],
                 )
             ),
@@ -761,8 +761,8 @@ fn test_diff_resolved() {
         (
             modified_path.to_owned(),
             (
-                Merge::resolved(before.path_value(modified_path)),
-                Merge::resolved(after.path_value(modified_path))
+                Merge::resolved(before.path_value(modified_path).unwrap()),
+                Merge::resolved(after.path_value(modified_path).unwrap())
             ),
         )
     );
@@ -771,7 +771,7 @@ fn test_diff_resolved() {
         (
             removed_path.to_owned(),
             (
-                Merge::resolved(before.path_value(removed_path)),
+                Merge::resolved(before.path_value(removed_path).unwrap()),
                 Merge::absent()
             ),
         )
@@ -782,7 +782,7 @@ fn test_diff_resolved() {
             added_path.to_owned(),
             (
                 Merge::absent(),
-                Merge::resolved(after.path_value(added_path))
+                Merge::resolved(after.path_value(added_path).unwrap())
             ),
         )
     );
@@ -869,7 +869,10 @@ fn test_diff_conflicted() {
         .map(|&path| {
             (
                 path.to_owned(),
-                (left_merged.path_value(path), right_merged.path_value(path)),
+                (
+                    left_merged.path_value(path).unwrap(),
+                    right_merged.path_value(path).unwrap(),
+                ),
             )
         })
         .collect_vec();
@@ -885,7 +888,10 @@ fn test_diff_conflicted() {
         .map(|&path| {
             (
                 path.to_owned(),
-                (right_merged.path_value(path), left_merged.path_value(path)),
+                (
+                    right_merged.path_value(path).unwrap(),
+                    left_merged.path_value(path).unwrap(),
+                ),
             )
         })
         .collect_vec();
@@ -985,6 +991,8 @@ fn test_diff_dir_file() {
         vec![right_base],
         vec![right_side1, right_side2],
     ));
+    let left_value = |path: &RepoPath| left_merged.path_value(path).unwrap();
+    let right_value = |path: &RepoPath| right_merged.path_value(path).unwrap();
 
     // Test the forwards diff
     {
@@ -994,58 +1002,37 @@ fn test_diff_dir_file() {
             .collect_vec();
         let expected_diff = vec![
             // path1: file1 -> directory1
-            (
-                path1.to_owned(),
-                (left_merged.path_value(path1), Merge::absent()),
-            ),
+            (path1.to_owned(), (left_value(path1), Merge::absent())),
             (
                 path1.join(file),
-                (Merge::absent(), right_merged.path_value(&path1.join(file))),
+                (Merge::absent(), right_value(&path1.join(file))),
             ),
             // path2: file1 -> directory1+(directory2-absent)
-            (
-                path2.to_owned(),
-                (left_merged.path_value(path2), Merge::absent()),
-            ),
+            (path2.to_owned(), (left_value(path2), Merge::absent())),
             (
                 path2.join(file),
-                (Merge::absent(), right_merged.path_value(&path2.join(file))),
+                (Merge::absent(), right_value(&path2.join(file))),
             ),
             // path3: file1 -> directory1+(file1-absent)
-            (
-                path3.to_owned(),
-                (
-                    left_merged.path_value(path3),
-                    right_merged.path_value(path3),
-                ),
-            ),
+            (path3.to_owned(), (left_value(path3), right_value(path3))),
             // path4: file1+(file2-file3) -> directory1+(directory2-directory3)
-            (
-                path4.to_owned(),
-                (left_merged.path_value(path4), Merge::absent()),
-            ),
+            (path4.to_owned(), (left_value(path4), Merge::absent())),
             (
                 path4.join(file),
-                (Merge::absent(), right_merged.path_value(&path4.join(file))),
+                (Merge::absent(), right_value(&path4.join(file))),
             ),
             // path5: directory1 -> file1+(file2-absent)
             (
                 path5.join(file),
-                (left_merged.path_value(&path5.join(file)), Merge::absent()),
-            ),
-            (
-                path5.to_owned(),
-                (Merge::absent(), right_merged.path_value(path5)),
+                (left_value(&path5.join(file)), Merge::absent()),
             ),
+            (path5.to_owned(), (Merge::absent(), right_value(path5))),
             // path6: directory1 -> file1+(directory1-absent)
             (
                 path6.join(file),
-                (left_merged.path_value(&path6.join(file)), Merge::absent()),
-            ),
-            (
-                path6.to_owned(),
-                (Merge::absent(), right_merged.path_value(path6)),
+                (left_value(&path6.join(file)), Merge::absent()),
             ),
+            (path6.to_owned(), (Merge::absent(), right_value(path6))),
         ];
         assert_eq!(actual_diff, expected_diff);
         diff_stream_equals_iter(&left_merged, &right_merged, &EverythingMatcher);
@@ -1061,55 +1048,34 @@ fn test_diff_dir_file() {
             // path1: file1 -> directory1
             (
                 path1.join(file),
-                (right_merged.path_value(&path1.join(file)), Merge::absent()),
-            ),
-            (
-                path1.to_owned(),
-                (Merge::absent(), left_merged.path_value(path1)),
+                (right_value(&path1.join(file)), Merge::absent()),
             ),
+            (path1.to_owned(), (Merge::absent(), left_value(path1))),
             // path2: file1 -> directory1+(directory2-absent)
             (
                 path2.join(file),
-                (right_merged.path_value(&path2.join(file)), Merge::absent()),
-            ),
-            (
-                path2.to_owned(),
-                (Merge::absent(), left_merged.path_value(path2)),
+                (right_value(&path2.join(file)), Merge::absent()),
             ),
+            (path2.to_owned(), (Merge::absent(), left_value(path2))),
             // path3: file1 -> directory1+(file1-absent)
-            (
-                path3.to_owned(),
-                (
-                    right_merged.path_value(path3),
-                    left_merged.path_value(path3),
-                ),
-            ),
+            (path3.to_owned(), (right_value(path3), left_value(path3))),
             // path4: file1+(file2-file3) -> directory1+(directory2-directory3)
             (
                 path4.join(file),
-                (right_merged.path_value(&path4.join(file)), Merge::absent()),
-            ),
-            (
-                path4.to_owned(),
-                (Merge::absent(), left_merged.path_value(path4)),
+                (right_value(&path4.join(file)), Merge::absent()),
             ),
+            (path4.to_owned(), (Merge::absent(), left_value(path4))),
             // path5: directory1 -> file1+(file2-absent)
-            (
-                path5.to_owned(),
-                (right_merged.path_value(path5), Merge::absent()),
-            ),
+            (path5.to_owned(), (right_value(path5), Merge::absent())),
             (
                 path5.join(file),
-                (Merge::absent(), left_merged.path_value(&path5.join(file))),
+                (Merge::absent(), left_value(&path5.join(file))),
             ),
             // path6: directory1 -> file1+(directory1-absent)
-            (
-                path6.to_owned(),
-                (right_merged.path_value(path6), Merge::absent()),
-            ),
+            (path6.to_owned(), (right_value(path6), Merge::absent())),
             (
                 path6.join(file),
-                (Merge::absent(), left_merged.path_value(&path6.join(file))),
+                (Merge::absent(), left_value(&path6.join(file))),
             ),
         ];
         assert_eq!(actual_diff, expected_diff);
@@ -1125,10 +1091,7 @@ fn test_diff_dir_file() {
             .collect_vec();
         let expected_diff = vec![
             // path1: file1 -> directory1
-            (
-                path1.to_owned(),
-                (left_merged.path_value(path1), Merge::absent()),
-            ),
+            (path1.to_owned(), (left_value(path1), Merge::absent())),
         ];
         assert_eq!(actual_diff, expected_diff);
         diff_stream_equals_iter(&left_merged, &right_merged, &matcher);
@@ -1145,7 +1108,7 @@ fn test_diff_dir_file() {
             // path1: file1 -> directory1
             (
                 path1.join(file),
-                (Merge::absent(), right_merged.path_value(&path1.join(file))),
+                (Merge::absent(), right_value(&path1.join(file))),
             ),
         ];
         assert_eq!(actual_diff, expected_diff);
@@ -1160,13 +1123,10 @@ fn test_diff_dir_file() {
             .map(|(path, diff)| (path, diff.unwrap()))
             .collect_vec();
         let expected_diff = vec![
-            (
-                path1.to_owned(),
-                (left_merged.path_value(path1), Merge::absent()),
-            ),
+            (path1.to_owned(), (left_value(path1), Merge::absent())),
             (
                 path1.join(file),
-                (Merge::absent(), right_merged.path_value(&path1.join(file))),
+                (Merge::absent(), right_value(&path1.join(file))),
             ),
         ];
         assert_eq!(actual_diff, expected_diff);
@@ -1183,10 +1143,7 @@ fn test_diff_dir_file() {
             .diff(&right_merged, &matcher)
             .map(|(path, diff)| (path, diff.unwrap()))
             .collect_vec();
-        let expected_diff = vec![(
-            path6.to_owned(),
-            (Merge::absent(), right_merged.path_value(path6)),
-        )];
+        let expected_diff = vec![(path6.to_owned(), (Merge::absent(), right_value(path6)))];
         assert_eq!(actual_diff, expected_diff);
         diff_stream_equals_iter(&left_merged, &right_merged, &matcher);
     }
