@@ -12,29 +12,25 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-// this was supposed to be fixed in 1.71.0, but barely missed the cut.
-// can be released after we bump MSRV to 1.72.0, see:
-// https://github.com/frondeus/test-case/issues/126#issuecomment-1635916592
-#![allow(clippy::items_after_test_module)]
-
 use std::path::Path;
 
 use assert_matches::assert_matches;
 use itertools::Itertools;
 use jj_lib::backend::{CommitId, MillisSinceEpoch, Signature, Timestamp};
 use jj_lib::commit::Commit;
+use jj_lib::fileset::FilesetExpression;
 use jj_lib::git;
 use jj_lib::git_backend::GitBackend;
+use jj_lib::graph::{GraphEdge, ReverseGraphIterator};
 use jj_lib::object_id::ObjectId;
 use jj_lib::op_store::{RefTarget, RemoteRef, RemoteRefState, WorkspaceId};
 use jj_lib::repo::Repo;
-use jj_lib::repo_path::RepoPath;
+use jj_lib::repo_path::{RepoPath, RepoPathUiConverter};
 use jj_lib::revset::{
     optimize, parse, DefaultSymbolResolver, FailingSymbolResolver, ResolvedExpression, Revset,
-    RevsetAliasesMap, RevsetExpression, RevsetFilterPredicate, RevsetParseContext,
-    RevsetResolutionError, RevsetWorkspaceContext,
+    RevsetAliasesMap, RevsetExpression, RevsetExtensions, RevsetFilterPredicate,
+    RevsetParseContext, RevsetResolutionError, RevsetWorkspaceContext, SymbolResolverExtension,
 };
-use jj_lib::revset_graph::{ReverseRevsetGraphIterator, RevsetGraphEdge};
 use jj_lib::settings::GitSettings;
 use jj_lib::workspace::Workspace;
 use test_case::test_case;
@@ -43,26 +39,32 @@ use testutils::{
     TestRepoBackend, TestWorkspace,
 };
 
-fn resolve_symbol(repo: &dyn Repo, symbol: &str) -> Result<Vec<CommitId>, RevsetResolutionError> {
-    let context = RevsetParseContext {
-        aliases_map: &RevsetAliasesMap::new(),
-        user_email: String::new(),
-        workspace: None,
-    };
+fn resolve_symbol_with_extensions(
+    repo: &dyn Repo,
+    extensions: &RevsetExtensions,
+    symbol: &str,
+) -> Result<Vec<CommitId>, RevsetResolutionError> {
+    let aliases_map = RevsetAliasesMap::default();
+    let context = RevsetParseContext::new(&aliases_map, String::new(), extensions, None);
     let expression = parse(symbol, &context).unwrap();
     assert_matches!(*expression, RevsetExpression::CommitRef(_));
-    let symbol_resolver = DefaultSymbolResolver::new(repo);
+    let symbol_resolver = DefaultSymbolResolver::new(repo, extensions.symbol_resolvers());
     match expression.resolve_user_expression(repo, &symbol_resolver)? {
         ResolvedExpression::Commits(commits) => Ok(commits),
         expression => panic!("symbol resolved to compound expression: {expression:?}"),
     }
 }
 
+fn resolve_symbol(repo: &dyn Repo, symbol: &str) -> Result<Vec<CommitId>, RevsetResolutionError> {
+    resolve_symbol_with_extensions(repo, &RevsetExtensions::default(), symbol)
+}
+
 fn revset_for_commits<'index>(
     repo: &'index dyn Repo,
     commits: &[&Commit],
 ) -> Box<dyn Revset + 'index> {
-    let symbol_resolver = DefaultSymbolResolver::new(repo);
+    let symbol_resolver =
+        DefaultSymbolResolver::new(repo, &([] as [&Box<dyn SymbolResolverExtension>; 0]));
     RevsetExpression::commits(commits.iter().map(|commit| commit.id().clone()).collect())
         .resolve_user_expression(repo, &symbol_resolver)
         .unwrap()
@@ -171,12 +173,13 @@ fn test_resolve_symbol_commit_id() {
 
     // Test present() suppresses only NoSuchRevision error
     assert_eq!(resolve_commit_ids(repo.as_ref(), "present(foo)"), []);
-    let symbol_resolver = DefaultSymbolResolver::new(repo.as_ref());
-    let context = RevsetParseContext {
-        aliases_map: &RevsetAliasesMap::new(),
-        user_email: settings.user_email(),
-        workspace: None,
-    };
+    let symbol_resolver = DefaultSymbolResolver::new(
+        repo.as_ref(),
+        &([] as [&Box<dyn SymbolResolverExtension>; 0]),
+    );
+    let aliases_map = RevsetAliasesMap::default();
+    let extensions = RevsetExtensions::default();
+    let context = RevsetParseContext::new(&aliases_map, settings.user_email(), &extensions, None);
     assert_matches!(
         optimize(parse("present(04)", &context).unwrap()).resolve_user_expression(repo.as_ref(), &symbol_resolver),
         Err(RevsetResolutionError::AmbiguousCommitIdPrefix(s)) if s == "04"
@@ -376,6 +379,41 @@ fn test_resolve_working_copy() {
     assert_eq!(resolve(ws2), vec![commit2.id().clone()]);
 }
 
+#[test]
+fn test_resolve_working_copies() {
+    let settings = testutils::user_settings();
+    let test_repo = TestRepo::init();
+    let repo = &test_repo.repo;
+
+    let mut tx = repo.start_transaction(&settings);
+    let mut_repo = tx.mut_repo();
+
+    let commit1 = write_random_commit(mut_repo, &settings);
+    let commit2 = write_random_commit(mut_repo, &settings);
+
+    // Add some workspaces
+    let ws1 = WorkspaceId::new("ws1".to_string());
+    let ws2 = WorkspaceId::new("ws2".to_string());
+
+    // add one commit to each working copy
+    mut_repo
+        .set_wc_commit(ws1.clone(), commit1.id().clone())
+        .unwrap();
+    mut_repo
+        .set_wc_commit(ws2.clone(), commit2.id().clone())
+        .unwrap();
+    let resolve = || -> Vec<CommitId> {
+        RevsetExpression::working_copies()
+            .evaluate_programmatic(mut_repo)
+            .unwrap()
+            .iter()
+            .collect()
+    };
+
+    // ensure our output has those two commits
+    assert_eq!(resolve(), vec![commit2.id().clone(), commit1.id().clone()]);
+}
+
 #[test]
 fn test_resolve_symbol_branches() {
     let settings = testutils::user_settings();
@@ -457,6 +495,7 @@ fn test_resolve_symbol_branches() {
         candidates: [
             "local",
             "local-remote@git",
+            "local-remote@mirror",
             "local-remote@origin",
             "remote@origin",
         ],
@@ -469,8 +508,6 @@ fn test_resolve_symbol_branches() {
     NoSuchRevision {
         name: "remote",
         candidates: [
-            "local-remote@origin",
-            "local-remote@untracked",
             "remote-conflicted@origin",
             "remote@origin",
         ],
@@ -795,13 +832,16 @@ fn test_resolve_symbol_git_refs() {
 
 fn resolve_commit_ids(repo: &dyn Repo, revset_str: &str) -> Vec<CommitId> {
     let settings = testutils::user_settings();
-    let context = RevsetParseContext {
-        aliases_map: &RevsetAliasesMap::new(),
-        user_email: settings.user_email(),
-        workspace: None,
-    };
+    let aliases_map = RevsetAliasesMap::default();
+    let revset_extensions = RevsetExtensions::default();
+    let context = RevsetParseContext::new(
+        &aliases_map,
+        settings.user_email(),
+        &revset_extensions,
+        None,
+    );
     let expression = optimize(parse(revset_str, &context).unwrap());
-    let symbol_resolver = DefaultSymbolResolver::new(repo);
+    let symbol_resolver = DefaultSymbolResolver::new(repo, revset_extensions.symbol_resolvers());
     let expression = expression
         .resolve_user_expression(repo, &symbol_resolver)
         .unwrap();
@@ -815,18 +855,25 @@ fn resolve_commit_ids_in_workspace(
     cwd: Option<&Path>,
 ) -> Vec<CommitId> {
     let settings = testutils::user_settings();
+    let path_converter = RepoPathUiConverter::Fs {
+        cwd: cwd.unwrap_or_else(|| workspace.workspace_root()).to_owned(),
+        base: workspace.workspace_root().to_owned(),
+    };
     let workspace_ctx = RevsetWorkspaceContext {
-        cwd: cwd.unwrap_or_else(|| workspace.workspace_root()),
+        path_converter: &path_converter,
         workspace_id: workspace.workspace_id(),
-        workspace_root: workspace.workspace_root(),
-    };
-    let context = RevsetParseContext {
-        aliases_map: &RevsetAliasesMap::new(),
-        user_email: settings.user_email(),
-        workspace: Some(workspace_ctx),
     };
+    let aliases_map = RevsetAliasesMap::default();
+    let extensions = RevsetExtensions::default();
+    let context = RevsetParseContext::new(
+        &aliases_map,
+        settings.user_email(),
+        &extensions,
+        Some(workspace_ctx),
+    );
     let expression = optimize(parse(revset_str, &context).unwrap());
-    let symbol_resolver = DefaultSymbolResolver::new(repo);
+    let symbol_resolver =
+        DefaultSymbolResolver::new(repo, &([] as [&Box<dyn SymbolResolverExtension>; 0]));
     let expression = expression
         .resolve_user_expression(repo, &symbol_resolver)
         .unwrap();
@@ -1166,14 +1213,14 @@ fn test_evaluate_expression_ancestors() {
 
     // The ancestors of the root commit is just the root commit itself
     assert_eq!(
-        resolve_commit_ids(mut_repo, ":root()"),
+        resolve_commit_ids(mut_repo, "::root()"),
         vec![root_commit.id().clone()]
     );
 
     // Can find ancestors of a specific commit. Commits reachable via multiple paths
     // are not repeated.
     assert_eq!(
-        resolve_commit_ids(mut_repo, &format!(":{}", commit4.id().hex())),
+        resolve_commit_ids(mut_repo, &format!("::{}", commit4.id().hex())),
         vec![
             commit4.id().clone(),
             commit3.id().clone(),
@@ -1186,7 +1233,7 @@ fn test_evaluate_expression_ancestors() {
     // Can find ancestors of parents or parents of ancestors, which may be optimized
     // to single query
     assert_eq!(
-        resolve_commit_ids(mut_repo, &format!(":({}-)", commit4.id().hex()),),
+        resolve_commit_ids(mut_repo, &format!("::({}-)", commit4.id().hex()),),
         vec![
             commit3.id().clone(),
             commit2.id().clone(),
@@ -1197,7 +1244,7 @@ fn test_evaluate_expression_ancestors() {
     assert_eq!(
         resolve_commit_ids(
             mut_repo,
-            &format!("(:({}|{}))-", commit3.id().hex(), commit2.id().hex()),
+            &format!("(::({}|{}))-", commit3.id().hex(), commit2.id().hex()),
         ),
         vec![
             commit2.id().clone(),
@@ -1208,7 +1255,7 @@ fn test_evaluate_expression_ancestors() {
     assert_eq!(
         resolve_commit_ids(
             mut_repo,
-            &format!(":(({}|{})-)", commit3.id().hex(), commit2.id().hex()),
+            &format!("::(({}|{})-)", commit3.id().hex(), commit2.id().hex()),
         ),
         vec![
             commit2.id().clone(),
@@ -1285,6 +1332,15 @@ fn test_evaluate_expression_range() {
         ]
     );
 
+    // Range including merge ancestors: commit4-- == root | commit2
+    assert_eq!(
+        resolve_commit_ids(
+            mut_repo,
+            &format!("{}--..{}", commit4.id().hex(), commit3.id().hex())
+        ),
+        vec![commit3.id().clone()]
+    );
+
     // Sibling commits
     assert_eq!(
         resolve_commit_ids(
@@ -1335,7 +1391,7 @@ fn test_evaluate_expression_dag_range() {
 
     // Can get DAG range of just the root commit
     assert_eq!(
-        resolve_commit_ids(mut_repo, "root():root()"),
+        resolve_commit_ids(mut_repo, "root()::root()"),
         vec![root_commit_id.clone()]
     );
 
@@ -1343,7 +1399,7 @@ fn test_evaluate_expression_dag_range() {
     assert_eq!(
         resolve_commit_ids(
             mut_repo,
-            &format!("{}:{}", root_commit_id.hex(), commit2.id().hex())
+            &format!("{}::{}", root_commit_id.hex(), commit2.id().hex())
         ),
         vec![
             commit2.id().clone(),
@@ -1356,14 +1412,14 @@ fn test_evaluate_expression_dag_range() {
     assert_eq!(
         resolve_commit_ids(
             mut_repo,
-            &format!("{}:{}", commit2.id().hex(), commit4.id().hex())
+            &format!("{}::{}", commit2.id().hex(), commit4.id().hex())
         ),
         vec![]
     );
 
     // Empty root
     assert_eq!(
-        resolve_commit_ids(mut_repo, &format!("none():{}", commit5.id().hex())),
+        resolve_commit_ids(mut_repo, &format!("none()::{}", commit5.id().hex())),
         vec![],
     );
 
@@ -1372,7 +1428,7 @@ fn test_evaluate_expression_dag_range() {
         resolve_commit_ids(
             mut_repo,
             &format!(
-                "({}|{}):{}",
+                "({}|{})::{}",
                 commit1.id().hex(),
                 commit2.id().hex(),
                 commit3.id().hex()
@@ -1389,7 +1445,7 @@ fn test_evaluate_expression_dag_range() {
     assert_eq!(
         resolve_commit_ids(
             mut_repo,
-            &format!("{}:{}", commit1.id().hex(), commit5.id().hex())
+            &format!("{}::{}", commit1.id().hex(), commit5.id().hex())
         ),
         vec![
             commit5.id().clone(),
@@ -1404,7 +1460,7 @@ fn test_evaluate_expression_dag_range() {
     assert_eq!(
         resolve_commit_ids(
             mut_repo,
-            &format!("{}:{}", commit2.id().hex(), commit5.id().hex())
+            &format!("{}::{}", commit2.id().hex(), commit5.id().hex())
         ),
         vec![
             commit5.id().clone(),
@@ -1503,6 +1559,164 @@ fn test_evaluate_expression_connected() {
     );
 }
 
+#[test]
+fn test_evaluate_expression_reachable() {
+    let settings = testutils::user_settings();
+    let test_repo = TestRepo::init();
+    let repo = &test_repo.repo;
+
+    let mut tx = repo.start_transaction(&settings);
+    let mut_repo = tx.mut_repo();
+
+    // Construct 3 separate subgraphs off the root commit.
+    // 1 is a chain, 2 is a merge, 3 is a pyramidal monstrosity
+    let graph1commit1 = write_random_commit(mut_repo, &settings);
+    let graph1commit2 = create_random_commit(mut_repo, &settings)
+        .set_parents(vec![graph1commit1.id().clone()])
+        .write()
+        .unwrap();
+    let graph1commit3 = create_random_commit(mut_repo, &settings)
+        .set_parents(vec![graph1commit2.id().clone()])
+        .write()
+        .unwrap();
+    let graph2commit1 = write_random_commit(mut_repo, &settings);
+    let graph2commit2 = write_random_commit(mut_repo, &settings);
+    let graph2commit3 = create_random_commit(mut_repo, &settings)
+        .set_parents(vec![graph2commit1.id().clone(), graph2commit2.id().clone()])
+        .write()
+        .unwrap();
+    let graph3commit1 = write_random_commit(mut_repo, &settings);
+    let graph3commit2 = write_random_commit(mut_repo, &settings);
+    let graph3commit3 = write_random_commit(mut_repo, &settings);
+    let graph3commit4 = create_random_commit(mut_repo, &settings)
+        .set_parents(vec![graph3commit1.id().clone(), graph3commit2.id().clone()])
+        .write()
+        .unwrap();
+    let graph3commit5 = create_random_commit(mut_repo, &settings)
+        .set_parents(vec![graph3commit2.id().clone(), graph3commit3.id().clone()])
+        .write()
+        .unwrap();
+    let graph3commit6 = create_random_commit(mut_repo, &settings)
+        .set_parents(vec![graph3commit3.id().clone()])
+        .write()
+        .unwrap();
+    let graph3commit7 = create_random_commit(mut_repo, &settings)
+        .set_parents(vec![graph3commit4.id().clone(), graph3commit5.id().clone()])
+        .write()
+        .unwrap();
+
+    // Domain is respected.
+    assert_eq!(
+        resolve_commit_ids(
+            mut_repo,
+            &format!(
+                "reachable({}, all() ~ ::{})",
+                graph1commit2.id().hex(),
+                graph1commit1.id().hex()
+            )
+        ),
+        vec![graph1commit3.id().clone(), graph1commit2.id().clone(),]
+    );
+    assert_eq!(
+        resolve_commit_ids(
+            mut_repo,
+            &format!(
+                "reachable({}, all() ~ ::{})",
+                graph1commit2.id().hex(),
+                graph1commit3.id().hex()
+            )
+        ),
+        vec![]
+    );
+
+    // Each graph is identifiable from any node in it.
+    for (i, commit) in [&graph1commit1, &graph1commit2, &graph1commit3]
+        .iter()
+        .enumerate()
+    {
+        assert_eq!(
+            resolve_commit_ids(
+                mut_repo,
+                &format!("reachable({}, all() ~ root())", commit.id().hex())
+            ),
+            vec![
+                graph1commit3.id().clone(),
+                graph1commit2.id().clone(),
+                graph1commit1.id().clone(),
+            ],
+            "commit {}",
+            i + 1
+        );
+    }
+
+    for (i, commit) in [&graph2commit1, &graph2commit2, &graph2commit3]
+        .iter()
+        .enumerate()
+    {
+        assert_eq!(
+            resolve_commit_ids(
+                mut_repo,
+                &format!("reachable({}, all() ~ root())", commit.id().hex())
+            ),
+            vec![
+                graph2commit3.id().clone(),
+                graph2commit2.id().clone(),
+                graph2commit1.id().clone(),
+            ],
+            "commit {}",
+            i + 1
+        );
+    }
+
+    for (i, commit) in [
+        &graph3commit1,
+        &graph3commit2,
+        &graph3commit3,
+        &graph3commit4,
+        &graph3commit5,
+        &graph3commit6,
+        &graph3commit7,
+    ]
+    .iter()
+    .enumerate()
+    {
+        assert_eq!(
+            resolve_commit_ids(
+                mut_repo,
+                &format!("reachable({}, all() ~ root())", commit.id().hex())
+            ),
+            vec![
+                graph3commit7.id().clone(),
+                graph3commit6.id().clone(),
+                graph3commit5.id().clone(),
+                graph3commit4.id().clone(),
+                graph3commit3.id().clone(),
+                graph3commit2.id().clone(),
+                graph3commit1.id().clone(),
+            ],
+            "commit {}",
+            i + 1
+        );
+    }
+
+    // Test a split of the pyramidal monstrosity.
+    assert_eq!(
+        resolve_commit_ids(
+            mut_repo,
+            &format!(
+                "reachable({}, all() ~ ::{})",
+                graph3commit4.id().hex(),
+                graph3commit5.id().hex()
+            )
+        ),
+        vec![
+            graph3commit7.id().clone(),
+            graph3commit4.id().clone(),
+            graph3commit1.id().clone(),
+        ]
+    );
+}
+
 #[test]
 fn test_evaluate_expression_descendants() {
     let settings = testutils::user_settings();
@@ -1537,7 +1751,7 @@ fn test_evaluate_expression_descendants() {
 
     // The descendants of the root commit are all the commits in the repo
     assert_eq!(
-        resolve_commit_ids(mut_repo, "root():"),
+        resolve_commit_ids(mut_repo, "root()::"),
         vec![
             commit6.id().clone(),
             commit5.id().clone(),
@@ -1551,7 +1765,7 @@ fn test_evaluate_expression_descendants() {
 
     // Can find descendants of a specific commit
     assert_eq!(
-        resolve_commit_ids(mut_repo, &format!("{}:", commit2.id().hex())),
+        resolve_commit_ids(mut_repo, &format!("{}::", commit2.id().hex())),
         vec![
             commit6.id().clone(),
             commit5.id().clone(),
@@ -1563,7 +1777,7 @@ fn test_evaluate_expression_descendants() {
     // Can find descendants of children or children of descendants, which may be
     // optimized to single query
     assert_eq!(
-        resolve_commit_ids(mut_repo, &format!("({}+):", commit1.id().hex())),
+        resolve_commit_ids(mut_repo, &format!("({}+)::", commit1.id().hex())),
         vec![
             commit6.id().clone(),
             commit5.id().clone(),
@@ -1573,7 +1787,7 @@ fn test_evaluate_expression_descendants() {
         ]
     );
     assert_eq!(
-        resolve_commit_ids(mut_repo, &format!("({}++):", commit1.id().hex())),
+        resolve_commit_ids(mut_repo, &format!("({}++)::", commit1.id().hex())),
         vec![
             commit6.id().clone(),
             commit5.id().clone(),
@@ -1583,7 +1797,7 @@ fn test_evaluate_expression_descendants() {
     assert_eq!(
         resolve_commit_ids(
             mut_repo,
-            &format!("(({}|{}):)+", commit4.id().hex(), commit2.id().hex()),
+            &format!("(({}|{})::)+", commit4.id().hex(), commit2.id().hex()),
         ),
         vec![
             commit6.id().clone(),
@@ -1594,7 +1808,7 @@ fn test_evaluate_expression_descendants() {
     assert_eq!(
         resolve_commit_ids(
             mut_repo,
-            &format!("(({}|{})+):", commit4.id().hex(), commit2.id().hex()),
+            &format!("(({}|{})+)::", commit4.id().hex(), commit2.id().hex()),
         ),
         vec![
             commit6.id().clone(),
@@ -2058,7 +2272,7 @@ fn test_evaluate_expression_merges() {
     );
     // Searches only among candidates if specified
     assert_eq!(
-        resolve_commit_ids(mut_repo, &format!(":{} & merges()", commit5.id().hex())),
+        resolve_commit_ids(mut_repo, &format!("::{} & merges()", commit5.id().hex())),
         vec![commit5.id().clone()]
     );
 }
@@ -2334,7 +2548,7 @@ fn test_evaluate_expression_union() {
     assert_eq!(
         resolve_commit_ids(
             mut_repo,
-            &format!(":{} | :{}", commit4.id().hex(), commit5.id().hex())
+            &format!("::{} | ::{}", commit4.id().hex(), commit5.id().hex())
         ),
         vec![
             commit5.id().clone(),
@@ -2351,7 +2565,7 @@ fn test_evaluate_expression_union() {
         resolve_commit_ids(
             mut_repo,
             &format!(
-                "(:{} ~ :{}) | :{}",
+                "(::{} ~ ::{}) | ::{}",
                 commit4.id().hex(),
                 commit2.id().hex(),
                 commit5.id().hex()
@@ -2372,7 +2586,7 @@ fn test_evaluate_expression_union() {
         resolve_commit_ids(
             mut_repo,
             &format!(
-                "(:{} ~ :{}) | {}",
+                "(::{} ~ ::{}) | {}",
                 commit4.id().hex(),
                 commit2.id().hex(),
                 commit5.id().hex(),
@@ -2406,7 +2620,7 @@ fn test_evaluate_expression_intersection() {
     assert_eq!(
         resolve_commit_ids(
             mut_repo,
-            &format!(":{} & :{}", commit4.id().hex(), commit5.id().hex())
+            &format!("::{} & ::{}", commit4.id().hex(), commit5.id().hex())
         ),
         vec![
             commit2.id().clone(),
@@ -2443,7 +2657,7 @@ fn test_evaluate_expression_difference() {
 
     // Difference from all
     assert_eq!(
-        resolve_commit_ids(mut_repo, &format!("~:{}", commit5.id().hex())),
+        resolve_commit_ids(mut_repo, &format!("~::{}", commit5.id().hex())),
         vec![commit4.id().clone(), commit3.id().clone()]
     );
 
@@ -2451,28 +2665,28 @@ fn test_evaluate_expression_difference() {
     assert_eq!(
         resolve_commit_ids(
             mut_repo,
-            &format!(":{} ~ :{}", commit4.id().hex(), commit5.id().hex())
+            &format!("::{} ~ ::{}", commit4.id().hex(), commit5.id().hex())
         ),
         vec![commit4.id().clone(), commit3.id().clone()]
     );
     assert_eq!(
         resolve_commit_ids(
             mut_repo,
-            &format!(":{} ~ :{}", commit5.id().hex(), commit4.id().hex())
+            &format!("::{} ~ ::{}", commit5.id().hex(), commit4.id().hex())
         ),
         vec![commit5.id().clone()]
     );
     assert_eq!(
         resolve_commit_ids(
             mut_repo,
-            &format!("~:{} & :{}", commit4.id().hex(), commit5.id().hex())
+            &format!("~::{} & ::{}", commit4.id().hex(), commit5.id().hex())
         ),
         vec![commit5.id().clone()]
     );
     assert_eq!(
         resolve_commit_ids(
             mut_repo,
-            &format!(":{} ~ :{}", commit4.id().hex(), commit2.id().hex())
+            &format!("::{} ~ ::{}", commit4.id().hex(), commit2.id().hex())
         ),
         vec![commit4.id().clone(), commit3.id().clone()]
     );
@@ -2482,7 +2696,7 @@ fn test_evaluate_expression_difference() {
         resolve_commit_ids(
             mut_repo,
             &format!(
-                ":{} ~ {} ~ {}",
+                "::{} ~ {} ~ {}",
                 commit4.id().hex(),
                 commit2.id().hex(),
                 commit3.id().hex()
@@ -2500,7 +2714,7 @@ fn test_evaluate_expression_difference() {
         resolve_commit_ids(
             mut_repo,
             &format!(
-                "(:{} ~ :{}) ~ (:{} ~ :{})",
+                "(::{} ~ ::{}) ~ (::{} ~ ::{})",
                 commit4.id().hex(),
                 commit1.id().hex(),
                 commit3.id().hex(),
@@ -2633,10 +2847,9 @@ fn test_evaluate_expression_file() {
 
     let resolve = |file_path: &RepoPath| -> Vec<CommitId> {
         let mut_repo = &*mut_repo;
-        let expression =
-            RevsetExpression::filter(RevsetFilterPredicate::File(Some(
-                vec![file_path.to_owned()],
-            )));
+        let expression = RevsetExpression::filter(RevsetFilterPredicate::File(
+            FilesetExpression::prefix_path(file_path.to_owned()),
+        ));
         let revset = expression.evaluate_programmatic(mut_repo).unwrap();
         revset.iter().collect()
     };
@@ -2668,7 +2881,7 @@ fn test_evaluate_expression_file() {
     assert_eq!(
         resolve_commit_ids_in_workspace(
             mut_repo,
-            &format!(r#"{}: & file("added_modified_clean")"#, commit2.id().hex()),
+            &format!(r#"{}:: & file("added_modified_clean")"#, commit2.id().hex()),
             &test_workspace.workspace,
             Some(test_workspace.workspace.workspace_root()),
         ),
@@ -2677,7 +2890,7 @@ fn test_evaluate_expression_file() {
 
     // empty() revset, which is identical to ~file(".")
     assert_eq!(
-        resolve_commit_ids(mut_repo, &format!("{}: & empty()", commit1.id().hex())),
+        resolve_commit_ids(mut_repo, &format!("{}:: & empty()", commit1.id().hex())),
         vec![commit4.id().clone()]
     );
 }
@@ -2753,7 +2966,7 @@ fn test_reverse_graph_iterator() {
         repo.as_ref(),
         &[&commit_a, &commit_c, &commit_d, &commit_e, &commit_f],
     );
-    let commits = ReverseRevsetGraphIterator::new(revset.iter_graph()).collect_vec();
+    let commits = ReverseGraphIterator::new(revset.iter_graph()).collect_vec();
     assert_eq!(commits.len(), 5);
     assert_eq!(commits[0].0, *commit_a.id());
     assert_eq!(commits[1].0, *commit_c.id());
@@ -2762,23 +2975,17 @@ fn test_reverse_graph_iterator() {
     assert_eq!(commits[4].0, *commit_f.id());
     assert_eq!(
         commits[0].1,
-        vec![RevsetGraphEdge::indirect(commit_c.id().clone())]
+        vec![GraphEdge::indirect(commit_c.id().clone())]
     );
     assert_eq!(
         commits[1].1,
         vec![
-            RevsetGraphEdge::direct(commit_e.id().clone()),
-            RevsetGraphEdge::direct(commit_d.id().clone()),
+            GraphEdge::direct(commit_e.id().clone()),
+            GraphEdge::direct(commit_d.id().clone()),
         ]
     );
-    assert_eq!(
-        commits[2].1,
-        vec![RevsetGraphEdge::direct(commit_f.id().clone())]
-    );
-    assert_eq!(
-        commits[3].1,
-        vec![RevsetGraphEdge::direct(commit_f.id().clone())]
-    );
+    assert_eq!(commits[2].1, vec![GraphEdge::direct(commit_f.id().clone())]);
+    assert_eq!(commits[3].1, vec![GraphEdge::direct(commit_f.id().clone())]);
     assert_eq!(commits[4].1, vec![]);
 }
 
@@ -2803,3 +3010,26 @@ fn test_no_such_revision_suggestion() {
         if name == "bax" && candidates == vec!["bar".to_string(), "baz".to_string()]
     );
 }
+
+#[test]
+fn test_revset_containing_fn() {
+    let settings = testutils::user_settings();
+    let test_repo = TestRepo::init();
+    let repo = &test_repo.repo;
+
+    let mut tx = repo.start_transaction(&settings);
+    let mut_repo = tx.mut_repo();
+    let commit_a = write_random_commit(mut_repo, &settings);
+    let commit_b = write_random_commit(mut_repo, &settings);
+    let commit_c = write_random_commit(mut_repo, &settings);
+    let commit_d = write_random_commit(mut_repo, &settings);
+    let repo = tx.commit("test");
+
+    let revset = revset_for_commits(repo.as_ref(), &[&commit_b, &commit_d]);
+
+    let revset_has_commit = revset.containing_fn();
+    assert!(!revset_has_commit(commit_a.id()));
+    assert!(revset_has_commit(commit_b.id()));
+    assert!(!revset_has_commit(commit_c.id()));
+    assert!(revset_has_commit(commit_d.id()));
+}
