@@ -947,9 +947,18 @@ fn test_rebase_descendants_contents() {
     let tree_c = commit_c.tree().unwrap();
     let tree_d = commit_d.tree().unwrap();
     let new_tree_c = new_commit_c.tree().unwrap();
-    assert_eq!(new_tree_c.path_value(path3), tree_c.path_value(path3));
-    assert_eq!(new_tree_c.path_value(path4), tree_d.path_value(path4));
-    assert_ne!(new_tree_c.path_value(path2), tree_b.path_value(path2));
+    assert_eq!(
+        new_tree_c.path_value(path3).unwrap(),
+        tree_c.path_value(path3).unwrap()
+    );
+    assert_eq!(
+        new_tree_c.path_value(path4).unwrap(),
+        tree_d.path_value(path4).unwrap()
+    );
+    assert_ne!(
+        new_tree_c.path_value(path2).unwrap(),
+        tree_b.path_value(path2).unwrap()
+    );
 }
 
 #[test]
@@ -1483,7 +1492,7 @@ fn test_rebase_descendants_update_checkout_abandoned_merge() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    // Checked-out merge commit D was abandoned. A parent commit should become
+    // Checked-out merge commit D was abandoned. A new merge commit should become
     // checked out.
     //
     // D
@@ -1518,7 +1527,10 @@ fn test_rebase_descendants_update_checkout_abandoned_merge() {
 
     let new_checkout_id = repo.view().get_wc_commit_id(&workspace_id).unwrap();
     let checkout = repo.store().get_commit(new_checkout_id).unwrap();
-    assert_eq!(checkout.parent_ids(), vec![commit_b.id().clone()]);
+    assert_eq!(
+        checkout.parent_ids(),
+        vec![commit_b.id().clone(), commit_c.id().clone()]
+    );
 }
 
 #[test_case(EmptyBehaviour::Keep; "keep all commits")]
